package jvpc.blitz;

import java.io.*;
import java.net.*;

import java.awt.*;
import java.awt.event.*;
import kero.awt.*;

import jvpc.vga.*;

public class Persistent extends MyFrame {
    MySocket socket;
    String game;
    int race;
    InputFileOpener ifo;
    int turnNumber = 0;
    Label deadline;
    TextArea message;
    int remaining = -1;
    Button request, submit;
    CountDown countdown;

    public Persistent(MySocket socket, String game, int race,
		    InputFileOpener ifo, FrameAdapter adapter) {
	super("Blitz Stand-Alone");
	addWindowListener(adapter);
	this.socket = socket;
	this.game = game;
	this.race = race;
	this.ifo = ifo;
	setLayout(new BorderLayout());
	deadline = new Label("No hosting deadline provided, yet");
	add(deadline, "North");
	Panel panel = new Panel(new GridLayout(1, 0));
	request = new Button("Request rst");
	addButtonTo(panel, request, KeyEvent.VK_R, "request");
	submit = new Button("Submit trn");
	addButtonTo(panel, submit, KeyEvent.VK_S, "submit");
	add(panel, "South");
	message = new TextArea("Message Area\n", 5, 40, TextArea.SCROLLBARS_VERTICAL_ONLY);
	add(message, "Center");
	pack();
	show();
	message.setCaretPosition(message.getText().length());
	countdown = new CountDown(this, deadline);
	countdown.start();
    }

    class CountDown extends Thread {
	Persistent persistent;
	Label deadline;
	public CountDown(Persistent p, Label deadline) {
	    this.persistent = p;
	    this.deadline = deadline;
	}
	public void run() {
	    while (true) {
		if (remaining == -1)
		    deadline.setText("Turn "+turnNumber+", No hosting scheduled");
		else
		    deadline.setText("Turn "+turnNumber+", Hosting in approx. "+remaining+" seconds");
		// persistent.pack();
		try {
		    Thread.sleep(1000);
		} catch (InterruptedException ie) {}
		if (remaining > 0) remaining--;
	    }
	}
    }

    public void setMessage(String str) {
	message.append(str+"\n");
    }

    public void setTurnNumber(int nr) {
	turnNumber = nr;
    }

    public void request() {
	try {
	    socket.write("rst "+game+" "+race+"\n");
	} catch (IOException ioe) {
	    System.err.println("Requesting RST failed: "+ioe);
	}
    }

    public void submit() {
	System.err.println("Submitting TRN...");
	try {
	    FileInputStream fis = ifo.open("player"+race+".trn", true);
	    File f = ifo.getLastOpenedFile();
	    int size = (int)f.length();
	    byte[] trn = new byte[size];
	    fis.read(trn);
	    fis.close();
	    socket.write("trn "+game+" "+race+" "+size+"\n");
	    socket.write(trn);
	    System.err.println("TRN Submitted!");
	} catch (IOException ioe) {
	    new MyDialog(this, ioe.toString(), "TRN not submitted");
	    System.err.println("Submitting TRN failed: "+ioe);
	}
    }
}

