// File: BaseFrame.java

package jvpc.jvpc;

import java.awt.*;
import java.awt.event.*;
import kero.awt.*;
import jvpc.vga.*;

/**
 * Presents a <CODE>Base</code> in a <CODE>Frame</code>. Does not have
 * <CODE>Container</code> functionality (explicitely dropped).
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since     4 II 2000
 * @version   18 III 2001
 */
public class BaseFrame
    extends MyFrame
    implements VGAPlanets, BaseListener, PlanetListener,
	       ActionListener, KeyListener
{
    JVPC jvpc;
    Game game;
    BData base;

    Label owner, damage;
    Button prev, next;
    InputChoice mission, fixRec;
    IntInput fixRecTarget, defense, fighters;
    Label tech;
    InputChoice buildHull, buildEngine, buildBeam, buildLaunch;
    int prevHullIndex = 0, prevEngineIndex = 0,
	prevBeamIndex = 0, prevLauncherIndex = 0;
    IntInput beamCount, launchCount;
    Label buildShip;
    Button planet, cargo, detail;

    // Need to be able to buy torpedoes...

    BaseFrame(BData frameBase, JVPC jvpc) {
	super();
	this.base = frameBase;
	this.game = base.getUniverse().getGame();
	this.jvpc = jvpc;

	base.getUniverse().addBaseListener(this);
	// base.getUniverse().addPlanetListener(this);

	setLayout(new GridLayout(0,1));

	Panel p = new Panel(new GridLayout(1, 0));
    prev = new Button("Prev");
    addButtonTo(p, prev, KeyEvent.VK_P, "prevBase");
    next = new Button("Next");
    addButtonTo(p, next, KeyEvent.VK_N, "nextBase");
    add(p);

    p = new Panel(new GridLayout(1, 0));
	owner = new Label("owner: 10");
	p.add(owner);
	damage = new Label("DAMAGE: 99%");
	p.add(damage);
	add(p);

	mission = new InputChoice("Mission:");
	for (int i=0; i<BaseMission.BASEMISSIONS; i++) {
	    mission.addSelection(BaseMission.name[i]);
	}
	addInput(mission, KeyEvent.VK_M, "selectBaseMission");

	FlowLayout flow = new FlowLayout();
	// flow.setHgap(0);
	// flow.setVgap(0);
	p = new Panel(new GridLayout(1, 0));
	fixRec = new InputChoice("");
	fixRec.addSelection("None");
	fixRec.addSelection("Fix:");
	fixRec.addSelection("Recycle:");
	addInputTo(p, fixRec, KeyEvent.VK_R, "selectFixRec");
	fixRecTarget = new IntInput("", 999, 1, 0);
	addNumberInputTo(p, fixRecTarget, KeyEvent.VK_3, "selectTarget");
	add(p);

	defense = new IntInput("Defense:", base.getBaseDefense(), 25, 2);
	addNumberInput(defense, KeyEvent.VK_D, "setBaseDefense");
	fighters = new IntInput("Fighters:", base.getFighters(), 25, 2);
	addNumberInput(fighters, KeyEvent.VK_F, "setBaseFighters");

	Game game = base.getGame();

	tech = new Label("hull: eng: beam: torp: ");
	add(tech);
	buildHull = new InputChoice("build Hull:");
	buildHull.addSelection("None");
	for (int i=1; i<=buildhulls; i++) {
	    Hull hull = game.getHull(base.getOwner(), i);
	    if (hull != null) {
		buildHull.addSelection(hull.getName());
	    }
	}
	addInput(buildHull, KeyEvent.VK_H, "selectBuildHull");

	buildEngine = new InputChoice("build Engines: 20");
	buildEngine.addSelection("None");
	for (int i=1; i<=engines; i++) {
	    Engine engine = game.getEngine(i);
	    buildEngine.addSelection(engine.getName());
	}
	addInput(buildEngine, KeyEvent.VK_E, "selectBuildEngine");

	buildBeam = new InputChoice("build Beam: 99");
	buildBeam.addSelection("None");
	for (int i=1; i<=beams; i++) {
	    Beam beam = game.getBeam(i);
	    buildBeam.addSelection(beam.getName());
	}
	addInput(buildBeam, KeyEvent.VK_B, "selectBuildBeam");

	buildLaunch = new InputChoice("build Launchers: 20");
	buildLaunch.addSelection("None");
	for (int i=1; i<=torpedoes; i++) {
	    Torpedo launcher = game.getTorpedo(i);
	    buildLaunch.addSelection(launcher.getName());
	}
	addInput(buildLaunch, KeyEvent.VK_L, "selectBuildLauncher");

	buildShip = new Label("not building a ship");
	add(buildShip);

	p = new Panel(new GridLayout(1, 0));
	cargo = new Button("carGo");
	addButtonTo(p, cargo, KeyEvent.VK_G, "viewCargoFrame");
	planet = new Button("Planet");
	addButtonTo(p, planet, KeyEvent.VK_P, "popupPlanet");
	detail = new Button("Torpedoes");
	addButtonTo(p, detail, KeyEvent.VK_T, "viewDetailFrame");
	add(p);

	setBase(frameBase, true);
	pack();
	limitSize();
	show();
    }


    // KeyListener methods

    public void keyPressed(KeyEvent ke) {
	super.keyPressed(ke);
	boolean altHeld = (ke.getModifiers() & KeyEvent.ALT_MASK) != 0;
	boolean ctrlHeld = (ke.getModifiers() & KeyEvent.CTRL_MASK) != 0;
	boolean shiftHeld = (ke.getModifiers() & KeyEvent.SHIFT_MASK) != 0;

	int keyCode = ke.getKeyCode();
	switch (keyCode) {
	case KeyEvent.VK_EQUALS:
	    if (!altHeld && !ctrlHeld) break;
	case KeyEvent.VK_PAGE_DOWN:
	    nextBase(!shiftHeld);
	    break;
	    // case KeyEvent.VK_DASH: // where is it... [Duke3D Nuke'm]
	    //   if (!altHeld && !ctrlHeld) break;
	case KeyEvent.VK_PAGE_UP:
	    prevBase(!shiftHeld);
	    break;
	default:
	    // ignore
	    break;
	}
    }

    //public void keyReleased(KeyEvent ke) {
    //  super.keyReleased(ke);
    //}

    //public void keyTyped(KeyEvent ke) {
    //  super.keyTyped(ke);
    //}


    public void prevBase() {
	prevBase(true);
    }

    public void prevBase(boolean visitAll) {
	int newId = base.getId() - 1;
	if (newId == 0) newId = planets;
	while (newId != base.getId()) {
	    BData newBase = base.getUniverse().getBData(newId);
	    if (newBase != null) {
		if ((newBase.getOwner() == base.getOwner()) || visitAll)
		    break;
	    }
	    newId--;
	    if (newId == 0) newId = planets;
	}
	setBase(base.getUniverse().getBData(newId), true);
    }

    public void nextBase() {
	nextBase(true);
    }

    public void nextBase(boolean visitAll) {
	int newId = base.getId() + 1;
	if (newId > planets) newId = 1;
	while (newId != base.getId()) {
	    BData newBase = base.getUniverse().getBData(newId);
	    if (newBase != null) {
		if ((newBase.getOwner() == base.getOwner()) || visitAll)
		  break;
	    }
	    newId++;
	    if (newId > planets) newId = 1;
	}
	setBase(base.getUniverse().getBData(newId), true);
    }


    public void report(String warning) {
	game.report(warning);
    }


    // BaseFrame methods

    public void setBaseDefense() {
	base.buildUp(Build.BASEDEFENSE, defense.getInt() - base.getBaseDefense());
	defense.setInt(base.getBaseDefense());
    }

    public void setBaseFighters() {
	base.buildFighters(fighters.getInt() - base.getFighters());
	fighters.setInt(base.getFighters());
    }

    public void selectBaseMission() {
	int index = mission.getIndex();
	//report("Selected base mission: "+index+"; "+BaseMission.name[index]);
	base.setMission(index);
    }

    public void selectFixRec() {
	int index = fixRec.getIndex();
	base.setFixRec(index);
    }

    public void selectTarget() {
	int target = fixRecTarget.getInt();
	base.setTarget(target);
    }

    protected void tryToBuildShip() {
	int hullIndex = buildHull.getIndex();
	if (hullIndex == 0) return;
	Hull hull = base.getGame().getHull(base.getOwner(), hullIndex);
	base.setBuildShip(hullIndex, buildEngine.getIndex(),
			  buildBeam.getIndex(), hull.getBeams(),
			  buildLaunch.getIndex(), hull.getLaunchs());
    }

    public void selectBuildHull() {
	// can build hull, but need to break down engine, beam + launch first
	// and only rebuild the selected items after the new hull is in
	// place (for we may need other amounts of the various items).
	int oldIndex = prevHullIndex;
	if (oldIndex > 0) {
	    Hull oldHull = base.getGame().getHull(base.getOwner(), oldIndex);
	    base.cancelBuildShip();
	    base.breakDownHulls(oldIndex, 1);
	    base.breakDownEngines(buildEngine.getIndex(), oldHull.getEngines());
	    base.breakDownBeams(buildBeam.getIndex(), oldHull.getBeams());
	    base.breakDownLaunchers(buildLaunch.getIndex(),oldHull.getLaunchs());
	    base.lowerHullTech(VGAPlanets.maxTechLevel);
	    base.lowerEngineTech(VGAPlanets.maxTechLevel);
	    base.lowerBeamTech(VGAPlanets.maxTechLevel);
	    base.lowerTorpTech(VGAPlanets.maxTechLevel);
	}
	int index = buildHull.getIndex();
	prevHullIndex = index;
	if (index > 0) {
	    warn("Trying to build new hull "+index);
	    Hull hull = base.getGame().getHull(base.getOwner(), index);
	    warn("Hull: "+hull);
	    if (hull.getTech() > base.getHullTech()) {
		warn("Must raise hull tech level first");
		base.raiseHullTech(hull.getTech() - base.getHullTech());
	    }
	    if (base.getHulls(index) == 0) base.buildHulls(index, 1);
	    doBuildEngines(hull);
	    doBuildBeams(hull);
	    doBuildLaunchers(hull);
	}
	tryToBuildShip();
    }

    protected void doBuildEngines(Hull hull) {
	int index = buildEngine.getIndex();
	if (index > 0) {
	    Engine engine = base.getGame().getEngine(index);
	    base.raiseEngineTech(engine.getTech() - base.getEngineTech());
	    int needEngines = hull.getEngines() - base.getEngines(index);
	    if (needEngines > 0) base.buildEngines(index, needEngines);
	}
    }

    public void selectBuildEngine() {
	int hullIndex = buildHull.getIndex();
	if (hullIndex == 0) return;
	Hull hull = base.getGame().getHull(base.getOwner(), hullIndex);
	// break down old engines
	if (prevEngineIndex > 0) {
	    base.breakDownEngines(prevEngineIndex, hull.getEngines());
	    base.lowerEngineTech(maxTechLevel);
	}
	prevEngineIndex = buildEngine.getIndex();
	doBuildEngines(hull);
	tryToBuildShip();
    }

    protected void doBuildBeams(Hull hull) {
	int index = buildBeam.getIndex();
	if (index > 0) {
	    Beam beam = base.getGame().getBeam(index);
	    base.raiseBeamTech(beam.getTech() - base.getBeamTech());
	    int needBeams = hull.getBeams() - base.getBeams(index);
	    if (needBeams > 0) base.buildBeams(index, needBeams);
	}
    }

    public void selectBuildBeam() {
	int hullIndex = buildHull.getIndex();
	if (hullIndex == 0) return;
	Hull hull = base.getGame().getHull(base.getOwner(), hullIndex);
	// break down old beams
	if (prevBeamIndex > 0) {
	    base.breakDownBeams(prevBeamIndex, hull.getBeams());
	    base.lowerBeamTech(maxTechLevel);
	}
	prevBeamIndex = buildBeam.getIndex();
	doBuildBeams(hull);
	tryToBuildShip();
    }

    protected void doBuildLaunchers(Hull hull) {
	int index = buildLaunch.getIndex();
	if (index > 0) {
	    Torpedo launcher = base.getGame().getTorpedo(index);
	    base.raiseTorpTech(launcher.getTech() - base.getTorpTech());
	    int needLaunchs = hull.getLaunchs() - base.getLaunchers(index);
	    if (needLaunchs > 0) base.buildLaunchers(index, needLaunchs);
	}
    }

    public void selectBuildLauncher() {
	int hullIndex = buildHull.getIndex();
	if (hullIndex == 0) return;
	Hull hull = base.getGame().getHull(base.getOwner(), hullIndex);
	// break down old launchs
	if (prevLauncherIndex > 0) {
	    base.breakDownLaunchers(prevLauncherIndex, hull.getLaunchs());
	    base.lowerTorpTech(maxTechLevel);
	}
	prevLauncherIndex = buildLaunch.getIndex();
	doBuildLaunchers(hull);
	tryToBuildShip();
    }

    public void popupPlanet() {
	jvpc.viewPlanet(base);
    }

    private void setBunchOfLabels() {
	owner.setText("owner: "+base.getOwner());
	tech.setText("hull: "+base.getHullTech()+
		     ", eng: "+base.getEngineTech()+
		     ", beam: "+base.getBeamTech()+
		     ", torp: "+base.getTorpTech());
	if (base.getDamage() > 0) {
	    damage.setForeground(Color.red);
	    damage.setText("DAMAGE: "+base.getDamage()+"%");
	} else {
	    damage.setForeground(Color.black);
	    damage.setText("no damage");
	}
    }

    public void setBase(BData newBase) {
	setBase(newBase, true);
    }

    public void setBase(BData newBase, boolean skipShipBuild) {
	base = newBase;
	report("BaseFrame.setBase("+(newBase.getId())+", "+skipShipBuild+")");
	setTitle("B"+base.getId()+": "+base.getName());
	setBunchOfLabels();
	defense.setInt(base.getBaseDefense());
	fighters.setInt(base.getFighters());
	mission.setSelection(base.getMission());
	fixRec.setSelection(base.getFixRec());
	fixRecTarget.setInt(base.getTarget());

	if (base.getBuildHullType() == 0) {
	    buildShip.setForeground(Color.red);
	    buildShip.setText("not building a ship!");
	} else {
	    buildShip.setForeground(Color.black);
	    buildShip.setText("building this ship!");
	}

	if (!skipShipBuild /* && (base.getBuildHullType() == 0) */) return;

	warn("newBase.hull: "+base.getBuildHullType());

	if (skipShipBuild) { // hrm, looks nicer
	    buildHull.removeAll();
	    buildHull.addSelection("None");
	    Game game = base.getUniverse().getGame();
	    for (int i=1; i<=buildhulls; i++) {
		Hull hull = game.getHull(base.getOwner(), i);
		if (hull != null) {
		    buildHull.addSelection(hull.getName());
		}
	    }
	}

	buildHull.setSelection(base.getBuildHullType());
	Hull hull = game.getHull(base.getOwner(), base.getBuildHullType());
	buildEngine.setLabel("build Engine: "+
			     (hull!=null? ""+hull.getEngines(): ""));
	buildEngine.setSelection(base.getBuildEngineType());
	buildBeam.setLabel("build Beam: "+(hull!=null? ""+hull.getBeams(): ""));
	buildBeam.setSelection(base.getBuildBeamType());
	buildLaunch.setLabel("build Launcher: "+base.getBuildLaunchers());
	buildLaunch.setSelection(base.getBuildLaunchType());
    }

    public void viewCargoFrame() {
	Location location = base.getLocation();
	jvpc.viewCargoFrame(location);
    }

    public void viewDetailFrame() {
	jvpc.viewDetail(base);
    }

    public void planetChanged(PlanetEvent pe) {
	if (pe.getId() == base.getId()) setBase(base, false);
    }

    public void baseChanged(BaseEvent be) {
	if (be.getId() == base.getId()) setBase(base, false);
    }
}
