// File: CargoFrame.java

package jvpc.jvpc;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import kero.util.*;
import kero.awt.*;
import jvpc.vga.*;

/**
 * Displays cargo
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since     XII 1999
 * @version   31 I 2000
 */
public class CargoFrame
    extends MyFrame
    implements VGAPlanets, ShipListener, PlanetListener, BaseListener,
	       ActionListener, WindowListener
{
  Location location;
  Label neutrlabel, tritlabel, durlabel, molyblabel;
  Label supplabel, mclabel, clanslabel, ammolabel;
  IntInput[][] cargo;
  // Add torpedoes when owner is the same ...
  Vector located;
  CargoTransferable active;
  int ammotype;
  Component planetbutton;
  int ships; // amount of ships (== length of shipbutton)
  Button[] shipbutton; // ship Buttons
  CargoTransferable shownplanet; // planet shown for cargo transfer from a ship

  public CargoFrame(Location location) {
    super(JVPC.program+" -- Cargo @("+location.getX()+","+location.getY()+")");
    setLayout(new GridLayout(0,9));

    neutrlabel = new Label("Neutr");
    tritlabel = new Label("Trit");
    durlabel = new Label("Dur");
    molyblabel = new Label("Molyb");
    supplabel = new Label("Supp");
    mclabel = new Label("MC");
    clanslabel = new Label("Clans");
    ammolabel = new Label("Ammo");

    setLocation(location);
    addWindowListener(this);
    location.getUniverse().addBaseListener(this);
    location.getUniverse().addPlanetListener(this);
    location.getUniverse().addShipListener(this);

    show();
  }

  public void setLocation(Location loc) {
    location = loc;
    ships = 0;

    int length = location.getShips().size() + 1;
    setLayout(new GridLayout(length+1, 9));
    cargo = new IntInput[length][Cargo.CARGOTYPES];

    located = new Vector();
    shipbutton = new Button[length-1];

    removeAll();
    add(new Label());
    add(neutrlabel); add(tritlabel); add(durlabel); add(molyblabel);
    add(supplabel); add(mclabel); add(clanslabel); add(ammolabel);

    setPlanet(location.getPlanet());
    Enumeration enum = location.getShips().elements();
    while (enum.hasMoreElements()) {
      Ship ship = (Ship)enum.nextElement();
      addShip(ship);
    }

    ammolabel.setText("Ammo");
    pack();
    limitSize();

    active = null;
    int index = 0;
    while (active == null) {
      setActive(index);
      index++;
    }
  }

  public void setPlanet(Planet p) {
    if (p == null) {
      planetbutton = new Label("Jettison");
    } else {
      Button button = new Button("Planet "+p.getId());
      button.addActionListener(this);
      planetbutton = (Component)button;
      if (!p.isOurPlanet()) planetbutton.disable();
    }
    located.addElement(planetbutton);
    add(planetbutton);

    IntInput input;
    for (int j=0; j<Cargo.CARGOTYPES; j++) {
      cargo[0][j] = new IntInput(null, p==null?0:p.getCargo(j), 2, 0);
      add(cargo[0][j]);
      cargo[0][j].addActionListener(this);
    }
  }

  public void addShip(Ship s) {
    Assert.assert(s != null);

    Button sb = new Button("Ship "+s.getId());
    add(sb);
    located.addElement(sb);
    sb.addActionListener(this);
    shipbutton[ships] = sb;
    ships++; // har har har
    if (!s.isOurShip()) sb.disable();

    IntInput input;
    for (int j=0; j<Cargo.CARGOTYPES; j++) {
      cargo[ships][j] = new IntInput(null, s.isOurShip()?s.getCargo(j):0, 2, 0);
      add(cargo[ships][j]);
      cargo[ships][j].addActionListener(this);
    }
  }


    /* retrieval of info */

  private CargoTransferable getIndexed(int index) {
    if (index == 0) {
      return shownplanet;
    } else {
      return (CargoTransferable)location.getShip(index-1);
    }
  }


    /* enabling and disabling of IntInputs */

  private void enablePlanetFields() {
    //report("setActivePlanetFields()");
    for (int j=0; j<Cargo.CARGOTYPES; j++) {
      cargo[0][j].setInt(shownplanet.getCargo(j));
    }
    if (shownplanet instanceof ToPlanet) {
      cargo[0][Cargo.MC].disable();
    } else {
      cargo[0][Cargo.MC].enable();
    }
    if ((ammotype != NONE) && (shownplanet instanceof BData)) {
      cargo[0][Cargo.AMMUNITION].enable();
      //cargo[0][Cargo.AMMUNITION].setInt(shownplanet.getCargo(Cargo.AMMUNITION));
    } else {
      cargo[0][Cargo.AMMUNITION].disable();
      cargo[0][Cargo.AMMUNITION].setInt(0);  // cargo can be >0
    }
  }

  /**
   * Hrm. Planets can only transfer to ships with the same owner. Ships can
   * transfer to ships with the same owner, the planet (topl) and one other
   * ship (tosh) under TimHost. Ships can transfer to any other ship in PHost
   * (messaging system; I intend to use this a.s.a.p).
   *
   * <P>For now, only the same-owner transfer is enabled/implemented.
   */
  private void enableShipFields(int i, int owner) {
    //report("setActiveShipFields()");
      Ship s = location.getShip(i);
      for (int j=0; j<Cargo.CARGOTYPES-1; j++) {
	if ((s.getOwner() == owner) ||
	    (active instanceof Ship && j!=Cargo.MC && j!=Cargo.AMMUNITION)
	) {
	  cargo[i+1][j].enable();
	  if (s.getOwner() == owner) {
	      cargo[i+1][j].setInt(s.getCargo(j));
	  } else if (active instanceof Ship) {
	      if (s.getId() == ((Ship)active).getToShip().getShipId()) {
		  // System.out.println("toship("+s.getToShip().getShipId()+").cargo["+j+"]: "+s.getToShip().getCargo(j));
		  cargo[i+1][j].setInt(((Ship)active).getToShip().getCargo(j));
	      } else {
		  cargo[i+1][j].setInt(0);
	      }
	  } else {
	      System.out.println("I don't get it");
	  }
	} else {
	  cargo[i+1][j].disable();
	  cargo[i+1][j].setInt(0);
	}
      }
      if ((s.getOwner() == owner) && (ammotype != NONE) &&
	  ((s.getTorpedoType() == ammotype) ||
	   ((s.getBays() > 0) && (ammotype == Cargo.FIGHTER)))) {
	cargo[i+1][Cargo.AMMUNITION].enable();
      } else {
	cargo[i+1][Cargo.AMMUNITION].disable();
      }
  }

    private void enableAllShipFields(int owner) {
	for (int i=0; i<location.getShips().size(); i++) {
	    enableShipFields(i, owner);
	}
    }
    
  private void setActivePlanet() {
    Planet planet = location.getPlanet();
    if ((planet == null) || !planet.isOurPlanet()) return;
    /* first item in located is the planet */
    shownplanet = (CargoTransferable)(PData)planet;
    ammolabel.setText("---");
    ammotype = NONE;
    int owner = planet.getOwner();
    planetbutton.setForeground(Color.yellow);
    for (int i=0; i<location.getShips().size(); i++) {
      shipbutton[i].setForeground(Color.black);
    }
    active = (CargoTransferable)(PData)planet;
    enablePlanetFields();
    enableAllShipFields(owner);
  }

  /**
   * @param index in the list of buttons, not in location.getShips()
   */
  private void setActiveShip(int index) {
    index--; // make it index in location.getShips()
    Ship ship = location.getShip(index);
    if (!ship.isOurShip()) return;
    /* the planet can always take part in transfer (toplanet for a foreign
     * ship), so leave the IntInputs enabled */
    planetbutton.setForeground(Color.black);
    ammotype = ship.getAmmoType();
    if (location.getPlanet() instanceof BData) {
      (location.getBase()).setAmmoType(ammotype);
    }
    switch (ammotype) {
    case Cargo.FIGHTER:
	ammolabel.setText("Fighters");
	break;
    case NONE:
	ammolabel.setText("no ammo");
	break;
    default:
	ammolabel.setText(ship.getGame().getTorpedo(ammotype).getName());
	break;
    }
    int owner = 0;
    for (int i=0; i<location.getShips().size(); i++) {
      if (i == index) {
	shipbutton[i].setForeground(Color.yellow);
	owner = location.getShip(i).getOwner();
	Planet planet = location.getPlanet();
	if ((planet != null) && (planet.getOwner() == owner)) {
	  shownplanet = (CargoTransferable)(PData)planet;
	} else {
	  shownplanet = location.getShip(i).getToPlanet();
	}
	active = location.getShip(i);
	enablePlanetFields();
      } else {
	shipbutton[i].setForeground(Color.black);
      }
    }
    enableAllShipFields(owner);
  }

  /**
   * @param index in the list of Buttons
   */
  private void setActive(int index) {
    //report("setActive("+index+")");
    if (index == 0) {
      if (location.getPlanet() != null) {
	setActivePlanet();
      }
    } else {
      setActiveShip(index);
    }
  }

  public void actionPerformed(ActionEvent ae) {
    // don't have any addInput*(), no reason to call super.actionPerformed()
    //System.err.println("CargoFrame.actionPerformed("+ae+")");
    Object source = ae.getSource();
    if (source instanceof Button) {
      setActive(located.indexOf(source));
    } else {
      IntInput ii = (IntInput)source;
      if (!ii.isEnabled()) return;
      for (int i=0; i<ships+1; i++) {
	for (int j=0; j<Cargo.CARGOTYPES; j++) {
	  if (cargo[i][j] == ii) {
	      CargoTransferable target = getIndexed(i);
	      if (target != active) {
		  if (active instanceof Ship && target instanceof Ship &&
		      ((Ship)target).getOwner() != ((Ship)active).getOwner()
		  ) {
		      System.err.println("Enemy ship-to-ship transfer! setting toship.id="+((Ship)target).getId());
		      int old_toship_id = ((Ship)active).getToShip().getShipId();
		      if (old_toship_id != ((Ship)target).getId() && ii.getInt() == 0) return; // hack to update fields 
		      if (!((Ship)target).getUniverse().getShip(((Ship)target).getId()).isSeenBy(((Owned)active).getOwner())) {
			  System.err.println("You don't see this ship, can't transfer cargo!");
			  return;
		      }
		      ((Ship)active).getToShip().setShipId(((Ship)target).getId());
		      target = ((Ship)active).getToShip();
		  } //else {
		  active.transferCargoTo(target, j, ii.getInt());
		  //}
	      }
	      cargo[i][j].setInt(target.getCargo(j));
	      enableAllShipFields(((Owned)active).getOwner());
	  }
	}
      }
    }
  }

  private void updatePlanet() {
    Planet planet = location.getPlanet();
    for (int j=0; j<Cargo.CARGOTYPES; j++) {
      cargo[0][j].setInt(planet.getCargo(j));
    }
  }

  public void planetChanged(PlanetEvent pe) {
    if (location.getPlanet() != null) {
      if (location.getPlanet().getId() == pe.getId()) {
	  // only when not ship-dump by ally!
	if (shownplanet == location.getPlanet()) updatePlanet();
      }
    }
  }

  public void baseChanged(BaseEvent be) {
    if (location.getPlanet() != null) {
      if (location.getPlanet().getId() == be.getId()) {
	if (shownplanet == location.getPlanet()) updatePlanet();
      }
    }
  }

  /**
   * @param index in the list of buttons (not in location.getShips())
   */
  private void updateShipAt(Ship ship, int index) {
    for (int j=0; j<Cargo.CARGOTYPES; j++) {
      cargo[index][j].setInt(ship.getCargo(j));
    }
  }

  public void shipChanged(ShipEvent se) {
    for (int i=0; i<location.getShips().size(); i++) {
      Ship ship = location.getShip(i);
      if (ship.getId() == se.getId()) {
	updateShipAt(ship, i+1);
	if (shownplanet == ship.getToPlanet()) enablePlanetFields();
      }
    }
  }


  // WindowListener methods

  public void windowIconified(WindowEvent we) {}
  public void windowDeiconified(WindowEvent we) {}

  public void windowActivated(WindowEvent we) {}
  public void windowDeactivated(WindowEvent we) {}

  public void windowOpened(WindowEvent we) {}
  public void windowClosed(WindowEvent we) {}
  public void windowClosing(WindowEvent we) {
    location.getUniverse().removePlanetListener(this);
    location.getUniverse().removeShipListener(this);
  }
}
