// File: JVPC.java

package jvpc.jvpc;

import java.lang.reflect.Method;
import java.util.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import kero.util.*;
import kero.awt.*;
import jvpc.vga.*;
import jvpc.extdata.*;

/**
 * This class forms the main program of JVPC, the Java-Client for Tim
 * Wissemans VGA-Planets. The purpose of this class is solely to provide the
 * main function, which is <CODE>static</code>, while all other classes are
 * not. It is possible (in theory) to run more than one game from this class.
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since     10 VII 1998
 * @version   25 II 2001
 */
public class JVPC
    extends MyFrame
    implements Reportable, VGAPlanets, ActionListener, WindowListener
{
    public static final String program="JVPC";
    public static final String version="0.94";

    static FrameAdapter adapter = new FrameAdapter();

    Game game;
    JVPCConfig config;

    TextArea paper;
    Button map, message, score, jvcr, load, save, saveAndQuit, quitNoSave;
    Class classjvcr;
    Panel buttonPanel;

    boolean[] wantsToPlay = new boolean[races+1];
    Vector frame;
    ShipFrame shipFrame = null;
    PlanetFrame planetFrame = null;
    BaseFrame baseFrame = null;
    BaseDetailFrame detailFrame = null;
    MessageFrame messageFrame = null;
    CargoFrame cargoFrame = null;

    // OK, this is not a good place to put it,
    // but I do not have a better one atm. FIXME
    DataBase database = new DataBase();

    public JVPC() {
	setTitle(JVPC.program+" "+JVPC.version);
	setLayout(new BorderLayout());

	paper = new TextArea(
	     JVPC.program+" v"+JVPC.version+"\n"+
	     "Copyright by Kero van Gelder and Rutger Nijlunsing\n"+
	     "Distributed under the GNU GPL.\n\n"+
	     "Initializing...",
	     12,50,TextArea.SCROLLBARS_VERTICAL_ONLY
	);
	paper.setEditable(false);
	paper.addKeyListener(this);
	add(paper, "Center");

	map = new Button("Map");
	message = new Button("mEssage");
	jvcr = new Button("jVcr");
	score = new Button("sCore");
	load = new Button("Load");
	save = new Button("Save");
	saveAndQuit = new Button("save & Quit");
	quitNoSave = new Button("qUit (no save)");
	buttonPanel = new Panel(new GridLayout(2, 4));
	addButtonTo(buttonPanel, map, KeyEvent.VK_M, "openMapFrame");
	addButtonTo(buttonPanel, message, KeyEvent.VK_E, "openMessageFrame");
	addButtonTo(buttonPanel, score, KeyEvent.VK_C, "openScoreFrame");
	buttonPanel.add(new Label());
	/* Not accepted by tvt-kaffe :-( FIXME
	   try {
	   classjvcr = Class.forName("com.planetsserver.bruecke.jvcr.jvcr.JVCR");
	   addButtonTo(buttonPanel, jvcr, KeyEvent.VK_V, "launchJVCR");
	   } catch (ClassNotFoundException cnfe) {
	   buttonPanel.add(new Label(""));
	   } */
	addButtonTo(buttonPanel, load, KeyEvent.VK_L, "load");
	addButtonTo(buttonPanel, save, KeyEvent.VK_S, "save");
	addButtonTo(buttonPanel, saveAndQuit, KeyEvent.VK_Q, "saveAndQuit");
	addButtonTo(buttonPanel, quitNoSave, KeyEvent.VK_U, "quit");
	add(buttonPanel, "South");

	addWindowListener(adapter);

	frame = new Vector();

	pack();
	limitSize();
	show();
	message.addFocusListener(new FocusAdapter() {
	    public void focusGained() {
		save.requestFocus();
	    }
	});
    }


    // Methods behind Buttons

    public void load() {
	game.readWork(wantsToPlay);
	// supposedly, nobody else edites extdata, so don't read it
	report("Data loaded");
    }

    public void save() {
	game.writeAll();
	database.writeExtData(game.ofo);
    }

    public void saveAndQuit() {
	killExcessiveFrames();
	game.setReportable(new Reporter()); // write success of upcoming saving to stderr, not GUI
	save();
	quit();
    }


    // WindowListener methods

    public void windowOpened(WindowEvent we) {}
    public void windowClosed(WindowEvent we) {
	MyFrame f = (MyFrame)we.getSource();
	if (f == planetFrame) {
	    planetFrame = null;
	} else if (f == shipFrame) {
	    shipFrame = null;
	} else if (f == messageFrame) {
	    messageFrame = null;
	} else if (f == cargoFrame) {
	    cargoFrame = null;
	} else if (f == detailFrame) {
	    detailFrame = null;
	}
	frame.removeElement(f);
    }

    public void windowClosing(WindowEvent we) {}
    public void windowIconified(WindowEvent we) {}
    public void windowDeiconified(WindowEvent we) {}
    public void windowActivated(WindowEvent we) {}
    public void windowDeactivated(WindowEvent we) {}


    // New frames stuff

    /** @param f Preferably a kero.awt.MyFrame */
    void openFrame(Frame f) {
	frame.addElement(f);
	f.addWindowListener(adapter);
	f.addWindowListener(this);
    }

    public void openScoreFrame() {
	report("Opening Score...");
	Universe u = game.getUniverse();
	ScoreFrame f = new ScoreFrame(
	    JVPC.program+" -- Scores, "+u.getTurnString(), u, this
	);
	openFrame(f);
	report(" Done!");
    }

    public void openMapFrame() {
	report("Opening Map...");
	Universe u = game.getUniverse();
	MapFrame mapFrame = new MapFrame("Map: "+u.getTurnString(), u, this);
	openFrame(mapFrame);
	report(" done!");
    }

    public void openShipFrame(Ship sh) {
	report("Opening Ship Frame...");
	shipFrame = new ShipFrame(sh,this);
	openFrame(shipFrame);
	report(" done!");
    }

    public void viewShip(Ship sh) {
	if (shipFrame == null) {
	    openShipFrame(sh);
	} else {
	    shipFrame.setShip(sh);
	    shipFrame.toFront();
	}
    }

    public void openPlanetFrame(Planet planet) {
	if (planet instanceof PData) {
	    PData pdata = (PData)planet;
	    planetFrame = new PlanetFrame(pdata, this);
	    openFrame(planetFrame);
	} else {
	    report("Planet is not your own; not editable");
	}
    }

    public void viewPlanet(Planet planet) {
	if (planet instanceof PData) {
	    PData pdata = (PData)planet;
	    if (planetFrame==null) {
		openPlanetFrame(pdata);
	    } else {
		planetFrame.setPlanet(pdata);
		planetFrame.toFront();
	    }
	}
    }

    public void openBaseFrame(BData base) {
	baseFrame = new BaseFrame(base, this);
	openFrame(baseFrame);
    }

    public void viewBase(BData base) {
	if (baseFrame == null) {
	    openBaseFrame(base);
	} else {
	    baseFrame.setBase(base);
	    baseFrame.toFront();
	}
    }

    public void openDetailFrame(BData base) {
	detailFrame = new BaseDetailFrame(base, this);
	openFrame(detailFrame);
    }

    public void viewDetail(BData base) {
	if (detailFrame == null) {
	    openDetailFrame(base);
	} else {
	    detailFrame.setBase(base);
	    detailFrame.toFront();
	}
    }

    public void openCargoFrame(Location location) {
	cargoFrame = new CargoFrame(location);
	openFrame(cargoFrame);
    }

    public void viewCargoFrame(Location location) {
	if (cargoFrame == null) {
	    openCargoFrame(location);
	} else {
	    cargoFrame.setLocation(location);
	    cargoFrame.toFront();
	}
    }

    public void openMessageFrame() {
	Universe u = game.getUniverse();
	MessageFrame messageFrame = new MessageFrame(
	    JVPC.program+" -- Message, "+u.getTurnString(), game
	);
	openFrame(messageFrame);
    }

    public void launchJVCR() {
	for (int i=0; i<races; i++) {
	    if (this.game.getUniverse().getGen(i) != null) {
		//new JVCR("."+File.separator, globalArgs[0], globalArgs[0], i);
		String initName = "init";
		Class[] paramSet = new Class[] {
		    String.class,
		    String.class,
		    String.class,
		    int.class
		};
		try {
		    report("Getting a new instance now!");
		    Object objectjvcr = classjvcr.newInstance();
		    Method method = classjvcr.getMethod(initName, paramSet);
		    if (method != null) try {
			//report("JVPC calling "+method.getName());
			Object[] param = new Object[4];
			param[0] = "./";
			param[1] = "/home/kero/DOS/games/vga";
			param[2] = "/home/kero/DOS/games/vga/game2";
			param[3] = new Integer(3);
			method.invoke(objectjvcr, param);
		    } catch (Exception e1) {
			crash(102, "Method.invoke() threw "+e1);
		    }
		} catch (NoSuchMethodException e3) {
		    warn("Could not find method "+initName+"("+paramSet+")");
		} catch (Exception e2) {
		    crash(102, "Class.newInstance() threw "+e2);
		}
	    }
	}
    }

    public void killExcessiveFrames() {
	for (int i=0; i<frame.size(); i++) {
	    MyFrame myFrame = (MyFrame)frame.elementAt(i);
	    if (myFrame != null) myFrame.dispose();
	}
    }


    // Reportable Overriding

    public void report(String message) {
	if (reportable != null) {
	    // well, not quite the intention, but...
	    reportable.report(message);
	} else {
	    paper.append(message);
	    int length = message.length();
	    if (length <= 1) return;
	    if ((length >= 3) &&
		(message.substring(length - 3, length).equals("..."))) return;
	    paper.appendText("\n");
	}
    }


    // Starting / Running stuff

    public void parseParameters(String[] arg) {
	// set wantsToPlay, ignore parameters for now.
	wantsToPlay = new boolean[races+1];
	for (int i=1; i<=races; i++) wantsToPlay[i] = true;
    }

    public void readGame(String[] arg) {
	switch (arg.length) {
	case 0:
	    game = new Game();
	    break;
	case 1:
	    game = new Game(arg[0]);
	    break;
	case 2:
	    game = new Game(arg[0],arg[1]);
	    break;
	default:
	    System.err.println("Only the first two parameters are used");
	    game = new Game(arg[0],arg[1]);
	    break;
	}
	game.setReportable(this);
	game.readAll(wantsToPlay);
	// database.readExtData();
    }

    public void readConfig() {
	config = new JVPCConfig();
	try {
	    InputStream inFile=game.getIFO().open("jvpc.cfg",true);
	    LineReader source=new LineReader(inFile);
	    config.readFrom(source);
	} catch (IOException ioe) {
	    report("No ``jvpc.cfg'' found, reverting to defaults"+ioe);
	}
	config.printConfig(System.err);
    }

    public JVPCConfig getConfig() {
	return config;
    }

    /**
     * Meant to be called at start-up.
     */
    public void run(String[] arg) {
	parseParameters(arg);
	readGame(arg);
	if (game == null) {
	    report("No universe found!");
	} else {
	    run(game);
	    openMapFrame();
	    // openBaseFrame(game.getUniverse().getBData(406));
	}
    }

    public void run(Game game) {
	this.game = game;
	readConfig();
	// frame.setBackground(config.getBackgroundColor());
	paper.setCaretPosition(paper.getText().length());
	buttonPanel.enable();
	report("Initialization done!");
    }

    public void quit() {
	System.exit(0);
    }


    // Main program

    public static void main(String[] arg) {
	new JVPC().run(arg);
    }
}
