// File: LocationList.java

package jvpc.jvpc;

import java.util.*;
import java.awt.*;
import kero.util.*;
import jvpc.vga.*;

/**
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since     ? VII 1998 ?
 * @version   24 III 2001
 */
public class LocationList implements VGAPlanets {
    Map map;
    int lightline = ILLEGAL; // static is not nice, can have >1 Map open

    public LocationList(Map m, Location loc) {
	map = m;
    }

    /**
     * Given map-coordinates, compute activates element in list at location.
     */
    public Located selectFromList(int mapX, int mapY, Location location) {
	Located result;
	int wx = mapX - map.toMapX(location.getX());
	int wy = mapY - map.toMapY(location.getY());
	int prev = lightline;
	int oldfontsize = map.fontsize;
	if (map.fontsize < map.minfontsize) {
	    oldfontsize = map.fontsize;
	    map.fontsize = map.minfontsize;
	}
	if (wy < -map.fontsize) {
	    lightline = ILLEGAL;
	    result = null;
	} else {
	    if (wy < 0) lightline = -1;
	    else lightline = wy/map.fontsize; // (wy-1) fix for some rounding error?
	    result = location.getElement(lightline);
	    if (result == null) lightline = ILLEGAL;
	}
	// System.err.println("Highlighted ("+lightline+"): "+result);
	map.fontsize = oldfontsize;
	if (prev != lightline) {
	    // argh! no!
	    // if (prev != ILLEGAL) getElement(prev).update(g, false);
	    // if (lightline != ILLEGAL) getElement(lightline).update(g, true);
	    map.drawList = true;
	    map.repaint();
	}
	return result;
    }

    public void unselect() {
	lightline = ILLEGAL;
    }

    public void update(
	Graphics g, Map map, Enumeration locateds, boolean drawHighlight
    ) {
	Dimension dim = map.getSize();
	boolean firstTime = true;
	int mapX = 0, mapY = 0;
	int count = 0;

	// if (location.getPlanet() != null) count = -1; // Hmmm...

	// Enumeration enum = location.locateds(true);
	while (locateds.hasMoreElements()) {
	    Located loc = (Located)locateds.nextElement();
	    if (firstTime) {
		mapX = map.toMapX(loc.getX());
		mapY = map.toMapY(loc.getY());
		firstTime = false;
	    }
	    MapLocated mapLoc = MapLocated.createMapLocated(map, loc);
	    Color c = mapLoc.getColor();
	    if (drawHighlight && (lightline == count)) c = Color.white;
	    g.setColor(c);

	    if ((map.fontsize >= map.minfontsize) &&
		(mapX >= 0) && (mapX < dim.width) &&
		(mapY >= 0) && (mapY < dim.height)
	    ) {
		String str = mapLoc.getString();
		if (str != null) {
		    g.drawString(str,
				 mapX + (int)(map.zoom/2+1),
				 mapY + (count+1)*map.fontsize);
		    count++;
		}
	    }
	    mapLoc.update(g, mapX, mapY);
	}
    }
}
