// File: Map.java

package jvpc.jvpc;

import java.util.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.lang.Math;
import kero.util.*;
import jvpc.vga.*;
import jvpc.extdata.*;

/**
 * Controls which part of the map is on-screen. Taken into consideration are a 
 * zoom-factor, the centered coordinate and the state of a vertical and a 
 * horizontal scrollbar. Gee... should have a MapPanel that solves placement
 * and manages the Scrollbars. Then Map itself can deal with displaying...
 *
 * <P> Note to programmers: The scrollbars do <EM>not</em> always behave like
 * the jdk 1.1.3 documentation makes us believe! In the example, the actual
 * width of the scrollbar I witnessed (HP-UX, suns own JVM) was 256, not
 * 256+64. This ``feature'' can be circumvented by setting<PRE>
 *   BuggyScrollbars = Yes
 * </pre>in <TT>jvpc.cfg</tt>.
 * <P>Windows'95 Scrollbar was 100% OK. Either Focus or KeyEvent did not work.
 * Note: later jdk has the documentation changed. So it's the docs that were
 * buggy, not the scrollbars.
 * 
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since     13 VII 1998
 */
public class Map
    extends Canvas
    implements VGAPlanets, AdjustmentListener, KeyListener, MouseListener, MouseMotionListener
{
    // states of the map, so far can not happen simultaneoulsy
    public static final int IDLE = 0;
    public static final int SELECTING = 1;
    public static final int WAYPOINT = 2;

    int mode = IDLE;

    boolean drawComplete, drawList, drawWaypoint;

    // For all data to be displayed
    Universe universe;

    MapFrame mapFrame;

    int west, east, north, south; // Borders of the starmap
    int westX, northY; // upper left corner of the screen

    JVPC applet;
    double zoom, minzoom, maxzoom;
    int fontsize, minfontsize, maxfontsize;
    boolean buggyscrollbars; // fixing the scrollbar problem

    // issues for selecting
    int mouseX, mouseY; // current mouse postion
    Located selected; // the selected object, which is centered when zooming
    Location location; // Location in which we are selecting
    Located current; // to be selected
    // for displaying all locations
    LocationList loclist; // wrong name FIXME
    // issues for waypoint
    MyPoint prevtarget=null, target;

    // dynamically adaptable display options
    boolean drawPlanetLabels = true;
    int planetLabel = 0;	// 0=name, 1=index, 2=user string,
    String planetstring;
    boolean drawShipLabels = true;
    int shipLabel = 1;		// 0=index, 1=ship name, 2=ship type, 3=string
    String shipstring;
    boolean drawCoverage = false;
    int coverageType = 1;	// 0 = sensor sweep, 1 = mine scan,
				// 2 = ship scan
    boolean drawMinefields = true;
    int drawMinefieldLabels = 1;// 0 = nothing, 1 = Minefield 123, 2 = m123
    boolean drawExplosions = true;
    boolean drawIonstorms = true;
    int ionstormLabel = 1;	// 0 = long, 1 = short
    boolean drawWormholes = true;
    int wormholeLabel = 1;	// 0 = long, 1 = short
    boolean drawGrid = true;
    int colorModel = 1;		// 0 = status, 1 = player
    boolean drawShipWaypoints = true;
    int shipWaypointsLength = 1;// # of turns to draw waypoints
				// 1 = only current turn

    /* Unfortunately, our JVM doesn't cache fonts for us, likely. So we
     * shouldn't say "new Font" more often than necessary. kaffe on iPAQ
     * queries many (hundreds?) of fonts to the X-server on each "new Font".
     */
    Font fontCache[] = null;
  
    public Map(MapFrame mapFrame, Universe u, JVPC ja) {
	this.mapFrame = mapFrame;
	this.applet = ja;
	setUniverse(u);

	JVPCConfig jc = applet.getConfig();
	westX = west = jc.getMinX();
	east = jc.getMaxX();
	northY = north = jc.getMaxY();
	south = jc.getMinY();
	minfontsize = jc.getMinFontSize();
	maxfontsize = jc.getMaxFontSize();
	if (fontCache == null) fontCache = new Font[maxfontsize-minfontsize+1];
	minzoom = jc.getMinZoom();
	zoom = 1.0;
	maxzoom = jc.getMaxZoom();
	buggyscrollbars = jc.getBuggyScrollbars();
	planetstring = jc.getPlanetString();
	shipstring = jc.getShipString();
	addKeyListener(this);
	addMouseListener(this);
	addMouseMotionListener(this);
	/* addFocusListener(new FocusAdapter() {
	   public void FocusGained(FocusEvent fe) {
	   System.err.println("Map.focusGained()");
	   }
	   });
	*/
    }

  public Font cachedFont(Font origFont, int size) {
    int index = size-minfontsize;
    if (fontCache[index] == null)
      fontCache[index] = new Font(origFont.getName(), Font.PLAIN, size);
    return fontCache[index];
  }
  
    public void setUniverse(Universe u) {
	if (u != null) {
	    universe = u;
	    loclist = new LocationList(this, null);
	    mapFrame.setTitle(JVPC.program+" -- Starmap, "+u.getTurnString());
	}
    }


    // AdjustmentListener methods

    public void adjustmentValueChanged(AdjustmentEvent e) {
	Scrollbar scroll = (Scrollbar)e.getAdjustable();
	int value = e.getValue();
	if (scroll == mapFrame.hscroll) {
	    readHScroll();
	} else {
	    readVScroll();
	}
    }


    // KeyListener methods

    public void keyTyped(KeyEvent e) {}

    public void keyPressed(KeyEvent e) {
	int code = e.getKeyCode();
	// int modif = e.getModifiers();
	boolean shifted = e.isShiftDown(); // (modif & InputEvent.SHIFT_MASK) != 0;
	boolean ctrled = e.isControlDown(); // (modif & InputEvent.CTRL_MASK) != 0;
	boolean alted = e.isAltDown() || e.isMetaDown(); // (modif & InputEvent.ALT_MASK) != 0;
	// System.err.print("["+code);
	switch (code) {
	case KeyEvent.VK_F1:
	    report("For manual, see http://jvpc.sourceforge.net/");
	    applet.report("For manual, see http://jvpc.sourceforge.net/");
	    break;
	case KeyEvent.VK_F8:
	case KeyEvent.VK_F:
	    openSettingsWindow();
	    break;
	case KeyEvent.VK_SPACE:
	    goSetWaypoint();
	    break;
	case KeyEvent.VK_ENTER:
	    enter();
	    break;
	case -34: // kaffe work-around :(
	case KeyEvent.VK_BACK_QUOTE:
	    // System.err.println("well?");
	    new JVPC().run(universe.getGame());
	    break;
	case KeyEvent.VK_TAB: // does this work practically?
	    if (ctrled) {
		setOverviewZoom();
	    } else {
		if (shifted) {
		    zoomOut();
		} else {
		    zoomIn();
		}
	    }
	    e.consume();
	    break;
	case KeyEvent.VK_COMMA:
	    zoomIn();
	    break;
	case KeyEvent.VK_PERIOD:
	    zoomOut();
	    break;
	case KeyEvent.VK_F5:
	case KeyEvent.VK_INSERT:
	    Marker marker;
	    if (current != null) {
		marker = new Marker(universe, current.getX(), current.getY());
	    } else {
		marker = new Marker(universe, toLocationX(mouseX), toLocationY(mouseY));
	    } 
	    applet.database.addMarker(marker);
	    new MarkerFrame(marker);
	    break;
	case KeyEvent.VK_UP:
	    // kick vscroll; // equivalent arrow-keys and actions for hscroll
	    break;
	case KeyEvent.VK_PAGE_UP:
	    // kick vscroll; // which key kicks hscroll, then?
	    break;
	case KeyEvent.VK_A:
	    nextColorModel();
	    break;
	case KeyEvent.VK_B:
	    if (selected instanceof BData) {
		BData bdata = (BData)selected;
		if (bdata.isTrn()) {
		    if (shifted || ctrled) applet.openBaseFrame(bdata);
		    else applet.openBaseFrame(bdata);
		} else report("not editable");
	    } else report("not a base!");
	    break;
	case KeyEvent.VK_C:
	    if (ctrled) {
		toggleDrawCoverage();
	    } else if (shifted) {
		prevCoverageType();
	    } else {
		nextCoverageType();
	    }
	    break;
	case KeyEvent.VK_E:
	    toggleDrawExplosions();
	    break;
	case KeyEvent.VK_G:
	    toggleDrawGrid();
	    break;
	case KeyEvent.VK_H:
	    hostUniverse();
	    break;
	case KeyEvent.VK_I:
	    if (ctrled) {
		toggleDrawIonstorms();
	    } else if (shifted) {
		prevIonstormLabel();
	    } else {
		nextIonstormLabel();
	    }
	    break;
	case KeyEvent.VK_J: // add?
	    advanceUniverse();
	    break;
	case KeyEvent.VK_K: // subtract?
	    earlierUniverse();
	    break;
	case KeyEvent.VK_M:
	    if (ctrled) toggleDrawMinefields();
	    else nextMinefieldLabel();
	    break;
	case KeyEvent.VK_P:
	    if (ctrled) {
		toggleDrawPlanetLabels();
	    } else if (shifted) {
		prevPlanetLabel();
	    } else {
		nextPlanetLabel();
	    }
	    break;
	case KeyEvent.VK_Q:
	    if (shifted) zoomOut(); else zoomIn();
	    break;
	case KeyEvent.VK_R: // ?
	    toggleRstTrn();
	    break;
	case KeyEvent.VK_S:
	    if (ctrled) {
		toggleDrawShipLabels();
	    } else if (shifted) {
		prevShipLabel();
	    } else {
		nextShipLabel();
	    }
	    break;
	case KeyEvent.VK_T:
	    if (ctrled) {
		toggleDrawShipWaypoints();
	    } else if (shifted) {
		incShipWaypoints();
	    } else {
		decShipWaypoints();
	    }
	    break;
	case KeyEvent.VK_V:
	    viewSelected(shifted || ctrled);
	    break;
	case KeyEvent.VK_W:
	    if (ctrled) {
		toggleDrawWormholes();
	    } else if (shifted) {
		prevWormholeLabel();
	    } else {
		nextWormholeLabel();
	    }
	    break;
	case KeyEvent.VK_Z:
	    centerSelected();
	    break;
	default:
	    break;
	}
    }
    
    public void keyReleased(KeyEvent e) {}


    // MouseListener methods

    public void mousePressed(MouseEvent me) {
	// find nearest object and make it the current object (not defined yet)
	findNearestTo(me.getX(),me.getY());
	//drawList = true;
	//repaint();
    }

    public void mouseClicked(MouseEvent e) {}

    public void mouseReleased(MouseEvent e) {
	selectedAt(e.getX(),e.getY());
    }

    public void mouseEntered(MouseEvent e) {}
    public void mouseExited(MouseEvent e) {}

    // MouseMotionListener methods

    public void mouseMoved(MouseEvent me) {
	switch (mode) {
	case WAYPOINT:
	    target = new MyPoint(toLocationX(me.getX()),toLocationY(me.getY()));
	    drawWaypoint = true;
	    repaint();
	    break;
//  	case SELECTING:
//  	    drawList = true;
//  	    repaint();
//  	    break;
	default:
	    current = null;
	    mouseX = me.getX();
	    mouseY = me.getY();
	    break;
	}
    }

    public void mouseDragged(MouseEvent me) {
	switch (mode) {
	case WAYPOINT:
	    target = new MyPoint(toLocationX(me.getX()),toLocationY(me.getY()));
	    drawWaypoint = true;
	    repaint();
	    break;
	case SELECTING:
	    current = loclist.selectFromList(me.getX(), me.getY(), location);
	    break;
	}
    }


    // Settings / Find window

    void openSettingsWindow() {
	new SettingsWindow(mapFrame, this);
	//find.addWindowListener(applet.adapter);
    }


    // waypoint methods

    public void goSetWaypoint() {
	switch (mode) {
	case IDLE:
	    if ((selected != null) && (selected instanceof Ship)) {
		mode=WAYPOINT;
		report("Setting waypoint...");
		Ship s=(Ship)selected;
		target=new MyPoint(s.getX()+s.getDX(),s.getY()+s.getDY());
		drawWaypoint=true;
		repaint();
	    }
	    break;
	case WAYPOINT:
	    mode=IDLE;
	    report("Waypoint not set");
	    target=null;
	    drawWaypoint=true;
	    repaint();
	    break;
	default:
	    // should not happen or whatever
	}
  }

  public void enter() {
    switch (mode) {
    case WAYPOINT:
      mode=IDLE;
      if (selected instanceof Ship) {
	((Ship)selected).setDX(target.getXint()-selected.getX());
	((Ship)selected).setDY(target.getYint()-selected.getY());
	report("Waypoint set");
      }
      target=null;
      drawComplete=true; // cancel old, draw new waypoint
      break;
    default:
      // dunno
    }
    repaint();
  }

  public void findNearestTo(int mapX,int mapY) {
    int locX = toLocationX(mapX);
    int locY = toLocationY(mapY);
    // System.err.println("map("+mapX+", "+mapY+"), location("+locX+", "+locY+")");
    switch (mode) {
    case WAYPOINT:
      Location loc = universe.getNearestTo(locX,locY);
      target = new MyPoint(loc.getX(),loc.getY());
      mouseX = toMapX(loc.getX()); // can be inaccurate...
      mouseY = toMapY(loc.getY()); // ...for inserting markers, improve there
      drawWaypoint=true;
      break;
    default:
      mode = SELECTING;
      location = universe.getNearestTo(locX,locY);
      mouseX = toMapX(location.getX());
      mouseY = toMapY(location.getY());
      current = loclist.selectFromList(mapX, mapY, location);
        if (!drawPlanetLabels || !drawShipLabels || (drawMinefieldLabels==0) ||
  	  !drawExplosions || !drawIonstorms || !drawWormholes) {
  	  drawComplete = true;
        } else {
  	  drawList = true;
        }
    }
    repaint();
  }

  public void selectedAt(int mapX,int mapY) {
    switch (mode) {
    case WAYPOINT:
      // do nothing
      break;
    case SELECTING:
      mode=IDLE;
      loclist.unselect();
      if (current!=null) {
	selected=current;
	if (selected instanceof Ship)
	  report(((Ship)selected).getReport());
	else if (selected instanceof Planet)
	  report(((Planet)selected).getReport());
	else
	  report("Selected "+selected);
      }
      if ((fontsize<minfontsize) ||
	  (!drawPlanetLabels || !drawShipLabels ||
	   !drawMinefields || (drawMinefieldLabels==0) ||
	   !drawIonstorms || !drawWormholes || !drawExplosions )) {
	drawComplete=true;
      } else {
	drawList=true;
      }
      repaint();
      break;
    default:
      // do I know?
    }
  }


  // switch and toggle methods

  public void toggleRstTrn() {
    Game game=universe.getGame();
    int turnnr=universe.getTurnNumber();
    int whence=universe.get();
    if (whence==RstOrTrn.RST) {
      Universe u=game.getUniverse(turnnr,RstOrTrn.TRN);
      if (u!=null) {
	setUniverse(u);
	report("Switched to TRN");
	drawComplete=true;
      } else report("No TRN available "+turnnr+" (internal error)");
    } else {
      Universe u=game.getUniverse(turnnr,RstOrTrn.RST);
      if (u!=null) {
	setUniverse(u);
	report("Switched to RST");
	drawComplete=true;
      } else report("No RST available for turn "+turnnr+" (internal error)");
    }
    repaint();
  }

  public void earlierUniverse() {
    Game game=universe.getGame();
    int turnnr=universe.getTurnNumber();
    int whence=universe.get();
    setUniverse(game.getPreviousUniverse(turnnr,whence));
    report("Switched to "+universe.getTurnString());
    drawComplete=true;
    repaint();
  }

  public void advanceUniverse() {
    Game game=universe.getGame();
    int turnnr=universe.getTurnNumber();
    int whence=universe.get();
    setUniverse(game.getNextUniverse(turnnr,whence));
    drawComplete=true;
    report("Switched to "+universe.getTurnString());
    repaint();
  }

  public void hostUniverse() {
    Game game=universe.getGame();
    int turnnr=universe.getTurnNumber();
    universe=game.getHostedNextUniverse(turnnr);
    report("Hosted, and switched to turn "+universe.getTurnNumber());
    drawComplete=true;
    repaint();
  }

  public Dimension getPreferredSize() {
    return new Dimension(240,240);
  }

  public void setSize(Dimension dim) {
      // super.setSize(dim.width,dim.height);
    drawComplete=true;
  }

  public void setSize(int width,int height) {
      // setSize(width,height);
    setZoom(zoom);
  }

  public void setOverviewZoom() {
    Dimension dim=getSize();
    double zoomx=(double)dim.width/(east-west+1);
    double zoomy=(double)dim.height/(north-south+1);
    setZoom(zoomx<zoomy?zoomx:zoomy);
  }

  /**
   * Sets the zoom-factor. <CODE>setSize()</code> has to be called at least
   * once, so that <CODE>minzoom</code> is set.
   */
  public void setZoom(double zoom) {
    double ozw,ozh,dx,dy;
    Dimension dim=getSize();
    ozw=((double)dim.width/this.zoom/2);
    ozh=((double)dim.height/this.zoom/2);
    // System.err.println("dim of map: "+ozw+"x"+ozh);
    if (zoom<minzoom) zoom=minzoom;
    if (zoom>maxzoom) zoom=maxzoom;
    if (this.zoom==0) {
      this.zoom=zoom;
      // System.err.println("First time zoom set: to "+zoom);
      return;
    }
    this.zoom=zoom;
    // System.err.println("Zoom set to: "+zoom);
    dx=((double)dim.width/zoom/2);
    dy=((double)dim.height/zoom/2);
    if (selected==null) {
      westX+=(int)(ozw-dx);
      northY-=(int)(ozh-dy);
    } else {
      switch (mode) {
      case WAYPOINT:
	westX=target.getXint()-(int)dx;
	northY=target.getYint()+(int)dy;
	break;
      default:
	westX=selected.getX()-(int)dx;
	northY=selected.getY()+(int)dy;
	break;
      }
    }
    checkAndSetView();
  }

  public void zoomIn() {
    setZoom(zoom*2);
  }

  public void zoomOut() {
    setZoom(zoom/2);
  }

  /**
   * Moves the point (locX,locY) from the starmap to the center of the Canvas.
   * <CODE>setZoom()</code> must have been called at least once before.
   */
  public void setCenter(int locX,int locY) {
    int d;
    Dimension dim=getSize();
    d=(int)((double)dim.width/zoom/2);
    westX=locX-d;
    d=(int)((double)dim.height/zoom/2);
    northY=locY+d;
    checkAndSetView();
  }

  /**
   * Moves the point (mapX,mapY) from the Canvas to the center of the Canvas.
   * <CODE>setZoom()</code> must have been called at least once before.
   */
  public void setScreenCenter(int mapX,int mapY) {
    setCenter(toLocationX(mapX),toLocationY(mapY));
  }

  private void checkAndSetView() {
    Dimension dim=getSize();
    int eastx,southy;
    double d=(double)dim.width/zoom;
    eastx=(int)((double)westX+d);
    if ((westX<west) && (eastx<east)) {
      int de=east-eastx;
      int dw=west-westX;
      int dmin=(de<dw)?de:dw;
      westX+=dmin;
    }
    if ((westX>west) && (eastx>east)) {
      int de=eastx-east;
      int dw=westX-west;
      int dmin=(de<dw)?de:dw;
      westX-=dmin;
    }
    Scrollbar hScroll = mapFrame.hscroll;
    if (buggyscrollbars) {
	hScroll.setValues(westX,(int)d,west,east); // wrong
    } else {
	hScroll.setValues(westX,(int)d,west,east-(int)d); // good
    }
    hScroll.setUnitIncrement((int)(d*0.1));
    hScroll.setBlockIncrement((int)(d*0.8));

    d=(double)dim.height/zoom;
    southy=(int)((double)northY-d);
    if ((northY>north) && (southy>south)) {
      int dn=northY-north;
      int ds=southy-south;
      int dmin=(dn<ds)?dn:ds;
      northY-=dmin;
    }
    if ((northY<north) && (southy<south)) {
      int dn=north-northY;
      int ds=south-southy;
      int dmin=(dn<ds)?dn:ds;
      northY+=dmin;
    }
    int ns2=(north-south)*2;
    Scrollbar vScroll = mapFrame.vscroll;
    if (buggyscrollbars) {
      vScroll.setValues(ns2-northY,(int)d,ns2-north,ns2-south); // wrong
    } else {
      vScroll.setValues(ns2-northY,(int)d,ns2-north,ns2-south-(int)d); // good
    }
    vScroll.setUnitIncrement((int)(d*0.1));
    vScroll.setBlockIncrement((int)(d*0.8));
    drawComplete=true;
    repaint();
  }

    public void viewSelected(boolean newFrame) {
	if (selected instanceof Ship) {
	    Ship ship = (Ship)selected;
	    // if (ship.isOurShip() && ship.isTrn()) {
	    if (newFrame) {
		applet.openShipFrame((Ship)selected);
	    } else {
		applet.viewShip((Ship)selected);
	    }
	    // } else report("not editable");
	} else { // if (selected instanceof PData) {
	    // PData pdata = (PData)selected;
	    // if (pdata.isTrn()) { // is own, editable planet...
	    if (newFrame) {
		//applet.openPlanetFrame((PData)selected);
		applet.openPlanetFrame((Planet)selected);
	    } else {
		//applet.viewPlanet((PData)selected);
		applet.viewPlanet((Planet)selected);
	    }
	    // } else report("not editable");
	} // else report("not editable");
    }

  public void startSetWaypoint() {
    if (selected instanceof Ship) {
      mode=WAYPOINT;
      prevtarget=new MyPoint(selected.getX(),selected.getY());
      drawWaypoint=true;
      repaint();
    }
  }

  public void centerSelected() {
    if ((mode==WAYPOINT) && (target!=null)) {
      int locX=target.getXint();
      int locY=target.getYint();
      setCenter(locX,locY);  
      drawComplete=true;
      repaint();
    } else if (selected!=null) {
      int locX=selected.getX();
      int locY=selected.getY();
      setCenter(locX,locY);
    }
  }

  public void nextColorModel() {
    colorModel++;
    colorModel %= 2;
    drawComplete=true;
    repaint();
  }

  public void toggleDrawIonstorms() {
    drawIonstorms = !drawIonstorms;
    report("Ionstorms are " + (drawIonstorms ? "" : "not ") + "drawn");
    drawComplete=true;
    repaint();
  }

  public void nextIonstormLabel() {
    ionstormLabel++;
    ionstormLabel %= 2;
    drawIonstorms = true;
    drawComplete=true;
    repaint();
  }

  public void prevIonstormLabel() {
    ionstormLabel += 2 - 1;
    ionstormLabel %= 2;
    drawIonstorms = true;
    drawComplete=true;
    repaint();
  }

  public void toggleDrawWormholes() {
    drawWormholes = !drawWormholes;
    report("Wormholes are " + (drawWormholes ? "" : "not ") + "drawn");
    drawComplete=true;
    repaint();
  }

  public void nextWormholeLabel() {
    wormholeLabel++;
    wormholeLabel %= 2;
    drawWormholes = true;
    drawComplete=true;
    repaint();
  }

  public void prevWormholeLabel() {
    wormholeLabel += 2 - 1;
    wormholeLabel %= 2;
    drawWormholes = true;
    drawComplete=true;
    repaint();
  }

  public void toggleDrawExplosions() {
    drawExplosions = !drawExplosions;
    report("Explosions are " + (drawExplosions?"":"not ") + "drawn");
    drawComplete=true;
    repaint();
  }

  public void toggleDrawMinefields() {
    drawMinefields = !drawMinefields;
    report("Minefields are " + (drawMinefields ? "" : "not ") + "drawn");
    drawComplete=true;
    repaint();
  }

  public void nextMinefieldLabel() {
    drawMinefieldLabels += 1;
    drawMinefieldLabels %= 3;
    drawComplete=true;
    repaint();
  }

  public void toggleDrawPlanetLabels() {
    drawPlanetLabels = !drawPlanetLabels;
    report("Planet labels are " + (drawPlanetLabels? "" : "not ") + "drawn");
    drawComplete=true;
    repaint();
  }

  public void nextPlanetLabel() {
    planetLabel++;
    planetLabel %= 3;
    drawPlanetLabels = true;
    drawComplete=true;
    repaint();
  }

  public void prevPlanetLabel() {
    planetLabel += 3 - 1;
    planetLabel %= 3;
    drawPlanetLabels = true;
    drawComplete=true;
    repaint();
  }

  public void setPlanetString(String str) {
    planetstring = str;
    drawComplete = true;
    repaint();
  }

  public void toggleDrawGrid() {
    drawGrid = !drawGrid;
    report("Grid is " + (drawGrid ? "" : "not ") + "drawn");
    drawComplete=true;
    repaint();
  }

  public void toggleDrawShipLabels() {
    drawShipLabels = !drawShipLabels;
    report("Ship labels are " + (drawShipLabels ? "" : "not ") + "drawn");
    drawComplete=true;
    repaint();
  }

  public void nextShipLabel() {
    shipLabel++;
    shipLabel %= 4;
    drawShipLabels = true;
    drawComplete=true;
    repaint();
  }

  public void prevShipLabel() {
    shipLabel++;
    shipLabel %= 4;
    drawShipLabels = true;
    drawComplete=true;
    repaint();
  }

  public void setShipString(String str) {
    shipstring = str;
    drawComplete = true;
    repaint();
  }

  public void toggleDrawShipWaypoints() {
    drawShipWaypoints = !drawShipWaypoints;
    report("Waypoints of ships are " + (drawShipWaypoints ? "" : "not ") + "drawn");
    drawComplete=true;
    repaint();
  }

  public void incShipWaypoints() {
    shipWaypointsLength = Math.min(shipWaypointsLength + 1, 5);
    drawShipWaypoints = true;
    drawComplete=true;
    repaint();
  }

  public void decShipWaypoints() {
    shipWaypointsLength = Math.max(shipWaypointsLength - 1, 1);
    drawShipWaypoints = true;
    drawComplete=true;
    repaint();
  }

  public void toggleDrawCoverage() {
    drawCoverage = !drawCoverage;
    report("Coverage is " + (drawCoverage ? "" : "not ") + "drawn");
    drawComplete=true;
    repaint();
  }

  public void setDrawCoverage(int newCoverageType) {
    String covMsg[] = {
      "of sensor sweep drawn",
      "of mine scan drawn",
      "of ship scan drawn"
    };
    coverageType = newCoverageType;
    report("Coverage " + covMsg[coverageType]);
    drawCoverage = true;
    drawComplete=true;
    repaint();
  }

  public void nextCoverageType() {
    setDrawCoverage((coverageType + 1) % 3);
  }

  public void prevCoverageType() {
    setDrawCoverage((coverageType + 3 - 1) % 3);
  }

  public void readHScroll() {
    int width = getSize().width;
    westX = mapFrame.hscroll.getValue();
    drawComplete = true;
    repaint();
  }

  public void readVScroll() {
    int height = getSize().height;
    northY = (north-south)*2 - mapFrame.vscroll.getValue();
    drawComplete = true;
    repaint();
  }

  public void report(String warning) {
    mapFrame.report(warning);
  }

  int toMapX(int locX) {
    return (int)((locX - westX) * zoom);
  }

  int toMapY(int locY) {
    return (int)((northY - locY) * zoom);
  }

  int toLocationX(int mapX) {
      /*  System.err.print("mapX: "+mapX);
	  System.err.print(", floated: "+((double)mapX + zoom/2.0));
	  System.err.print(", zoomed: "+(((double)mapX + zoom/2.0)/zoom));
	  System.err.println(", zoom2d: "+(1/zoom * ((double)mapX + zoom/2.0)));
      */
      return westX + (int)(((double)mapX+zoom/2.0)/zoom);
  }

  int toLocationY(int mapY) {
    return northY-(int)(((double)mapY+zoom/2.0)/zoom);
  }

  public void drawGrid(Graphics g, int size) {
    g.setColor(new Color(64, 64, 64));
    int width = getSize().width;
    int height = getSize().height;
    int eastx = (int)(westX + width / zoom);
    int southy = (int)(northY - height / zoom);
    int gridspacing;
    gridspacing = (int)(100 / zoom);
    if (gridspacing > 100)
      gridspacing = 100;
    else if (gridspacing > 10)
      gridspacing = 10;
    else
      gridspacing = 1;
    for (int x=westX - (westX % gridspacing); x<=eastx; x += gridspacing) {
      int mapX=toMapX(x);
      g.drawLine(mapX, 0, mapX, height);
    }
    for (int y=southy - (southy % gridspacing); y<=northY; y += gridspacing) {
      int mapY=toMapY(y);
      g.drawLine(0, mapY, width, mapY);
    }
  }

  public void drawSensorSweepCoverage(Graphics g) {
    int width = getSize().width;
    int height = getSize().height;
    g.setColor(new Color(30, 30, 30));
    g.fillRect(0,0,width,height);
    g.setColor(Color.black);
    // For each sensor sweeping ship, indicate
    // the area it sweepes.
    int sweeprange = universe.getConfig().sensorRange();
    int maprange = (int)(zoom * sweeprange);
    for (int s=0; s<ships; s++) {
      Ship ship=universe.getShip(s);
      if (ship == null) continue;
      if (ship.getMission() == Mission.SENSORSWEEP) {
	int mapX = toMapX(ship.getX());
	int mapY = toMapY(ship.getY());
	if ((mapX+maprange < 0) || (mapX-maprange >= width)) continue;
	if ((mapY+maprange < 0) || (mapY-maprange >= height)) continue;
	g.fillOval(toMapX(ship.getX() - sweeprange),
		   toMapY(ship.getY() + sweeprange),
		   (int)(2 * sweeprange * zoom),
		   (int)(2 * sweeprange * zoom));
      }
    }
  }

  public void drawMineScanCoverage(Graphics g) {
    int width = getSize().width;
    int height = getSize().height;
    g.setColor(new Color(30, 30, 30));
    g.fillRect(0,0,width,height);
    g.setColor(Color.black);
    // For each minesweeping ship, indicate
    // the area it scans for mines.
    int scanrange = universe.getConfig().mineScanRange();
    int maprange = (int)(zoom * scanrange);
    for (int s=0; s<ships; s++) {
      Ship ship=universe.getShip(s);
      if (ship == null) continue;
      if (ship.getMission() == Mission.MINESWEEP) {
	int mapX = toMapX(ship.getX());
	int mapY = toMapY(ship.getY());
	if ((mapX+maprange < 0) || (mapX-maprange >= width)) continue;
	if ((mapY+maprange < 0) || (mapY-maprange >= height)) continue;
	g.fillOval(toMapX(ship.getX() - scanrange),
		   toMapY(ship.getY() + scanrange),
		   (int)(2 * scanrange * zoom),
		   (int)(2 * scanrange * zoom));
      }
    }
  }

  public void drawShipScanCoverage(Graphics g) {
    int width = getSize().width;
    int height = getSize().height;
    g.setColor(new Color(30, 30, 30));
    g.fillRect(0,0,width,height);
    g.setColor(Color.black);
    // For each planet and ship, indicate the area it scans for ships.
    int scanrange = universe.getConfig().scanRange();
    int maprange = (int)(zoom * scanrange);
    Enumeration enum = universe.getLocations();
    while (enum.hasMoreElements()) {
	Location loc = (Location)enum.nextElement();
	boolean scanning = false;
	if (loc.getPlanet() != null) {
	    Planet planet = loc.getPlanet();
	    if (planet.isOurPlanet()) scanning = true;
	}
	Enumeration shipEnum = loc.getShips().elements();
	while (shipEnum.hasMoreElements()) {
	    Ship ship = (Ship)shipEnum.nextElement();
	    if (ship.isOurShip()) scanning = true;
	}
	if (scanning) {
	int mapX = toMapX(loc.getX());
	int mapY = toMapY(loc.getY());
	if ((mapX+maprange < 0) || (mapX-maprange >= width)) continue;
	if ((mapY+maprange < 0) || (mapY-maprange >= height)) continue;
	g.fillOval(toMapX(loc.getX() - scanrange),
		   toMapY(loc.getY() + scanrange),
		   (int)(2 * scanrange * zoom),
		   (int)(2 * scanrange * zoom));
	}
    }
  }

  public void redraw() {
    drawComplete=true;
    // repaint();
  }

  public void updateLocation(Graphics g, Location loc, boolean drawHighlight) {
      // Location loc=list.getLocation();
    Dimension dim=getSize();
    int mapX = toMapX(loc.getX());
    if ((mapX<0) || (mapX>=dim.width)) return; // in ``area'' on-screen?
    int mapY = toMapY(loc.getY());
    if ((mapY<0) || (mapY>=dim.height)) return; //in ``area'' on-screen?
    int count = 1;
    int size = (int)Math.ceil(zoom);
    Color c;

    if ((mode == SELECTING) &&
	(location.getX() == loc.getX()) && (location.getY() == loc.getY())) {
	loclist.update(g, this, loc.locateds(true, true, true), drawHighlight);
    } else {
	loclist.update(g, this, loc.locateds(drawPlanetLabels, drawShipLabels,
					     drawMinefields), drawHighlight);
    }
  }

  public void update(Graphics g) {
    Font font=g.getFont();
    fontsize=(int)Math.round(8*zoom);
    if (fontsize>maxfontsize) fontsize=maxfontsize;
    if (!drawComplete && !drawList && !drawWaypoint) {
      // System.err.print("!draw ");
      drawComplete=true;
    }
    if (drawComplete) {
      if (fontsize >= minfontsize) g.setFont(cachedFont(font, fontsize));
      // do normal stuff
      double d;
      Dimension dim=getSize();
      if (drawCoverage) {
	switch (coverageType) {
	case 0:
	  drawSensorSweepCoverage(g);
	  break;
	case 1:
	  drawMineScanCoverage(g);
	  break;
	case 2:
	  drawShipScanCoverage(g);
	  break;
	default:
	  Assert.assert(false);
	  break;
	}
      } else {
	g.setColor(Color.black);
	g.fillRect(0,0,dim.width,dim.height);
      }
      if (drawGrid) {
	drawGrid(g, (int)Math.ceil(zoom));
      }
      Enumeration enum = universe.getLocations();// loclist.elements();
      while (enum.hasMoreElements()) {
	  Location list = (Location)enum.nextElement();
	  updateLocation(g, list, (list==location));
      }
//        Enumeration enum = universe.getLocations();
//        while (enum.hasMoreElements()) {
//  	  Location loc = (Location)enum.nextElement();
//  	  updateLocation(g, loc.locateds(true));
//        }
      prevtarget=null;
      if (mode==WAYPOINT) drawWaypoint=true;
      drawComplete=false;
    }

    if (drawList) {
      int oldfontsize=fontsize;
      if (fontsize<minfontsize) {
	fontsize=minfontsize;
      }
      g.setFont(cachedFont(font, fontsize));
      //updateLocation(g,((LocationList)loclist.get(location)).getLocation());
      updateLocation(g, location, true);
      fontsize=oldfontsize;
      drawList=false;
    }

    if (drawWaypoint) {
      g.setColor(Color.black);
      g.setXORMode(Color.white);
      if (prevtarget!=null) {
	g.drawLine(toMapX(selected.getX()),toMapY(selected.getY()),
		   toMapX(prevtarget.getXint()),toMapY(prevtarget.getYint()));
      }
      if (target!=null) {
	g.drawLine(toMapX(selected.getX()),toMapY(selected.getY()),
		   toMapX(target.getXint()),toMapY(target.getYint()));
	int dx = target.getXint() - selected.getX();
	int dy = target.getYint() - selected.getY();
	int distance2 = (dx*dx) + (dy*dy);
	double distance = Math.sqrt(distance2);
	report("Distance "+Mymath.formatNumber(distance, 2)+"ly; "+
	       distance2+" mines; "+
	       ((selected instanceof Ship)?
	           (((Ship)selected).neutrLeftWhenMoving(dx, dy)+" fuel left"):
		   "")
	      );
      }
      prevtarget=target;
      g.setPaintMode();
      drawWaypoint=false;
    }
  }

  public void paint(Graphics g) {
    update(g);
  }
}
