// File: MapFrame.java

package jvpc.jvpc;

import java.awt.*;
import java.awt.event.*;
import kero.awt.*;
import jvpc.vga.*;

/**
 * Contains the on-screen starmap. This formed by a <CODE>Canvas</code> and two
 * <CODE>Scrollbar</code>s.
 *
 * <P>Programmers: for bugs on Scrollbars, @see Map
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since     10 VII 1998
 * @version   3 III 2001
 */
public class MapFrame
    extends MyFrame
    implements VGAPlanets, PlanetListener, ShipListener
{
    private static final int scrollWidth = 16;

    // GUI objects
    Scrollbar hscroll, vscroll;
    Map map;
    Label label;

    Panel buttonPanel;
    Button earlier, advance, view, cargo, setWaypoint, cancelWaypoint;
    InputChoice coverage;

    SettingsWindow find;

    /** Disabled constructor. Use <CODE>MapFrame(Game,Universe)</code> instead.
     */
    private MapFrame() {}
  
    public MapFrame(String name, Universe universe, JVPC ja) {
	super(name);
	GridBagLayout gbl = new GridBagLayout();
	setLayout(gbl);
	buttonPanel = new Panel(new GridLayout(1, 0));
	earlier = new Button("<");
	addButtonTo(buttonPanel, earlier, KeyEvent.VK_K, "earlierTurn");
	advance = new Button(">");
	addButtonTo(buttonPanel, advance, KeyEvent.VK_J, "advanceTurn");
	coverage = new InputChoice("");
	coverage.addSelection("No Scan");
	coverage.addSelection("Ship Scan");
	coverage.addSelection("Sensor Scan");
	coverage.addSelection("Mine Scan");
	// addInputTo(panel, coverage, KeyEvent.VK_C, "viewScan");
	view = new Button("View");
	addButtonTo(buttonPanel, view, KeyEvent.VK_V, "viewSelected");
	cargo = new Button("Cargo");
	addButtonTo(buttonPanel, cargo, KeyEvent.VK_C, "viewCargo");
	setWaypoint = new Button("Waypoint");
	addButtonTo(buttonPanel, setWaypoint, KeyEvent.VK_SPACE, "toggleSetWaypoint");
	cancelWaypoint = new Button("Cancel WPt");
	cancelWaypoint.disable();
	addButtonTo(buttonPanel, cancelWaypoint, KeyEvent.VK_SPACE, "cancelWaypoint");
	map = new Map(this, universe, ja);
	hscroll = new Scrollbar(Scrollbar.HORIZONTAL);
	hscroll.addAdjustmentListener(map);
	vscroll = new Scrollbar(Scrollbar.VERTICAL);
	vscroll.addAdjustmentListener(map);
	label = new Label("Launched turn "+universe.getTurnNumber());

	GridBagConstraints gbc = new GridBagConstraints();
	gbc.gridheight = 1; gbc.gridwidth = 1;
	gbc.gridx = 0; gbc.gridy = 0;
	gbc.weightx = 0.0; gbc.weighty = 0.0;
	gbc.fill = GridBagConstraints.HORIZONTAL;
	add(buttonPanel, gbc);

	gbc.gridwidth = 1;
	gbc.gridx = 0; gbc.gridy = 1;
	gbc.fill = GridBagConstraints.BOTH;
	gbc.weightx = 1.0; gbc.weighty = 1.0;
	add(map, gbc);

	gbc.fill = GridBagConstraints.VERTICAL;
	gbc.gridx = 1;
	gbc.weightx = 0.01; gbc.weighty = 0.01;
	add(vscroll, gbc);

	gbc.gridx = 0; gbc.gridy = 2;
	gbc.fill = GridBagConstraints.HORIZONTAL;
	add(hscroll, gbc);

	gbc.gridy = 3; gbc.gridwidth = 2;
	add(label, gbc);
	pack();
	limitSize();
	show();

	// add planet/ship/etc listeners
	universe.addPlanetListener(this);
	universe.addShipListener(this);

	// hack to circumvent Kaffe focus problems
	addFocusListener(new FocusAdapter() {
	    public void focusGained(FocusEvent fe) {
		map.requestFocus();
	    }
	});
    }

    public void earlierTurn() {
	map.earlierUniverse();
	map.requestFocus();
    }

    public void advanceTurn() {
	map.advanceUniverse();
	map.requestFocus();
    }

    public void viewScan() {
	int index = coverage.getIndex();
	map.drawCoverage = (index != 0);
	if (map.drawCoverage) map.coverageType = (index - 1);
	map.repaint();  // baaaah
	map.requestFocus();
    }

    public void viewSelected() {
	map.requestFocus();
	map.viewSelected(false);
    }

    public void viewCargo() {
	map.requestFocus();
	if (map.selected != null) {
	    Location loc = map.selected.getLocation();
	    map.applet.viewCargoFrame(loc);
	}
    }

    public void toggleSetWaypoint() {
	switch (map.mode) {
	case Map.IDLE:
	    cancelWaypoint.enable();
	    map.requestFocus();
	    map.goSetWaypoint();
	    break;
	case Map.WAYPOINT:
	    map.enter();
	    map.requestFocus();
	    break;
	default: // SELECTING would be sweet
	    map.requestFocus();
	    break;
	}
	// already has repaint()
    }

    public void cancelWaypoint() {
	cancelWaypoint.disable();
	if (map.mode == Map.WAYPOINT) map.goSetWaypoint();
	map.requestFocus();
    }

    // ship/planet/etc changed events

    public void planetChanged(PlanetEvent pe) {
	if (map.selected instanceof Planet) {
	    Planet planet = (Planet)map.selected;
	    if (planet.getId() == pe.getId()) {
		report(planet.getReport());
	    }
	    map.repaint();
	}
    }

    public void shipChanged(ShipEvent se) {
	if (map.selected instanceof Ship) {
	    Ship ship = (Ship)map.selected;
	    if (ship.getId() == se.getId()) {
		report(ship.getReport());
	    }
	    map.repaint();
	}
    }


    // MapFrame methods

    public void report(String message) {
	label.setText(message);
    }
}
