// File: MessageFrame.java

package jvpc.jvpc;

import java.awt.*;
import java.awt.event.*;
import kero.awt.*;
import jvpc.vga.*;

/**
 * Presents a <CODE>Message</code> in a <CODE>Frame</code>. Does not have
 * <CODE>Container</code> functionality (explicitely dropped).
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since     13 II 1999
 * @version   10 III 2001
 */
public class MessageFrame
  extends MyFrame
  implements VGAPlanets, KeyListener, ItemListener, TextListener
{
  Game game;
  Universe universe;
  boolean showincoming;
  int currmdata,currmess;
  Message message;
  Label from,to;
  TextArea textarea;
  Panel fromtopanel;
  Panel panel, buttonPanel, outgoingPanel;
  Button prevturn,nextturn,incoming,outgoing;
  IntInput messnr;
  Button reply,forward;
  Button writeMess,copy;

  int fromPlayer=0,toPlayer=0;
  Checkbox[] sender,receiver;
  CheckboxGroup sendgroup,receivegroup;
  Button doneFromTo, removeMess;
  Panel sendPanel,receivePanel,newPanel;

  //ScrollPane sp;
  //Panel panel;

  MessageFrame(String name, Game g) {
    super(name);
    game=g;
    universe=game.getUniverse(); // current universe is TRN
    showincoming=false;
    currmdata=currmess=0;

    // sp = new ScrollPane();
    setLayout(new BorderLayout());
    panel = new Panel(new GridLayout(0,1));
    super.add(panel, "North");

    buttonPanel=new Panel(new GridLayout(0,2));
    //buttonPanel.setInsets(new Insets(0,0,0,0));
    prevturn=new Button("Previous turn");
    addButtonTo(buttonPanel, prevturn, KeyEvent.VK_P, "previousTurn");
    nextturn=new Button("Next turn");
    addButtonTo(buttonPanel, nextturn, KeyEvent.VK_N, "nextTurn");
    panel.add(buttonPanel);

    buttonPanel=new Panel(new GridLayout(0,2));
    incoming=new Button("Incoming");
    addButtonTo(buttonPanel, incoming, KeyEvent.VK_I, "switchToIncoming");
    outgoing=new Button("Outgoing");
    addButtonTo(buttonPanel, outgoing, KeyEvent.VK_O, "switchToOutgoing");
    panel.add(buttonPanel);

    messnr = new IntInput("Message nr:", 1, 10, 3);
    addNumberInputTo(panel, messnr, KeyEvent.VK_M, "setMessage");

    from=new Label("From: ??");
    to=new Label("To: ??");
    fromtopanel=new Panel(new GridLayout(1,2));
    fromtopanel.add(from);
    fromtopanel.add(to);
    panel.add(fromtopanel);

    textarea=new TextArea("Hello, world!",20,40,TextArea.SCROLLBARS_BOTH);
    // textarea.setFont(new Font("Courier", Font.PLAIN, 8));
    textarea.setEditable(true); // current universe is TRN (see few lines up)
    textarea.addTextListener(this);
    addKeyListener(this); // listen to yourself???
    super.add(textarea,"Center");

    outgoingPanel = new Panel(new GridLayout(0, 1));
    writeMess=new Button("Write msg");
    addButtonTo(outgoingPanel, writeMess, KeyEvent.VK_W, "setFromTo");
    outgoingPanel.add(new Label("Copy msg"));
    outgoingPanel.add(new Label("Reply msg"));
    outgoingPanel.add(new Label("Fwd msg"));
    removeMess = new Button("Del msg");
    addButtonTo(outgoingPanel, removeMess, KeyEvent.VK_D, "removeMessage");

    FlowLayout flow=new FlowLayout();

    // prepare send/receive radio buttons for a new message
    sendgroup=new CheckboxGroup();
    sendPanel=new Panel(new GridLayout(0,1));
    // sendPanel.setLayout(new BoxLayout(sendPanel,BoxLayout.Y_AXIS));
    sender=new Checkbox[races+1];
    int count=0;
    for (int i=1;i<=races;i++) {
      if (universe.getGen(i)!=null) {
	count++;
	fromPlayer=i;
	sender[i]=new
	  Checkbox(universe.getPlayer(i).getShortname(),true,sendgroup);
	sender[i].addItemListener(this);
	sendPanel.add(sender[i]);
      }
    }
    receivegroup=new CheckboxGroup();
    receivePanel=new Panel(new GridLayout(0,1));
    // receivePanel.setLayout(new BoxLayout(receivePanel,BoxLayout.Y_AXIS));
    receiver=new Checkbox[races+1];
    for (int i=1;i<=races;i++) {
      // if (``player plays in game'') {
	receiver[i]=new
	  Checkbox(universe.getPlayer(i).getShortname(),true,receivegroup);
	receiver[i].addItemListener(this);
	receivePanel.add(receiver[i]);
      // }
    }
    doneFromTo=new Button("Recipient(s) set");
    addButtonTo(receivePanel, doneFromTo, KeyEvent.VK_R, "writeMessage");
    newPanel=new Panel(new GridLayout(1, 0)); // auto FlowLayout, OK
    if (count>1) newPanel.add(sendPanel);
    newPanel.add(receivePanel);

    // and then I'd love to have a _horizontal_ set of radio buttons instead of
    // the Label(To/From) for all the players involved. So you could change
    // them _after_ the message was written. Mind there will have to be one
    // message written for each recipient. It would be great if reading message
    // would recognize similar messages and regroup them again :-) Of course, a
    // COPY mechanism would be necessary. FORWARD and REPLY are recommended as
    // well :-)

    //super.add(sp);

    switchToIncoming();

    pack();
    limitSize();
    show();
  }
/*
  public Dimension getPreferredSize() {
    return cont.getPreferredSize();
  }
*/
  // MessagePanel specific methods

  private void showMessage() {
    if (message==null) {
      from.setText("From: Nobody");
      to.setText("To: Nobody");
      textarea.setText("No messages");
      textarea.setEditable(false);
    } else {
      int fromPlayer=message.getSender();
      int toPlayer=message.getRecipient();
      from.setText("From: "+(fromPlayer==UNKNOWN?"?":""+fromPlayer+" "+universe.getPlayer(fromPlayer).getLongname()));
      to.setText("To: "+(toPlayer==UNKNOWN?"?":""+toPlayer+" "+universe.getPlayer(toPlayer).getLongname()));
      textarea.setText(message.getContent());
      if (showincoming) messnr.setInt(currmdata);
      else messnr.setInt(currmess);
    }
  }

  public void previousTurn() {
    Game game=universe.getGame();
    int turnnr=universe.getTurnNumber();
    int whence=universe.get();
    universe=game.getPreviousUniverse(turnnr,whence);
    String turnstr="turn "+universe.getTurnNumber();
    setTitle(JVPC.program+" -- Message, "+turnstr);
    showMessage();
  }

  public void nextTurn() {
    Game game=universe.getGame();
    int turnnr=universe.getTurnNumber();
    int whence=universe.get();
    universe=game.getNextUniverse(turnnr,whence);
    String turnstr="turn "+universe.getTurnNumber();
    setTitle(JVPC.program+" -- Message, "+turnstr);
    showMessage();
  }

  public void switchToIncoming() {
    if (showincoming) return; // already viewing incoming messages
    showincoming=true;
    super.remove(outgoingPanel);
    // this.invalidate();
    pack();
    limitSize();
    universe=game.getUniverse(universe.getTurnNumber(),RstOrTrn.RST);
    textarea.setEditable(false);
    message=universe.getMessage(currmdata);
    showMessage();
  }

  public void switchToOutgoing() {
    if (!showincoming) return; // already viewing outgoing messages
    showincoming=false;
    super.add(outgoingPanel, "East");
    // this.invalidate();
    pack();
    limitSize();
    universe=game.getUniverse(universe.getTurnNumber(),RstOrTrn.TRN);
    textarea.setEditable(true);
    message=universe.getMessage(currmess);
    showMessage();
  }

  public void setMessage() {
    setMessage(messnr.getInt(), false);
  }

  public void setMessage(int newNr) {
    setMessage(newNr, false);
  }

  public void setMessage(int newnr, boolean forcedRedraw) {
    int current;
    if (showincoming) current=currmdata;
    else current=currmess;

    int total = universe.getMessageCount();
    if (total == 0) {
	newnr = -1;
	//removeMess.disable();
    } else {
	if (newnr < 0) newnr = newnr%total + total;
	if (newnr >= total) newnr %= total;
	//removeMess.enable();
    }
    messnr.setInt(newnr);
    if ((newnr == current) && !forcedRedraw) return;

    current = newnr;
    message = (current < 0)? null: universe.getMessage(current);
    if (showincoming) currmdata=current;
    else currmess=current;
    showMessage();
  }

  public void setFromTo() {
    showincoming=false;
    panel.remove(fromtopanel);
    super.remove(textarea);
    super.remove(outgoingPanel); // .disable();
    super.add(newPanel,"Center");
    pack();
    limitSize();
  }

  public void writeMessage() {
    toPlayer = 0;
    for (int i=1;i<=races;i++) {
      if (receiver[i].getState()) toPlayer=i;
    }
    if (toPlayer != 0) {
	message=new Message(universe);
	message.setSender(fromPlayer);
	message.setRecipient(toPlayer);
// message.setContent("Error: Only last recipient gets message (check)");
	universe.putMessage(message); // should be TRN universe FIXME check...
	currmess=universe.getMessageCount()-1;
	textarea.setEditable(true);
    }
    super.remove(newPanel);
    panel.add(fromtopanel);
    super.add(textarea,"Center");
    super.add(outgoingPanel, "East"); // .enable();
    pack();
    limitSize();
    showMessage();
  }

    public void removeMessage() {
	universe.removeMessageAt(currmess); // should be TRN universe
	setMessage(currmess-1, true);
    }


  // KeyListener methods

  public void keyTyped(KeyEvent ke) {}
  public void keyReleased(KeyEvent ke) {}

  public void keyPressed(KeyEvent ke) {
    super.keyPressed(ke);
    int code=ke.getKeyCode();
    switch (code) {
    case KeyEvent.VK_UP:
    case KeyEvent.VK_PAGE_UP:
      // case KeyEvent.VK_DASH: // which bloody key is this?
    case KeyEvent.VK_K:
      if (showincoming) setMessage(currmdata-1); else setMessage(currmess-1);
      break;
    case KeyEvent.VK_DOWN:
    case KeyEvent.VK_PAGE_DOWN:
    case KeyEvent.VK_EQUALS:
    case KeyEvent.VK_J:
      if (showincoming) setMessage(currmdata+1); else setMessage(currmess+1);
      break;
    default:
      break;
    }
  }


  // ItemListener method

  public void itemStateChanged(ItemEvent ie) {
    Checkbox source=(Checkbox)ie.getSource();
    for (int i=1;i<=races;i++) {
      if (sender[i]==source && ie.getStateChange() == ItemEvent.SELECTED) {
	fromPlayer=i;
      }
    }
    for (int i=1;i<=races;i++) {
	//if (receiver[i]==source) toPlayer=i;
    }
    // report(""+source);
  }


  // TextListener method

  public void textValueChanged(TextEvent te) {
    TextComponent comp=(TextComponent)te.getSource();
    System.err.println("outgoing message: "+te);
    if (comp==textarea) {
      if (!showincoming) {
	if (message!=null) message.setContent(textarea.getText());
	// report("Received: "+textarea.getText());
      }
    }
  }


  // Overridden methods

  public Component add(Component comp) {
    report("MessagePanel.add(Component) -- no Container functionality");
    return comp;
  }

  // MessagePanel methods
  public void report(String warning) {
    System.err.println(warning);
  }
}
