// File: PlanetFrame.java

package jvpc.jvpc;

import java.awt.*;
import java.awt.event.*;
import kero.awt.*;
import jvpc.vga.*;

/**
 * Presents a <CODE>Planet</code> in a <CODE>Frame</code>. Does not have
 * <CODE>Container</code> functionality (explicitely dropped).
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since     25 VII 1998
 * @version   18 III 2001
 */
public class PlanetFrame
  extends MyFrame
  implements
    VGAPlanets, PlanetListener,
    FocusListener, ActionListener, KeyListener
{
  JVPC applet;
  Game game;
  PData planet;
  Label name;
  Button prev, next;
  Label temperature, owner;
  InputField fcode;
  Label natives, nathappy, colonists, colhappy;
  IntInput nattax, coltax;
  IntInput supplies, money, mines, factories, defense;
  Button base;
  Button cargo;
  
  PlanetFrame(PData frameplanet, JVPC applet) {
    super();
    this.planet = frameplanet;
    this.game = planet.getUniverse().getGame();
    this.applet = applet;

    planet.getUniverse().addPlanetListener(this);

    setLayout(new GridLayout(0,1));
    Panel f = new Panel(new GridLayout(1, 0));
    name = new Label("12345678901234567890"); f.add(name);
    prev = new Button("Prev");
    addButtonTo(f, prev, KeyEvent.VK_P, "prevPlanet");
    next = new Button("Next");
    addButtonTo(f, next, KeyEvent.VK_N, "nextPlanet");
    add(f);
    temperature = new Label();
    add(temperature);
    owner = new Label();
    add(owner);
    fcode = new InputField("friendly Code:", planet.getFC());
    addInputField(fcode, KeyEvent.VK_C, "setPlanetFC");
    natives = new Label();
    add(natives);
    nathappy = new Label();
    add(nathappy);
    nattax = new IntInput("   native Tax:", 0, 5, 2);
    addNumberInput(nattax, KeyEvent.VK_T, "setPlanetNatTax");
    colonists = new Label();
    add(colonists);
    colhappy = new Label();
    add(colhappy);

    coltax = new IntInput("   Colonist tax:", planet.getColTax(), 5, 2);
    addNumberInput(coltax, KeyEvent.VK_C, "setPlanetColTax");

    supplies = new IntInput("Supplies:", planet.getSupplies(), 100, 2);
    addNumberInput(supplies, KeyEvent.VK_S, "setPlanetSupplies");
    money = new IntInput("money($):", planet.getMC(), 100, 2);
    addNumberInput(money, KeyEvent.VK_4, "setPlanetMoney");
    mines = new IntInput("Mines:", planet.getMines(), 100, 2);
    addNumberInput(mines, KeyEvent.VK_M, "setPlanetMines");
    factories = new IntInput("Factories:", planet.getFactories(), 100, 2);
    addNumberInput(factories, KeyEvent.VK_F, "setPlanetFactories");
    defense = new IntInput("Defense:", planet.getDefense(), 100, 2);
    addNumberInput(defense, KeyEvent.VK_D, "setPlanetDefense");

    Panel p = new Panel(new GridLayout(1, 0));
    cargo = new Button("carGo");
    addButtonTo(p, cargo, KeyEvent.VK_G, "viewCargoFrame");
    base = new Button("Build Base");
    addButtonTo(p, base, KeyEvent.VK_B, "baseButtonPressed");
    add(p);

    setPlanet(frameplanet);
    pack();
    limitSize();
    show();
    fcode.requestFocus();
  }

  // FocusListener methods
  public void focusGained(FocusEvent e) {
    // report("PlanetFrame -- "+e);
  }
  public void focusLost(FocusEvent e) {
    // report("PlanetFrame -- "+e);
  }

  // ActionListener method
  public void actionPerformed(ActionEvent ae) {
    super.actionPerformed(ae);
  }


  // KeyListener methods

  public void keyPressed(KeyEvent ke) {
    super.keyPressed(ke);
    boolean altheld = (ke.getModifiers() & KeyEvent.ALT_MASK) != 0;
    boolean ctrlheld = (ke.getModifiers() & KeyEvent.CTRL_MASK) != 0;
    boolean shiftheld = (ke.getModifiers() & KeyEvent.SHIFT_MASK) != 0;

    int keycode = ke.getKeyCode();
    switch (keycode) {
    case KeyEvent.VK_EQUALS:
      if (!altheld && !ctrlheld) break;
      // case KeyEvent.VK_DASH: // where is it... [Duke3D Nuke'm]
      //   if (!altheld && !ctrlheld) break;
    default:
      // ignore
      break;
    }
  }

  public void keyReleased(KeyEvent ke) {
    super.keyReleased(ke);
  }

  public void keyTyped(KeyEvent ke) {
    super.keyTyped(ke);
  }

  public void prevPlanet() {
    prevPlanet(true);
  }

  public void prevPlanet(boolean visitAlly) {
    int newid = planet.getId() - 1;
    if (newid == 0) newid = planets;
    while (newid != planet.getId()) {
      Planet newplanet = planet.getUniverse().getPlanet(newid);
      if ((newplanet != null) && newplanet.isOurPlanet()) {
	if ((newplanet.getOwner() == planet.getOwner()) || visitAlly) break;
      }
      newid--;
      if (newid == 0) newid = planets;
    }
    setPlanet((PData)planet.getUniverse().getPlanet(newid));
  }

  public void nextPlanet() {
    nextPlanet(true);
  }

  public void nextPlanet(boolean visitAlly) {
    int newid = planet.getId() + 1;
    if (newid > planets) newid = 1;
    while (newid != planet.getId()) {
      Planet newplanet = planet.getUniverse().getPlanet(newid);
      if ((newplanet != null) && newplanet.isOurPlanet()) {
	if ((newplanet.getOwner() == planet.getOwner()) || visitAlly) break;
      }
      newid++;
      if (newid > planets) newid = 1;
    }
    setPlanet((PData)planet.getUniverse().getPlanet(newid));
  }


  // Overridden methods

  public void paint(Graphics gc) {
    validate();
    super.paint(gc);
  }
  /*
  public Component add(Component comp) {
    report("PlanetFrame.add(Component) -- no Container functionality");
    return comp;
  }
  */
  public void report(String warning) {
    game.report(warning);
  }


  // PlanetFrame methods

  public void setPlanetFC() {
    planet.setFC(fcode.getText());
  }

  public void setPlanetNatTax() {
    planet.setNatTax(nattax.getInt());
  }

  public void setPlanetColTax() {
    planet.setColTax(coltax.getInt());
  }

  public void setPlanetSupplies() {
    planet.buySupplies(supplies.getInt() - planet.getSupplies());
    supplies.setInt(planet.getSupplies());
  }

  public void setPlanetMoney() {
    planet.sellSupplies(money.getInt() - planet.getMC());
    money.setInt(planet.getMC());
  }

  public void setPlanetMines() {
    planet.buildMines(mines.getInt() - planet.getMines());
    mines.setInt(planet.getMines());
  }

  public void setPlanetFactories() {
    planet.buildFactories(factories.getInt() - planet.getFactories());
    factories.setInt(planet.getFactories());
  }

  public void setPlanetDefense() {
    planet.buildDefense(defense.getInt() - planet.getDefense());
    defense.setInt(planet.getDefense());
  }

  public void setBuildBase() {
    if (planet.hasBase()) {
      base.setLabel("go to Base");
      base.enable();
    } else if (planet.buildingBase()) {
      base.setLabel("stop Building");
      base.enable();
    } else {
      base.setLabel("build Base");
      if (planet.canBuildBase()) base.enable(); else base.disable();
    }
  }
  
  public void baseButtonPressed() {
    if (planet.hasBase()) {
      applet.openBaseFrame(planet.getUniverse().getBData(planet.getId()));
    } else {
	if (planet.buildingBase()) {
	    planet.breakDown(Build.BASE, 1);
	} else {
	    planet.buildUp(Build.BASE, 1);
	}
    }
  }

  public void setBunchOfLabels() {
    temperature.setText("Temperature: "+planet.getTemp()+" degrees");
    owner.setText("Owner: "+planet.getOwner());
    if (planet.getNatClans() == 0) {
      natives.setText("no natives");
      nathappy.setText("   all very happy");
    } else {
      natives.setText(""+planet.getNatClans()+
		      " "+Government.getName(planet.getNatGov())+
		      " "+Natives.getName(planet.getNatSort())+" clans");
      nathappy.setText("   which are "+planet.getNatHappy()+
		       "% happy ("+planet.getNatHappyNT()+"%, "+
		       planet.getNatTaxMC()+"MC)");
    }
    colonists.setText(""+planet.getColClans()+" colonist clans");
    colhappy.setText("   which are "+planet.getColHappy()+
		     "% happy (will be "+planet.getColHappyNT()+"%, "+
		     planet.getColTaxMC()+"MC)");
    setBuildBase();
  }

  public void setPlanet(PData newplanet) {
    disableEvents();
    planet = newplanet;
    setTitle("P"+planet.getId()+": "+planet.getName());
    name.setText(planet.getName());
    fcode.setText(planet.getFC());
    setBunchOfLabels();
    nattax.setInt(planet.getNatTax());
    coltax.setInt(planet.getColTax());
    supplies.setInt(planet.getSupplies());
    money.setInt(planet.getMC());
    mines.setInt(planet.getMines());
    factories.setInt(planet.getFactories());
    defense.setInt(planet.getDefense());
    enableEvents();
  }

  public void viewCargoFrame() {
    Location location = planet.getLocation();
    applet.viewCargoFrame(location);
  }

  public void planetChanged(PlanetEvent pe) {
    if (pe.getId() == planet.getId()) setPlanet(planet);
  }
}
