// File: DoubleInput.java

package kero.awt;

import java.util.*;
import java.awt.*;
import java.awt.event.*;

/**
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Library Public License applies)
 * @since     14 XI 1999
 * @version   8 I 2000
 */
public class DoubleInput
  extends NumberInput
{
  static final double defaultstep = 1.0;
  static final double defaultchange = 10.0;
  static final int defaultamount = 2;

  double number;
  double step; // small step for decrease/increase (used for `<' and `>')
  double change; // change*step is bigger than step (used for `<<' and `>>')

  protected DoubleInput(String labeltext, double initNumber) {
    this(labeltext, initNumber, defaultstep, defaultchange, defaultamount);
  }

  /**
   * A subclass calling this constructor (recommended :-) should ensure that
   * the method getDouble() returns something special.
   */
  public DoubleInput(String labeltext, double initNumber,
			double step, double change, int amount) {
    number = initNumber;
    this.step = Math.abs(step);
    if (change >= 1.0) {
      this.change = change;
    } else {
      change = 1.0;
    }
    constructUI(labeltext, amount);
  }

  public void setDouble(double number) {
    if (this.number != number) {
      this.number = number;
      String numberstr = String.valueOf(number);
      textfield.setText(numberstr);
      throwActionPerformed(numberstr);
    }
  }

  /**
   * return the double in the form of a String (which is the only ``general''
   * form of a double). subclasses are encouraged to write functions like
   * getInt() for IntField, getDouble for DoubleField etc.
   */
  public double getDouble() {
    return number;
  }

  public String getNumber() {
    return String.valueOf(number);
  }

  public void setNumberFromField() {
    double newnumber = Double.valueOf(textfield.getText()).doubleValue();
    if (newnumber != number) {
      number = newnumber;
      // throw ActionEvent ?
    }
  }

  public void lowerNumber(int steps) {
    setDouble(number - change * Math.pow(step, steps));
  }

  public void raiseNumber(int steps) {
    setDouble(number + change * Math.pow(step, steps));
  }
}
