// File: FrameAdapter.java

package kero.awt;

import java.awt.*;
import java.awt.event.*;
import kero.util.*;

/**
 * Handles <CODE>WindowEvent</code>s when appropriate for <CODE>MyFrame</code>.
 * @see MyFrame
 * @see java.awt.WindowAdapter
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Library Public License applies)
 * @since     6 IX 1998
 * @version   21 XI 1999
 */
public class FrameAdapter
  extends Reporter
  implements WindowListener
{
  private int framecount;

  public FrameAdapter() {
    framecount=0;
  }

  public int getFrameCount() {
    return framecount;
  }

  public void setFrameCount(int count) {
    framecount=count;
  }

  public void windowIconified(WindowEvent we) {}
  public void windowDeiconified(WindowEvent we) {}

  public void windowActivated(WindowEvent we) {}
  public void windowDeactivated(WindowEvent we) {}

  public void windowOpened(WindowEvent we) {
    framecount++;
  }

  public void windowClosed(WindowEvent we) {
    framecount--;
    if (framecount==0) {
      // All MyFrames are closed, quit the application
      quit();
    }
  }

  public void windowClosing(WindowEvent we) {
    MyFrame myframe=(MyFrame)we.getWindow();
    myframe.closeFrame(); // default is:  dispose();
  }

  /**
   * Finishes the application. This method is called when there are no <CODE>
   * MyFrame</code>s left. Any subclass should override this method to call the
   * proper routines of the application. The default action is terminating the
   * application and the JVM. This method is not <CODE>static</code>, because a
   * call to the real finishing methods can be made anyway.
   */
  public void quit() {
    System.exit(0);
  }

  public void quit(Reportable r) {
    System.exit(0);
  }
}	
