// File: InputComponent.java

package kero.awt;

import java.util.*;
import java.awt.*;
import java.awt.event.*;

/**
 * Combines a Label and some Component into one Component. This is tha paent
 * class for classes that use a specific component.
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Library Public License applies)
 * @since     7 I 2001
 * @version   7 I 2001
 */
public class InputComponent
    extends Panel
{
    Label label;
    Component comp;

    public InputComponent(String labelText, Component c) {
	super(new FlowLayout(1,0,0));
	label = new Label(labelText);
	super.add(label);
	comp = c;
	super.add(comp);
    }


    // addition and removal of several event listeners

    public void addFocusListener(FocusListener listener) {
	comp.addFocusListener(listener);
    }

    public void removeFocusListener(FocusListener listener) {
	comp.removeFocusListener(listener);
    }

    public void addKeyListener(KeyListener listener) {
	comp.addKeyListener(listener);
    }

    public void removeKeyListener(KeyListener listener) {
	comp.removeKeyListener(listener);
    }


    // Directly passable requests

    public void requestFocus() {
	comp.requestFocus();
    }


    // Label methods

    public void setLabel(String labeltext) {
	label.setText(labeltext);
    }

    public String getLabel() {
	return label.getText();
    }

    public void setLabelFont(Font font) {
	label.setFont(font);
    }


    // overridden Container methods (really?)

    public final Component add(Component component) {
	System.err.println("An InputComponent is not a Container as such.\n"+
			   "Component not added.\n");
	return component;
    }

    public Dimension getPreferredSize() {
	Dimension labelSize = label.getPreferredSize();
	Dimension textSize = comp.getPreferredSize();
	int prefWidth = labelSize.width + textSize.width;
	int prefHeight = labelSize.height;
	if (textSize.height > prefHeight) prefHeight = textSize.height;
	return new Dimension(prefWidth, prefHeight);
    }

    public void doLayout() {
	//System.err.println("doLayout()\n"+
	//		       "size: "+getSize()+"\n"+
	//	               "  insets: "+getInsets()+"\n");
	Dimension ownSize=getSize();
	Dimension labelSize=label.getPreferredSize();
	Dimension textSize=comp.getPreferredSize();
	label.setSize(labelSize.width, textSize.height);
	label.setLocation(0, 0);
	comp.setSize(ownSize.width-labelSize.width, textSize.height);
	comp.setLocation(labelSize.width, 0);
    }
}
