// File: InputLayout.java

package kero.awt;

import java.util.*;
import java.awt.*;
import java.awt.event.*;

public class InputLayout implements LayoutManager {
    public void addLayoutComponent (String name, Component component) {}
    public void removeLayoutComponent (Component component) {}
    public Dimension preferredLayoutSize (Container target) {
	Component children[] = target.getComponents();
	int w=0, h=0;
	for (int i=0; i<children.length; i++) {
	    Dimension prefSize = children[i].getPreferredSize();
	    w += prefSize.width;
	    if (prefSize.height > h) h = prefSize.height;
	}
	return new Dimension(w, h);
    }
    public Dimension minimumLayoutSize (Container target) {
	Component children[] = target.getComponents();
	return children[children.length / 2].getPreferredSize();
    }
    public void layoutContainer (Container target) {
	Dimension dim = target.getSize();
	int width=dim.width, height = dim.height;
	Component children[] = target.getComponents();
	Component label = null;
	if (children.length % 2 == 0) label = children[0];
	int textIndex = children.length / 2;
	Component input = children[textIndex];
	int inputWidth = input.getPreferredSize().width;
	int i;
	double factor = 1.0;

	int totalWidth = 0;
	for (i=0; i<children.length; i++) {
	    totalWidth += children[i].getPreferredSize().width;
	}
	if (width >= totalWidth) {
	    // happy, grow input
	} else if (width >= inputWidth) {
	    // not enough space, shrink label, dec and inc
	    factor = (double)(width - inputWidth) / (totalWidth - inputWidth);
	} else {
	    // very unhappy, can't even fit input completely
	    factor = 0.0;
	}

	int w, x;
	x = 0;
	if (factor > 0.0) {
	    if (label != null) {
		w = (int)(factor * label.getPreferredSize().width);
		label.setSize(w, height);
		label.setLocation(x, 0);
		x += w;
	    }
	    for (i=1; i<textIndex; i++) {
		w = (int)(factor * children[i].getPreferredSize().width);
		children[i].setSize(w, height);
		children[i].setLocation(x, 0);
		x += w;
	    }
	}
	if (factor == 1.0) w = width - (totalWidth - inputWidth);
	else if (factor > 0.0) w = inputWidth;
	else w = width;
	input.setSize(w, height);
	input.setLocation(x, 0);
	x += w;
	if (factor > 0.0) {
	    for (i=textIndex+1; i<children.length; i++) {
		w = (int)(factor * children[i].getPreferredSize().width);
		children[i].setSize(w, height);
		children[i].setLocation(x, 0);
		x += w;
	    }
	}
    }
}
