// File: IntInput.java

package kero.awt;

import java.util.*;
import java.awt.*;
import java.awt.event.*;

/**
 * Integer version of NumberInput. Small decrease and increase of course are of
 * step 1, larger in/decrease is configurable at constructor-call.
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Library Public License applies)
 * @since     14 XI 1999
 * @version   14 III 2001
 */
public class IntInput
  extends NumberInput
{
  static final int defaultchange = 10;
  static final int defaultamount = 2;

  int number;
  int change; // change is bigger than step (used for `<<' and `>>')

  protected IntInput(String labeltext, int initnr) {
    this(labeltext, initnr, defaultchange, defaultamount);
    System.err.println("IntInput {1} constructed!!!");
  }

  /**
   * A subclass calling this constructor (recommended :-) should ensure that
   * the method getInt() returns something special.
   */
  public IntInput(String labeltext, int initnr, int change, int amount) {
    super();
    number = initnr;
    if (change < 2) change = 2; // factor of one is senseless
    this.change = change;
    constructUI(labeltext, amount);
    setInt(number);
  }

    public void setInt(int number) {
	if (this.number != number) {
	    setIntNoAction(number);
	    String numberstr = String.valueOf(number);
	    throwActionPerformed(numberstr);
	}
    }

    public void setIntNoAction(int number) {
	this.number = number;
	String numberstr = String.valueOf(number);
	textfield.setText(numberstr);
    }

  /**
   * return the int in the form of a String (which is the only ``general''
   * form of a int). subclasses are encouraged to write functions like
   * getInt() for IntField, getInt for IntField etc.
   */
  public int getInt() {
    return number;
  }

  public String getNumber() {
    return String.valueOf(number);
  }

  public void setNumberFromField() {
      int oldint = getInt();
      try {
	  setInt(Integer.parseInt(textfield.getText()));
      } catch (NumberFormatException nfe) {
	  if ("".equals(textfield.getText())) {
	      setInt(0);
	  } else {
	      setInt(oldint); // or set illegal?
	  }
      }
  }

  public void lowerNumber(int step) {
    setInt(number - (int)Math.pow(change, step));
  }

  public void raiseNumber(int step) {
    setInt(number + (int)Math.pow(change, step));
  }
}
