// File: MyScrollbar.java

package kero.awt;

import java.awt.Scrollbar;

/**
 * Replacement for java.awt.Scrollbar, because the behaviour of the constructor
 * changed <EM>slightly</em> from jdk 1.1 to 1.2. Look up the API on the web if
 * you want to know exactly what I mean:
 * @see http://java.sun.com/products/jdk/1.1/docs/api/java.awt.Scrollbar.html
 * @see http://java.sun.com/products/jdk/1.2/docs/api/java/awt/Scrollbar.html
 *
 * @author Kero van Gelder
 * @copyright Kero van Gelder (the GNU Lesser Public License applies)
 * @since 3 III 2001
 * @version 3 III 2001
 */
public class MyScrollbar extends Scrollbar {
    static final int jdk11 = 11;
    static final int jdk12 = 12;

    static int behaviour;

    static {
	Scrollbar bar = new Scrollbar(Scrollbar.HORIZONTAL, 0, 64, 0, 255);
	int max = bar.getMaximum();
	if (max == 255) {
	    System.err.println("Scrollbar behaves as jdk 1.1");
	    behaviour = jdk11;
	} else if (max == 255 - 64) {
	    System.err.println("Scrollbar behaves as jdk 1.2/3");
	    behaviour = jdk12; // jdk 1.2/3 behaviour, [0..255-64]
	} else {
	    // What the ...
	    System.err.println("Constructing Scrollbar(HOR, 0, 64, 0, 255)"+
			       "returns unintelligable maximum of: "+max);
	    System.exit(1);
	}
    }

    public MyScrollbar() {
	super();
    }

    public MyScrollbar(int orientation) {
	super(orientation);
    }

    public MyScrollbar(int orientation,
		       int value,
		       int visible, int minimum, int maximum) {
	super(orientation,
	      value,
	      visible, minimum, maximum + (behaviour==jdk11? 0: visible));
    }

    public static void main(String[] arg) {}
}
