package kero.awt;

import java.awt.TextField;
import java.awt.Color;
import kero.awt.event.*;

/**
 * @author Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since 12 I 2001
 * @version 13 I 2001
 */
public class TimeField extends MyTextField implements InputListener {
    int hours, minutes;

    public TimeField(String s) {
	super(s);
	illegal = false;
	checkValidity();
	parseContent();
	setupInputListener();
    }

    public TimeField(int hours, int minutes) {
	super(""+hours+":"+minutes);
	illegal = false;
	this.hours = hours;
	this.minutes = minutes;
	setupInputListener();
    }

    public int getHours() {
	return hours;
    }

    public int getMinutes() {
	return minutes;
    }

    protected void parseContent() {
	String content = getText();
	int colonIndex = content.indexOf(':');

	String hourString = content.substring(0, colonIndex);
	System.err.println("hour: "+hourString);
	try {
	    hours = Integer.parseInt(hourString);
	} catch (NumberFormatException nfe) {
	    // ignore
	}

	String minuteString = content.substring(colonIndex+1, content.length());
	System.err.println("minute: "+minuteString);
	try {
	    minutes = Integer.parseInt(minuteString);
	} catch (NumberFormatException nfe) {
	    // ignore
	}
    }

    /**
     * Verifies whether the String in this field suffices a time-string, that
     * is two digits followed by a colon and two more digits.
     * <P>If there are any clocks with time not being built like this (or
     * representable by this), I am sorry.
     */
    private void checkValidity() {
	StringBuffer str = new StringBuffer(getText());
	int i = 0;

	int digits = 0;
	while ((i<str.length()) && Character.isDigit(str.charAt(i))) {
	    digits++;
	    i++;
	}
	if (digits == 0) setIllegal();

	while ((i<str.length()) && !Character.isDigit(str.charAt(i))) {
	    if (str.charAt(i) == ':') {
		i++;
	    } else {
		// str.deleteCharAt(i);
		i++;
	    }
	}
	if (i == digits) str.insert(i, ":");

	digits = 0;
	while ((i<str.length()) && Character.isDigit(str.charAt(i))) {
	    digits++;
	    i++;
	}
	if (digits == 0) setIllegal();
    }

    public void inputGiven(InputEvent ie) {
	parseContent();
    }
}
