// File: InputChoice.java

package kero.awt;

import java.util.*;
import java.awt.*;
import java.awt.event.*;

/**
 * Combines a Label and a Choice into one Component, such that it functions
 * as an input choice. When an ActionEvent from the Choice is caught, the
 * event is passed onto objects that registered as ActionListener to an
 * instance of this class.
 *
 * <P>The constuctions for passing events from the choice to the listeners
 * of the entire InputChoice are bad; they should use the mechanism provided by
 * the jdk. Then the for-loop can be synchronized. Only then the events are
 * completely thread-safe.
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Library Public License applies)
 * @since     8 I 2000
 * @version   7 I 2001
 */
public class InputChoice
    extends InputComponent
	    // implements ItemListener, KeyListener, FocusListener
{
    Choice choice;

    public InputChoice(String labelText) {
	super(labelText, new Choice());
	choice = (Choice)comp;
    }


    // addition and removal of several event listeners

    public void addItemListener(ItemListener listener) {
	choice.addItemListener(listener);
    }

    public void removeItemListener(ItemListener listener) {
	choice.removeItemListener(listener);
    }


    /**
     * Preferably do not use this method. It may disappear in future versions.
     */
    public Choice getChoice() {
	return choice;
    }

    public String getSelection() {
	return choice.getSelectedItem();
    }

    public int getIndex() {
	return choice.getSelectedIndex();
    }

    public void setSelection(String text) {
	choice.select(text);
    }

    public void setSelection(int index) {
	choice.select(index);
    }

    public void setTextFont(Font font) {
	choice.setFont(font);
    }

    public void addSelection(String text) {
	choice.addItem(text);
    }

    public void removeAll() {
	choice.removeAll();
    }
}
