// File: InputField.java

package kero.awt;

import java.util.*;
import java.awt.*;
import java.awt.event.*;

/**
 * Combines a Label and a TextField into one Component, such that it functions
 * as an input field. Interesting events can be subscribed to; the request is
 * pased to the TextField.
 *
 * <P>Note that when focus is lost, you will be interested in the value, too.
 * MyContainer and InputCollector take care of that.
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Library Public License applies)
 * @since     11 IX 1998
 * @version   7 I 2001
 */
public class InputField
    extends Panel
    implements ActionListener, FocusListener
{
    Label label;
    TextField textfield;
    String value;

    public InputField(String labeltext, String fieldtext) {
	super(new FlowLayout(1,0,0));
	label = new Label(labeltext);
	super.add(label);
	textfield = new TextField(fieldtext);
	textfield.addActionListener(this);
	textfield.addFocusListener(this);
	super.add(textfield);
	value = fieldtext;
	activate();
    }


    // do I still need these two...

    public void deactivate() {
	textfield.setEditable(false);
	//textfield.disable();
	textfield.removeActionListener(this);
    }

    public void activate() {
	textfield.setEditable(true);
	//textfield.enable();
	textfield.addActionListener(this);
    }


    // admission and withdrawal of several event listeners

    public void addActionListener(ActionListener listener) {
	textfield.addActionListener(listener);
    }

    public void removeActionListener(ActionListener listener) {
	textfield.removeActionListener(listener);
    }

    public void addFocusListener(FocusListener listener) {
	textfield.addFocusListener(listener);
    }

    public void removeFocusListener(FocusListener listener) {
	textfield.removeFocusListener(listener);
    }

    public void addKeyListener(KeyListener listener) {
	textfield.addKeyListener(listener);
    }

    public void removeKeyListener(KeyListener listener) {
	textfield.removeKeyListener(listener);
    }


    // other various methods passed straight the TextField

    public void selectAll() {
	textfield.selectAll();
    }


    // one ActionListener method

    public void actionPerformed(ActionEvent ae) {
	value = ae.getActionCommand();
    }


    // two FocusListener methods plus additional Focus method

    public void focusGained(FocusEvent fe) {
	// ignore for self
    }

    public void focusLost(FocusEvent fe) {
	// set value for self
	value = textfield.getText();
    }

    public void requestFocus() {
	textfield.requestFocus();
    }


    // Normal interaction methods

    public void setLabel(String labeltext) {
	label.setText(labeltext);
    }

    public String getLabel() {
	return label.getText();
    }

    public void setLabelFont(Font font) {
	label.setFont(font);
    }

    /**
     * Preferably do not use this method. It may disappear in future versions.
     */
    public TextField getTextField() {
	return textfield;
    }

    public String getText() {
	return textfield.getText();
    }

    public void setText(String text) {
	textfield.setText(text);
    }

    public void setTextFont(Font font) {
	textfield.setFont(font);
    }


    // the (il)legal part does not function well !!!
  
    /**
     * Internal reverter of setIllegal()
     */
    protected void setLegal() {
	textfield.setForeground(Color.black);
    }

    /**
     * Meant to be called by inputFieldEvent-receiver when input-value was
     * illegal. Will be unset when new input has been given (needs to be tested
     * with a test-run...).
     */
    public void setIllegal() {
	// Hm.. What is illegal for a TextField? [Kero]
	textfield.selectAll();
	textfield.setForeground(Color.red);
	// erm, set some internal Boolean?
    }


    // overridden Container methods

    public final Component add(Component component) {
	System.err.println("An InputField is not a Container as such.\n"+
			   "Component not added.\n");
	return component;
    }

    public Dimension getPreferredSize() {
	Dimension labelsize = label.getPreferredSize();
	Dimension textsize = textfield.getPreferredSize();
	int prefwidth = labelsize.width + textsize.width;
	int prefheight = labelsize.height;
	if (textsize.height > prefheight) prefheight = textsize.height;
	return new Dimension(prefwidth, prefheight);
    }

    public void doLayout() {
	//System.err.println("doLayout()\n"+
	//		       "size: "+getSize()+"\n"+
	//	               "  insets: "+getInsets()+"\n");
	Dimension ownsize=getSize();
	Dimension labelsize=label.getPreferredSize();
	Dimension textsize=textfield.getPreferredSize();
	label.setSize(labelsize.width, textsize.height);
	label.setLocation(0,0);
	textfield.setSize(ownsize.width-labelsize.width, textsize.height);
	textfield.setLocation(labelsize.width,0);
    }
}
