// File BinaryNode.java

package kero.util;

import java.util.Enumeration;

/**
 * BinaryNode with parent pointer.
 *
 * @author Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Library Public License applies)
 * @since 17 V 2000
 * @version 17 IX 2000
 */
public class BinaryNode
    extends Reporter
{
    protected BinaryNode parent;
    protected BinaryNode left, right;


    // get/inquiring methods

    public BinaryNode getParent() {
	return parent;
    }

    public BinaryNode getLeft() {
	return left;
    }

    public BinaryNode getRight() {
	return right;
    }

    public BinaryNode getPrev() {
	BinaryNode node = this;
	if (node.left != null) {
	    // we've had the right branch, now take the left branch
	    node = node.left;
	    while (node.right != null) node = node.right;
	} else if (node.parent == null) {
	    // we're done
	    node = null;
	} else {
	    // we've had this left branch, go up until you're a right branch
	    while (node.parent != null && node == node.parent.left) {
		node = node.parent;
	    }
	    node = node.parent; // can be null
	}
	return node;
    }

    public BinaryNode getNext() {
	BinaryNode node = this;
	if (node.right != null) {
	    // we've had the left branch, now take the right branch
	    node = node.right;
	    while (node.left != null) node = node.left;
	} else if (node.parent == null) {
	    // we're done
	    node = null;
	} else {
	    // we've had this right branch, go up until you're a left branch
	    while (node.parent != null && node == node.parent.right) {
		node = node.parent;
	    }
	    node = node.parent; // can be null
	}
	return node;
    }

    public BinaryNode getRoot() {
	BinaryNode result = this;
	while (result.parent != null) result = result.parent;
	return result;
    }

    public boolean isAncestorOf(BinaryNode node) {
	if (node == this) return false;
	while (node != null) {
	    if (node == this) return true;
	    node = node.parent; // abuse parameter
	}
	return false;
    }

    public boolean isOnlyNode() {
	return ((parent == null) && (left == null) && (right == null));
    }

    public Enumeration elements() {
	return new BinaryNodeEnumeration(this);
    }


    // setting methods

    void setParent(BinaryNode node) {
	parent = node;
    }

    void setLeft(BinaryNode node) {
	left = node;
    }

    void setRight(BinaryNode node) {
	right = node;
    }

    public void insertLeft(BinaryNode node) {
	setLeft(node);
	node.parent = this;
    }

    public void insertRight(BinaryNode node) {
	setRight(node);
	node.parent = this;
    }

    public String toString(int depth) {
	StringBuffer result = new StringBuffer();
	if (left != null) result.append(left.toString(depth+1));
	for (int i=0; i<depth; i++) result.append('\t');
	result.append(toString());
	result.append('\n');
	if (right != null) result.append(right.toString(depth+1));
	return result.toString();
    }
}


class BinaryNodeEnumeration implements Enumeration {
    BinaryNode current;

    public BinaryNodeEnumeration(BinaryNode root) {
	current = root;
	if (current != null) {
	    while (current.left != null) current = current.left;
	}
    }

    public boolean hasMoreElements() {
	return (current != null);
    }

    public Object nextElement() {
	BinaryNode result = current;
	if (current != null) current = current.getNext();
	return result;
    }
}
