// File: LineReader.java

package kero.util;

import java.io.*;

/**
 * Reads lines from an InputStream.
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since     18 XII 1998
 * @version   25 X 1999
 */
public class LineReader extends Reader {
  private InputStream is;
  private char sym; // looks one character ahead

  public LineReader(InputStream is) throws IOException {
    this.is=is;
    nextSym();
  }

  public int available() throws IOException {
    return is.available();
  }

  public boolean eofReached() throws IOException {
    return (available() == 0);
  }
  
  public void close() throws IOException {
    is.close();
  }

  protected void nextSym() throws IOException {
    sym=(char)is.read(); // What's the use if this does not throw at EOF???
    // System.err.print(sym);
  }
  
  /**
   * Skips a number of bytes.
   *
   * @param size Amount of skipped bytes.
   */
  public long skip(long size) throws IOException {
    if (size<=0) return 0;
    long result=is.skip(size-1);
    nextSym();
    return result;
  }

  public char readChar() throws IOException {
    char ch=sym;
    skipSpace();
    nextSym();
    return ch;
  }

  public char peek() {
    return sym;
  }

  public String readAlphaNumericString() throws IOException {
    StringBuffer result=new StringBuffer();
    skipSpace();
    while (Character.isLetterOrDigit(sym)) {
      result.append(sym);
      nextSym();
    }
    // System.err.print(result+",");
    return result.toString();
  }

  public String readRestOfLine() throws IOException {
    StringBuffer result=new StringBuffer();
    while ((sym!=(char)10) && (sym!=(char)13) && ((short)sym!=-1)) {
      result.append(sym);
      nextSym();
    }
    switch (sym) {
    case (char)10:
      nextSym();
      if (sym==(char)13) nextSym();
      break;
    case (char)13:
      sym=(char)is.read();
      if (sym==(char)10) nextSym();
      break;
    default:
      // occurs only at EOF
      break;
    }
    return result.toString();
  }

  public void skipSpace() throws IOException {
    while ((sym!=-1) && Character.isWhitespace(sym)) nextSym();
  }

  public void skipLine() throws IOException {
    while ((sym!=(char)10) && (sym!=(char)13) && ((short)sym!=-1)) {
      nextSym();
    }
    switch (sym) {
    case (char)10:
      nextSym();
      if (sym==(char)13) nextSym();
      break;
    case (char)13:
      nextSym();
      if (sym==(char)10) nextSym();
      break;
    default:
      // occurs only at EOF
      break;
    }
  }

  public void passChar(char ch) throws IOException {
    while ((sym!=ch) && ((short)sym!=-1)) {
      nextSym();
    }
    nextSym(); // char ch itself
  }

  public StringBuffer readUntilChar(char ch) throws IOException {
    StringBuffer result = new StringBuffer(256);
    while ((sym!=ch) && ((short)sym!=-1)) {
      result.append(sym);
      nextSym();
    }
    return result;
  }

  public int read(char[] string, int size, int amount) throws IOException {
    throw new IOException("Not implemented");
  }
}
