package kero.util;

import java.util.*;

/**
 * Provides extra functionality on top of a Date, for convenience.
 *
 * @author Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since 4 I 2001
 * @version 13 I 2001
 */
public class MyDate extends java.util.GregorianCalendar implements Cloneable {
    public static final String shortWeekString[] = {
	"zero", "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat",
    };
    public static final String shortMonthString[] = {
	"Jan", "Feb", "Mar", "Apr", "May", "Jun",
	"Jul", "Aug", "Sep", "Oct", "Nov", "Dec",
    };

    public MyDate() {
	super();
	/* We look only at date and time down to minute accuracy */
	set(Calendar.SECOND, 0);
	set(Calendar.MILLISECOND, 0);
    }

    public MyDate(int year, int month, int dayOfMonth) {
	super();
	set(year, month, dayOfMonth);
	set(Calendar.SECOND, 0);
	set(Calendar.MILLISECOND, 0);
    }

    public MyDate(int hour, int minute) {
	super();
	set(Calendar.HOUR_OF_DAY, hour);
	set(Calendar.MINUTE, minute);
	set(Calendar.SECOND, 0);
	set(Calendar.MILLISECOND, 0);
    }

    public Object clone() {
	// return (MyDate)(super.clone()); // Kaffe can't handle :-(
	MyDate result = new MyDate();
	result.setTime(getTime());
	return result;
    }

    public MyDate prevDay() {
	MyDate result = (MyDate)clone();
	result.add(Calendar.DAY_OF_MONTH, -1);
	return result;
    }

    public MyDate nextDay() {
	MyDate result = (MyDate)clone();
	result.add(Calendar.DAY_OF_MONTH, 1);
	return result;
    }

    public String getDateString() {
	return ""+get(Calendar.DAY_OF_MONTH)+
	    " "+shortMonthString[get(Calendar.MONTH)]+
	    " "+get(Calendar.YEAR);
    }

    public String getTimeString() {
	int hour = get(Calendar.HOUR_OF_DAY);
	int minute = get(Calendar.MINUTE);
	return ((hour<10)?"0":"")+hour+":"+((minute<10)?"0":"")+minute;
    }

    public String toString() {
	return getTime().toString();
	//Calendar.get(DAY_OF_WEEK, this)+" "+getDateString();
    }
}
