// File: Reportable.java

package kero.util;

/**
 * Provides a set of functions to report to the programmer during debugging,
 * but also to the user during execution (perhaps the latter should be moved to
 * another interface, sometime).
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Library Public License applies)
 * @since     V 1998
 * @version   21 XII 1999
 */
public interface Reportable {
  final static int noError = 0; // Succesful run
  final static int javaError = 1; // Java failed somehow
  final static int javaLibError = 10; // java.* mistake
  final static int extLibError = 100; // third party package mistake
  final static int internalError = 1000; // I made a mistake

  /*
    public static final Reportable BLACKHOLE = new Reporter() {
      public void setReportable(Reportable r) {}
      public void unsetReportable() {}
      public void report(String message) {}
      public void warn(String warning) {}
      public void crash(int errno, String error) {
	     System.err.println("Fatal error "+errno+": "+error);
      }
    };
  */
  /**
   * Provide another channel to report to.
   * Most likely not used by warn() and crash()
   */
  public void setReportable(Reportable r);

  /**
   * Drop any previously set report channel
   * Most likely not used by warn() and crash()
   */
  public void unsetReportable();

  /**
   * Report some information to the user. Do it nicely. Some day I'll finish
   * my Messaging System, then there will be a fileMessage(), as well.
   */
  public void report(String message);

  /**
   * Report some information to the programmer.
   */
  public void warn(String warning);

  /**
   * Report some information to the programmer, then quit the program.
   */
  public void crash(int errno, String error);
}
