// File: Reporter.java

package kero.util;

/**
 * Default implementation of the following:
 * <UL>
 *   <LI>Reportable
 *   <LI>public static void assert()
 * </ul>
 *
 * <P>Carries a static BLACKHOLE, which I'd rather see in Reportable, but some
 * java compilers have trouble with that.
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Library Public License applies)
 * @since     12 IX 1999
 * @version   2 IV 2000
 */
public class Reporter implements Reportable {

  public static final Reportable BLACKHOLE = new Reportable() {
    public void setReportable(Reportable r) {}
    public void unsetReportable() {}
    public void report(String message) {}
    public void warn(String warning) {}
    public void crash(int errno, String error) {
      System.err.println("Fatal error "+errno+": "+error);
    }
  };


  Reportable reportable;

  public void setReportable(Reportable r) {
    reportable = r;
  }

  public void unsetReportable() {
    reportable = null;
  }

  public void report(String message) {
    if (reportable != null) {
      reportable.report(message);
    } else {
      System.err.print(message);
      if ((message.length()>1) || (message.indexOf("...")<0)) {
	System.err.print("\n");
      }
    }
  }

  public void warn(String message) {
    System.err.print(message);
    if ((message.length()>1) || (message.indexOf("...")<0)) {
      System.err.print("\n");
    }
  }

  public void crash(int errno, String message) {
    System.err.println("\nFatal Error "+errno+":\n"+message+"\n");
    Thread.dumpStack();
    System.exit(errno);
  }

  public static void assert(boolean correct) {
    if (!correct) {
      Thread.dumpStack();
      System.exit(internalError);
    }
  }
}
