// File: Ring.java

package kero.util;

/**
 * A Ring is a doubly linked list with its tail connected to the head.
 */
public class Ring extends Reporter {
  protected RingElement head;
  protected RingElement current;
  protected int elements;

  public Ring() {}

  public boolean isEmpty() {
    return (elements == 0);
  }

  /**
   * Add the new element directly after the current one. The current element
   * becomes the new element.
   */
  public void addElement(RingElement element) {
    if (isEmpty()) {
      head = element;
      element.next = element;
      element.prev = element;
    } else {
      element.next = current.next;
      element.prev = current;
      element.next.prev = element;
      current.next = element;
    }
    current = element;
    elements++;
  }

  /**
   * Add the element just before the head (i.e. after the tail). Both head and
   * current remain the same.
   */
  public void addElementAsLast(RingElement element) {
    if (isEmpty()) {
      head = current = element;
      element.next = element;
      element.prev = element;
    } else {
      element.prev = head.prev;
      element.next = head;
      element.prev.next = element;
      head.prev = element;
    }
    elements++;
  }

  /**
   * Removes the current element from the ring. This element is returned. The
   * current element becomes the next element from the ring.
   */
  public RingElement removeCurrent() {
    if (isEmpty()) return null;
    RingElement result = current;
    if (current.prev == current) {
      head = current = null;
    } else {
      current.prev.next = current.next;
      current.next.prev = current.prev;
      current = current.next;
      result.prev = result.next = null;
    }
    elements--;
    return result;
  }

  public void moveCurrentLower() {
    if (isEmpty()) return;
    current = current.prev;
  }

  public void moveCurrentUp() {
    if (isEmpty()) return;
    current = current.next;
  }

  public RingElement getHead() {
    return head;
  }

  public RingElement getCurrent() {
    return current;
  }

  public String toString() {
    String result = "Ring has "+elements+" elements";
    result += "\nCurrent: "+current;
    result += "\nHead: "+head;
    RingElement element = head.next;
    while (element != head) {
      result += "\n"+element;
      if (element.prev.next != element) {
	System.err.println(result);
	Assert.assert(false);
      }
      element = element.next;
    }
    result += "\n";
    return result;
  }
}
