// File: Alliance.java

package jvpc.vga;

import java.io.*;

/**
 * Contains a complete alliance report, as given in util.dat record 22.
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since     15 I 1999
 * @version   6 II 1999
 */
public class Alliance
  extends RstOrTrn
{
    private static final int SHIPLEVEL=1;
    private static final int PLANETLEVEL=2;
    private static final int MINEFIELDLEVEL=4;
    private static final int COMBATLEVEL=8;
    private static final int VISIONLEVEL=16;
    private static final int ACTIVEBIT=32;

    int player;

    byte[] offerto;
    byte[] offerfrom;
    byte[] conditionalto;
    byte[] conditionalfrom;

    public Alliance() {
	this(null,0);
    }

    public Alliance(Universe u,int p) {
	super(u);
	player=p;
	offerto=new byte[races+1];
	offerfrom=new byte[races+1];
	conditionalto=new byte[races+1];
	conditionalfrom=new byte[races+1];
    }

    public void readFrom(LSBReader source) throws IOException {
	int i;
	for (i=1;i<=races;i++) offerto[i]=source.readByte();
	for (i=1;i<=races;i++) offerfrom[i]=source.readByte();
	for (i=1;i<=races;i++) conditionalto[i]=source.readByte();
	for (i=1;i<=races;i++) conditionalfrom[i]=source.readByte();
	// Now fill Player from Universe to some sensible default
	// values.
	Universe u = getUniverse();
	Player me = u.getPlayer(player);
	me.setStatus(Player.SELF);
	for (i=1; i<=races; i++) {
	    Player you = u.getPlayer(i);
	    int newstatus = Player.NEUTRAL;
	    if (allowsShipAlly(i) &&
		allowsPlanetAlly(i) &&
		allowsMinefieldAlly(i) &&
		allowsCombatAlly(i) &&
		allowsVisionAlly(i))
		newstatus = Player.ALLY;
	    if (allowsPlanetAlly(i)) {
		you.setPlanetStatus(false, newstatus);
	    }
	    if (allowsMinefieldAlly(i)) {
		you.setMinefieldStatus(false, newstatus);
	    }
	    if (allowsCombatAlly(i)) {
		// Combat makes other-ones ships more friendly
		you.setShipStatus(false, newstatus);
	    }
	}
    }

    public boolean offerTo(int player) {
	return (offerto[player] & ACTIVEBIT) == ACTIVEBIT;
    }

    public boolean offerFrom(int player) {
	return (offerfrom[player] & ACTIVEBIT) == ACTIVEBIT;
    }

    /**
     * Returns true if player gets ally message from each other
     * WARNING: this does not mean there is an actual _active_
     * alliance on some level.
     */
    public boolean isAlly(int player) {
	return offerTo(player) && offerFrom(player);
    }

    /**
     * Returns true if current player offers a ship alliance to player
     * @param player
     */
    public boolean offersShipAlly(int player) {
	return isAlly(player) &&
	    (((offerto[player] & SHIPLEVEL) |
	      (conditionalto[player] & conditionalfrom[player] & SHIPLEVEL))>0);
    }

    public boolean allowsShipAlly(int player) {
	return isAlly(player) &&
	    (((offerfrom[player] & SHIPLEVEL) |
	      (conditionalto[player] & conditionalfrom[player] & SHIPLEVEL))>0);
    }

    public boolean offersPlanetAlly(int player) {
	return isAlly(player) &&
	    (((offerto[player] & PLANETLEVEL) |
	      (conditionalto[player] & conditionalfrom[player] & PLANETLEVEL))>0);
    }

    public boolean allowsPlanetAlly(int player) {
	return isAlly(player) &&
	    (((offerfrom[player] & PLANETLEVEL) |
	      (conditionalto[player] & conditionalfrom[player] & PLANETLEVEL))>0);
    }

    public boolean offersMinefieldAlly(int player) {
	return isAlly(player) &&
	    (((offerto[player] & MINEFIELDLEVEL) |
	      (conditionalto[player] & conditionalfrom[player] & MINEFIELDLEVEL))>0);
    }

    public boolean allowsMinefieldAlly(int player) {
	return isAlly(player) &&
	    (((offerfrom[player] & MINEFIELDLEVEL) |
	      (conditionalto[player] & conditionalfrom[player] & MINEFIELDLEVEL))>0);
    }

    public boolean offersCombatAlly(int player) {
	return isAlly(player) &&
	    (((offerto[player] & COMBATLEVEL) |
	      (conditionalto[player] & conditionalfrom[player] & COMBATLEVEL))>0);
    }

    public boolean allowsCombatAlly(int player) {
	return isAlly(player) &&
	    (((offerfrom[player] & COMBATLEVEL) |
	      (conditionalto[player] & conditionalfrom[player] & COMBATLEVEL))>0);
    }

    public boolean offersVisionAlly(int player) {
	return isAlly(player) &&
	    (((offerto[player] & VISIONLEVEL) |
	     (conditionalto[player] & conditionalfrom[player] & VISIONLEVEL))>0);
    }

    public boolean allowsVisionAlly(int player) {
	return isAlly(player) &&
	    (((offerfrom[player] & VISIONLEVEL) |
	     (conditionalto[player] & conditionalfrom[player] & VISIONLEVEL))>0);
    }
}
