// File: BData.java

package jvpc.vga;

import java.io.*;

/**
 * Implements a planet of your own <SMALL>from <CODE>BDATA??.DAT</code>, hence
 * the ugly name</small>.
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since     24 II 1998
 * @version   18 III 2001
 */
public class BData extends PData implements Build{
    public static int fixShip = 1;
    public static int recycleShip = 2;

    int damage;
    int techLevel[];
    //int[] engine,hull,beam,launch; // stuff in storage
    int fixrec,target,mission;
    int bhulltype,bengtype,bbeamtype,bbeams,btorptype,blaunchs; // build ship
    int unknown;

    //public BData() {
    //  this(null);
    //}

    /**
     * Constructs the default <CODE>BData</code> object. All fields are <CODE>
     * UNKNOWN</code>.
     */
    public BData(Universe u) {
	super(u);
	struct = new int[Build.BDATABUILDTYPES]; // override PData length
	id = owner = damage = UNKNOWN;
	techLevel = new int[nrOfTechLevels];
	for (int i=0; i<nrOfTechLevels; i++) techLevel[i] = UNKNOWN;
	for (int i=BASICBDATABUILDTYPES; i<BDATABUILDTYPES; i++) {
	    struct[i] = UNKNOWN;
	}
	fixrec = target = mission = UNKNOWN;
	bhulltype = bengtype = bbeamtype = bbeams = btorptype = blaunchs = UNKNOWN;
	unknown = UNKNOWN; // :-)
    }

    public BData(Planet p) {
	this(p.getUniverse());
	copyPlanetFields(p);
    }

    private void copyPlanetFields(Planet p) {
	universe = p.getUniverse();
	// Located
	x = p.getX();
	y = p.getY();
	// Owned
	id = p.getId();
	owner = p.getOwner();
	// CargoTransfer
	for (int i=0; i<Cargo.CARGOTYPES; i++) cargo[i].amount = p.getCargo(i);
	// Planet
	name = new StringBuffer(p.getName());
	// PData
	fc = new StringBuffer(p.getFC());
	struct[Build.MINE] = p.getMines();
	struct[Build.FACTORY] = p.getFactories();
	struct[Build.DEFENSE] = p.getDefense();
	struct[Build.BASE] = 1; // of course there is a base on this planet
	coltax = p.getColTax();
	colhappy = p.getColHappy();
	nattax = p.getNatTax();
	nathappy = p.getNatHappy();
	natgov = p.getNatGov();
	natclans = p.getNatClans();
	natsort = p.getNatSort();
	temp = p.getTemp();
	coreneutr = p.getCoreNeutronium();
	densneutr = p.getNeutroniumDensity();
	coretrit = p.getCoreTritanium();
	denstrit = p.getTritaniumDensity();
	coredur = p.getCoreDuranium();
	densdur = p.getDuraniumDensity();
	coremolyb = p.getCoreMolybdenium();
	densmolyb = p.getMolybdeniumDensity();
    }

    public Object clone() throws CloneNotSupportedException {
	BData base = (BData)(super.clone());
	base.techLevel = new int[nrOfTechLevels];
	for (int j=0; j<nrOfTechLevels; j++) base.techLevel[j] = techLevel[j];
	//base.copyPlanetFields(this);
	return base;
    }

    /**
     * Read from a stream, assuming it has been positioned properly.
     */
    public void readFrom(LSBReader source) throws IOException {
	int j;
	// id has been read by BDataGen
	owner = source.readShort();
	struct[Build.BASEDEFENSE] = source.readShort();
	damage = source.readShort();
	for (j=0; j<nrOfTechLevels; j++) techLevel[j] = source.readShort();
	for (j=1; j<=engines; j++) struct[ENGOFFSET+j] = source.readShort();
	for (j=1; j<=buildhulls; j++) struct[HULLOFFSET+j]=source.readShort();
	for (j=1; j<=beams; j++) struct[BEAMOFFSET+j] = source.readShort();
	for (j=1; j<=torpedoes; j++) struct[LAUNCHOFFSET+j]=source.readShort();
	for (j=1; j<=torpedoes; j++) struct[TORPOFFSET+j] = source.readShort();
	struct[FIGHTER] = source.readShort();
	target = source.readShort();
	fixrec = source.readShort();
	mission = source.readShort();
	bhulltype = source.readShort();
	bengtype = source.readShort();
	bbeamtype = source.readShort();
	bbeams = source.readShort();
	btorptype = source.readShort();
	blaunchs = source.readShort();
	unknown = source.readShort();
    }

    // do not override PData.writeTo(LSBWriter)
    public void writeBaseTo(LSBWriter sink) throws IOException {
	int j;
	sink.writeShort(id);
	sink.writeShort(owner);
	sink.writeShort(struct[Build.BASEDEFENSE]);
	sink.writeShort(damage);
	for (j=0; j<nrOfTechLevels; j++) sink.writeShort(techLevel[j]);
	for (j=1; j<=engines; j++) sink.writeShort(struct[ENGOFFSET+j]);
	for (j=1; j<=buildhulls; j++) sink.writeShort(struct[HULLOFFSET+j]);
	for (j=1; j<=beams; j++) sink.writeShort(struct[BEAMOFFSET+j]);
	for (j=1; j<=torpedoes; j++) sink.writeShort(struct[LAUNCHOFFSET+j]);
	for (j=1; j<=torpedoes; j++) sink.writeShort(struct[TORPOFFSET+j]);
	sink.writeShort(struct[FIGHTER]);
	sink.writeShort(target);
	sink.writeShort(fixrec);
	sink.writeShort(mission);
	sink.writeShort(bhulltype);
	sink.writeShort(bengtype);
	sink.writeShort(bbeamtype);
	sink.writeShort(bbeams);
	sink.writeShort(btorptype);
	sink.writeShort(blaunchs);
	sink.writeShort(unknown);
    }

    public BData getRstBData() {
	if (isRst()) return null;
	Game game = universe.getGame();
	Universe rstuniv = game.getUniverse(getTurnNumber(), RstOrTrn.RST);
	return rstuniv.getBData(getId());
    }

    public void throwEvent() {
	universe.throwBaseEvent(id);
    }

    public int getHulls(int hulltype) {
	if (hulltype <= 0) return 0;
	if (hulltype > buildhulls) return 0;
	return struct[HULLOFFSET+hulltype];
    }

    public int getEngines(int engtype) {
	if (engtype <= 0) return 0;
	if (engtype > engines) return 0;
	return struct[ENGOFFSET+engtype];
    }

    public int getBeams(int beamtype) {
	if (beamtype <= 0) return 0;
	if (beamtype > beams) return 0;
	return struct[BEAMOFFSET+beamtype];
    }

    public int getLaunchers(int torptype) {
	if (torptype <= 0) return 0;
	if (torptype > torpedoes) return 0;
	return struct[LAUNCHOFFSET+torptype];
    }

    public int isBuildingShip() {
	if (bhulltype==UNKNOWN) return UNKNOWN;
	if (bhulltype==0) return FALSE;
	return TRUE;
    }

    /**
     * @return index from 1 to 20 in players hull list.
     */
    public int getBuildHullType() {
	return bhulltype;
    }

    public int getBuildEngineType() {
	return bengtype;
    }

    public int getBuildBeamType() {
	return bbeamtype;
    }

    public int getBuildBeams() {
	return bbeams;
    }

    public int getBuildLaunchType() {
	return btorptype;
    }

    public int getBuildLaunchers() {
	return blaunchs;
    }


    // tech level stuff

    public int highestHullLevelBuilt() {
	int result = 0;
	BData rstBase = getRstBData();
	for (int i=1; i<=VGAPlanets.buildhulls; i++) {
	    if (getHulls(i) > rstBase.getHulls(i)) {
		int level = getGame().getHull(owner, i).getTech();
		if (result < level) result = level;
	    }
	}
	return result;
    }

    public int highestEngineLevelBuilt() {
	int result = 0;
	BData rstBase = getRstBData();
	for (int i=1; i<=VGAPlanets.engines; i++) {
	    if (getEngines(i) > rstBase.getEngines(i)) {
		int level = getGame().getEngine(i).getTech();
		if (result < level) result = level;
	    }
	}
	return result;
    }

    public int highestBeamLevelBuilt() {
	int result = 0;
	BData rstBase = getRstBData();
	for (int i=1; i<=VGAPlanets.beams; i++) {
	    if (getBeams(i) > rstBase.getBeams(i)) {
		int level = getGame().getBeam(i).getTech();
		if (result < level) result = level;
	    }
	}
	return result;
    }

    public int highestTorpedoLevelBuilt() {
	int result = 0;
	BData rstBase = getRstBData();
	for (int i=1; i<=VGAPlanets.torpedoes; i++) {
	    int level = getGame().getTorpedo(i).getTech();
	    if (getLaunchers(i) > rstBase.getLaunchers(i)) {
		if (result < level) result = level;
	    }
	    if (getLocation().getTorpedoes(i) > getRstLocation().getTorpedoes(i)) {
		if (result < level) result = level;
	    }
	}
	return result;
    }

    public int getHullTech() {
	return getTechLevel(hullTech);
    }

    public int getEngineTech() {
	return getTechLevel(engineTech);
    }

    public int getTorpTech() {
	return getTechLevel(torpedoTech);
    }

    public int getBeamTech() {
	return getTechLevel(beamTech);
    }

    public int getTechLevel(int whichTech) {
	if ((whichTech < 0) || (whichTech >= nrOfTechLevels)) return UNKNOWN;
	return techLevel[whichTech];
    }

    public int getDamage() {
	return damage;
    }

    // Do not override Planet.getDefense()
    public int getBaseDefense() {
	return getStructAmount(Build.BASEDEFENSE);
    }

    public int getFighters() {
	return struct[Build.FIGHTER];
    }

    public int getTarget() {
	return target;
    }

    public int getFixRec() {
	return fixrec;
    }

    public int getMission() {
	return mission;
    }

    public int getTorpedoes(int torptype) {
	if (torptype <= 0) return 0;
	if (torptype > torpedoes) return 0;
	return struct[torptype + Build.TORPOFFSET];
    }


    // Setting, Building and breaking down methods

    public void raiseHullTech(int increment) {
	raiseTechLevel(hullTech, increment);
    }

    public void lowerHullTech(int decrement) {
	lowerTechLevel(hullTech, decrement);
    }

    public void raiseEngineTech(int increment) {
	raiseTechLevel(engineTech, increment);
    }

    public void lowerEngineTech(int decrement) {
	lowerTechLevel(engineTech, decrement);
    }

    public void raiseTorpTech(int increment) {
	raiseTechLevel(torpedoTech, increment);
    }

    public void lowerTorpTech(int decrement) {
	lowerTechLevel(torpedoTech, decrement);
    }

    public void raiseBeamTech(int increment) {
	raiseTechLevel(beamTech, increment);
    }

    public void lowerBeamTech(int decrement) {
	lowerTechLevel(beamTech, decrement);
    }

    private void raiseTechLevel(int whichTech, int increment) {
	if ((whichTech < 0) || (whichTech >= nrOfTechLevels)) return;
	if (increment < 0) {
	    lowerTechLevel(whichTech, -increment);
	    return;
	}
	if (isEditable()) {
	    int oldLevel = techLevel[whichTech];
	    if (oldLevel + increment > maxTechLevel) {
		increment = maxTechLevel - oldLevel;
	    }
	    int newLevel = techLevel[whichTech] + increment;
	    int needMC = totalTechLevelCost[newLevel]-totalTechLevelCost[oldLevel];
	    if (getMC()+getSupplies() >= needMC) {
		if (getMC() < needMC) int_sellSupplies(needMC - getMC());
		techLevel[whichTech] = newLevel;
		setMC(getMC() - needMC);
		universe.throwPlanetEvent(id);
		universe.throwBaseEvent(id);
	    }
	    getGame().unlock();
	}
    }

    private void lowerTechLevel(int whichTech, int decrement) {
	if ((whichTech < 0) || (whichTech >= nrOfTechLevels)) return;
	if (decrement < 0) {
	    raiseTechLevel(whichTech, -decrement);
	    return;
	}
	if (isEditable()) {
	    int oldLevel = techLevel[whichTech];
	    int rstLevel = getRstBData().getTechLevel(whichTech);
	    int buildLevel = 10;
	    switch (whichTech) {
	    case hullTech:
		buildLevel = highestHullLevelBuilt();
		break;
	    case engineTech:
		buildLevel = highestEngineLevelBuilt();
		break;
	    case beamTech:
		buildLevel = highestBeamLevelBuilt();
		break;
	    case torpedoTech:
		buildLevel = highestTorpedoLevelBuilt();
		break;
	    default: crash(1001, "lowerTechLevel() non-tech-level");
		break;
	    }
	    int minLevel = Math.max(rstLevel, buildLevel);
	    if (oldLevel - decrement < minLevel) decrement = oldLevel - minLevel;
	    int newLevel = oldLevel - decrement;
	    if (newLevel != oldLevel) {
		int diffMC = totalTechLevelCost[oldLevel]-totalTechLevelCost[newLevel];
		techLevel[whichTech] = newLevel;
		setMC(getMC() + diffMC);
		universe.throwPlanetEvent(id);
		universe.throwBaseEvent(id);
	    }
	    getGame().unlock();
	}
    }


    // (fix/rec / target) mission stuff
    
    public void setTarget(int newTarget) {
	if ((newTarget <= 0) || (newTarget > ships)) newTarget = 0;
	if (isEditable()) {
	    Ship ship = universe.getShip(newTarget);
	    if ((ship == null) || (ship.getLocation() != getLocation())) {
		report("Ship "+newTarget+" not at the same location");
		newTarget = 0;
	    }
	    if (newTarget != target) {
		target = newTarget;
		universe.throwBaseEvent(id);
	    }
	    getGame().unlock();
	}
    }

    public void setFixRec(int newRec) {
	if ((newRec < 0) || (newRec > recycleShip)) return;
	if (isEditable()) {
	    if (newRec != fixrec) {
		fixrec = newRec;
		universe.throwBaseEvent(id);
	    }
	    getGame().unlock();
	}
    }

    public void setMission(int newmission) {
	if (isEditable()) {
	    if (newmission != mission) {
		mission = newmission;
		universe.throwBaseEvent(id);
	    }
	    getGame().unlock();
	}
    }

    /**
     * Build fighters on the base. These cost 3 trit, 2 molyb and 100 MC.
     * @param toBeBuilt amount of fighters to be built; can be negative.
     */
    public void buildFighters(int toBeBuilt) {
	buildUp(Build.FIGHTER, toBeBuilt);
    }

    public void breakDownFighters(int toBeBroken) {
	breakDown(Build.FIGHTER, toBeBroken);
    }

    public void buildBaseDefense(int toBeBuilt) {
	buildUp(Build.BASEDEFENSE, toBeBuilt);
    }

    public void breakDownBaseDefense(int toBeBroken) {
	breakDown(Build.BASEDEFENSE, toBeBroken);
    }

    /**
     * @param torptype Starts at one, like Tim is used to.
     */
    public void buildTorpedoes(int torptype, int toBeBuilt) {
	if (torptype <= 0) return;
	if (torptype > torpedoes) return;
	buildUp(TORPOFFSET+torptype, toBeBuilt);
    }

    public void breakDownTorpedoes(int torptype, int toBeBroken) {
	if (torptype <= 0) return;
	if (torptype > torpedoes) return;
	breakDown(TORPOFFSET+torptype, toBeBroken);
    }

    public void buildEngines(int engtype, int toBeBuilt) {
	if (engtype <= 0) return;
	if (engtype > engines) return;
	buildUp(ENGOFFSET+engtype, toBeBuilt);
    }

    public void breakDownEngines(int engtype, int toBeBroken) {
	if (engtype <= 0) return;
	if (engtype > engines) return;
	breakDown(ENGOFFSET+engtype, toBeBroken);
    }

    public void buildHulls(int hulltype, int toBeBuilt) {
	if (hulltype <= 0) return;
	if (hulltype > buildhulls) return;
	buildUp(HULLOFFSET+hulltype, toBeBuilt);
    }

    public void breakDownHulls(int hulltype, int toBeBroken) {
	if (hulltype <= 0) return;
	if (hulltype > buildhulls) return;
	breakDown(HULLOFFSET+hulltype, toBeBroken);
    }

    public void buildBeams(int beamtype, int toBeBuilt) {
	if (beamtype <= 0) return;
	if (beamtype > beams) return;
	buildUp(BEAMOFFSET+beamtype, toBeBuilt);
    }

    public void breakDownBeams(int beamtype, int toBeBroken) {
	if (beamtype <= 0) return;
	if (beamtype > beams) return;
	breakDown(BEAMOFFSET+beamtype, toBeBroken);
    }

    public void buildLaunchers(int launchtype, int toBeBuilt) {
	if (launchtype <= 0) return;
	if (launchtype > torpedoes) return;
	buildUp(LAUNCHOFFSET+launchtype, toBeBuilt);
    }

    public void breakDownLaunchers(int launchtype, int toBeBroken) {
	if (launchtype <= 0) return;
	if (launchtype > torpedoes) return;
	breakDown(LAUNCHOFFSET+launchtype, toBeBroken);
    }

    public void buildUp(int type, int toBeBuilt) {
	if (type >= BASICBDATABUILDTYPES) {
	    if (type <= HULLOFFSET) {
		/* torps */
		Torpedo t = getGame().getTorpedo(type-TORPOFFSET);
		if (t.tech > techLevel[torpedoTech]) {
		    report("Torpedo tech level too low to build ammo");
		    return;
		}
	    } else if (type <= ENGOFFSET) {
		/* hulls etc */
		Hull h = getGame().getHull(owner, type-HULLOFFSET);
		if (h == null) {
		    report("Invalid hull build type for player, "+
			   "your client layer on top of jvpc.vga is mistaken");
		    return;
		}
		if (h.tech > techLevel[hullTech]) {
		    report("Hull tech level too low to build hull");
		    return;
		}
	    } else if (type <= BEAMOFFSET) {
		Engine e = getGame().getEngine(type-ENGOFFSET);
		if (e.tech > techLevel[engineTech]) {
		    report("Engine tech level too low to build engine");
		    return;
		}
	    } else if (type <= LAUNCHOFFSET) {
		Beam b = getGame().getBeam(type-BEAMOFFSET);
		if (b.tech > techLevel[beamTech]) {
		    report("Beam tech level too low to build beam");
		    return;
		}
	    } else if (type < BDATABUILDTYPES) {
		Torpedo t = getGame().getTorpedo(type-LAUNCHOFFSET);
		if (t.tech > techLevel[torpedoTech]) {
		    report("Torpedo tech level too low to build launcher");
		    return;
		}
	    } else {
		warn("base tries to build unknown type "+type);
	    }
	}
	super.buildUp(type, toBeBuilt);
    }

    /* No tech level required for breaking down hull parts, no need to override
     * breakDown().
     * Or is check (universe.getGame().truehull[owner].build[hulltype]==0)
     * necessary? Nah, there are no hulls available of those types :-)
     */
    
    public void setBuildShip(
	int hulltype, int engtype,
	int beamtype, int weapons,
	int torptype, int launchs
    ) {
	// Basic checking on parameters
	if ((hulltype < 1) || (hulltype > buildhulls)) return;
	if ((engtype < 1) || (engtype > engines)) return;
	if ((beamtype < 0) || (beamtype > beams)) return;
	if ((torptype < 0) || (torptype > torpedoes)) return;
	if (beamtype == 0) weapons = 0;
	if (torptype == 0) launchs = 0;

	if (isEditable()) {
	    // Cancel current build
	    bhulltype = bengtype = bbeamtype = bbeams = btorptype = blaunchs = 0;

	    // Semantic & race-based checking on parameters
	    Hull hull = universe.getGame().getHull(owner, hulltype);
	    if (hull != null) {
		if (weapons < 0) {
		    weapons = 0;
		} else if (weapons > hull.getBeams()) {
		    weapons = hull.getBeams();
		}

		if (launchs < 0) {
		    launchs = 0;
		} else if (hull.getBays() > 0) { // this if doesnt seem to be necessary
		    launchs = 0; // no meaning
		} else if (launchs > hull.getLaunchs()) {
		    launchs = hull.getLaunchs();
		}

		warn("Trying to build "+hulltype+" "+engtype+" "+
		     beamtype+" "+weapons+" "+torptype+" "+launchs);

		// Do we have the parts?
		int i;
		System.err.print("hulls available:");
		for (i=1; i<=20; i++) System.err.print(" "+getHulls(i));
		System.err.print("\nengines:");
		for (i=1; i<=engines; i++) System.err.print(" "+getEngines(i));
		System.err.print("\nbeams:");
		for (i=1; i<=beams; i++) System.err.print(" "+getBeams(i));
		System.err.print("\nlaunchers:");
		for (i=1; i<=torpedoes; i++) System.err.print(" "+getLaunchers(i));
		System.err.println();
		System.err.println(""+getHulls(hulltype)+" >= 1");
		System.err.println(""+getEngines(engtype)+" >= "+hull.getEngines());
		System.err.println("("+beamtype+" || ("+getBeams(beamtype)+" >= "+weapons+")");
		System.err.println("("+torptype+" || ("+getLaunchers(torptype)+" >= "+launchs+"))");
		if ((getHulls(hulltype) >= 1) &&
		    (getEngines(engtype) >= hull.getEngines()) &&
		    ((beamtype == 0) || (getBeams(beamtype) >= weapons)) &&
		    ((torptype == 0) || (getLaunchers(torptype) >= launchs))) {
		    System.err.println("We're building!");
		    // place order
		    bhulltype = hulltype;
		    bengtype = engtype;
		    bbeamtype = beamtype;
		    bbeams = weapons;
		    btorptype = torptype;
		    blaunchs = launchs;
		}
		universe.throwBaseEvent(id);
	    }
	    getGame().unlock();
	}
    }

    public void cancelBuildShip() {
	if (isEditable()) {
	    if (
		bhulltype != 0 || bengtype != 0 ||
		bbeamtype != 0 || bbeams != 0   ||
		btorptype != 0 || blaunchs != 0
		) {
		bhulltype = bengtype = 0;
		bbeamtype = bbeams = 0;
		btorptype = blaunchs = 0;
		universe.throwBaseEvent(id);
	    }
	    getGame().unlock();
	}
    }


    // Cargo interface stuff

    int ammotype;

    public int getCargo(int cargotype) {
	if (cargotype == Cargo.AMMUNITION) {
	    switch (ammotype) {
	    case NONE:
		return 0;
	    case Cargo.FIGHTER:
		return struct[Build.FIGHTER];
	    default:
		return struct[Build.TORPOFFSET + ammotype];
	    }
	} else {
	    return super.getCargo(cargotype);
	}
    }

    protected void setCargo(int cargotype, int newamount) {
	if (cargotype == Cargo.AMMUNITION) {
	    report("BData.setCargo("+ammotype+")");
	    switch (ammotype) {
	    case NONE:
		// right. set what?
		break;
	    case Cargo.FIGHTER:
		struct[Build.FIGHTER] = newamount;
		cargo[Cargo.AMMUNITION].amount = newamount;
		break;
	    default:
		struct[Build.TORPOFFSET + ammotype] = newamount;
		cargo[Cargo.AMMUNITION].amount = newamount;
		break;
	    }
	} else {
	    super.setCargo(cargotype, newamount);
	}
    }

    // Help, do I smell race conditions? (yes, they're there, if two threads
    // start actually editing simultaneously.)
    public void setAmmoType(int ammotype) {
	this.ammotype = ammotype;
	report("BData.setAmmoType("+ammotype+")");
	switch (ammotype) {
	case NONE:
	    cargo[Cargo.AMMUNITION].amount = 0;
	    break;
	case Cargo.FIGHTER:
	    cargo[Cargo.AMMUNITION].amount = struct[Build.FIGHTER];
	    break;
	default:
	    cargo[Cargo.AMMUNITION].amount = struct[Build.TORPOFFSET+ammotype];
	    break;
	}
    }


    // Build interface stuff

    public int minStructAmount(int buildType) {
	int firstIndex = Build.TORPOFFSET+1;
	int lastIndex = Build.TORPOFFSET+VGAPlanets.torpedoes;
	if ((buildType >= firstIndex ) && (buildType <= lastIndex)) {
	    Location loc = getLocation();
	    int index = buildType-Build.TORPOFFSET;
	    int result = getRstLocation().getTorpedoes(index) -
		loc.getTorpedoes(index) + getTorpedoes(index);
	    if (result < 0) result = 0;
	    return result;
	} else if (buildType == Build.FIGHTER) {
	    /*     rstLoc.fighters <= trnLoc.fighters
	     * <-> rstLoc.fighters <= trnLoc.fighters-basefighters+basefighters
	     *  -> basefighters >= rstLoc.fighters-trnLoc.fighters+basefighters
	     * looks weird, huh? well, in fact it reads
	     *     baseFighters >= rstLoc.fighters - trnLoc.shipFighters
	     */
	    Location loc = getLocation();
	    int result = loc.getRstLocation().getFighters() -
		loc.getFighters() + getFighters();
	    if (result < 0) result = 0;
	    return result;
	} else {
	    return super.minStructAmount(buildType);
	}
    }

    public int maxStructAmount(int buildtype) {
	if (buildtype < BASICBDATABUILDTYPES) {
	    return super.maxStructAmount(buildtype);
	} else {
	    return INFINITY;
	}
    }

    public int needTrit(int buildtype) {
	if (buildtype < BASICBDATABUILDTYPES) return super.needTrit(buildtype);
	Game game = universe.getGame();
	if (buildtype <= HULLOFFSET) {
	    return 1; // torpedoes cost 1 trit, dur and molyb per definition
	}
	if (buildtype <= ENGOFFSET) {
	    Hull hull = universe.getGame().getHull(owner, buildtype-HULLOFFSET);
	    if (hull == null) return getTritanium()+1;
	    return hull.trit;
	}
	if (buildtype <= BEAMOFFSET) {
	    return game.engine[buildtype-ENGOFFSET].trit;
	}
	if (buildtype <= LAUNCHOFFSET) {
	    return game.beam[buildtype-BEAMOFFSET].trit;
	}
	if (buildtype < BDATABUILDTYPES) {
	    return game.torpedo[buildtype-LAUNCHOFFSET].trit;
	}
	return getTritanium() + 1; // you don't have the tritanium -> no build
    }

    public int needDur(int buildtype) {
	if (buildtype < BASICBDATABUILDTYPES) return super.needDur(buildtype);
	Game game = universe.getGame();
	if (buildtype <= HULLOFFSET) {
	    return 1;
	}
	if (buildtype <= ENGOFFSET) {
	    Hull hull = universe.getGame().getHull(owner,buildtype-HULLOFFSET);
	    if (hull == null) return getDuranium()+1;
	    return hull.dur;
	}
	if (buildtype <= BEAMOFFSET) {
	    return game.engine[buildtype-ENGOFFSET].dur;
	}
	if (buildtype <= LAUNCHOFFSET) {
	    return game.beam[buildtype-BEAMOFFSET].dur;
	}
	if (buildtype < BDATABUILDTYPES) {
	    return game.torpedo[buildtype-LAUNCHOFFSET].dur;
	}
	return getDuranium() + 1;
    }

    public int needMolyb(int buildtype) {
	if (buildtype < BASICBDATABUILDTYPES) {
	    return super.needMolyb(buildtype);
	}
	Game game = universe.getGame();
	if (buildtype <= HULLOFFSET) {
	    return 1;
	}
	if (buildtype <= ENGOFFSET) {
	    Hull hull = universe.getGame().getHull(owner,buildtype-HULLOFFSET);
	    if (hull == null) return getMolybdenium()+1;
	    return hull.molyb;
	}
	if (buildtype <= BEAMOFFSET) {
	    return game.engine[buildtype-ENGOFFSET].molyb;
	}
	if (buildtype <= LAUNCHOFFSET) {
	    return game.beam[buildtype-BEAMOFFSET].molyb;
	}
	if (buildtype < BDATABUILDTYPES) {
	    return game.torpedo[buildtype-LAUNCHOFFSET].molyb;
	}
	return getMolybdenium() + 1;
    }

    public int needSupp(int buildtype) {
	if (buildtype < BASICBDATABUILDTYPES) return super.needSupp(buildtype);
	// no supplies, only MC required for ship parts
	if (buildtype < BDATABUILDTYPES) return 0;
	return getMC() + getSupplies() + 1;
    }

    public int needMC(int buildtype) {
	if (buildtype < BASICBDATABUILDTYPES) return super.needMC(buildtype);
	Game game = universe.getGame();
	if (buildtype <= HULLOFFSET) {
	    return game.torpedo[buildtype-TORPOFFSET].unitmc;
	}
	if (buildtype <= ENGOFFSET) {
	    Hull hull = universe.getGame().getHull(owner, buildtype-HULLOFFSET);
	    if (hull == null) return getMC()+1;
	    return hull.mc;
	}
	if (buildtype <= BEAMOFFSET) {
	    return game.engine[buildtype-ENGOFFSET].mc;
	}
	if (buildtype <= LAUNCHOFFSET) {
	    return game.beam[buildtype-BEAMOFFSET].mc;
	}
	if (buildtype < BDATABUILDTYPES) {
	    return game.torpedo[buildtype-LAUNCHOFFSET].mc;
	}
	return getMC() + getSupplies() + 1;
    }


    // documentary values

    public String toString() {
        return "Base "+toBDataString("#@df%");
    }

    public String toBDataString(String userline) {
	StringBuffer result = new StringBuffer(256);
	for (int i=0; i<userline.length(); i++) {
	    result.append(userline.charAt(i));
	    switch (userline.charAt(i)) {
	    case '#':
		result.append(getId());
		break;
	    case '@':
		result.append('(');
		result.append(getX());
		result.append(',');
		result.append(getY());
		result.append(')');
		break;
	    case 'd':
		result.append(getBaseDefense());
		break;
	    case 'f':
		result.append(getFighters());
		break;
	    case 'm':
		result.setLength(result.length()-1);
		result.append(BaseMission.name[getMission()]);
		break;
	    case 't':
		result.setLength(result.length()-1);
		for (int j=0; j<nrOfTechLevels; j++) {
		    result.append((j==0)? "[": ", ");
		    result.append(techLevel[j]);
		}
		result.append(']');
		break;
	    case '%':
		result.append(getDamage());
		break;
	    }
	    result.append(' ');
	}
	return result.toString();
    }

    //public String getReport() {
    //return "Base "+toString("#@df%tm");
    //}
}
