// File: BdataGen.java

package jvpc.vga;

import java.util.Hashtable;
import java.io.*;

/**
 * Generates <CODE>Bdata</code>objects.
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since     24 II 1998
 * @version   21 XII 1998
 */
public class BDataGen extends Generator {
  final int basecount;
  /**
   * Constructs a Bdata generator.
   * @param is The <CODE>InputStream</code> from which the <CODE>source</code>
   * can be created.
   */
  public BDataGen(InputStream is) throws IOException {
    super(is);
    basecount=source.readShort();
  }

  public BDataGen(Generator g) throws IOException {
    super(g);
    basecount=source.readShort();
  }

  /**
   * Generates all <CODE>Bdata</code> objects found in <CODE>pis</code>.
   * Puts them at the proper place in <CODE>base[]</code>
   */
  public void generateAll(Universe u) {
    int i,owner,id;
    BData bdata;

    for (i=1;i<=basecount;i++) {
      try {
	id=source.readShort();
	bdata=u.getBData(id);
	if (bdata==null) {
	  if (u.getPlanet(id) != null) {
	    bdata = new BData(u.getPlanet(id));
	  } else {
	    bdata=new BData(u);
	  }
	}
	bdata.id=id;
	bdata.readFrom(source);
	u.putBData(bdata);
      } catch (IOException ioe) {
	crash(11, "Error while reading bdata: "+ioe);
      }
    }
  }

  public int getBaseCount() {
    return basecount;
  }
}
