// File: Build.java

package jvpc.vga;

/**
 * Enables abstraction from the <EM>type</em> of building, i.e, need not care
 * about factories, fighters, two types of defense and such when building them.
 *
 * @author Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since   5 II 2000
 * @version 20 II 2001
 */
public interface Build extends VGAPlanets {
  // planet == PData building structures
  public static final int MINE = 0;
  public static final int FACTORY = 1;
  public static final int DEFENSE = 2;
  public static final int BASE = 3;
  
  public static final int PDATABUILDTYPES = 4;

  // starbase == BData building structures
  public static final int BASEDEFENSE = 4;
  public static final int FIGHTER = 5;
  static final int BASICBDATABUILDTYPES = 6;

  // start counting torp types at ONE
  // BData assumes these values are in this order, don't mess with them!
  public static final int TORPOFFSET = BASICBDATABUILDTYPES - 1;
  public static final int HULLOFFSET = TORPOFFSET + torpedoes;
  public static final int ENGOFFSET = HULLOFFSET + buildhulls;
  public static final int BEAMOFFSET = ENGOFFSET + engines;
  public static final int LAUNCHOFFSET = BEAMOFFSET + beams;
  public static final int BDATABUILDTYPES = LAUNCHOFFSET + torpedoes + 1;

  //static final int FUNCTION = -42659;

  // for the first few (up to BASICBDATA) buildings, the actual values
  // starbase building parts will compute the values in functions like needTrit
  static final int needtrit[] =  {0, 0,  0, 402,   0,   3 };
  static final int needdur[] =   {0, 0,  0, 120,   1,   0 };
  static final int needmolyb[] = {0, 0,  0, 340,   0,   2 };
  static final int needsupp[] =  {1, 1,  1,   0,   0,   0 };
  static final int needmc[] =    {4, 3, 10, 900,  10, 100 };
  static final int maxamount[] = {0, 0,  0,   1, 200,  60 };
  /* There is no field minamount[], because in general the minimum is zero, in
   * play the minimum depends on the RST and transferability of the buildable
   * items (which are subject to Cargo as well).
   */

  public int needTrit(int buildtype);
  public int needDur(int buildtype);
  public int needMolyb(int buildtype);
  public int needSupp(int buildtype);
  public int needMC(int buildtype);

  public int getStructAmount(int type);

  /**
   * @return The maximum amount of the given type that can be made.
   */
  public int maxStructAmount(int buildtype);

  /**
   * @return The minimum amount of the given type that can exist (that is, can
   * exist on the place where you are currently building/breaking down).
   */
  public int minStructAmount(int buildtype);

  /**
   * Build/buy any of the [0, BUILDTYPES> things that can be built.
   * <BR><B>NOTE: this function is not bothered by tech-levels.</b> UIs should
   * call functions like buildEngines(type, amount) to take tech-levels into
   * account.
   * @param type The type, reaching from 0 to BDATABUILDTYPES-1.
   * @param amount The amount to be built.
   */
  public void buildUp(int type, int amount);

  /**
   * As buildUp(), but break down/sell.
   */
  public void breakDown(int type, int amount);
}
