// File: CargoTransfer.Java 

package jvpc.vga;

import kero.util.*;

/**
 * Defines the interface for transfering cargo from ships to planets and vice
 * versa.
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since     23 I 2000
 * @version   14 III 2001
 */
public abstract class CargoTransfer
    extends Owned
{
    Cargo[] cargo;

    private void constructCargo() {
	cargo = new Cargo[Cargo.CARGOTYPES];
	for (int i=0; i<Cargo.CARGOTYPES; i++) cargo[i] = new Cargo(0);
    }

    public CargoTransfer() {}

    public CargoTransfer(Universe u) {
	super(u);
	constructCargo();
    }

    public CargoTransfer(Universe u, int newid) {
	super(u, newid);
	constructCargo();
    }

    public Object clone() throws CloneNotSupportedException {
	CargoTransfer result = (CargoTransfer)super.clone();
	result.cargo = new Cargo[Cargo.CARGOTYPES];
	for (int i=0; i<Cargo.CARGOTYPES; i++) {
	    result.cargo[i] = new Cargo();
	    result.cargo[i].amount = cargo[i].amount;
	}
	return result;
    }

    public abstract int maxCargo(int cargoType);

    public int cargoLeft(int cargoType) {
	int maxcargo = maxCargo(cargoType);
	if (maxcargo == INFINITY) return INFINITY;
	int cargoused = cargoUsed(cargoType);
	return maxcargo - cargoused;
    }

    public int cargoUsed(int cargoType) {
	int g = Cargo.group[cargoType];
	int used = 0;
	for (int i=0; i<Cargo.CARGOTYPES; i++) {
	    if (Cargo.group[i] == g) used += getCargo(i);
	}
	return used;
    }

    public int getCargo(int cargoType) {
	return cargo[cargoType].amount;
    }

    protected abstract void setCargo(int cargoType, int newAmount);

    protected abstract void throwEvent();

    public void transferCargoTo(CargoTransferable partner, int type, int newAmount) {
	if (newAmount < 0) newAmount = 0;
	// Assert.assert(isOurs());
	// Assert.assert(getLocation() == partner.getLocation())
	if (isEditable()) {
	    report("transferCargoTo("+partner+","+type+","+newAmount+")");
	    // System.err.println("self: "+this.getOwner()+", other: "+partner.getOwner());
	    if ((this instanceof Planet && this.getOwner() == 0) ||
		(partner instanceof Planet && ((Planet)partner).getOwner() == 0)) {
		getGame().unlock();
		return;
	    }
	    if (type == Cargo.MC) {
		if (((CargoTransfer) partner).owner != owner) {
		    getGame().unlock();
		    return;
		}
	    } else if (type == Cargo.AMMUNITION) {
		// OK, what follows here is a lot of hacking. It is not clean, but it is
		// the cleanest way in the current class hierarchy. Passing to subclasses
		// would take more code. Hey, I never said that Tim's structures allow a
		// perfect OO hierarchy!
		// Hm, may be could use Cargo.frombase[]...
		Ship s;
		CargoTransfer other;
		if (this instanceof Ship) {
		    s = (Ship)this;
		    other = (CargoTransfer)partner;
		} else {
		    s = (Ship)partner;
		    other = this;
		}
		if (s.getAmmoType() == NONE) {
		    warn("transferCargoTo() ship has no armament, bailing out");
		    getGame().unlock();
		    return;
		}
		if (other instanceof BData) {
		    // a base can deal all kinds of ammunition,
		    // irrespective of tech-levels
		    ((BData)other).setAmmoType(s.getAmmoType()); // Race Conditions!!![Kero]
		} else {
		    // either a planet, which is not allowed to transfer anyway, or a Ship
		    if (other.getAmmoType() != s.getAmmoType()) {
			warn("mismatch of ammo types");
			getGame().unlock();
			return;
		    }
		}
	    }
	    int toBeTransfered = newAmount - partner.getCargo(type);
	    //report("to be transfered: "+toBeTransfered);

	    // two checks on this side of the transfer
	    if (toBeTransfered > getCargo(type)) toBeTransfered = getCargo(type);
	    //report("to be transfered: "+toBeTransfered);
	    int cargoLeft = partner.cargoLeft(type);
	    if (cargoLeft != INFINITY) {
		if (toBeTransfered > cargoLeft) toBeTransfered = cargoLeft;
	    }
	    //report("to be transfered: "+toBeTransfered);

	    // two checks on the side of the partner
	    cargoLeft = cargoLeft(type);
	    if (cargoLeft != INFINITY) {
		if (-toBeTransfered > cargoLeft) toBeTransfered = -cargoLeft;
	    }
	    // report("to be transfered: "+toBeTransfered);
	    if (-toBeTransfered > partner.getCargo(type))
		toBeTransfered = -partner.getCargo(type);
	    report("to be transfered: "+toBeTransfered);
	    if (toBeTransfered != 0) {
		((CargoTransfer)partner).setCargo(type, partner.getCargo(type) + toBeTransfered);
		report("part two of transfer ahead");
		setCargo(type, getCargo(type) - toBeTransfered);
		report("throwing the events, checking consistency...");
		((CargoTransfer)partner).throwEvent();
		report("explain...");
		throwEvent();
	    }
	    getGame().unlock();
	}
    }

    public int getAmmoType() {
	return NONE;
    }

    public int getNeutronium() {
	return getCargo(Cargo.NEUTRONIUM);
    }

    void setNeutronium(int newNeutr) {
	setCargo(Cargo.NEUTRONIUM, newNeutr);
    }

    public int getTritanium() {
	return getCargo(Cargo.TRITANIUM);
    }

    void setTritanium(int newTrit) {
	setCargo(Cargo.TRITANIUM, newTrit);
    }

    public int getDuranium() {
	return getCargo(Cargo.DURANIUM);
    }

    void setDuranium(int newdur) {
	setCargo(Cargo.DURANIUM, newdur);
    }

    public int getMolybdenium() {
	return getCargo(Cargo.MOLYBDENIUM);
    }

    void setMolybdenium(int newMolyb) {
	setCargo(Cargo.MOLYBDENIUM, newMolyb);
    }

    public int getSupplies() {
	return getCargo(Cargo.SUPPLIES);
    }

    void setSupplies(int newSupplies) {
	setCargo(Cargo.SUPPLIES, newSupplies);
    }

    public int getMC() {
	return getCargo(Cargo.MC);
    }

    void setMC(int newMc) {
	setCargo(Cargo.MC, newMc);
    }

    public int getColClans() {
	return getCargo(Cargo.COLCLANS);
    }

    public void setColClans(int newColClans) {
	setCargo(Cargo.COLCLANS, newColClans);
    }

    public int getAmmunition() {
	return getCargo(Cargo.AMMUNITION);
    }

    void setAmmunition(int newTorps) {
	setCargo(Cargo.AMMUNITION, newTorps);
    }

    public boolean equals(CargoTransfer transfer) {
	for (int i=0; i<Cargo.CARGOTYPES; i++) {
	    if (cargo[i].amount != transfer.cargo[i].amount)  return false;
	}
	return true;
    }
}
