// File: FizzBinGen.java

package jvpc.vga;

import java.util.*;
import java.io.*;

/**
 * Reads the registry key from the file FIZZ.BIN and can rebuild the file from
 * available game data. Because registering VGA-Planets earns Tim Wisseman
 * money, I can not parse and show it (whereas PLANETS.EXE can).
 *
 * <P><B>Note</b>:Since there are scores involved, fizz is universe dependent.
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since     19 XII 1999
 * @version   22 XII 1999
 */
public class FizzBinGen extends Generator {
    private final static char charSharewareRegKey[] = { /* 204 chars */
	0x5e, 0x04, 0x00, 0x00, 0x36, 0x07, 0x00, 0x00, 0xe7, 0x09, 0x00, 0x00,
	0x80, 0x06, 0x00, 0x00, 0x50, 0x14, 0x00, 0x00, 0xe8, 0x20, 0x00, 0x00,
	0x7b, 0x22, 0x00, 0x00, 0xb0, 0x2c, 0x00, 0x00, 0x29, 0x2e, 0x00, 0x00,
	0xe8, 0x3a, 0x00, 0x00, 0x3d, 0x40, 0x00, 0x00, 0x80, 0x13, 0x00, 0x00,
	0xeb, 0x4b, 0x00, 0x00, 0xf0, 0x49, 0x00, 0x00, 0xe3, 0x49, 0x00, 0x00,
	0xa0, 0x5c, 0x00, 0x00, 0x31, 0x57, 0x00, 0x00, 0xc6, 0x6c, 0x00, 0x00,
	0x97, 0x5d, 0x00, 0x00, 0xc8, 0x73, 0x00, 0x00, 0xb5, 0x6b, 0x00, 0x00,
	0xc0, 0x23, 0x00, 0x00, 0x60, 0x25, 0x00, 0x00, 0x00, 0x27, 0x00, 0x00,
	0xa0, 0x28, 0x00, 0x00, 0x5e, 0x04, 0x00, 0x00, 0x42, 0x0a, 0x00, 0x00,
	0x5e, 0x11, 0x00, 0x00, 0x5c, 0x17, 0x00, 0x00, 0xa9, 0x1a, 0x00, 0x00,
	0xd2, 0x21, 0x00, 0x00, 0x1a, 0x27, 0x00, 0x00, 0x00, 0x0d, 0x00, 0x00,
	0x4f, 0x17, 0x00, 0x00, 0x5c, 0x17, 0x00, 0x00, 0xd0, 0x1a, 0x00, 0x00,
	0x40, 0x1d, 0x00, 0x00, 0x20, 0x15, 0x00, 0x00, 0xc0, 0x16, 0x00, 0x00,
	0x60, 0x18, 0x00, 0x00, 0x00, 0x1a, 0x00, 0x00, 0xa0, 0x1b, 0x00, 0x00,
	0x40, 0x1d, 0x00, 0x00, 0xe0, 0x1e, 0x00, 0x00, 0x80, 0x20, 0x00, 0x00,
	0x20, 0x22, 0x00, 0x00, 0xc0, 0x23, 0x00, 0x00, 0x60, 0x25, 0x00, 0x00,
	0x00, 0x27, 0x00, 0x00, 0xa0, 0x28, 0x00, 0x00, 0xfb, 0xd5, 0x07, 0x00
    };

    public final static byte[] sharewareRegKey = new String(charSharewareRegKey).getBytes();
  
    public FizzBinGen(InputStream is) throws IOException {
	super(is);
    }

    public byte[] generate() {
	try {
	    // first, FIZZ.BIN contains scores with some offset
	    source.skip(races*3*4); // 3 scores per race
	    // skip an uninteresting 4 bytes
	    source.skip(4);
	    // read the registry key
	    return source.readByteArray(FIZZSIZE);
	} catch (IOException ioe) {
	    return null;
	}
    }
}
