// File: Gen.java

package jvpc.vga;

import java.io.*;
import kero.util.*;

/**
 * A <CODE>Gen</code> object contains gen (general?) information of a player.
 * It is used to check passwords, set new passwords, verify data-integrity and
 * to find out which data-files are actual (instead of using the file
 * <CODE>init.tmp</code> which get dirty easily in team-games.
 *
 * <P>Since information like this is read <EM>before</em> any object of the
 * class <CODE>Dated</code> can be constructed, it implements
 * <CODE>VGAPlanets</code> directly.
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since     29 XI 1998
 * @version   21 XII 1999
 */
public class Gen
  extends Reporter
  implements VGAPlanets
{
  static String defaultpwd="NOPASSWORD";

  boolean isplaying;
  StringBuffer timestamp,pwd,newpwd,dissig;
  Score[] score;
  boolean hasnewpwd;
  int shipcs,planetcs,basecs;
  short turnnr,hostid; // Well, not really host-id, but who cares
  boolean passed; // no password or user gave correct password
  boolean userst,usedata,usetrn,useutil;

  byte phostMajor, phostMinor;
  char phostRelease;

  public Gen() {
    isplaying=false;
    score=new Score[races+1];
    passed=false;
  }

  public boolean isPlaying() {
    return isplaying;
  }

  protected boolean checkPassword(String checkpwd) {
    // check whether checkpwd fits encrypted strings
    passed=true; // hehehe...
    return passed;
  }

    // race conditions! Fix when Gen is used everywhere, properly...
    public void setNewPwd(StringBuffer np) {
	//if (isEditable()) {
	    newpwd=np;
	    // not nice; check length, check for same-as-oldpwd...
	    hasnewpwd=true;
	    //getGame().unlock();
	//}
    }

  public void setShipChecksum(int cs) {
    shipcs=cs;
  }

  public void setPlanetChecksum(int cs) {
    planetcs=cs;
  }

  public void setBaseChecksum(int cs) {
    basecs=cs;
  }

  public int getPlanetChecksum() {
    return planetcs;
  }

  public int getShipChecksum() {
    return shipcs;
  }

  public int getBaseChecksum() {
    return basecs;
  }

  public String getDisSig() {
    return dissig.toString();
  }

  public String getDatSig() {
    // Adapt dissig, return it.
    StringBuffer datsig=new StringBuffer(dissig.toString());
    for (int i=0;i<PWDSIZE;i++) datsig.setCharAt(i,(char)(datsig.charAt(i)+i+1));
    return datsig.toString();
  }

  public int getTurnNumber() {
    return turnnr;
  }

  public String getTimestamp() {
    return timestamp.toString();
  }

    public Score getScore(int player) {
	return score[player];
    }

  /**
   * Read a complete Gen-object from a source.
   * @param source The source from which the data can be read.
   * @param isreal Whether the gen-info is real or from a result.
   */
  public void readGen(LSBReader source,int player,boolean isreal) {
    try {
      timestamp=source.readString(TIMESTAMPSIZE);
      // System.err.println("Timestamp: "+timestamp);
      for (int i=1;i<=races;i++) {
	score[i]=new Score();
	score[i].readGenScore(source);
      }
      short pl=source.readShort();
      if (pl!=player) {
	System.err.println("gen"+player+".dat contains data for player "+pl);
	System.exit(1);
      }
      pwd=source.readString(PWDSIZE); // encoded pwd
      dissig=source.readString(PWDSIZE); // dissig
      if (!checkPassword(defaultpwd)) {
	// get real pwd from the user and check it. !!!
        // if (!checkPassword(userpwd)) { 
	//   report("Failed to give proper pwd"); 
	// }
      }
      if (isreal) source.skip(1); // some useless byte; why, Tim?
      source.skip(12); // the three checksums, we don't need them
      if (isreal) {
	hasnewpwd=(source.readShort()!=0);
	newpwd=source.readString(PWDSIZE);
      } else {
	hasnewpwd=false;
      }
      turnnr=source.readShort();
      hostid=source.readShort();
      isplaying=true;
    } catch(IOException ioe) {
      System.err.println("readGen() -- "+ioe);
      System.exit(1);
    }
  }

  public void readPHostControl(LSBReader source,int player) throws IOException {
    int p=source.readShort();
    if (p!=player) crash(1002, "util"+player+".dat from player "+p);
    phostMajor=source.readByte();
    phostMinor=source.readByte();
    int hullspecdigest=source.readInt();
    int engspecdigest=source.readInt();
    int beamspecdigest=source.readInt();
    int torpspecdigest=source.readInt();
    int truehulldigest=source.readInt();
    int planetxydigest=source.readInt();
    int configdigest=source.readInt();
    int racenamedigest=source.readInt();
    StringBuffer gamename=source.readString(32);
    phostRelease = source.readString(1).charAt(0);
    warn("PHost version: "+phostMajor+"."+phostMinor+phostRelease);
  }

    public boolean atLeastPHostVersion(byte major, byte minor, char release) {
       if (phostMajor < major) return false;
       if (phostMajor > major) return true;
       if (phostMinor < minor) return false;
       if (phostMinor > minor) return true;
       if (phostRelease < release) return false;
       return true;
  }

  public void writeGen(LSBWriter sink,short player) {
    try {
      sink.writeString(timestamp,TIMESTAMPSIZE);
      for (int i=1;i<=races;i++) {
	score[i].writeGenScore(sink);
      }
      sink.writeShort(player);
      sink.writeString(pwd,PWDSIZE); // encoded pwd
      sink.writeString(dissig,PWDSIZE); // dissig
      sink.skip(1); // argh, Tim!
      sink.writeInt(shipcs);
      sink.writeInt(planetcs);
      sink.writeInt(basecs);
      if (hasnewpwd) {
	sink.writeUnsignedShort(1);
	sink.writeString(newpwd,PWDSIZE);
      } else {
	sink.writeUnsignedShort(0);
	sink.skip(PWDSIZE);
      }
      sink.writeShort(turnnr);
      sink.writeShort(hostid);
      sink.close();
    } catch(IOException ioe) {
      System.err.println("writeGen() -- "+ioe);
      System.exit(1);
    }
  }

  public void scanTurn() {}
  public void scanUtil() {}
}
