// File: Hull.java

package jvpc.vga;

import java.io.*;
import java.util.*;

/**
 * One hull from HULLSPEC.DAT
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since     17 I 1999
 * @version   15 I 2001
 */
public class Hull
    implements VGAPlanets
{
    Game game;
    int id;
    String name, shortname;
    int picture,unknown;
    int trit,dur,molyb,tank;
    int crew,engines,mass,tech,cargo,bays,launchs,beams,mc;

    public Hull(Game g,int newid) {
	game=g;
	id=newid;
    }

    public void readFrom(LSBReader source) throws IOException {
	name=source.readString(HULLNMSIZE).toString();
	shortname=name.substring(0,name.indexOf(' '));
	picture=source.readShort();
	unknown=source.readShort();
	trit=source.readShort();
	dur=source.readShort();
	molyb=source.readShort();
	tank=source.readShort();
	crew=source.readShort();
	engines=source.readShort();
	mass=source.readShort();
	tech=source.readShort();
	cargo=source.readShort();
	bays=source.readShort();
	launchs=source.readShort();
	beams=source.readShort();
	mc=source.readShort();
    }

    public String getName() {
	return name;
    }

    public String getShortName() {
	return shortname;
    }

    public int getMass() {
	return mass;
    }

    public int getTech() {
	return tech;
    }

    public int getTank() {
	return tank;
    }

    public int getCargo() {
	return cargo;
    }

    public int getCrew() {
	return crew;
    }

    public int getEngines() {
	return engines;
    }

    public int getBeams() {
	return beams;
    }

    public int getLaunchs() {
	return launchs;
    }

    public int getBays() {
	return bays;
    }

    /**
     * Can a ship cloak (including advanced cloak).
     */
    public boolean canCloak() {
	Hullfunc hullfunc=game.getHullfunc();
	Vector cloakvec=(Vector)hullfunc.getFunctionVector(funccloak);
	for (int i=cloakvec.size()-1; i>=0; i--) {
	    if (((Hullfunc.Allowed)cloakvec.elementAt(i)).hull==id) return true;
	}
	// cloaking includes advanced cloak
	cloakvec=(Vector)hullfunc.getFunctionVector(funcadvancedcloak);
	for (int i=cloakvec.size()-1; i>=0; i--) {
	    if (((Hullfunc.Allowed)cloakvec.elementAt(i)).hull==id) return true;
	}
	return false;
    }

    public boolean canCloakAdvanced() {
	Hullfunc hullfunc=game.getHullfunc();
	Vector cloakvec=(Vector)hullfunc.getFunctionVector(funcadvancedcloak);
	for (int i=cloakvec.size()-1; i>=0; i--) {
	    if (((Hullfunc.Allowed)cloakvec.elementAt(i)).hull==id) return true;
	}
	return false;
    }

    public boolean canDecloak() {
	Hullfunc hullfunc=game.getHullfunc();
	Vector cloakvec=(Vector)hullfunc.getFunctionVector(funcdecloak);
	for (int i=cloakvec.size()-1; i>=0; i--) {
	    if (((Hullfunc.Allowed)cloakvec.elementAt(i)).hull==id) return true;
	}
	return false;
    }

    /**
     * Can perform bioscan with 20% chance of detection (standard bioscan)
     */
    public boolean canBioscan20() {
	Hullfunc hullfunc=game.getHullfunc();
	Vector cloakvec=(Vector)hullfunc.getFunctionVector(funcbioscan20);
	for (int i=cloakvec.size()-1; i>=0; i--) {
	    if (((Hullfunc.Allowed)cloakvec.elementAt(i)).hull==id) return true;
	}
	return false;
    }

    /**
     * Can perform bioscan with 100% chance of detection (Robot Pawn IIRC)
     */
    public boolean canBioscan100() {
	Hullfunc hullfunc=game.getHullfunc();
	Vector cloakvec=(Vector)hullfunc.getFunctionVector(funcbioscan100);
	for (int i=cloakvec.size()-1; i>=0; i--) {
	    if (((Hullfunc.Allowed)cloakvec.elementAt(i)).hull==id) return true;
	}
	return false;
    }

    public boolean canHeatTo50() {
	Hullfunc hullfunc=game.getHullfunc();
	Vector cloakvec=(Vector)hullfunc.getFunctionVector(funcheatsto50);
	for (int i=cloakvec.size()-1; i>=0; i--) {
	    if (((Hullfunc.Allowed)cloakvec.elementAt(i)).hull==id) return true;
	}
	return false;
    }

    public boolean canHeatTo100() {
	Hullfunc hullfunc=game.getHullfunc();
	Vector cloakvec=(Vector)hullfunc.getFunctionVector(funcheatsto100);
	for (int i=cloakvec.size()-1; i>=0; i--) {
	    if (((Hullfunc.Allowed)cloakvec.elementAt(i)).hull==id) return true;
	}
	return false;
    }

    public boolean canCoolTo50() {
	Hullfunc hullfunc=game.getHullfunc();
	Vector cloakvec=(Vector)hullfunc.getFunctionVector(funccoolsto50);
	for (int i=cloakvec.size()-1; i>=0; i--) {
	    if (((Hullfunc.Allowed)cloakvec.elementAt(i)).hull==id) return true;
	}
	return false;
    }

    /**
     * Can perform normal alchemy (Merlin)
     */
    public boolean isAlchemy() {
	Hullfunc hullfunc=game.getHullfunc();
	Vector cloakvec=(Vector)hullfunc.getFunctionVector(funcalchemy);
	for (int i=cloakvec.size()-1; i>=0; i--) {
	    if (((Hullfunc.Allowed)cloakvec.elementAt(i)).hull==id) return true;
	}
	return false;
    }

    /**
     * Can perform neutronium alchemy, 1 supply + 1 mineral (Neutronic Refinery)
     */
    public boolean isRefinery() {
	Hullfunc hullfunc=game.getHullfunc();
	Vector cloakvec=(Vector)hullfunc.getFunctionVector(funcrefinery);
	for (int i=cloakvec.size()-1; i>=0; i--) {
	    if (((Hullfunc.Allowed)cloakvec.elementAt(i)).hull==id) return true;
	}
	return false;
    }

    /**
     * Can perform advanced neutronic alchemy: 1 mineral (Aries)
     */
    public boolean isAdvancedRefinery() {
	Hullfunc hullfunc=game.getHullfunc();
	Vector cloakvec=(Vector)hullfunc.getFunctionVector(funcadvancedrefinery);
	for (int i=cloakvec.size()-1; i>=0; i--) {
	    if (((Hullfunc.Allowed)cloakvec.elementAt(i)).hull==id) return true;
	}
	return false;
    }

    /**
     * Has a glory device, which does still 20% damage to your own ships.
     */
    public boolean hasGlory20() {
	Hullfunc hullfunc=game.getHullfunc();
	Vector cloakvec=(Vector)hullfunc.getFunctionVector(funcglory20);
	for (int i=cloakvec.size()-1; i>=0; i--) {
	    if (((Hullfunc.Allowed)cloakvec.elementAt(i)).hull==id) return true;
	}
	return false;
    }

    /**
     * Has a glory device, which does only 10% damage to your own ships.
     */
    public boolean hasGlory10() {
	Hullfunc hullfunc=game.getHullfunc();
	Vector cloakvec=(Vector)hullfunc.getFunctionVector(funcglory10);
	for (int i=cloakvec.size()-1; i>=0; i--) {
	    if (((Hullfunc.Allowed)cloakvec.elementAt(i)).hull==id) return true;
	}
	return false;
    }

    public boolean hasGravitonic() {
	Hullfunc hullfunc=game.getHullfunc();
	Vector cloakvec=(Vector)hullfunc.getFunctionVector(funcgravitonic);
	for (int i=cloakvec.size()-1; i>=0; i--) {
	    if (((Hullfunc.Allowed)cloakvec.elementAt(i)).hull==id) return true;
	}
	return false;
    }

    /**
     * Has a casino, <ABBR>i.e.</abbr> is a gambling ship.
     */
    public boolean hasCasino() {
	Hullfunc hullfunc=game.getHullfunc();
	Vector cloakvec=(Vector)hullfunc.getFunctionVector(funcgamble);
	for (int i=cloakvec.size()-1; i>=0; i--) {
	    if (((Hullfunc.Allowed)cloakvec.elementAt(i)).hull==id) return true;
	}
	return false;
    }

    public boolean canChunnel() {
	Hullfunc hullfunc=game.getHullfunc();
	Vector cloakvec=(Vector)hullfunc.getFunctionVector(funcchunnel);
	for (int i=cloakvec.size()-1; i>=0; i--) {
	    if (((Hullfunc.Allowed)cloakvec.elementAt(i)).hull==id) return true;
	}
	return false;
    }

    public boolean hasImperialTroopers() {
	Hullfunc hullfunc=game.getHullfunc();
	Vector cloakvec=(Vector)hullfunc.getFunctionVector(funcassault);
	for (int i=cloakvec.size()-1; i>=0; i--) {
	    if (((Hullfunc.Allowed)cloakvec.elementAt(i)).hull==id) return true;
	}
	return false;
    }

    public boolean canHyperjump() {
	Hullfunc hullfunc=game.getHullfunc();
	Vector cloakvec=(Vector)hullfunc.getFunctionVector(funchyperjump);
	for (int i=cloakvec.size()-1; i>=0; i--) {
	    if (((Hullfunc.Allowed)cloakvec.elementAt(i)).hull==id) return true;
	}
	return false;
    }

    public boolean canScoopFuel() {
	Hullfunc hullfunc=game.getHullfunc();
	Vector cloakvec=(Vector)hullfunc.getFunctionVector(funcfuelscoop);
	for (int i=cloakvec.size()-1; i>=0; i--) {
	    if (((Hullfunc.Allowed)cloakvec.elementAt(i)).hull==id) return true;
	}
	return false;
    }

    public boolean canScanAllWormholes() {
	Hullfunc hullfunc=game.getHullfunc();
	Vector cloakvec=(Vector)hullfunc.getFunctionVector(funcwormholescan);
	for (int i=cloakvec.size()-1; i>=0; i--) {
	    if (((Hullfunc.Allowed)cloakvec.elementAt(i)).hull==id) return true;
	}
	return false;
    }

    public String toString() {
	return name;
    }
}
