// File: Hullfunc.java

package jvpc.vga;

import java.io.*;
import java.util.*;

/**
 * All hull-functions from HULLFUNC.DAT. These functions are given by
 * a list of hulls and apply to a set of players.
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since     17 I 1999
 * @version   13 V 1999
 */
public class Hullfunc
  implements VGAPlanets
{
  static int[] alchemists={ 105 };
  static int[] refineries={ 104 };
  static int[] advancedrefineries={ 97 };
  static int[] heatsto50={ 3 };
  static int[] coolsto50={ 8 };
  static int[] heatsto100={ 64 };
  static int[] hyperjumpers={ 51, 77, 87 };
  static int[] gravitonics={ 44, 45, 46 };
  static int[] wormholescanners={ 3 };
  static int[] gamblers={ 42 };
  static int[] decloakers={ 7 };
  static int[] assaulters={ 69 };
  static int[] chunnelers={ 56 };
  static int[] fuelscoopers={ 96 };
  static int[] bioscanner100={ 84 };
  static int[] advancedcloakers={ 29, 31 };
  static int[] cloakers={ 21, 22, 25, 26, 27, 28, 32, 33, 36, 38, 43, 44, 45, 46, 47 };
  static int[] bioscanner20={ 9, 96 };
  static int[] glory20={ 39 };
  static int[] glory10={ 41 };

  public class Allowed {
    int hull;
    int allowed;

    public Allowed(int nr,int allowed) {
      hull=nr;
      this.allowed=allowed;
    }
  }

  Hashtable table;

  public Hullfunc() {
    table=new Hashtable();
    Vector vector;
    int i;
    // function: 
    vector=new Vector();
    for (i=0; i<alchemists.length; i++) {
      vector.addElement(new Allowed(alchemists[i], (1<<races)-1));
    }
    table.put(new Integer(funcalchemy),vector);
    // function: 
    vector=new Vector();
    for (i=0; i<refineries.length; i++) {
      vector.addElement(new Allowed(refineries[i], (1<<races)-1));
    }
    table.put(new Integer(funcrefinery),vector);
    // function: 
    vector=new Vector();
    for (i=0; i<advancedrefineries.length; i++) {
      vector.addElement(new Allowed(advancedrefineries[i], (1<<races)-1));
    }
    table.put(new Integer(funcadvancedrefinery),vector);
    // function: 
    vector=new Vector();
    for (i=0; i<heatsto50.length; i++) {
      vector.addElement(new Allowed(heatsto50[i], (1<<races)-1));
    }
    table.put(new Integer(funcheatsto50),vector);
    // function: 
    vector=new Vector();
    for (i=0; i<coolsto50.length; i++) {
      vector.addElement(new Allowed(coolsto50[i], (1<<races)-1));
    }
    table.put(new Integer(funccoolsto50),vector);
    // function: 
    vector=new Vector();
    for (i=0; i<heatsto100.length; i++) {
      vector.addElement(new Allowed(heatsto100[i], (1<<races)-1));
    }
    table.put(new Integer(funcheatsto100),vector);
    // function: 
    vector=new Vector();
    for (i=0; i<hyperjumpers.length; i++) {
      vector.addElement(new Allowed(hyperjumpers[i], (1<<races)-1));
    }
    table.put(new Integer(funchyperjump),vector);
    // function: 
    vector=new Vector();
    for (i=0; i<gravitonics.length; i++) {
      vector.addElement(new Allowed(gravitonics[i], (1<<races)-1));
    }
    table.put(new Integer(funcgravitonic),vector);
    // function: 
    vector=new Vector();
    for (i=0; i<wormholescanners.length; i++) {
      vector.addElement(new Allowed(wormholescanners[i], (1<<races)-1));
    }
    table.put(new Integer(funcwormholescan),vector);
    // function: 
    vector=new Vector();
    for (i=0; i<gamblers.length; i++) {
      vector.addElement(new Allowed(gamblers[i], (1<<races)-1));
    }
    table.put(new Integer(funcgamble),vector);
    // function: 
    vector=new Vector();
    for (i=0; i<decloakers.length; i++) {
      vector.addElement(new Allowed(decloakers[i], (1<<races)-1));
    }
    table.put(new Integer(funcdecloak),vector);
    // function: 
    vector=new Vector();
    for (i=0; i<assaulters.length; i++) {
      vector.addElement(new Allowed(assaulters[i], (1<<races)-1));
    }
    table.put(new Integer(funcassault),vector);
    // function: 
    vector=new Vector();
    for (i=0; i<chunnelers.length; i++) {
      vector.addElement(new Allowed(chunnelers[i], (1<<races)-1));
    }
    table.put(new Integer(funcchunnel),vector);
    // function: 
    vector=new Vector();
    for (i=0; i<fuelscoopers.length; i++) {
      vector.addElement(new Allowed(fuelscoopers[i], (1<<races)-1));
    }
    table.put(new Integer(funcfuelscoop),vector);
    // function: 
    vector=new Vector();
    for (i=0; i<bioscanner100.length; i++) {
      vector.addElement(new Allowed(bioscanner100[i], (1<<races)-1));
    }
    table.put(new Integer(funcbioscan100),vector);
    // function: advanced cloak 
    vector=new Vector();
    for (i=0; i<advancedcloakers.length; i++) {
      vector.addElement(new Allowed(advancedcloakers[i], (1<<races)-1));
    }
    table.put(new Integer(funcadvancedcloak),vector);
    // function: cloak 
    vector=new Vector();
    for (i=0; i<cloakers.length; i++) {
      vector.addElement(new Allowed(cloakers[i], (1<<races)-1));
    }
    table.put(new Integer(funccloak),vector);
    // function: 
    vector=new Vector();
    for (i=0; i<bioscanner20.length; i++) {
      vector.addElement(new Allowed(bioscanner20[i], (1<<races)-1));
    }
    table.put(new Integer(funcbioscan20),vector);
    // function: 
    vector=new Vector();
    for (i=0; i<glory20.length; i++) {
      vector.addElement(new Allowed(glory20[i], (1<<races)-1));
    }
    table.put(new Integer(funcglory20),vector);
    // function: 
    vector=new Vector();
    for (i=0; i<glory10.length; i++) {
      vector.addElement(new Allowed(glory10[i], (1<<races)-1));
    }
    table.put(new Integer(funcglory10),vector);
  }

  public void clear() {
    table.clear();
    for (int i=0;i<hullfunctions;i++) {
      table.put(new Integer(i),new Vector());
    }
  }

  public Vector getFunctionVector(int function) {
    return (Vector)table.get(new Integer(function));
  }

  public void putFunctionAllowed(int function, int hull, int allowed) {
    Vector vec=getFunctionVector(function);
    vec.addElement(new Hullfunc.Allowed(hull,allowed));
  }

  public void print() {
    Vector vec;
    Hullfunc.Allowed allowed;
    for (int i=0; i<hullfunctions; i++) {
      vec=getFunctionVector(i);
      System.out.print("{");
      for (int j=0; j<vec.size()-1; j++) {
	allowed=(Hullfunc.Allowed)vec.elementAt(j);
	System.out.print(" "+allowed.hull+",");
      }
      if (vec.size()>0) {
	allowed=(Hullfunc.Allowed)vec.elementAt(vec.size()-1);
	System.out.print(" "+allowed.hull);
      }
      System.out.println("}");
    }
  }
}
