// File: HullfuncDatGen.java

package jvpc.vga;

import java.util.*;
import java.io.*;

/**
 * Generates Hull functions from the binary file "hullfunc.dat".
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since     13 V 1999
 * @version   21 XII 1999
 */
public class HullfuncDatGen extends Generator {
  /**
   * @param is The <CODE>InputStream</code> from which the <CODE>source</code>
   * can be created.
   */
  public HullfuncDatGen(InputStream is) throws IOException {
    super(is);
  }

  /**
   * Generates 10 <CODE>Hull</code> objects.
   */
  public void generateAll(Game game) {
    Hullfunc hullfunc=game.getHullfunc();
    hullfunc.clear();
    try {
      int magicnumber=source.readInt();
      if (magicnumber!=0xB1297F35) {
	crash(1002, "HULLFUNC.DAT is not a (binary) hullfunc file.\n"+
	      "Make sure this file was generated by running:\n"+
	      "  phost -l [gamedir [globaldir]]\n"+
	      "It is generated automatically!! (don't redirect output)\n"
	      );
      }
      int phostversion=source.readShort();
      source.readString(32); // so?
      for (int i=0;i<8;i++) source.readShort(); // 8 reserved words, should be zero
      int records=source.readShort();
      Hull hull;
      for (int i=0;i<records;i++) {
	int hullnr=source.readShort();
	int functions=source.readShort();
	for (int j=0;j<functions;j++) {
          int function=source.readShort();
	  int playersallowed=source.readShort(); // erm 11 bits in an LSB word....
	  hullfunc.putFunctionAllowed(function,hullnr,playersallowed);
	  // System.err.println("Function "+function+" for hull "+hullnr+" and players "+playersallowed);
	}
      }
      magicnumber=source.readInt();
      if (magicnumber!=0x1F0C219A) {
	crash(1002, "HULLFUNC.DAT corrupted. Remove or rebuild (phost -l).");
      }
      int checksum=source.readInt(); // forget it. I don't do little endian arithmetic
    } catch (IOException ioe) {
      crash(11, "Error while reading hullfunc.dat: "+ioe);
    }
  }
}
