// File: InputFileOpener.java

package jvpc.vga;

import java.io.*;

/**
 * Opens an <CODE>FileInputStream</code> for JVPC. It looks in the directory
 * with the game first, then optionally in the global-data directory as well.
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since     10 VII 1998
 * @version   19 XII 1999
 */
public final class InputFileOpener
{
  private String workdir,globaldir;
  static private File file;

  /**
   * Constructs an <CODE>InputFileOpener</code>.
   * @param workdir Name of the directory with all player files.
   * @param globaldir Name of the directory where global data can be found.
   */
  public InputFileOpener(String wd,String gd) throws IOException {
    this.workdir=wd;
    if (workdir.charAt(workdir.length()-1) != File.separatorChar)
	workdir += File.separator;
    //FileInputStream fis=new FileInputStream(workdir); // is a directory?
    this.globaldir=gd;
    if (globaldir.charAt(globaldir.length()-1) != File.separatorChar)
	globaldir += File.separator;
    //fis=new FileInputStream(globaldir); // check for being a directory !!!
  }

  /**
   * Opens an <CODE>FileInputStream</code> for JVPC. It looks in the directory
   * with the game first, then optionally in the global-data directory, too.
   * Both the lower-case and upper-case name are checked, starting with the
   * lower-case name.
   * @param filename Name of the file to be opened.
   * @param global If <CODE>true</code>, look in <CODE>globaldir</code> if the
   * file is not found in <CODE>workdir</code>.
   * @return The <CODE>FileInputStream</code> associated with the filename.
   */
  public FileInputStream open(String filename,boolean global) throws IOException {
    StringBuffer lower=new StringBuffer(filename);
    StringBuffer upper=new StringBuffer(filename);
    int i;
    try {
      // change filename to lower-case
      for (i=0;i<lower.length();i++)
	lower.setCharAt(i,Character.toLowerCase(filename.charAt(i)));
      //System.err.println("Trying to open: "+lower);
      return doOpen(lower.toString(),global);
    } catch (IOException ioe) {
      // ignore
    }
    try {
      // change filename to upper-case
      for (i=0;i<upper.length();i++)
	upper.setCharAt(i,Character.toUpperCase(filename.charAt(i)));
      //System.err.println("Trying to open: "+upper);
      return doOpen(upper.toString(),global);
    } catch (IOException ioe) {
      throw new IOException("Could open neither "+lower+" nor "+upper+":"+ioe);
    }
  }

  protected FileInputStream doOpen(String filename,boolean global) throws IOException {
    FileInputStream fis=null;
    try {
      //System.err.println("Trying to open: "+workdir+filename);
      fis=new FileInputStream(workdir+filename);
      //System.err.println("File work/``"+filename+"''"+fis.available());
      file = new File(workdir+filename);
      return fis;
    } catch (IOException ioe1) {
      if (global) {
	try {
	  //System.err.println("Trying to open: "+globaldir+filename);
	  fis=new FileInputStream(globaldir+filename);
	  //System.err.println("File global``"+filename+"''"+fis.available());
	  file = new File(globaldir+filename);
	  return fis;
	} catch (IOException ioe2) {
	  throw new IOException("File "+filename+" not found, "+
				"neither in "+workdir+", nor in "+globaldir);
	}
      } else {
	throw new IOException("File "+workdir+filename+" not found");
      }
    }
  }

  public String getWorkDir() {
    return workdir;
  }

  public void setWorkDir(String workdir) {
    this.workdir = workdir;
  }

  static public File getLastOpenedFile() {
    return file;
  }
}
