// File: Ionstorm.java

package jvpc.vga;

import java.util.*;
import java.io.*;

/**
 * Contains an ionstorm for JVPC.
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the Lesser GNU General Public License applies)
 * @since     21 May 2004
 */
public class Ionstorm
  extends Located
  implements Intersection
{
  int id, voltage, heading, warp, radius;

  public Vector getTypes() {
    Vector res = super.getTypes();
    res.addElement("voltage");
    res.addElement("heading");
    res.addElement("warp");
    res.addElement("radius");
    return res;
  }

  public Vector getTypeMethods(String typeName) {
    Vector res = super.getTypeMethods(typeName);
    if ("voltage".equals(typeName)) {
      res.addElement("getVoltage");
    } else if ("heading".equals(typeName)) {
      res.addElement("getHeading");
    } else if ("warp".equals(typeName)) {
      res.addElement("getWarp");
    } else if ("radius".equals(typeName)) {
      res.addElement("getRadius");
    }
    return res;
  }

  public Ionstorm(Universe u, int id) {
    super(u);
    this.id = id;
  }

  public Object clone() throws CloneNotSupportedException {
    return (Ionstorm)(super.clone());
  }

  public int getId() { return id; }
  public int getVoltage() { return voltage; }
  public int getHeading() { return heading; }
  public int getWarp() { return warp; }
  public double getRadius() {  return (double)radius; }

  public Circle getCircle() {
    Circle c = new Circle(new MyPoint(getX(), getY()));
    c.setRadius(getRadius());
    return c;
  }

  /**
   * Returns true if point p lies on this object
   */
  public boolean liesOn(MyPoint p) {
    return getCircle().liesOn(p);
  }

  /**
   * Returns true if point p lies within this object
   */
  public boolean liesWithin(MyPoint p) {
    return getCircle().liesWithin(p);
  }

  /**
   * Returns true if line l lies (partly) within this object
   */
  public boolean liesWithin(Line l) {
    return getCircle().liesWithin(l);
  }

  /**
   * Returns true if rectangle r lies (partly) within this object
   */
  public boolean liesWithin(MyRectangle r) {
    return getCircle().liesWithin(r);
  }

  public String getKey() {
    return ionstormhashprefix + getId();
  }

  public String toString() {
    return toString(true);
  }

  public String toString(boolean showXY) {
    return "Ion"+getId()+(showXY?"@("+x+","+y+")":" ")+
      getVoltage()+"V, w"+getWarp()+", head="+getHeading()+" r="+getRadius();
  }
}
