// File: MapSearch.java

package jvpc.vga;

import java.util.*;
import kero.util.Assert;

/**
 * Contains a set of Locations, which are ordered in some sophisticated way,
 * such that finding the Location nearest to some set of coordinates can be
 * found relatively fast.
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since     16 III 1999
 * @version   14 III 2000
 */
public class MapSearch {
    Hashtable locations = new Hashtable();

    static String makeKey(int x, int y) {
	return "@" + x + "," + y;
    }

    protected void putLocation(Location loc) {
	int x = loc.getX();
	int y = loc.getY();
	// if (getLocationAt(x,y) != null) return;
	String key = makeKey(x, y);
	Object obj = locations.put(key, loc);
	Assert.assert(obj == null);
    }

    public void putLocated(Located located) {
	if (located instanceof Location) return; // prevent recursion
	int x = located.getX();
	int y = located.getY();
	Location location = getLocationAt(x, y);
	if (location == null) {
	    location = new Location(located.getUniverse(), x, y);
	    putLocation(location);
	}
	// put Located in Location only if it is not in there, yet
	if (!location.contains(located)) location.putLocated(located);
    }

    public Location getLocationAt(int x, int y) {
	String key = makeKey(x, y);
	return (Location)locations.get(key);
    }

    /*  public Location getLocationAt(MyPoint mp) {
	return getLocationAt(mp.getXint(), mp.getYint());
    }*/

  public Location getNearestTo(int x, int y) {
    // System.err.print("MapSearch.getNearestTo("+x+","+y+")");
    Location nearest = getLocationAt(x, y);
    if (nearest != null) return nearest;
    Location other;
    int neardist, dist; // distance squared
    Enumeration enum = locations.elements();
    if (enum.hasMoreElements()) {
      nearest = (Location)enum.nextElement();
      int dx = nearest.getX() - x;
      int dy = nearest.getY() - y;
      neardist = dx*dx + dy*dy;
      while (enum.hasMoreElements()) {
	other = (Location)enum.nextElement();
	dx = other.getX() - x;
	dy = other.getY() - y;
	dist = dx*dx + dy*dy;
	if (dist < neardist) {
	  nearest = other;
	  neardist = dist;
	}
      }
    }
    return nearest;
  }

    public Location getNearestTo(MyPoint mp) {
	return getNearestTo(mp.getXint(), mp.getYint());
    }

    public Enumeration getLocations() {
	return locations.elements();
    }
}
