// File: MessGen.java

package jvpc.vga;

import java.io.*;

/**
 * Generates messages; opens file ``mess??.dat'' and reads outgoing messages.
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since     12 II 1999
 * @version   21 XII 1999
 */
public class MessGen extends Generator {
  private int messages;
  private OutputFileOpener ofo;

  /**
   * Constructor for reading messages only.
   */
  public MessGen(InputStream is) throws IOException {
    super(is);
    messages=source.readShort();
  }

  /**
   * Constructor for writing messages.
   * (odd, that I need two instances of one generator for that...)
   */
  public MessGen(OutputFileOpener o) {
    super();
    ofo=o;
  }

  public void generateAll(Universe u,int player) {
    int i;
    int[] pointer=new int[messages];
    int[] length=new int[messages];
    int[] from=new int[messages];
    int[] to=new int[messages];
    try {
      // current ``posistion'' is 0
      for (i=0;i<messages;i++) {
	pointer[i]=source.readInt()-1;
	length[i]=source.readShort();
	from[i]=source.readShort();
	if (from[i]!=player) crash(1002, "Somebody's been messing with the data-files");
	to[i]=source.readShort();
      }
      // current ``position'' is 2 + messages * (4+2+2+2)
      int curpos=2+messages*10;
      for (i=0;i<messages;i++) {
	if (curpos<pointer[i]) {
	  source.skip(pointer[i]-curpos);
	  curpos=pointer[i];
	}
	Message message=new Message(u);
	message.readMessFrom(source,length[i],player);
	curpos+=length[i]; // contents
	message.setFromAndTo(from[i],to[i]);
	u.putMessage(message);
      }
    } catch (IOException ioe) {
      crash(11, "Could not read mess, probably truncated file");
    }
  }

  public void saveMess(Universe u) throws IOException {
    OutputStream outfile;
    LSBWriter[] lsbw=new LSBWriter[races+1];
    int[] count=new int[races+1];
    int[] pointer=new int[races+1];
    int[] length=new int[races+1];
    int i,j,player;

    // count messages per player
    for (i=0;i<u.getMessageCount();i++) {
      Message message=u.getMessage(i);
      int owner=message.getSender();
      count[owner]++;
    }

    for (j=1;j<=races;j++) {
      if (u.getGen(j)!=null) { // isPlaying()...
	outfile=ofo.open("mess"+j+".dat");
	lsbw[j]=new LSBWriter(outfile);
	lsbw[j].writeShort((short)count[j]);
	pointer[j]=2+10*count[j]+1;
      }
    }

    // write pointers and lengths
    for (i=0;i<u.getMessageCount();i++) {
      Message message=u.getMessage(i);
      int owner=message.getSender();
      lsbw[owner].writeInt(pointer[owner]);
      lsbw[owner].writeShort((short)message.getLength());
      lsbw[owner].writeShort((short)message.getSender());
      lsbw[owner].writeShort((short)message.getRecipient());
      pointer[owner]+=message.getLength();
    }

    // write contents
    for (i=0;i<u.getMessageCount();i++) {
      Message message=u.getMessage(i);
      int owner=message.getSender();
      try {
	u.getMessage(i).writeMessageTo(lsbw[owner]);
      } catch (IOException ioe) {
	// hubba
	crash(11, "Failed to write message "+i+" to file; "+ioe);
      }
    }

    // no signature needed, just close file
    for (i=0;i<races;i++) {
      player=(short)(i+1);
      if (lsbw[player]!=null) try {
	lsbw[player].close();
      } catch (Exception ioe) {
	crash(11, "Failed to finish incoming??.dat: "+ioe);
      }
    }
  }
}
