// File: Mission.java

package jvpc.vga;

import java.util.*;

/**
 * Names of missions, parameters of missions and queries about missions (often
 * in realtion with player/race).
 *
 * @author Rutger Nijlunsing
 * @author Kero van Gelder
 * @copyright Rutger & Kero; the GNU General Public License applies
 * @version 12 IV 2000
 */
public class Mission
  implements VGAPlanets
{
  // Meaning of mission field of a Ship
  public final static int NONE = 0;
  public final static int EXPLORE = 1;
  public final static int MINESWEEP = 2;
  public final static int LAYMINES = 3;
  public final static int KILL = 4;
  public final static int SENSORSWEEP = 5;
  public final static int COLONIZE = 6;
  public final static int TOW = 7;
  public final static int INTERCEPT = 8;
  public final static int SPECIAL = 9;
  public final static int CLOAK = 10;
  public final static int BEAMUPFUEL = 11;
  public final static int BEAMUPDUR = 12;
  public final static int BEAMUPTRIT = 13;
  public final static int BEAMUPMOLYB = 14;
  public final static int BEAMUPSUPP = 15;

  public final static int TIMMISSIONS = 16;
  
  /**
   * Returns true iff special mission is
   * lay web minefield for this player
   */
  public static boolean specialIsLayWebMinefield(PConfig config, int player) {
    return config.getPlayerRace(player) == Race.CRYSTALS;
  }

  public static int buildFromCargo(PConfig config) {
    return config.extMissionsStartAt();
  }

  public static int layMinefield(PConfig config) {
    return config.extMissionsStartAt() + 1;
  }

  public static int layWebMinefield(PConfig config) {
    return config.extMissionsStartAt() + 2;
  }

  public static int scoopTorps(PConfig config) {
    return config.extMissionsStartAt() + 3;
  }

  public static int gatherBuildTorpedos(PConfig config) {
    return config.extMissionsStartAt() + 4;
  }

  public static int beamDownMCs(PConfig config) {
    return config.extMissionsStartAt() + 5;
  }

  public static int transferTorpedos(PConfig config) {
    return config.extMissionsStartAt() + 6;
  }

  public static int transferFighters(PConfig config) {
    return config.extMissionsStartAt() + 7;
  }

  public static int transferCredits(PConfig config) {
    return config.extMissionsStartAt() + 8;
  }

  public static int standardSuperSpy(PConfig config) {
    return config.extMissionsStartAt() + 9;
  }

  public static int cloak(PConfig config) {
    return config.extMissionsStartAt() + 10;
  }

  public static int special(PConfig config) {
    return config.extMissionsStartAt() + 11;
  }

  public static int gatherBuildFighters(PConfig config) {
    return config.extMissionsStartAt() + 12;
  }

  public static int beamUpMCs(PConfig config) {
    return config.extMissionsStartAt() + 13;
  }

  public static int beamUpClans(PConfig config) {
    return config.extMissionsStartAt() + 14;
  }

  public static int beamUpMultiple(PConfig config) {
    return config.extMissionsStartAt() + 15;
  }

  public static int layMinesIn(PConfig config) {
    return config.extMissionsStartAt() + 16;
  }

  public static int layWebMinesIn(PConfig config) {
    return config.extMissionsStartAt() + 17;
  }

  /**
   * Returns true iff mission produces a (web)minefield
   */
  public static boolean missionLaysMines(int mission, PConfig config, int player) {
    return
      (mission == LAYMINES) ||
      ((mission == SPECIAL) && specialIsLayWebMinefield(config, player)) ||
      (mission == layMinefield(config)) ||
      (mission == layWebMinefield(config)) ||
      (mission == layMinesIn(config)) ||
      (mission == layWebMinesIn(config));
  }

  /**
   * Returns true iff mission produces a web-minefield
   */
  public static boolean missionLaysWebMines(int mission, PConfig config, int player) {
    return
      ((mission == SPECIAL) && specialIsLayWebMinefield(config, player)) ||
      (mission == layWebMinefield(config)) ||
      (mission == layWebMinesIn(config));
  }

  // and now for the real Missions!
  
  int id;
  String name;
  String inthint,targethint;

  public static Vector setupTimMissions() {
    Vector result=new Vector();
    // add list below in increasing order; works. setElementSt(new Mission(),Mission.MISSID) would be better
    result.addElement(new Mission(Mission.NONE,"None",null,null));
    result.addElement(new Mission(Mission.EXPLORE,"Explore",null,null));
    result.addElement(new Mission(Mission.MINESWEEP,"Minesweep",null,null));
    result.addElement(new Mission(Mission.LAYMINES,"Lay Mines",null,null));
    result.addElement(new Mission(Mission.KILL,"Kill",null,null));
    result.addElement(new Mission(Mission.SENSORSWEEP,"Sensor Sweep",null,null));
    result.addElement(new Mission(Mission.COLONIZE,"Colonize Ship",null,null));
    result.addElement(new Mission(Mission.TOW,"Tow",null,"Ship ID"));
    result.addElement(new Mission(Mission.INTERCEPT,"Intercept","Ship ID",null));
    result.addElement(new Mission(Mission.SPECIAL,"Special",null,null));
    result.addElement(new Mission(Mission.CLOAK,"Cloak Ship",null,null));
    result.addElement(new Mission(Mission.BEAMUPFUEL,"Beam Up Fuel",null,null));
    result.addElement(new Mission(Mission.BEAMUPDUR,"Beam Up Duranium",null,null));
    result.addElement(new Mission(Mission.BEAMUPTRIT,"Beam Up Tritanium",null,null));
    result.addElement(new Mission(Mission.BEAMUPMOLYB,"Beam Up Molybdenium",null,null));
    result.addElement(new Mission(Mission.BEAMUPSUPP,"Beam Up Supplies",null,null));
    return result;
  }
  
  public Mission(int id,String name,String inthint,String targethint) {
    this.id=id;
    this.name=name;
    this.inthint=inthint;
    this.targethint=targethint;
  }

  public int getId() {
    return id;
  }

  public String getName() {
    return name;
  }

  public String getIntHint() {
    return inthint;
  }

  public String getTowHint() {
    return targethint;
  }

  public String toString() {
    return name + "(" + id + ")";
  }
}
