// File: OutputFileOpener.java

package jvpc.vga;

import java.io.*;

/**
 * Opens an <CODE>FileOutputStream</code> for JVPC. It looks in the directory
 * with the game first, then optionally in the global-data directory as well.
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since     10 VII 1998
 * @version   20 XII 1998
 */
public final class OutputFileOpener
{
  private String workdir,globaldir;

  /**
   * Constructs an <CODE>OutputFileOpener</code>.
   * @param workdir Name of the directory with all player files.
   * @param globaldir Name of the directory where global data can be found.
   */
  public OutputFileOpener(String workdir,String globaldir) {
    this.workdir=workdir;
    this.globaldir=globaldir;
  }

  /**
   * Opens an <CODE>FileOutputStream</code> for JVPC associated with the given
   * filename (in lowecase). It looks in the directory with the game only.
   * @param filename Name of the file to be opened, converted to lowercase.
   * @param global If <CODE>true</code>, look in <CODE>globaldir</code>. If the
   * file is not found, open/create it in <CODE>workdir</code>.
   * @return The <CODE>FileOutputStream</code> associated with the filename.
   */
  public FileOutputStream open(String filename) {
    StringBuffer lower=new StringBuffer(filename);
    StringBuffer upper=new StringBuffer(filename);
    FileOutputStream file;
    int i;
    for (i=0;i<lower.length();i++)
      lower.setCharAt(i,Character.toLowerCase(filename.charAt(i)));
    for (i=0;i<upper.length();i++)
      upper.setCharAt(i,Character.toUpperCase(filename.charAt(i)));
    try {
      FileInputStream infile=new FileInputStream(workdir+lower);
      infile.close();
    } catch (IOException ioe1) {
      // lower case name not found; try upper case
      try {
	FileInputStream infile=new FileInputStream(workdir+upper);
	// When we are here, uppercase was found; open for writing
	infile.close();
	file=new FileOutputStream(workdir+upper);
	return file;
      } catch (IOException ioe) {
	// either upper case didn't exist, or we can not write; write lowercase
	try {
	  file=new FileOutputStream(workdir+lower);
	  return file;
	} catch (IOException ioe2) {
	  // we can't write anything
	  return null;
	}
      }
    }
    // lowercase found (all catch-statements return)
    try {
      file=new FileOutputStream(workdir+lower);
      return file;
    } catch (IOException ioe) {
      // we can't write a write-protected file (or something)
      return null;
    }
  }
}
