// File: PConfig.java

package jvpc.vga;

import java.util.Hashtable;
import java.io.*;
import kero.util.*;

/**
 * Can read pconfig.src like files. Has an interface for pconfig-specific
 * values.
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since     18 XII 1998
 * @version   5 II 2001
 */
public class PConfig
    extends Config
{
    /**
     * Return the race of player p
     */
    public int getPlayerRace(int player) {
	try {
	    return getIntArrayElement("PlayerRace", player);
	} catch (NoConfigStringException ncse) {
	    return player;
	}
    }

    public String getGameName() {
	return (String)config.get("GameName");
    }


    // Some Movement stuff

    public int gravityWellRange() {
	try {
	    return getInteger("GravityWellRange");
	} catch (NoConfigStringException ncse) {
	    return 3;
	}
    }

    public boolean allowGravityWells() {
	try {
	    return getBoolean("AllowGravityWells");
	} catch (NoConfigStringException ncse) {
	    return true;
	}
    }

    public boolean roundGravityWell() {
	try {
	    return getBoolean("RoundGravityWell");
	} catch (NoConfigStringException ncse) {
	    return false;
	}
    }


    // Some Racial Stuff

    public boolean crystalsPreferDeserts() {
	try {
	    return getBoolean("CrystalsPreferDeserts");
	} catch (NoConfigStringException ncse) {
	    return true; // default host setting
	}
    }

    public boolean crystalSinTempBehavior() {
	try {
	    return getBoolean("CrystalSinTempBehavior");
	} catch (NoConfigStringException ncse) {
	    return true; // default host setting
	}
    }

    public int mineScanRange() {
	try {
	    return getInteger("MineScanRange");
	} catch (NoConfigStringException ncse) {
	    return 200;
	}
    }

    public int mineSweepRange() {
	try {
	    return getInteger("MineSweepRange");
	} catch (NoConfigStringException ncse) {
	    return 200;
	}
    }

    public int mineDecayRate() {
	try {
	    return getInteger("MineDecayRate");
	} catch (NoConfigStringException ncse) {
	    return 5;
	}
    }

    public int webMineSweepRange() {
	try {
	    return getInteger("WebMineSweepRange");
	} catch (NoConfigStringException ncse) {
	    return 20;
	}
    }

    public int webMineDecayRate() {
	try {
	    return getInteger("WebMineDecayRate");
	} catch (NoConfigStringException ncse) {
	    return 5;
	}
    }

    public int sensorRange() {
	try {
	    return getInteger("SensorRange");
	} catch (NoConfigStringException ncse) {
	    return 200;
	}
    }

    public int scanRange() {
	try {
	    return getInteger("ScanRange");
	} catch (NoConfigStringException ncse) {
	    return 200;
	}
    }

    public int cloakFuelBurn() {
	try {
	    return getInteger("CloakFuelBurn");
	} catch (NoConfigStringException ncse) {
	    return 5;
	}
    }


    // Some planetary stuff

    public int minesForDetectable() {
	try {
	    return getInteger("MinesForDetectable");
	} catch (NoConfigStringException ncse) {
	    return 20;
	}
    }

    public int factoriesForDetectable() {
	try {
	    return getInteger("FactoriesForDetectable");
	} catch (NoConfigStringException ncse) {
	    return 15;
	}
    }

    public int defenseForUndetectable() {
	try {
	    return getInteger("DefenseForUndetectable");
	} catch (NoConfigStringException ncse) {
	    return 15;
	}
    }

    public int maxColTempSlope() {
	try {
	    return getInteger("MaxColTempSlope");
	} catch (NoConfigStringException ncse) {
	    return 10;
	}
    }


    public boolean climateLimitsPopulation() {
	try {
	    return getBoolean("ClimateLimitsPopulation");
	} catch (NoConfigStringException ncse) {
	    return true;
	}
    }

    public int colonistTaxRate(int player) {
	if (player == 0) {
	    return 0;               // No tax for No-one :)
	} else {
	    try {
		return getIntArrayElement("ColonistTaxRate", player);
	    } catch (NoConfigStringException ncse) {
		return 100;
	    }
	}
    }

    public int nativeTaxRate(int player) {
	if (player == 0) {
	    return 0;               // No tax for No-one :)
	} else {
	    try {
		return getIntArrayElement("NativeTaxRate", player);
	    } catch (NoConfigStringException ncse) {
		return 100;
	    }
	}
    }

    public int extMissionsStartAt() {
	try {
	    return getInteger("ExtMissionsStartAt");
	} catch (NoConfigStringException ncse) {
	    return 20;
	}
    }

    public int wrmEntryPowerX100() {
	try {
	    return getInteger("WrmEntryPowerX100");
	} catch (NoConfigStringException ncse) {
	    return 25;
	}
    }

    public int unitsPerTorpRate(int player) {
	try {
	    return getIntArrayElement("UnitsPerTorpRate", player);
	} catch (NoConfigStringException ncse) {
	    return 1;
	}
    }

    public boolean useAccurateFuelModel() {
	try {
	    return getBoolean("UseAccurateFuelModel");
	} catch (NoConfigStringException ncse) {
	    return false;
	}
    }

    public boolean allowHiss() {
	try {
	    return getBoolean("AllowHiss");
	} catch (NoConfigStringException ncse) {
	    return true;
	}
    }

    public int hissEffectRate() {
	if (!allowHiss()) return 0;
	try {
	    return getInteger("HissEffectRate");
	} catch (NoConfigStringException ncse) {
	    return 5;
	}
    }

    public int maxShipsHissing() {
	try {
	    return getInteger("MaxShipsHissing");
	} catch (NoConfigStringException ncse) {
	    return 10;
	}
    }
}
