// File: PData.java

package jvpc.vga;

import kero.util.*;
import java.io.*;

/**
 * Implements a planet of your own (from <CODE>PDATA??.DAT</code>, hence
 * the ugly name).
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since     10 VII 1998
 * @version   14 III 2001
 */
public class PData
  extends Planet
  implements CargoTransferable, Build
{
  protected StringBuffer fc;
  int[] struct = new int[Build.PDATABUILDTYPES];
  protected int coltax,colhappy;
  protected int nattax,nathappy,natgov;
  protected int natclans;
  protected int natsort;
  protected int temp;
  protected int coreneutr,densneutr;
  protected int coretrit,denstrit;
  protected int coredur,densdur;
  protected int coremolyb,densmolyb;

  // who saw what; in order to save unowned planets in pdata??.dat
  private short seenby;

  public PData() {
  }

  /**
   * Constructs the default <CODE>PData</code> object. All fields are <CODE>
   * UNKNOWN</code>.
   */
  public PData(Universe u) {
    super(u);
    fc=null;
    for (int i=0; i<Build.PDATABUILDTYPES; i++) struct[i] = UNKNOWN;
    struct[Build.BASE] = 0;
    coltax=colhappy=UNKNOWN;
    nattax=nathappy=natgov=natclans=natsort=UNKNOWN;
    temp=UNKNOWN;
    cargo[Cargo.NEUTRONIUM].amount = UNKNOWN;
    coreneutr=densneutr=UNKNOWN;
    coretrit=denstrit=UNKNOWN;
    coredur=densdur=UNKNOWN;
    coremolyb=densmolyb=UNKNOWN;
  }

  /**
   * Constructs a <CODE>PData</code> object from a <CODE>Planet</code>.
   * All common fields are copied, the rest is taken from <CODE>this()</code>.
   */
  public PData(Planet planet) {
    this(null, planet);
  }

  /**
   * Constructs a <CODE>PData</code> object from a <CODE>Planet</code>.
   * All common fields are copied, the rest is taken from <CODE>this()</code>.
   */
  public PData(Universe u,Planet planet) {
    this(u);
    id = planet.id;
    owner = planet.owner;
    // id and owner might be filled in somewhere else (by PDataGen, probably)
    setX(planet.x);
    setY(planet.y);
    name = planet.name; // reference is perfect!
  }

  public Object clone() throws CloneNotSupportedException {
    PData result = (PData)super.clone();
    result.struct = new int[struct.length];
    for (int i=0; i<struct.length; i++) result.struct[i] = struct[i];
    return result;
  }

  /**
   * Read from a stream, assuming it has been positioned properly.
   */
  public void readFrom(LSBReader source,int player) throws IOException {
    // owner and id have been read by PDataGen
    fc=source.readString(FCSIZE);
    struct[Build.MINE]=source.readShort();
    struct[Build.FACTORY]=source.readShort();
    struct[Build.DEFENSE]=source.readShort();
    cargo[Cargo.NEUTRONIUM].amount = source.readInt();
    cargo[Cargo.TRITANIUM].amount = source.readInt();
    cargo[Cargo.DURANIUM].amount = source.readInt();
    cargo[Cargo.MOLYBDENIUM].amount = source.readInt();
    cargo[Cargo.COLCLANS].amount = source.readInt();
    cargo[Cargo.SUPPLIES].amount = source.readInt();
    cargo[Cargo.MC].amount = source.readInt();
    coreneutr=source.readInt();
    coretrit=source.readInt();
    coredur=source.readInt();
    coremolyb=source.readInt();
    densneutr=source.readShort();
    denstrit=source.readShort();
    densdur=source.readShort();
    densmolyb=source.readShort();
    coltax=source.readShort();
    nattax=source.readShort();
    colhappy=source.readShort();
    nathappy=source.readShort();
    natgov=source.readShort();
    natclans=source.readInt();
    natsort=source.readShort();
    temp=100-source.readShort(); // nice joke, Tim
    // prevent PData from (re)setting #bases to 0
    int buildbase = (source.readShort()>0) ? 1 : 0; // normalize to 1
    if ((struct[Build.BASE] == UNKNOWN) || (buildbase != 0)) {
      struct[Build.BASE] = buildbase;
    }
    setSeenBy(player);
  }

  public void setSeenBy(int player) {
    seenby |= 1<<player;
  }

  public boolean isSeenBy(int player) {
    return (seenby & (1<<player))!=0;
  }

  public void writeTo(LSBWriter sink) throws IOException {
    sink.writeShort((short)owner);
    sink.writeShort((short)id);
    sink.writeString(fc,FCSIZE);
    sink.writeShort((short)struct[Build.MINE]);
    sink.writeShort((short)struct[Build.FACTORY]);
    sink.writeShort((short)struct[Build.DEFENSE]);
    sink.writeInt(getNeutronium());
    sink.writeInt(getTritanium());
    sink.writeInt(getDuranium());
    sink.writeInt(getMolybdenium());
    sink.writeInt(getColClans());
    sink.writeInt(getSupplies());
    sink.writeInt(getMC());
    sink.writeInt(coreneutr);
    sink.writeInt(coretrit);
    sink.writeInt(coredur);
    sink.writeInt(coremolyb);
    sink.writeShort((short)densneutr);
    sink.writeShort((short)denstrit);
    sink.writeShort((short)densdur);
    sink.writeShort((short)densmolyb);
    sink.writeShort((short)coltax);
    sink.writeShort((short)nattax);
    sink.writeShort((short)colhappy);
    sink.writeShort((short)nathappy);
    sink.writeShort((short)natgov);
    sink.writeInt(natclans);
    sink.writeShort((short)natsort);
    sink.writeShort((short)(100-temp)); // nice joke, Tim
    sink.writeShort((short)(buildingBase() ? 1 : 0));
  }

  public PData getRstPData() {
    if (isRst()) return null;
    Game game = universe.getGame();
    Universe rstuniv = game.getUniverse(getTurnNumber(), RstOrTrn.RST);
    return (PData)rstuniv.getPlanet(getId());
  }

  public boolean isOurPlanet() {
    return true;
  }
  
  // SurfaceMineral interface stuff
  public int getSurfNeutronium() {
    return getNeutronium();
  }
  public int getSurfTritanium() {
    return getTritanium();
  }
  public int getSurfDuranium() {
    return getDuranium();
  }
  public int getSurfMolybdenium() {
    return getMolybdenium();
  }

  // CoreMineral interface stuff
  public int getCoreNeutronium() {
    return coreneutr;
  }
  public int getCoreTritanium() {
    return coretrit;
  }
  public int getCoreDuranium() {
    return coredur;
  }
  public int getCoreMolybdenium() {
    return coremolyb;
  }

  public int getNeutroniumDensity() {
    return densneutr;
  }
  public int getTritaniumDensity() {
    return denstrit;
  }
  public int getDuraniumDensity() {
    return densdur;
  }
  public int getMolybdeniumDensity() {
    return densmolyb;
  }

  // TotalMineral interface stuff
  public int getTotalNeutronium() {
    return coreneutr+getNeutronium();
  }
  public int getTotalTritanium() {
    return coretrit+getTritanium();
  }
  public int getTotalDuranium() {
    return coredur+getDuranium();
  }
  public int getTotalMolybdenium() {
    return coremolyb+getMolybdenium();
  }

  public int getMines() {
    return struct[Build.MINE];
  }

  public int getFactories() {
    return struct[Build.FACTORY];
  }
  
  public int getDefense() {
    return struct[Build.DEFENSE];
  }


  // some very generic build/break methods

  public int minStructAmount(int buildtype) {
    return getRstPData().getStructAmount(buildtype);
  }

  public int maxStructAmount(int buildtype) {
    if (Build.maxamount[buildtype] > 0) return Build.maxamount[buildtype];
    switch (buildtype) {
    case Build.MINE:
      return getMaxMines();
    case Build.FACTORY:
      return getMaxFactories();
    case Build.DEFENSE:
      return getMaxDefense();
    default:
      return UNKNOWN;
    }
  }

  public int getStructAmount(int buildtype) {
    return struct[buildtype];
  }

  void setStructAmount(int buildtype, int amount) {
      struct[buildtype] = amount;
  }

  public int needTrit(int buildtype) {
    if (buildtype < 0) crash(1000, "needTrit("+buildtype+") buildtype <0");
    if (buildtype >= BASICBDATABUILDTYPES)
      crash(1000, "PData.needTrit() BData");
    return Build.needtrit[buildtype];
  }

  public int needDur(int buildtype) {
    if (buildtype < 0) crash(1000, "needDur("+buildtype+") buildtype <0");
    if (buildtype >= BASICBDATABUILDTYPES)
      crash(1000, "PData.needDur() BData");
    return Build.needdur[buildtype];
  }

  public int needMolyb(int buildtype) {
    if (buildtype < 0) crash(1000, "needMolyb("+buildtype+") buildtype <0");
    if (buildtype >= BASICBDATABUILDTYPES)
      crash(1000, "PData.needMolyb() BData");
    return Build.needmolyb[buildtype];
  }

  public int needSupp(int buildtype) {
    if (buildtype < 0) crash(1000, "needSupp("+buildtype+") buildtype <0");
    if (buildtype >= BASICBDATABUILDTYPES)
      crash(1000, "PData.needSupp() BData");
    return Build.needsupp[buildtype];
  }

  public int needMC(int buildtype) {
    if (buildtype < 0) crash(1000, "needMC("+buildtype+") buildtype <0");
    if (buildtype >= BASICBDATABUILDTYPES) crash(1000, "PData.needMC() BData");
    return Build.needmc[buildtype];
  }

  public void buildUp(int type, int tobebuilt) {
    if (type >= PDATABUILDTYPES) Assert.assert(this instanceof BData);
    if (tobebuilt < 0) {
      breakDown(type, -tobebuilt);
      return;
    }
    if (isEditable()) {
	if (maxStructAmount(type) != INFINITY) {
	    if (getStructAmount(type) + tobebuilt > maxStructAmount(type)) {
		tobebuilt = maxStructAmount(type) - getStructAmount(type);
	    }
	}
	// current can be above max, yet equal to min; do not break down structs
	// e.g. #fact on planets w/ Amorphus
	if (getStructAmount(type) + tobebuilt < minStructAmount(type)) {
	    tobebuilt = minStructAmount(type) - getStructAmount(type);
	}
	if (maxStructAmount(type) != INFINITY) {
	    if (getStructAmount(type) + tobebuilt > maxStructAmount(type)) {
		tobebuilt = 0;
	    }
	}
	int availtrit = getTritanium();
	if (tobebuilt*needTrit(type) > availtrit)
	    tobebuilt = availtrit/needTrit(type);
	int availdur = getDuranium();
	if (tobebuilt*needDur(type) > availdur)
	    tobebuilt = availdur/needDur(type);
	int availmolyb = getMolybdenium();
	if (tobebuilt*needMolyb(type) > availmolyb)
	    tobebuilt = availmolyb/needMolyb(type);
	int availsupp = getSupplies();
	if (tobebuilt*needSupp(type) > availsupp)
	    buySupplies(tobebuilt*needSupp(type) - availsupp);
	availsupp = getSupplies();
	if (tobebuilt*needSupp(type) > availsupp)
	    tobebuilt = availsupp/needSupp(type);
	int availmc = getSupplies() + getMC();
	if (tobebuilt * (needSupp(type)+needMC(type)) > availmc)
	    tobebuilt = availmc / (needSupp(type)+needMC(type));
	if (tobebuilt != 0) {
	    if (tobebuilt*needMC(type) > getMC()) {
		// we know the suppplies are actually available
		int_sellSupplies(tobebuilt*needMC(type) - getMC());
	    }
	    cargo[Cargo.TRITANIUM].amount = getTritanium() - tobebuilt*needTrit(type);
	    cargo[Cargo.DURANIUM].amount = getDuranium() - tobebuilt*needDur(type);
	    cargo[Cargo.MOLYBDENIUM].amount = getMolybdenium() - tobebuilt*needMolyb(type);
	    cargo[Cargo.SUPPLIES].amount = getSupplies() - tobebuilt*needSupp(type);
	    cargo[Cargo.MC].amount = getMC() - tobebuilt*needMC(type);
	    setStructAmount(type, getStructAmount(type) + tobebuilt);
	    universe.throwPlanetEvent(id);
	}
	getGame().unlock();
    }
  }

  public void breakDown(int type, int tobebroken) {
    if (tobebroken < 0) {
      buildUp(type, -tobebroken);
      return;
    }
    if (getStructAmount(type) - tobebroken < minStructAmount(type)) {
      tobebroken = getStructAmount(type) - minStructAmount(type);
    }
    // no extra check whether we're above max. Min can not be above current.
    if (isEditable()) {
      cargo[Cargo.TRITANIUM].amount = getTritanium() + tobebroken*needTrit(type);
      cargo[Cargo.DURANIUM].amount = getDuranium() + tobebroken*needDur(type);
      cargo[Cargo.MOLYBDENIUM].amount = getMolybdenium() + tobebroken*needMolyb(type);
      cargo[Cargo.SUPPLIES].amount = getSupplies() + tobebroken*needSupp(type);
      cargo[Cargo.MC].amount = getMC() + tobebroken*needMC(type);
      setStructAmount(type, getStructAmount(type) - tobebroken);
      universe.throwPlanetEvent(id);
      getGame().unlock();
    }
  }

  public void buildMines(int amount) {
    buildUp(Build.MINE, amount);
  }

  public void breakDownMines(int amount) {
    breakDown(Build.MINE, amount);
  }

  public void buildFactories(int amount) {
    buildUp(Build.FACTORY, amount);
  }

  public void breakDownFactories(int amount) {
    breakDown(Build.FACTORY, amount);
  }

  public void buildDefense(int amount) {
    buildUp(Build.DEFENSE, amount);
  }

  public void breakDownDefense(int amount) {
    breakDown(Build.DEFENSE, amount);
  }

  public String getFC() {
    if (fc == null) return null;
    return fc.toString();
  }

  public void setFC(String newfc) {
    setFC(new StringBuffer(newfc));
  }

  public void setFC(StringBuffer newfc) {
    if (newfc.length() != FCSIZE) newfc.setLength(3);
    if ((fc != null) && fc.equals(newfc)) return;
    if (isEditable()) {
	fc = newfc;
	universe.throwPlanetEvent(id);
	getGame().unlock();
    }
  }

  protected void setCargo(int cargotype, int newamount) {
      System.err.println("Planet #"+id+", cargo "+cargotype+" from "+cargo[cargotype].amount+" to "+newamount);
    if (cargo[cargotype].amount == newamount) return;
    cargo[cargotype].amount = newamount;
  }

  void setSurfNeutronium(int amount) {
    setNeutronium(amount);
  }

  void setSurfTritanium(int amount) {
    setTritanium(amount);
  }

  void setSurfDuranium(int amount) {
    setDuranium(amount);
  }

  void setSurfMolybdenium(int amount) {
    setMolybdenium(amount);
  }

  /**
   * convert supplies to MC or vice versa, until there are the specified amount
   * of supplies.
   */

    protected void int_sellSupplies(int amount) {
	cargo[Cargo.MC].amount += amount; // prevent event...
	setSupplies(getSupplies() - amount); // ...throw here
    }

    public void sellSupplies(int amount) {
	if (amount < 0) {
	    buySupplies(-amount);
	    return;
	}
	if (isEditable()) {
	    if (amount > getSupplies()) amount = getSupplies();
	    int_sellSupplies(amount);
	    universe.throwPlanetEvent(id);
	    getGame().unlock();
	}
    }

  public void buySupplies(int amount) {
    if (amount < 0) {
      sellSupplies(-amount);
      return;
    }
    if (isEditable()) {
	Location trnloc = getLocation();
	PData rstplanet = getRstPData();
	Location rstloc = trnloc.getRstLocation();
	// use superficial knowledge about supplies only being needed for MFD:
	int maxsupplies = rstloc.getSupplies() +
	    rstplanet.getMines() - getMines() +
	    rstplanet.getFactories() - getFactories() +
	    rstplanet.getDefense() - getDefense();
	if (amount + trnloc.getSupplies() > maxsupplies)
	    amount = maxsupplies - trnloc.getSupplies();
	if (amount > getMC()) amount = getMC();
	cargo[Cargo.MC].amount -= amount;
	setSupplies(getSupplies() + amount);
	universe.throwPlanetEvent(id);
	getGame().unlock();
    }
  }

  public boolean buildingBase() {
    if (hasBase()) return false;
    return (struct[Build.BASE] > 0);
  }

  // irregularity, because for a base, it is interesting to know whether we
  // can build _one_. In fact this function should point to a general
  // canBuild()
  public boolean canBuildBase() {
    if (hasBase()) return false;
    if (getTritanium() < 402) return false;
    if (getDuranium() < 120) return false;
    if (getMolybdenium() < 340) return false;
    if (getSupplies()+getMC() < 900) return false;
    return true;
  }

  public void buildBase() {
    buildUp(Build.BASE, 1);
  }

  public void breakDownBase() {
    breakDown(Build.BASE, 1);
  }

  public int getColTax() {
    return coltax;
  }

  public void setColTax(int newcoltax) {
    if (newcoltax<0) newcoltax = 0;
    if (newcoltax>100) newcoltax = 100;
    if (isEditable()) {
	if (this.coltax != newcoltax) {
	    coltax = newcoltax;
	    universe.throwPlanetEvent(id);
	}
	getGame().unlock();
    }
  }

    public int getColTaxMC() {
	if (colhappy < 30) return 0;
	int taxMC = (getColClans() * coltax * 5 + 2500) / 5000;
	int taxRate = getConfig().colonistTaxRate(owner);
	return (taxMC * taxRate + 50) / 100;
    }

    public int getNatTax() {
	return nattax;
    }

  public void setNatTax(int newnattax) {
    if (newnattax<0) newnattax = 0;
    if (newnattax>100) newnattax = 100;
    if (isEditable()) {
	if (this.nattax != newnattax) {
	    this.nattax = newnattax;
	    universe.throwPlanetEvent(id);
	}
	getGame().unlock();
    }
  }

    /**
     * Return tax collected from natives in MCs
     */
    public int getNatTaxMC() {
	int taxMC = 0;
	if (nathappy < 30) return 0;
	taxMC = (natclans * natgov * nattax + 2500) / 5000;
	if (taxMC > getColClans()) taxMC = getColClans();
	if (natsort == Natives.INSECTOID) taxMC *= 2;
	int taxRate = getConfig().nativeTaxRate(getOwner());
	taxMC = (taxMC * taxRate + 50) / 100;
	return taxMC;
    }

  /**
   * Return tax collected from natives in MCs disregarding number of colonists
   */
  public int getNatTaxMCwoCol() {
    int res = 0;
    if (nathappy < 30)
      return 0;
    res = (int)((natclans * natgov * nattax + 2500) / 5000);
    if (natsort == Natives.INSECTOID)
      res *= 2;
    res = (int)((res * getConfig().nativeTaxRate(getOwner()) + 50) / 100);
    return res;
  }

  /**
   * Return tax collected from natives in MCs disregarding number of colonists,
   * ignoring the owner and assuming 100% nativeTaxRate().
   */
  public int getNatTaxMCwoColMax() {
    int res = 0;
    if (nathappy < 30) return 0;
    res = (int)((getNatClans() * getNatGov() * getNatTax() + 2500) / 5000);
    if (natsort == Natives.INSECTOID) res *= 2;
    res = (int)((res * 100 + 50) / 100);
    return res;
  }

  public int getNatGov() {
    return natgov;
  }

  public int getNatSort() {
    return natsort;
  }

  public int getNatClans() {
    return natclans;
  }

  /**
   * Return the temperature (0-99) of the planet
   */
  public int getTemp() {
    return temp;
  }

  public void setTemp(int newTemp) {
    temp = newTemp;
  }

    public int getColHappy() {
	return colhappy;
    }

    /**
     * Return the happiness of the colonists next turn
     */
    public int getColHappyNT() {
	int targettemp = 50;
	int tempdivisor = 33;
	if (getRaceOwner() == Race.CRYSTALS) {
	    if (getConfig().crystalsPreferDeserts()) {
		targettemp = 100;
		tempdivisor = 66;
	    }
	}
	int deltaColHappy = Mymath.trunc(
	    10.0 - Math.sqrt(getColClans() / 10000.0)
	     - (double)Math.abs(temp - targettemp) / tempdivisor
	     - (double)(getMines() + getFactories()) / 300
	     - ((double)coltax * 0.8)
	    ) + getLocation().shipsHissing() * getConfig().hissEffectRate();
	return Mymath.bound(colhappy + deltaColHappy, 0, 100);
    }

    public int getNatHappy() {
	return nathappy;
    }

    public int getNatHappyNT() {
	/*System.err.println("Original: " + (
	    (double)natgov/2.0 - Math.sqrt((double)natclans/10000) - 
	    (double)(getMines() + getFactories())/200.0 - (0.85 * nattax)
	    ));
	System.err.println("Math.ceil: " + Math.ceil(
	    (double)natgov/2.0 - Math.sqrt((double)natclans/10000) - 
	    (double)(getMines() + getFactories())/200.0 - (0.85 * nattax)
	    ));
	System.err.println("My-math.trunc: " + Mymath.trunc(
	    (double)natgov/2.0 - Math.sqrt((double)natclans/10000) - 
	    (double)(getMines() + getFactories())/200.0 - (0.85 * nattax)
	    )); */
	int deltaNatHappy = Mymath.trunc(5.0 +
	    (double)natgov/2.0 - Math.sqrt((double)natclans/10000) -
	        (double)(getMines() + getFactories())/200.0 - (0.85 * nattax)
	    ) +
	    (natsort == Natives.AVIAN ? 10 : 0) +
	    getLocation().shipsHissing() * getConfig().hissEffectRate();
	return Mymath.bound(nathappy + deltaNatHappy, -300, 100);
    }

    public static void main(String arg[]) {
	System.err.println("5/2 = "+(5/2));
	System.err.println("(int)3.8 = "+(int)3.8);
	System.err.println("(int)-3.8 = "+(int)-3.8);
    }
    
  int baseMaxCol() {
    PConfig config = getConfig();
    int basemaxcol = UNKNOWN;
    int race = getRaceOwner();
    if ((race == Race.CRYSTALS) &&
	(config.crystalsPreferDeserts())) {
      if (config.crystalSinTempBehavior()) {
	if (temp >= 15) {
	  // Crystals, Sin Temp, warm
	  return (int)Math.max(10000000 * Math.sin(temp * Math.PI / 200),
			       100);
	};
	// Crystals, Sin Temp, cold
	return 300 + config.maxColTempSlope() * temp;
      } else {
	// Crystals, no Sin Temp Behaviour.
	basemaxcol = Math.max(100000 * temp, 100);
      }
    }
    if ((temp < 20) && (race == Race.REBELS)) {
      return 9000000;
    }
    if (temp < 15) {
      return 300 + config.maxColTempSlope() * temp;
    }
    if (temp > 84) {
      basemaxcol = 100 + (100 - temp) * config.maxColTempSlope();
      if ((race == Race.KLINGONS) ||
	  (race == Race.CYLONS) ||  // only for THost?
	  (race == Race.REBELS) ||
	  (race == Race.COLONIES)) {
	if (basemaxcol < 6000) {
	  basemaxcol = 6000;
	}
      }
      return basemaxcol;
    }
    return (int) (Math.sin(temp * Math.PI / 100) * 10000000);
  }

  public int getMaxClans() {
    PConfig config = getConfig();
    return ((config.climateLimitsPopulation()) ?
	    baseMaxCol() : 25000000) / 100;
  }
}
