// File: PDataGen.java

package jvpc.vga;

import java.util.Hashtable;
import java.io.*;

/**
 * Generates <CODE>PData</code>objects.
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since     25 VII 1998
 * @version   21 XII 1999
 */
public class PDataGen extends Generator {
  final int planetcount;
  /**
   * Constructs a PData generator.
   * @param is The <CODE>InputStream</code> from which the <CODE>source</code>
   * can be created.
   */
  public PDataGen(InputStream is) throws IOException {
    super(is);
    planetcount=source.readShort();
  }

  public PDataGen(Generator g) throws IOException {
    super(g);
    planetcount=source.readShort();
  }

  /**
   * Generates all <CODE>PData</code> objects found in <CODE>pis</code>.
   * Puts them at the proper place in <CODE>planet[]</code>
   */
  public void generateAll(Universe u,int player) {
    int i,owner,id;
    Planet planet;
    PData pdata;
    for (i=1;i<=planetcount;i++) {
      try {
	owner=source.readShort();
	id=source.readShort();
	planet=u.getPlanet(id);
	if (planet!=null) {
	  if (planet instanceof PData) pdata=(PData)planet;
	  else pdata=new PData(u,planet);
	} else {
	  pdata=new PData(u);
	}
	pdata.owner=owner; // ugly...
	pdata.id=id;
	pdata.readFrom(source,player);
	u.removePlanet(id); // removes it when available
	u.putPlanet(pdata);
      } catch (IOException ioe) {
	crash(11, "Error while reading pdata: "+ioe);
      }
    }
  }

  public int getPlanetCount() {
    return planetcount;
  }
}
