// File: Planet.java

package jvpc.vga;

import java.util.*;
import java.io.*;
import kero.util.*;

/**
 * Provides plenty of interfaces for all kind of planets. Most return <CODE>
 * UNKNOWN</code> due to lack in information. These interfaces are implemented
 * here, so that the utility can always invoke them on a planet-like object in
 * space, regardless of its exact species. Subclasses should override as many
 * of these interface functions as they can.
 *
 * <P>This class should <EM>not</em> be used as CargoTransfer, but the class
 * hierarchy grew like this. It is easier; be warned.
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since     10 VII 1998
 * @version   24 III 2001
 */
public class Planet
    extends CargoTransfer
    implements SurfaceMineral, CoreMineral, TotalMineral
{
    static int realPlanets = 0;
    static int minDim = 500;
    static int maxDim = 3500;
    
  protected StringBuffer name;

  // Interface Tabulatable
  public Vector getTypes() {
    Vector res = super.getTypes();
    res.addElement("mc");
    res.addElement("s");
    res.addElement("n");
    res.addElement("t");
    res.addElement("d");
    res.addElement("m");
    res.addElement("name");
    return res;
  }

  public Vector getTypeMethods(String typeName) {
    Vector res = super.getTypeMethods(typeName);
    if ("n".equals(typeName)) {
      res.addElement("getSurfNeutronium");
      res.addElement("getCoreNeutronium");
      res.addElement("getTotalNeutronium");
    } else if ("t".equals(typeName)) {
      res.addElement("getSurfTritanium");
      res.addElement("getCoreTritanium");
      res.addElement("getTotalTritanium");
    } else if ("d".equals(typeName)) {
      res.addElement("getSurfDuranium");
      res.addElement("getCoreDuranium");
      res.addElement("getTotalDuranium");
    } else if ("m".equals(typeName)) {
      res.addElement("getSurfMolybdenium");
      res.addElement("getCoreMolybdenium");
      res.addElement("getTotalMolybdenium");
    } else if ("s".equals(typeName)) {
      res.addElement("getSupplies");
    } else if ("mc".equals(typeName)) {
      res.addElement("getMC");
    } else if ("name".equals(typeName)) {
      res.addElement("getName");
    }
    return res;
  }

  public Planet() {
    this(null);
  }

  public Planet(Universe u) {
    super(u);
    name=null;
    cargo[Cargo.NEUTRONIUM].amount = UNKNOWN;
  }

    //public Object clone() throws CloneNotSupportedException {
    //return super.clone();
    //}

  /**
   * Reads the X and Y coordinate.
   * @param source Source from which the coordinates are read.
   * @exception IOException Just passed on from source, 
   * @see LSBReader
   */
  public void readXYFrom(LSBReader source) throws IOException {
      // support explorer
      int newX = source.readShort();
      int newY = source.readShort();
      if ((newX<minDim) || (newX>maxDim) || (newY<minDim) || (newY>maxDim)) return;

      setX(newX);
      setY(newY);
      realPlanets++;
  }

  public void readNameFrom(LSBReader source) throws IOException {
    if (name==null) {
      name=source.readString(PLANETNMSIZE);
    } else {
      StringBuffer newname=source.readString(PLANETNMSIZE);
      if (!newname.equals(name)) {
	crash(1002, "Two different names for one planet");
      }
    }
  }

    public static int realPlanets() {
	return realPlanets;
    }

    public void throwEvent() {
	universe.throwPlanetEvent(id);
    }

  public String getFC() { return null; }

  public int getCoreNeutronium() { return UNKNOWN; }
  public int getCoreTritanium() { return UNKNOWN; }
  public int getCoreDuranium() { return UNKNOWN; }
  public int getCoreMolybdenium() { return UNKNOWN; }

  public int getNeutroniumDensity() { return UNKNOWN; }
  public int getTritaniumDensity() { return UNKNOWN; }
  public int getDuraniumDensity() { return UNKNOWN; }
  public int getMolybdeniumDensity() { return UNKNOWN; }

  public int getTotalNeutronium() { return UNKNOWN; }
  public int getTotalTritanium() { return UNKNOWN; }
  public int getTotalDuranium() { return UNKNOWN; }
  public int getTotalMolybdenium() { return UNKNOWN; }

  public int getSurfNeutronium() { return getNeutronium(); }
  public int getSurfTritanium() { return getTritanium(); }
  public int getSurfDuranium() { return getDuranium(); }
  public int getSurfMolybdenium() { return getMolybdenium(); }

  public final String getName() {
    if (name == null) {
      report("No name for planet "+getId()+" "+this);
    //report((this instanceof PData?"pdata":"planet") + (hasBase()?"+b":"-b"));
    //report((this instanceof BData?"bdata":"------"));
      Assert.assert(false);
    }
    return name.toString();
  }

  public boolean isOurPlanet() { return false; }

  public int getTemp() { return UNKNOWN; }
  
  public int getMines() { return UNKNOWN; }
  public int getFactories() { return UNKNOWN; }
  public int getDefense() { return UNKNOWN; }

  public int getMaxMines() {
    int nocol = getColClans();
    if (nocol == UNKNOWN) return UNKNOWN;
    if (nocol>200) {
      return 200 + (int)(0.5 + Math.sqrt(nocol-200));
    } else {
      return nocol;
    }
  }

  public int getMaxFactories() {
    int nocol = getColClans();
    if (nocol == UNKNOWN) return UNKNOWN;
    if (nocol>100) {
      return 100 + (int)(0.5 + Math.sqrt(nocol-100));
    } else {
      return nocol;
    }
  }

  public int getMaxDefense() {
    int nocol = getColClans();
    if (nocol == UNKNOWN) return UNKNOWN;
    if (nocol>50) {
      return 50 + (int)(0.5 + Math.sqrt(nocol-50));
    } else {
      return nocol;
    }
  }

  public int getColHappy() { return UNKNOWN; }
  public int getColHappyNT() { return UNKNOWN; }
  public int getColTax() { return UNKNOWN; }
  public int getColTaxMC() { return UNKNOWN; }

  public int getNatHappy() { return UNKNOWN; }
  public int getNatHappyNT() { return UNKNOWN; }
  public int getNatTax() { return UNKNOWN; }
  // public boolean setNatTax(int tax) { return false; } // needed ???
  public int getNatTaxMC() { return UNKNOWN; }
  public int getNatTaxMCwoCol() { return UNKNOWN; }

  /**
   * Return the type of government of the natives.
   * See class Government.
   */
  public int getNatGov() { return UNKNOWN; }

  /**
   * Return the type of the natives.
   * See class Natives.
   */
  public int getNatSort() { return UNKNOWN; }

  public int getNatClans() { return UNKNOWN; }

  /**
   * Return the maximum number of colonist clans possible
   * on this planet based on the temperature and race.
   */
  public int getMaxClans() { return UNKNOWN; }

  public boolean buildingBase() { return false; }
  public boolean hasBase() { return (getBase()!=null); }
  
  public BData getBase() {
    return universe.getBData(id);
  }


  // CargoTransfer methods

  protected void setCargo(int cargotype, int newamount) {
    Assert.assert(false);
  }

  // how the hell will you override this properly for BData?
  public int getCargo(int cargotype) {
    if (cargotype == Cargo.AMMUNITION) return 0;
    return super.getCargo(cargotype);
  }

  public int maxCargo(int cargotype) {
    return INFINITY;
  }


    // reporting methods

    public String toString() {
	return "Planet "+toString("#");
    }

    /**
     * Produce a big one-line report of this ship
     */
    public String getReport() {
	return "Planet "+toString("#obecntdms$Lia");
    }

    public String toString(String planetString) {
	return toPlanetString(planetString);
    }

    /**
     * Returns user configurable string about the planet.
     */
    public String toPlanetString(String userline) {
	StringBuffer result = new StringBuffer();
	result.ensureCapacity(256); // user should not want longer strings
	for (int i=0; i<userline.length(); i++) {
	    result.append(userline.charAt(i));
	    switch (userline.charAt(i)) {
	    case '@':
		result.append("("+getX()+","+getY()+")");
		break;
	    case 'o':
		if (getOwner()==UNKNOWN) result.append('?');
		else result.append(getOwner());
		break;
	    case '#':
		result.append(getId());
		break;
	    case 'a': // name
		result.setLength(result.length()-1);
		result.append('"');
		result.append(getName());
		result.append('"');
		break;
	    case 'e': // temperature
		result.setLength(result.length()-1);
		result.append("temp");
		if (getTemp()==UNKNOWN) result.append('?');
		else result.append(getTemp());
		break;
	    case 'c':
	    case 'C':
		result.setLength(result.length()-1);
		result.append("`"+getFC()+"'");
		break;
	    case 'f':
		result.append(getFactories());
		break;
	    case 'n':
		if (getSurfNeutronium()==UNKNOWN) result.append('?');
		else result.append(getSurfNeutronium());
		break;
	    case 't':
		if (getSurfTritanium()==UNKNOWN) result.append('?');
		else result.append(getSurfTritanium());
		break;
	    case 'd':
		if (getSurfDuranium()==UNKNOWN) result.append('?');
		else result.append(getSurfDuranium());
		break;
	    case 'm':
		if (getSurfMolybdenium()==UNKNOWN) result.append('?');
		else result.append(getSurfMolybdenium());
		break;
	    case 'N':
		if (getCoreNeutronium()==UNKNOWN) result.append('?');
		else {
		    result.append(getCoreNeutronium());
		    result.append(',');
		    result.append(getNeutroniumDensity());
		    result.append('%');
		}
		break;
	    case 'T':
		if (getCoreTritanium()==UNKNOWN) result.append('?');
		else {
		    result.append(getCoreTritanium());
		    result.append(',');
		    result.append(getTritaniumDensity());
		    result.append('%');
		}
		break;
	    case 'D':
		if (getCoreDuranium()==UNKNOWN) result.append('?');
		else {
		    result.append(getCoreDuranium());
		    result.append(',');
		    result.append(getDuraniumDensity());
		    result.append('%');
		}
		break;
	    case 'M':
		if (getSurfMolybdenium()==UNKNOWN) result.append('?');
		else {
		    result.append(getCoreMolybdenium());
		    result.append(',');
		    result.append(getMolybdeniumDensity());
		    result.append('%');
		}
		break;
	    case 's':
	    case 'S':
		if (getSupplies()==UNKNOWN) result.append('?');
		else result.append(getSupplies());
		break;
	    case '$':
		if (getMC()==UNKNOWN) result.append('?');
		else result.append(getMC());
		break;
	    case 'L':
		result.setLength(result.length()-1);
		if (getColClans()==UNKNOWN) result.append('?');
		else if (getColClans()==0) result.append("noCol");
		else result.append(getColClans()+"col");
		break;
	    case 'i': // natives
		result.setLength(result.length()-1);
		if (getNatClans()==UNKNOWN) result.append('?');
		else if (getNatClans()==0) result.append("noNativ");
		else {
		    result.append(getNatClans());
		    result.append(" "+Government.getShortName(getNatGov()));
		    result.append(" "+Natives.getShortName(getNatSort()));
		}
		break;
	    case 'b':
	    case 'B':
		result.setLength(result.length()-1);
		if (buildingBase()) result.append('b');
		if (hasBase()) result.append('B');
		break;
	    default:
		result.append("<?>");
		break;
	    }
	    result.append(' ');
	}
	return result.toString();
    }
}
