// File: PlanetNMGen.java

package jvpc.vga;

import java.util.Hashtable;
import java.io.*;

/**
 * Adds planet names to the planets that are known thus far. Therefore, when
 * reading data for a <CODE>Universe</code>, it is recommended to generate the
 * planet names last.
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since     26 VII 1998
 * @version   21 XII 1999
 */
public class PlanetNMGen extends Generator {
  /**
   * Constructs a Planet generator.
   * @param is The <CODE>InputStream</code> from which the <CODE>source</code>
   * can be created.
   */
  public PlanetNMGen(InputStream is) {
    super(is);
  }

  /**
   * Generates 500 <CODE>Planet</code> objects with only their name.
   */
  public void generateAll(Universe u) {
    int i;
    Planet planet;
    for (i=1;i<=planets;i++) {
      try {
	if (u.containsPlanet(i)) {
	  planet=u.getPlanet(i);
	  planet.readNameFrom(source);
	} else {
	  /* If this was done after the startmap (xyplan(??).dat) was read,
	   * then it is nice NOT to insert a new planet, because it is not
	   * ``known'' through explorer (hence it ``does not exist'').
	   */
	  // How to check whether starmap _was_ read before?
	  source.skip(PLANETNMSIZE);
	}
      } catch (IOException ioe) {
	crash(11, "Error while generating planet names: "+ioe);
      }
    }
  }
}
