// File: Ship.java

package jvpc.vga;

import java.util.*;
import java.io.*;
import java.awt.*;
import kero.util.*;

/**
 * Contains all data of a ship of your own.
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since     20 IX 1998
 * @version   14 III 2001
 */
public class Ship
  extends CargoTransfer
  implements CargoTransferable
{

  // Interface Tabulatable
  public Vector getTypes() {
    Vector res = super.getTypes();
    res.addElement("mc");
    res.addElement("s");
    res.addElement("n");
    res.addElement("t");
    res.addElement("d");
    res.addElement("m");
    res.addElement("mass");
    res.addElement("name");
    return res;
  }

  public Vector getTypeMethods(String typeName) {
    Vector res = super.getTypeMethods(typeName);
    if ("n".equals(typeName)) {
      res.addElement("getTotalNeutronium");
    } else if ("t".equals(typeName)) {
      res.addElement("getTotalTritanium");
    } else if ("d".equals(typeName)) {
      res.addElement("getTotalDuranium");
    } else if ("m".equals(typeName)) {
      res.addElement("getTotalMolybdenium");
    } else if ("s".equals(typeName)) {
      res.addElement("getSupplies");
    } else if ("mc".equals(typeName)) {
      res.addElement("getMC");
    } else if ("mass".equals(typeName)) {
      res.addElement("getMass");
      res.addElement("getEmptyMass");
      res.addElement("getCargoMass");
    } else if ("name".equals(typeName)) {
      res.addElement("getName");
    }
    return res;
  }


  // ``ship'' fields
  boolean isship=false; // is this a ship from ship.dis/dat?
                        // if so, then we know all data-fields
  StringBuffer name=null;
  StringBuffer fc=null;			// Friendly code
  int warp = UNKNOWN;
  int dx=UNKNOWN,dy=UNKNOWN;		// Waypoint (dx,dy)
  int hulltype=UNKNOWN;
  int engtype=UNKNOWN;			// Engine type; see class Engine
  int beamtype=UNKNOWN;
  int beams=UNKNOWN;			// # of beams
  int bays=UNKNOWN;			// # of fighter bays
  int torptype=UNKNOWN;		// 1..10
  //int torps=UNKNOWN;		// # of torpedoes OR fighters, now cargo
  int launchs=UNKNOWN;			// # of torpedo launchers
  int mission=UNKNOWN;			// Mission; see class Mission
  int enemy=UNKNOWN;			// Primary enemy player #
  int inttarget=UNKNOWN;	// For other uses, see MISSION.INI
  int towtarget=UNKNOWN;	// For other uses, see MISSION.INI
  int damage=UNKNOWN;			// Damage: 0-100%
  int crew=UNKNOWN;
  // cargo from CargoTransfer
  ToPlanet toplanet;
  ToShip toship;
  // ``target'' fields
  int heading=UNKNOWN; // 0: straight north, clockwise up to 359 degrees
                       // -1: unknown heading
  int weight=UNKNOWN;  // mass+neutr+cargo
  private short seenby; // 11 bits telling who sees this target

  public Ship() {}

//  public Ship(Ship ship) {
//  }

//  public Ship(Universe u,Ship ship) {
//    super(u);
//    // copy all fields from ship?
//  }

  public Ship(Universe u,int newid) {
    super(u,newid); // including Dated & RstOrTrn fields
    toplanet = new ToPlanet(this);
    toship = new ToShip(this);
  }

  public Object clone() throws CloneNotSupportedException {
    Ship copy = (Ship)(super.clone());
    copy.toplanet = (ToPlanet)toplanet.clone();
    copy.toship = (ToShip)toship.clone();
    return copy;
  }

  public void copyFromBattle(Vcr.Combatant combatant) {
    crew=combatant.crew;
    beamtype=combatant.beamtype;
    beams=combatant.beams;
    torptype=combatant.torptype;
    launchs=combatant.launchs;
    bays=combatant.bays;
    damage=combatant.damageafter;
    cargo[Cargo.AMMUNITION].amount = combatant.torpsafter;
    // or, if it is fighters, put #fighters in torps field
    if (combatant.fightersafter!=0) {
      cargo[Cargo.AMMUNITION].amount = combatant.fightersafter;
    }
  }

  public void readShipFrom(LSBReader source) throws IOException {
    // id has been read by ShipGen
    owner = source.readShort();
    fc = source.readString(FCSIZE);
    warp = source.readShort();
    dx = source.readShort();
    dy = source.readShort();
    setX(source.readShort());
    setY(source.readShort());
    engtype = source.readShort();
    hulltype = source.readShort();
    beamtype = source.readShort();
    beams = source.readShort();
    bays = source.readShort();
    torptype = source.readShort();
    cargo[Cargo.AMMUNITION].amount = source.readShort();
    launchs = source.readShort();
    mission = source.readShort();
    enemy = source.readShort();
    towtarget = source.readShort();
    damage = source.readShort();
    crew = source.readShort();
    cargo[Cargo.COLCLANS].amount = source.readShort();
    name = source.readString(SHIPNMSIZE);
    cargo[Cargo.NEUTRONIUM].amount = source.readShort();
    cargo[Cargo.TRITANIUM].amount = source.readShort();
    cargo[Cargo.DURANIUM].amount = source.readShort();
    cargo[Cargo.MOLYBDENIUM].amount = source.readShort();
    cargo[Cargo.SUPPLIES].amount = source.readShort();
    toplanet.readFrom(source);
    toship.readFrom(source);
    inttarget = source.readShort();
    cargo[Cargo.MC].amount = source.readShort();
    // is ``real'' ship
    isship = true;
    setSeenBy(owner);
  }

  /*
   * There are max 50 targets in Tims files, but more in PHost results.
   * vputil stores some of them in target.ext, which planets does not honour.
   */
  public void readTargetFrom(LSBReader source,int player) throws IOException {
    // id has been read by TargetGen
    owner=source.readShort();
    warp=source.readShort();
    setX(source.readShort());
    setY(source.readShort());
    hulltype=source.readShort();
    heading=source.readShort();
    name=source.readString(SHIPNMSIZE);
    setSeenBy(player);
  }

  /*
   * Alwyas 500 shipxy records are read.
   */
  public void readShipxyFrom(LSBReader source, int player) throws IOException {
    int newx=source.readShort();
    int newy=source.readShort();
    int newowner=source.readShort();
    int newweight=source.readShort();
    if (newweight==0) return; // datafile has no info
    if (weight==newweight) return; // we already know the info
    if (weight==UNKNOWN) { // we did not know yet
      x=newx;
      y=newy;
      owner=newowner;
      weight=newweight;
      setSeenBy(player);
    } else { // we knew something else! Should not occur!
      System.err.println(" Inconsistent weight for ShipXY!"+
			   id+", "+"old weight: "+weight+" -> "+
			   "new weight: "+newweight);
    }
  }

  void setSeenBy(int player) {
    seenby |= 1<<player;
  }

  public boolean isSeenBy(int player) {
    return (seenby & (1<<player))!=0;
  }

  public boolean isOurShip() {
    return isship;
  }

  public void writeShipTo(LSBWriter sink) throws IOException {
    sink.writeShort(id);
    sink.writeShort(owner);
    sink.writeString(fc,FCSIZE);
    sink.writeShort(warp);
    sink.writeShort(dx);
    sink.writeShort(dy);
    sink.writeShort(x);
    sink.writeShort(y);
    sink.writeShort(engtype);
    sink.writeShort(hulltype);
    sink.writeShort(beamtype);
    sink.writeShort(beams);
    sink.writeShort(bays);
    sink.writeShort(torptype);
    sink.writeShort(cargo[Cargo.AMMUNITION].amount);
    sink.writeShort(launchs);
    sink.writeShort(mission);
    sink.writeShort(enemy);
    sink.writeShort(towtarget);
    sink.writeShort(damage);
    sink.writeShort(crew);
    sink.writeShort(getColClans());
    sink.writeString(name);
    sink.writeShort(getNeutronium());
    sink.writeShort(getTritanium());
    sink.writeShort(getDuranium());
    sink.writeShort(getMolybdenium());
    sink.writeShort(getSupplies());
    toplanet.writeTo(sink);
    toship.writeTo(sink);
    sink.writeShort(inttarget);
    sink.writeShort(getMC());
  }

  public void writeTargetTo(LSBWriter sink) throws IOException {
    sink.writeShort(id);
    sink.writeShort(owner);
    sink.writeShort(warp);
    sink.writeShort(x);
    sink.writeShort(y);
    sink.writeShort(hulltype);
    sink.writeShort(heading);
    sink.writeString(name,SHIPNMSIZE);
  }

  public void writeShipxyTo(LSBWriter sink) throws IOException {
    sink.writeShort(x);
    sink.writeShort(y);
    sink.writeShort(owner);
    sink.writeShort(weight);
  }

  public static void writeDummyShipxyTo(LSBWriter sink) throws IOException {
      for (int i=0;i<4;i++) sink.writeShort(0); // shipxysize/2 would be nice
  }

  public void throwEvent() {
    universe.throwShipEvent(id);
  }

  public Ship getRstShip() {
    if (isRst()) return null;
    Game game = universe.getGame();
    Universe rstuniv = game.getUniverse(getTurnNumber(), RstOrTrn.RST);
    return rstuniv.getShip(getId());
  }

  public int getHulltype() {
    return hulltype;
  }

  public Hull getHull() {
    if (hulltype == UNKNOWN) return null;
    return getGame().getHull(getHulltype());
  }

  public int getEngtype() {
    return engtype;
  }

  public Engine getEngine() {
    return getGame().getEngine(getEngtype());
  }

  public int getBeamCount() {
    return beams;
  }
  
  public int getBeamtype() {
    return beamtype;
  }

  /**
   * Return the type of the beam
   */
  public Beam getBeam() {
    return getGame().getBeam(getBeamtype());
  }

  public boolean hasBays() {
    return (bays > 0);
  }

  public int getBays() {
    return bays;
  }

  /**
   * Has torpedo launchers on board?
   * @see hasBays
   */
  public boolean hasLaunchers() {
    return (launchs > 0);
  }

  public int getFighters() {
    if (bays == 0) return 0;
    return getCargo(Cargo.AMMUNITION);
  }

  public int getLaunchCount() {
    return launchs;
  }

  public int getTorpedoes() {
    if (launchs == 0) return 0;
    return getCargo(Cargo.AMMUNITION);
  }

  public int getTorpedoType() {
    return torptype;
  }

  /**
   * Return the type of the torpedo tube
   */
  public Torpedo getTorpedo() {
    return getGame().getTorpedo(getTorpedoType());
  }

  public int getAmmoType() {
    if (bays > 0) return Cargo.FIGHTER;
    if (launchs > 0) return torptype;
    return NONE;
  }

  public int getAmmo() {
    return getCargo(Cargo.AMMUNITION);
  }

  public int maxCargo(int cargotype) {
    int g = Cargo.group[cargotype];
    switch (g) {
    case Cargo.FUEL:
      return getHull().getTank();
    case Cargo.CARGO:
      return getHull().cargo;
    case Cargo.MONEY:
      return 10000;
    default:
      crash(1000, "Unknown cargo type "+g+" encountered");
      return ILLEGAL; // shuts up the compiler
    }
  }

    public boolean isFreighter() {
	return (!hasLaunchers() && !hasBays() && (getBeamtype() == 0));
    }

    public String getName() {
	return (name == null ? null : name.toString());
    }

    public void setName(String newname) {
	if (isEditable()) {
	    if (!name.equals(newname)) {
		name = new StringBuffer(newname);
		if (name.length() > SHIPNMSIZE) name.setLength(SHIPNMSIZE);
		while (name.length() < SHIPNMSIZE) name.append(" ");
		universe.throwShipEvent(id);
	    }
	    getGame().unlock();
	}
    }

    public String getFC() {
	return (fc == null ? null : fc.toString());
    }

    /* mismatch with PData.setFC(): both return type and behaviour are
     * different. I have no idea which one is ``best'' or whether the
     * difference really matters. [Kero]
     */
    public void setFC(String newfc) {
	if (isEditable()) {
	    if (!newfc.equals(fc.toString())) {
		fc = new StringBuffer(newfc);
		if (fc.length()>FCSIZE) fc.setLength(FCSIZE);
		while (fc.length()<FCSIZE) fc.append(" ");
		universe.throwShipEvent(id);
	    }
	    getGame().unlock();
	}
    }

  public int getTotalNeutronium() {
    return getNeutronium();
  }

  public int getTotalTritanium() {
    return getTritanium();
  }

  public int getTotalDuranium() {
    return getDuranium();
  }

  public int getTotalMolybdenium() {
    return getMolybdenium();
  }

  public int getDX() {
    if (dx!=UNKNOWN) return dx; // own ship, return accurate value
    // return what you do know about enemy ship
    if ((warp==UNKNOWN) || (warp==0)) return 0; // no waypoint
    if ((heading==UNKNOWN) || (heading==-1)) return 0; // no heading
    // best estimate for enemy ship
    return (int)(warp*warp*Math.sin(Math.PI*heading/180));
  }

    public void setDX(int newdx) {
	if (isEditable()) {
	    if (newdx != dx) {
		dx = newdx;
		universe.throwShipEvent(id);
	    }
	    getGame().unlock();
	}
    }

    public void setDY(int newdy) {
	if (isEditable()) {
	    if (newdy != dy) {
		dy = newdy;
		universe.throwShipEvent(id);
	    }
	    getGame().unlock();
	}
    }

  public int getDY() {
    if (dy!=UNKNOWN) return dy;
    if ((warp==UNKNOWN) || (warp==0)) return 0;
    if ((heading==UNKNOWN) || (heading==-1)) return 0;
    return (int)(warp*warp*Math.cos(Math.PI*heading/180));
  }

  // public void setWaypoint(int nx,int ny) {} semantics?

  public int getEnemy() {
    return enemy;
  }

  public void setEnemy(int e) {
      if (e < 0) e = 0;
      if (e > races) e = races;
    if (isEditable()) {
	if (e != enemy) {
	    enemy=e;
	    universe.throwShipEvent(id);
	}
	getGame().unlock();
    }
  }

  public int getTowTarget() {
    return towtarget;
  }

  public void setTowTarget(int tt) {
    if (tt < 0) tt = 0; // PHost towtarget can be 0 to maxint
    // if ((mission == Mission.TOW) && ((tt < 1) || (tt > ships))) return;
    if (isEditable()) {
	if (tt != towtarget) {
	    if (mission == Mission.TOW) {
		Ship ship = universe.getShip(tt);
		if (ship == null) {
		    report("You can not see that ship!");
		    tt = 0; // blunt, but effective
		} else if (!ship.isSeenBy(getOwner())) {
		    report("Some other player sees this ship, you don't.");
		    tt = 0;
		}
	    }
	    towtarget = tt;
	    universe.throwShipEvent(id);
	}
	getGame().unlock();
    }
  }

  public int getIntTarget() {
    return inttarget;
  }

  public void setIntTarget(int it) {
    if (it < 0) it = 0;
    // if ((mission == Mission.INTERCEPT) && ((it < 1) || (it > ships))) return;
    if (isEditable()) {
	if (it != inttarget) {
	    if (mission == Mission.INTERCEPT) {
		Ship ship = universe.getShip(it);
		if (ship == null) {
		    report("You can not see that ship!");
		    it = 0; // blunt, but effective
		} else if (!ship.isSeenBy(getOwner())) {
		    report("Some other player sees this ship, you don't.");
		    it = 0;
		}
	    }
	    inttarget = it;
	    universe.throwShipEvent(id);
	}
	getGame().unlock();
    }
  }

  public int getCrew() {
    return crew;
  }

  public int getDamage() {
    return damage;
  }

  /**
   * Return the maxTravelDistance of this ship
   */
  public int maxTravelDistance() {
    int res = warp * warp;
    if (getHull().hasGravitonic()) res *= 2;
    return res;
  }

  /**
   * Return the movement by warp engine of this ship
   */
  public MyPoint getMovement(int dx, int dy) {
    MyPoint res;
    double d = Math.sqrt(dx*dx + dy*dy);
    int maxTravelDistance = maxTravelDistance();
    if (d > maxTravelDistance) {
      // Limit traveled distance
      double heading = Math.atan2(dy, dx);
      double tdx = maxTravelDistance * Math.cos(heading);
      double tdy = maxTravelDistance * Math.sin(heading);
      
      if (x == 0)
	tdx = 0;
      if (y == 0)
	tdy = 0;

      res = new MyPoint(Mymath.trunc(tdx + Mymath.sign(tdx)),
			Mymath.trunc(tdy + Mymath.sign(tdy)));
      return res;
    } else {
      res = new MyPoint(dx, dy);
    }
    return res;
  }

  /**
   * Return the position of this ship next turn,
   * regarding the current position, the waypoint and the warp.
   */
  public MyPoint getPositionNT() {
    return getMovement(dx, dy).add(x, y);
  }

  /**
   * Return number of turns before ship reaches waypoint
   * with current warp. Fuel usage is not looked at.
   */
  public int getTurnsUntilWaypoint() {
    // Save current state
    Ship ship = new Ship();
    ship.x = x;
    ship.y = y;
    ship.dx = dx;
    ship.dy = dy;

    if ((warp == 0) && ((dx != 0) || (dy != 0)))
      return INFINITY;
    int turns = 0;
    while (turns < 10) {  // FIXME: should not be fixed
      if ((dx == 0) && (dy == 0)) {
	// Ship arrived at waypoint
	break;
      } 
      turns++;
      MyPoint npos = getMovement(dx, dy);
      dx -= npos.getXint();
      dy -= npos.getYint();
      x += npos.getXint();
      y += npos.getYint();
    }

    // Restore ship
    x = ship.x;
    y = ship.y;
    dx = ship.dx;
    dy = ship.dy;

    return turns;
  }

    /**
     * Return fuel left when ship arrives at waypoint
     * with current warp.
     */
    public int getNeutrAtWaypoint() {
	return neutrLeftWhenMoving(dx, dy);
    }

    public int neutrLeftWhenMoving(int dx, int dy) {
	if (
	    (dx == UNKNOWN) || (dy == UNKNOWN) ||
	    (getNeutronium() == UNKNOWN) ||
	    (warp == UNKNOWN)
	) {
	    return UNKNOWN;
	}
	Ship ship;
	try {
	    ship = (Ship)clone();
	    ship.dx = dx;
	    ship.dy = dy;
	} catch (CloneNotSupportedException cnse) {
	    Assert.assert(false);
	    return UNKNOWN;
	}
	int turns = 0;
	if ((ship.warp == 0) && ((ship.dx != 0) || (ship.dy != 0)))
	    return 0;
	while (turns < 10) {  // FIXME: should not be fixed
	    if ((ship.dx == 0) && (ship.dy == 0)) {
		// Ship arrived at waypoint
		break;
	    }
	    turns++;
	    ship.cargo[Cargo.NEUTRONIUM].amount = ship.getNeutrNT();
	    MyPoint npos = ship.getMovement(ship.dx, ship.dy);
	    ship.dx -= npos.getXint();
	    ship.dy -= npos.getYint();
	    ship.x += npos.getXint();
	    ship.y += npos.getYint();
	}
	int res = ship.getNeutronium();
	return res;
    }

  int getCargoMass() {
    int result = 0;
    for (int j=0; j<Cargo.MC; j++) result += cargo[j].amount;
    for (int j=Cargo.MC+1; j<Cargo.CARGOTYPES; j++) result += cargo[j].amount;
    return result;
  }

  /**
   * Get mass of ship without fuel and cargo
   */
  int getEmptyMass() {
    int res = getHull().getMass();
    if (beams > 0)
      res += beams * getBeam().getMass();
    if (launchs > 0)
      res += launchs * getTorpedo().getMass();
    return res;
  }

  /**
   * Get total mass of ship
   */
  public int getMass() {
    if (weight != UNKNOWN)  return weight;
    return
      getEmptyMass() +
      getCargoMass() +
      getNeutronium();
  }

  /**
   * Return true iff ship is cloaking this turn
   */
  boolean isCloaking() {
    // FIXME: check for cloaking able ship
    return 
      (mission == Mission.CLOAK) ||
      (mission == Mission.cloak(getConfig())) ||
      (mission == Mission.standardSuperSpy(getConfig()));
  }

    /**
     * Return neutronium next turn in ship.
     * <=0 iff out-of-fuel.
     */
    public int getNeutrNT() {
	if (getNeutronium() == UNKNOWN)
	    return UNKNOWN;
	if ((warp == 0) || (warp == UNKNOWN))
	    return getNeutronium();
	int res = getNeutronium();
	MyPoint move = getMovement(dx, dy);
	double dist = move.length();
	double engUsage = getEngine().getNeutrUsage(warp) /
	    (getHull().hasGravitonic()? 200000.0: 100000.0);
	int usage;
	// FIXME: add towing ships
	int massToMove = getMass();
	if (getMission() == Mission.TOW) {
	    Ship s = universe.getShip(towtarget);
	    if (s != null) {
		massToMove += s.getMass();
	    }
	}
	if (getConfig().useAccurateFuelModel()) {
	    usage = (int)(massToMove * (1 - Math.exp(-engUsage * dist)));
	} else {
	    usage = (int)(Mymath.ernd(massToMove / 10.0) * 10 *
			  engUsage * (int)Math.floor(dist));
	}
	res -= usage;
	if (isCloaking()) {
	    int cloakFuelBurn = getConfig().cloakFuelBurn();
	    int massToCloak = Math.max(getHull().getMass(), 100);
	    res -= Mymath.trunc(massToCloak * cloakFuelBurn / 100);
	}
	return res;
    }

  protected void setCargo(int cargotype, int newamount) {
    if (cargo[cargotype].amount == newamount) return;
    Assert.assert(newamount != UNKNOWN);
    //if (isEditable()) {
	cargo[cargotype].amount = newamount;
	// if (throwEvent) universe.throwShipEvent(id);
    //    getGame().unlock();
    //}
  }

  void setFighters(int newfighters) {
    if (bays == 0) return;
    setCargo(Cargo.AMMUNITION, newfighters);
  }

  void setTorpedoes(int newtorps) {
    if (launchs == 0) return;
    setCargo(Cargo.AMMUNITION, newtorps);
  }

  public ToPlanet getToPlanet() {
    return toplanet;
  }

  public ToShip getToShip() {
    return toship;
  }

  public int getMission() {
    return mission;
  }

  public void setMission(int newmis) {
    // FIXME is this a valid mission?
    // there is a gap between normal and PHost missions
    if (isEditable()) {
	if (newmis != mission) {
	    mission = newmis;
	    if (mission == Mission.TOW) setTowTarget(towtarget);
	    if (mission == Mission.INTERCEPT) setIntTarget(inttarget);
	    universe.throwShipEvent(id);
	}
	getGame().unlock();
    }
  }

  public int getWarp() { return warp; }

  public int getMaxWarp() {
    int res = 10 - (damage / 10);
    if (getRaceOwner() == Race.LIZARDS) {
      res += 5;
    }
    return Math.min(res, maxWarp);
  }

  public void setWarp(int newWarp) {
    if (newWarp < 0) newWarp = 0;
    int maxWarp = getMaxWarp();
    if (newWarp > maxWarp) newWarp = maxWarp;
    if (isEditable()) {
	if (newWarp != warp) {
	    warp = newWarp;
	    universe.throwShipEvent(id);
	}
	getGame().unlock();
    }
  }


  // some additional functions

  public void setWebDrainComplete() {
    cargo[Cargo.NEUTRONIUM].amount = 0;
    // is done when reading data, no need to throw event.
  }

  public String getKey() {
    return shiphashprefix + getId();
  }

  public int getTorpsLaid() {
    int mission = getMission();
    if (mission != UNKNOWN) {
      if (Mission.missionLaysMines(mission, getConfig(), getOwner())) {
	if ((mission == Mission.LAYMINES) || (mission == Mission.SPECIAL)) {
	  return cargo[Cargo.AMMUNITION].amount;
	} else {
	  return inttarget;
	}
      }
    }
    return 0;
  }

  /**
   * Return # of units laid during next turn.
   */
  public int getUnitsLaid() {
    int laid = getTorpsLaid();
    if (laid != UNKNOWN)
      return getTorpsLaid() *
	getConfig().unitsPerTorpRate(getOwner()) *
	torptype * torptype;
    else
      return UNKNOWN;
  }

  /**
   * Return owner of minefield laid during next turn 
   */
  public int getOwnerLaid() {
    PConfig config = getConfig();
    int owner = getOwner();
    if ((mission == Mission.layMinefield(config)) ||
	(mission == Mission.layWebMinefield(config))) {
      return towtarget == 0 ? owner : towtarget;
    }
    return owner;
  }

  /**
   * Produce a one-liner report of this ship
   */
  public String getReport() {
    return "Ship "+toString("#oc@w%heblntdm$LsMEW");
  }

  public String toString() {
    return "Ship "+toString("#@");
  }

  public String toString(String userline) {
    StringBuffer result = new StringBuffer();
    result.ensureCapacity(256); // if a user wants longer strings on screen :-(
    Hull hull = getHull();
    for (int i=0; i<userline.length(); i++) {
      result.append(userline.charAt(i));
      switch (userline.charAt(i)) {
      case '#':
	result.append(getId());
	break;
      case '@':
	result.append("("+getX()+","+getY()+")");
	break;
      case 'o':
	if (getOwner() != UNKNOWN) result.append(getOwner());
	else result.append('?');
	break;
      case 'c':
	result.setLength(result.length()-1);
	if (getFC() != null) result.append("`"+getFC()+"'");
	else result.append('?');
	break;
      case 'w':
	if ((getWarp() != UNKNOWN) || (getDX() != UNKNOWN)) {
	  if (getWarp() != UNKNOWN) result.append(getWarp());
	  if (getDX() != UNKNOWN)
	    result.append("->("+getDX()+","+getDY()+")");
	  else if (heading!=UNKNOWN) result.append("/h"+heading); // + unicode o/degree
	}
	break;
      case '%':
	if (getDamage() != UNKNOWN) result.append(getDamage());
	else result.append('?');
	break;
      case 'h':
	result.setLength(result.length()-1);
	if ((getHulltype()!=UNKNOWN) && (getHulltype()!=0)) {
	  StringBuffer func = new StringBuffer(20);
	  if (hull.canCloakAdvanced()) {
	    func.append('C');
	  } else {
	    if (hull.canCloak()) func.append('c');
	  }
	  if (hull.canDecloak()) func.append('L');
	  if (hull.isAlchemy()) func.append('a');
	  if (hull.isRefinery()) func.append('r');
	  if (hull.isAdvancedRefinery()) func.append('R');
	  if (hull.canHeatTo50()) func.append('+');
	  if (hull.canHeatTo100()) func.append("++");
	  if (hull.canCoolTo50()) func.append('-');
	  if (hull.canBioscan20()) func.append('b');
	  if (hull.canBioscan100()) func.append('B');
	  if (hull.hasGlory20()) func.append('x');
	  if (hull.hasGlory10()) func.append('X');
	  if (hull.hasGravitonic()) func.append('G');
	  if (hull.hasCasino()) func.append('g');
	  if (hull.canChunnel()) func.append('H');
	  if (hull.canHyperjump()) func.append('h');
	  if (hull.canScoopFuel()) func.append('s');
	  if (hull.canScanAllWormholes()) func.append('w');
	  if (hull.hasImperialTroopers()) func.append('i');
	  if (func.length() != 0) result.append("["+func+"]");
	  else result.setLength(result.length()-1);
	} else {
	  result.append('?');
	}
	break;
      case 'e':
	if (hull != null) {
	  result.append(hull.getEngines());
	  if (engtype!=UNKNOWN) result.append("x"+engtype);
	}
	break;
      case 'b':
	if (beams != UNKNOWN) {
	  if (beams!=0) {
	    result.append(beams);
	    if (beamtype != UNKNOWN) result.append("x" + beamtype);
	  }
	} else {
	  if (hull != null) {
	    int b = hull.getBeams();
	    if (b!=0) result.append(b);
	  }
	}
	break;
      case 'l':
	result.setLength(result.length()-1);
	if (launchs != UNKNOWN) {
	  if (launchs!=0) {
	    result.append("T" + ((getTorpedoes() == UNKNOWN)? "?": ""+getTorpedoes()));
	    result.append("/" + ((launchs == UNKNOWN)? "?": ""+launchs));
	    result.append("x" + ((torptype == UNKNOWN)? "?": ""+torptype));
	  }
	} else {
	  if (hull != null) {
	    int lau = hull.getLaunchs();
	    if (lau!=0) result.append("T"+lau);
	  }
	}
	if (bays!=UNKNOWN) {
	  if (bays!=0) {
	    result.append("F" + ((getFighters() == UNKNOWN)? "?": ""+getFighters()));
	    result.append("/" + ((bays == UNKNOWN)? "?": ""+bays));
	  }
	} else {
	  if (hull != null) {
	    int b = getHull().getBays();
	    if (b!=0) result.append(" F/"+b);
	  }
	}
	break;
      case 'n':
	if (getNeutronium() != UNKNOWN) result.append(getNeutronium());
	else result.append('?');
	break;
      case 't':
	if (getTritanium() != UNKNOWN) result.append(getTritanium());
	else result.append('?');
	break;
      case 'd':
	if (getDuranium() != UNKNOWN) result.append(getDuranium());
	else result.append('?');
	break;
      case 'm':
	if (getMolybdenium() != UNKNOWN) result.append(getMolybdenium());
	else result.append('?');
	break;
      case '$':
	if (getMC() != UNKNOWN) result.append(getMC());
	else result.append('?');
	break;
      case 's':
	if (getSupplies() != UNKNOWN) result.append(getSupplies());
	else result.append('?');
	break;
      case 'L':
	result.setLength(result.length()-1);
	if (getColClans()==UNKNOWN) result.append('?');
	else if (getColClans()==0) result.append("noCol");
	else result.append(getColClans()+"col");
	break;
      case 'W':
	result.setLength(result.length()-1);
	result.append("w"+weight); // weight always known
	if (hull != null) result.append("/m"+hull.getMass());
	break;
      case 'M':
	result.setLength(result.length()-1);
	if (mission!=UNKNOWN) {
	  Mission m = universe.getGame().getMission(mission);
	  result.append(m.getName());
	  if (m.getIntHint()!=null) result.append("/i"+inttarget);
	  if (m.getTowHint()!=null) result.append("/t"+towtarget);
	} else {
	  result.append('?');
	}
	break;
      case 'E':
	if (enemy!=UNKNOWN) result.append(enemy); else result.append('?');
	break;
      default:
	result.append("<unknown field>");
	break;
      }
      result.append(' ');
    }
    return result.toString();
  }
}
