// File: ShipGen.java

package jvpc.vga;

import java.util.Hashtable;
import java.io.*;

/**
 * Generates <CODE>Ship</code> objects.
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since     20 XII 1998
 * @version   14 III 2001
 */
public class ShipGen extends Generator {
  final int shipcount;
  /**
   * Constructs an Ship generator.
   * @param is The <CODE>InputStream</code> from which the <CODE>source</code>
   * can be created.
   */
  public ShipGen(InputStream is) throws IOException {
    super(is);
    shipcount=source.readShort();
  }

  public ShipGen(Generator g) throws IOException {
    super(g);
    shipcount=source.readShort();
  }

  public int getShipcount() {
    return shipcount;
  }

  /**
   * Generates all <CODE>Ship</code> objects found in <CODE>pis</code>.
   * Puts them at the proper place in <CODE>ship[]</code>
   */
  public void generateAll(Universe u) {
    int i,id;
    Ship ship;
    for (i=1;i<=shipcount;i++) {
      try {
	id=source.readShort();
	if (u.containsShip(id)) {
	  ship=u.getShip(id);
	} else {
	  ship=new Ship(u,id);
	}
	ship.readShipFrom(source);
	u.putShip(ship); // should not be in there (anymore)
      } catch (IOException ioe) {
	crash(11, "Error while reading shipdata: "+ioe);
      }
    }
  }
}
