// File: Table.java

package jvpc.vga;

import java.lang.reflect.*;
import java.util.*;
import kero.util.*;

/**
 * Generates a table of objects
 * The table can be sorted.
 * The fields to be displayed can be tailored with the TableConfig.
 *
 * The first step is rendering of all cells
 * The next step is sorting.
 *
 * @author Rutger Nijlunsing
 * @since 19990513
 */
public class Table implements VGAPlanets
{
    Object object[];		// All object in table (y-axis)
    Vector sortOn;		// Vector of Integer's; columns to sort on.
    TableConfig config;		// Fields to show in table
    MyPoint src;		// For fields needing a src point

    boolean renderedValid = false; // true if rendered is valid
    Vector rendered;		// Vector of Object[]; The rendered, unsorted table
    boolean sortedValid = false; // true if sorted is valid
    Object sorted[];		// All objects[] sorted

    public Table() {
	object = new Object[0];	// No object, but a valid table
	sortOn = new Vector();
	config = null;
	src = new MyPoint(2000, 2000);
	renderedValid = false;
	sortedValid = false;
    }

    /**
     * Set the objects to be displayed in the table
     */
    public void setObjects(Vector v) {
	int len = v.size();
	object = new Object[len];
	for (int i=0; i<len; i++)
	    object[i] = v.elementAt(i);
	renderedValid = false;
    }
    
    /**
     * Sort first on column @param column.
     */
    public void addSortColumn(int column) {
	// Find previous column, and remove that one
	Integer col = new Integer(column);
	int idx = sortOn.indexOf(col);
	if (idx != -1)
	    sortOn.removeElementAt(idx);
	sortOn.insertElementAt(col, 0);
	sortedValid = false;
    }

    /**
     * No sorting is performed
     */
    public void resetSortColumn() {
	sortOn = null;
	sortedValid = false;
    }

    public void setConfig(TableConfig tc) {
	config = tc;
	renderedValid = false;
    }

    public void setSource(MyPoint p) {
	src = p;
	renderedValid = false;
    }

    /**
     * Update rendered
     */
    private void updateRender() {
	rendered = new Vector(object.length);
	for (int i=0; i<object.length; i++) {
	    rendered.addElement(renderObject(object[i], config, src));
	}
	renderedValid = true;
    }

    /**
     * Update sorting
     */
    private void updateSort() {
	sorted = new Object[object.length];
	System.arraycopy(object, 0, sorted, 0, object.length);
	// TODO: perform sorting
	sortedValid = true;
    }

    /**
     * Render one object
     */
    static public Object[] renderObject(Object o, TableConfig tc, MyPoint src) {
	int fields = tc.getFields();
	Object res[] = new Object[fields];
	try {
	    for (int i=0; i<fields; i++) {
		int ci = tc.findClass(o.getClass());
		Method m = tc.getMethod(ci, i);
		if (m != null) {
		    int type = tc.getMethodType(ci, i);
		    switch (type) {
		    case TableConfig.NONE:
			res[i] = m.invoke(o, new Object[0]);
			break;
		    case TableConfig.POINT:
			res[i] = m.invoke(o, new Object[] {src});
			break;
		    default:
			Assert.assert(false);
			break;
		    }
		    if (res[i] instanceof Integer)
			if (((Integer)res[i]).intValue() == UNKNOWN)
			    res[i] = null;
		} else {
		    // No method.
		    res[i] = null;
		}
	    }
	} catch (Exception e) {
	    e.printStackTrace();
	    Assert.assert(false);
	}
	return res;
    }

    /**
     * Render current objects
     *
     * Returns a Vector of Object[]
     */
    public Vector render() {
	if (!renderedValid) {
	    sortedValid = false;
	    updateRender();
	}
	if (!sortedValid) {
	    updateSort();
	}
	// TODO: return rendered _sorted_
	return rendered;
    }
}
