// File: TableConfig.java

package jvpc.vga;

import java.lang.reflect.*;
import java.util.*;
import kero.util.*;

/**
 * The config of the table of objects 
 *
 *
 * @author Rutger Nijlunsing
 * @since 19990430
 */

public class TableConfig {
    Class cls[];		// y axis: classes
    int classes;		// # of elements used in cls[]
    String field[];		// x axis: field names
    int fields;			// # of elements used in column[]
    Method contents[][];	// field (class, column)
				//  null for none.
    int type[][];		// Parameters needed for method
    final static int NONE = 1;	//  No parameters needed
    final static int POINT = 2; //  MyPoint as parameter needed

    /**
     * Return the index of class @param c
     * -1 if not found.
     */
    public int findClass(Class c) {
	for (int i=0; i<classes; i++)
	    if (cls[i] == c)
		return i;
	Assert.assert(false);
	return -1;
    }

    /**
     * Return the index of field @param f
     * -1 if not found.
     */
    public int findField(String f) {
	for (int i=0; i<fields; i++)
	    if (field[i].equals(f))
		return i;
	Assert.assert(false);
	return -1;
    }

    /**
     * Set contents of table of class with index @param ci, and field
     * with index @param fi to possibility @param idx (0..).
     * @param idx = 0 implies the default method.
     * @param idx = -1 implies 'none'
     *
     * The possibities can be retrieved with c.getTypeMethods().
     * @param ci can be found with findClass
     * @param fi can be found with findField
     */
    public void set(int ci, int fi, int idx) {
	try {
	    Class c = cls[ci];
	    String f = field[fi];
	    Tabulatable t = (Tabulatable)c.newInstance();
	    Vector v = t.getTypeMethods(f);
	    Method m;
	    if (v.size() == 0)
		idx = -1;
	    if (idx != -1) {
		String methodname = (String)v.elementAt(idx);
		m = c.getMethod(methodname, null);
	    } else {
		m = null;
	    }
	    contents[ci][fi] = m;
	    type[ci][fi] = NONE;
	    // TODO: add other parameters
	} catch (Exception e) {
	    e.printStackTrace();
	    Assert.assert(false);
	}
    }

    public Method getMethod(int ci, int fi) {
	return contents[ci][fi];
    }

    public int getMethodType(int ci, int fi) {
	return type[ci][fi];
    }

    /**
     * Return empty TableConfig, with max space for
     * @param maxclasses classes, and @param maxfields columns.
     */
    public TableConfig(int maxclasses, int maxfields) {
	cls = new Class[maxclasses];
	classes = 0;
	field = new String[maxfields];
	fields = 0;
	contents = new Method[maxclasses][maxfields];
	type = new int[maxclasses][maxfields];
    }

    /**
     * Add class
     * Return index in table
     */
    public int addClass(Class newclass) {
	cls[classes] = newclass;
	for (int i=0; i<fields; i++)
	    set(classes, i, 0);
	classes++;
	return classes - 1;
    }
    
    /**
     * Add field
     * Return index in table
     */
    public int addField(String fieldname) {
	field[fields] = fieldname;
	for (int i=0; i<classes; i++)
	    set(i, fields, 0);
	fields++;
	return fields - 1;
    }

    public int getFields() {
	return fields;
    }

}
