// File: TrnGen.java

package jvpc.vga;

import java.util.Hashtable;
import java.io.*;
import kero.util.*;

/**
 * Changes information in a TRN Universe, as found in a TRN file. 
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since     14 III 1999
 * @version   14 III 2001
 */
public class TrnGen
  extends Generator
{
  int commandcount;
  private boolean numberread=false;

  public TrnGen(InputStream is) throws IOException {
    super(is);
  }

  public String getTimestamp(int player) throws IOException {
    Assert.assert(!numberread);
    numberread=true;
    Assert.assert(source.readShort()==player);
    commandcount=source.readInt();
    return source.readString(TIMESTAMPSIZE).toString();
  }

    public void generateAll(Universe u,int player) throws IOException {
    report("Reading TRN (player "+player+", turn "+u.getTurnNumber()+")");
    Assert.assert(numberread);
    int[] pointer;
    int commandnr,id;
    int i,j;
    Gen gen=u.getGen(player);

    try {
      source.skip(2); // useless
      gen.hostid=source.readShort();
      if (commandcount>0) source.skip(1); // useless

      pointer=new int[commandcount];
      for (i=0;i<commandcount;i++) pointer[i]=source.readInt()-1;
      for (i=0;i<commandcount;i++) {
	source.skipUntil(pointer[i]);
	commandnr=source.readShort();
	// report("decompiling "+i+"th command: nr "+commandnr);
	id=source.readShort();
	switch (commandnr) {
	case 1: {
	  u.getShip(id).fc = source.readString(FCSIZE);
	  break;
	}
	case 2: {
	  u.getShip(id).warp = source.readShort();
	  break;
	}
	case 3: {
	  Ship s = u.getShip(id);
	  s.dx = source.readShort();
	  s.dy = source.readShort();
	  break;
	}
	case 4: {
	  u.getShip(id).mission = source.readShort();
	  break;
	}
	case 5: {
	  u.getShip(id).enemy = source.readShort();
	  break;
	}
	case 6: {
	  u.getShip(id).towtarget = source.readShort();
	  break;
	}
	case 7: {
	  u.getShip(id).name = source.readString(SHIPNMSIZE);
	  break;
	}
	case 8: {
	  Ship ship = u.getShip(id);
	  ship.toplanet.readFrom(source);
	  break;
	}
	case 9: {
	  Ship ship = u.getShip(id);
	  ship.toship.readFrom(source);
	  break;
	}
	case 10: {
	  u.getShip(id).inttarget = source.readShort();
	  break;
	}
	case 11: {
	  u.getShip(id).cargo[Cargo.NEUTRONIUM].amount = source.readShort();
	  break;			       
	}				       
	case 12: {		       
	  u.getShip(id).cargo[Cargo.TRITANIUM].amount = source.readShort();
	  break;			       
	}				       
	case 13: {		       
	  u.getShip(id).cargo[Cargo.DURANIUM].amount = source.readShort();
	  break;			       
	}				       
	case 14: {		       
	  u.getShip(id).cargo[Cargo.MOLYBDENIUM].amount = source.readShort();
	  break;			       
	}				       
	case 15: {		       
	  u.getShip(id).cargo[Cargo.SUPPLIES].amount = source.readShort();
	  break;			       
	}				       
	case 16: {		       
	  u.getShip(id).cargo[Cargo.COLCLANS].amount = source.readShort();
	  break;
	}
	case 17: {
	  u.getShip(id).cargo[Cargo.AMMUNITION].amount = source.readShort();
	  break;
	}
	case 18: {
	  u.getShip(id).cargo[Cargo.MC].amount = source.readShort();
	  break;
	}

	case 21: {
	  ((PData)u.getPlanet(id)).fc = source.readString(FCSIZE);
	  break;
	}
	case 22: {
	  ((PData)u.getPlanet(id)).struct[Build.MINE] = source.readShort();
	  break;
	}		  
	case 23: {  
	  ((PData)u.getPlanet(id)).struct[Build.FACTORY] = source.readShort();
	  break;
	}		  
	case 24: {  
	  ((PData)u.getPlanet(id)).struct[Build.DEFENSE] = source.readShort();
	  break;
	}
	case 25: {
	  ((PData)u.getPlanet(id)).cargo[Cargo.NEUTRONIUM].amount = source.readInt();
	  break;
	}
	case 26: { 
	  ((PData)u.getPlanet(id)).cargo[Cargo.TRITANIUM].amount = source.readInt();
	  break;
	}
	case 27: { 
	  ((PData)u.getPlanet(id)).cargo[Cargo.DURANIUM].amount = source.readInt();
	  break;	 
	}
	case 28: { 
	  ((PData)u.getPlanet(id)).cargo[Cargo.MOLYBDENIUM].amount = source.readInt();
	  break;	 
	}
	case 29: { 
	  ((PData)u.getPlanet(id)).cargo[Cargo.COLCLANS].amount = source.readInt();
	  break;	 
	}
	case 30: { 
	  ((PData)u.getPlanet(id)).cargo[Cargo.SUPPLIES].amount = source.readInt();
	  break;	 
	}
	case 31: { 
	  ((PData)u.getPlanet(id)).cargo[Cargo.MC].amount = source.readInt();
	  break;
	}
	case 32: {
	  ((PData)u.getPlanet(id)).coltax = source.readShort();
	  break;
	}
	case 33: {
	  ((PData)u.getPlanet(id)).nattax = source.readShort();
	  break;
	}
	case 34: {
	  ((PData)u.getPlanet(id)).struct[Build.BASE] = 1;
	  break;
	}

	case 40: {
	  u.getBData(id).struct[Build.BASEDEFENSE] = source.readShort();
	  break;
	}
	case 41: {
	  u.getBData(id).techLevel[engineTech] = source.readShort();
	  break;	  
	}		  
	case 42: {  
	  u.getBData(id).techLevel[hullTech] = source.readShort();
	  break;	  
	}		  
	case 43: {  
	  u.getBData(id).techLevel[beamTech] = source.readShort();
	  break;
	}
	case 44: {
	  BData base=u.getBData(id);
	  for (j=1; j<=engines; j++)
	    base.struct[Build.ENGOFFSET+j] = source.readShort();
	  break;
	}
	case 45: {
	  BData base=u.getBData(id);
	  for (j=1; j<=buildhulls; j++)
	    base.struct[Build.HULLOFFSET+j] = source.readShort();
	  break;
	}
	case 46: {
	  BData base=u.getBData(id);
	  for (j=1; j<=beams; j++)
	    base.struct[Build.BEAMOFFSET+j] = source.readShort();
	  break;
	}
	case 47: {
	  BData base=u.getBData(id);
	  for (j=1; j<=torpedoes; j++)
	    base.struct[Build.LAUNCHOFFSET+j] = source.readShort();
	  break;
	}
	case 48: {
	  BData base=u.getBData(id);
	  for (j=1; j<=torpedoes; j++) {
	    base.struct[Build.TORPOFFSET+j] = source.readShort();
	  }
	  break;
	}
	case 49: {
	  u.getBData(id).struct[Build.FIGHTER] = source.readShort();
	  break;
	}
	case 50: {
	  u.getBData(id).target = source.readShort();
	  break;
	}
	case 51: {
	  u.getBData(id).fixrec = source.readShort();
	  break;
	}
	case 52: {
	  u.getBData(id).mission = source.readShort();
	  break;
	}
	case 53: {
	  BData base=u.getBData(id);
	  base.bhulltype=source.readShort();
	  base.bengtype=source.readShort();
	  base.bbeamtype=source.readShort();
	  base.bbeams=source.readShort();
	  base.btorptype=source.readShort();
	  base.blaunchs=source.readShort();
	  base.unknown=source.readShort();
	  break;
	}
	case 54: {
	  u.getBData(id).techLevel[torpedoTech] = source.readShort();
	  break;
	}

	case 60: {
	  int length=id;
	  Message m=new Message(u);
	  m.readFromTrn(source,length,player);
	  u.putMessage(m);
	  break;
	}
	case 61: {
	  /* id contains 0 (zero), why? */
	  gen.setNewPwd(source.readString(PWDSIZE));
	  break;
	}
	default: {
	  report("TrnGen.readTurn("+player+") -- Command "+commandnr+" not recognized");
	  return; // yeah, a return
	}
	}
      }
      source.skip(8); // checksum and 4 weird bytes
      byte[] fizz = source.readByteArray(FIZZSIZE);
      Game game = u.getGame();
      if (game.fizz.equals(FizzBinGen.sharewareRegKey)) game.fizz = fizz;
      // 11 checksums from your ``allies''; check them
    } catch (IOException ioe) {
      crash(11, "Could not read player"+player+".trn: "+ioe);
    }

    copyFileToDir(InputFileOpener.getLastOpenedFile().getAbsolutePath(), u.game.workdir.toString()+u.getTurnNumber());
  }
}
