// File: Universe.java

package jvpc.vga;

import java.util.*;
import java.io.*;
import kero.util.*;

/**
 * Contains an entire universe (i.e. all data from one specific turn).
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since     10 VII 1998
 * @version   14 III 2001
 */
public class Universe
    extends Reporter
    implements VGAPlanets
{
  Game game;
  int turnnr;
  int from; // RstOrTrn.RST or TRN
  Hashtable thing; // Hashtable? or some Vectors?
  Hashtable locations = new Hashtable(); // Hashtable with Location
  Gen[] gen;
  Planet[] planet;
  // BData[] bdata; // [snip]
  Ship[] ship;
  Player[] player = new Player[races + 1];
  Vector vcr;
  Vector message;
    int minefieldcount=0; // != minefield.size()
  Vector minefield;
  Vector explosion;
  int ionstormcount=0;
  Vector ionstorm;
    int wormholecount=0; // != wormhole.size()
  Vector wormhole;
  Alliance[] alliance;
  // TODO: add scores
  // TODO: add password
  PConfig config; // TODO: should be Config
  ShipGen[] shipgen=new ShipGen[races+1];
  PDataGen[] pdatagen=new PDataGen[races+1];
  TargetGen[] targetgen=new TargetGen[races+1];
  VcrGen[] vcrgen=new VcrGen[races+1];
  // user information
  Reportable reportable;
  // easy searching
  MapSearch mapsearch = new MapSearch();
  // ``static'' reminder
  boolean configread=false;

  // evenlisteners
  Vector planetlistener=new Vector();
  Vector baselistener=new Vector();
  Vector shiplistener=new Vector();
  
  public Universe(Game g,int turnnr,int from) {
    this(g,null,turnnr,from);
  }

  /**
   * Copy this Universe. That is, copy the links to the Game and to all(?)
   * objects in its Hashtable. This constructor is meant to initialize a
   * starmap.
   */
  protected Universe(Universe u,int turnnr,int from) {
    this(null,u,turnnr,from);
  }

  /**
   * Make a new Universe.
   * @param u Universe to copy some fields from (but definitely not turnnr)
   */
  protected Universe(Game g,Universe u,int turnnr,int from) {
    if (g!=null) {
	game=g;
    } else if (u==null) {
	// (g == null) && (u == null)
	throw new Error("Internal error; can't init Universe");
    } else {
	game=u.game;
    }
    this.turnnr = turnnr;
    this.from = from;
    // RstOrTrn set later by creator
    thing=new Hashtable();
    planet=new Planet[planets+1];
    // bdata=new BData[planets+1];
    ship=new Ship[ships+1];
    vcr=new Vector();
    message=new Vector();
    minefield=new Vector();
    explosion=new Vector();
    ionstorm=new Vector();
    wormhole=new Vector();
    gen=new Gen[races+1];
    alliance=new Alliance[races+1];
    if (u!=null) {
      for (int i=1;i<=planets;i++) {
	Planet obj=u.getPlanet(i);
	if (obj!=null) {
	  try {
	    obj=(Planet)obj.clone();
	  } catch (CloneNotSupportedException cnse) {
	    // ignore
	  }
	  obj.setUniverse(this);
	  putPlanet(obj);
	}
      }
    }
    for (int i=1; i<=players; i++) {
      player[i] = new Player(this, i);
    }
  }

    public Universe toTrnUniverse() {
	//warn("Copying RST("+getTurnNumber()+")->TRN universe");
	Assert.assert(get() == RstOrTrn.RST);
	Universe u = deepCopy();
	u.from = RstOrTrn.TRN;
	return u;
    }

   /**
     * Return a copy of the current universe.
     * Many important contained objects are also copied (some are just
     * linked!!!) Use with care!
     */
    public Universe deepCopy() {
	// link gen-info, alliance-info, scores(?), password(?), config,
	// players and game (do _not_ pass this as universe parameter).
	Universe u = new Universe(game, getTurnNumber(), get());
	u.gen = gen; // ???
	u.alliance = alliance; // ???
	// scores?
	// password?
	u.config=config;
	// race names?
	u.configread=configread;
	u.player=player; // ???
	// copy the other fields and make them editable, but not the Messages
	try {
	    // bases are extended planets
	    for (int i=1;i<=planets;i++) {
		Planet obj=getPlanet(i);
		if (obj!=null) {
		    obj=(Planet)obj.clone();
		    // obj.setUniverse(u);
		    u.putPlanet(obj);
		}
	    }
	    for (int i=1;i<=ships;i++) {
		Ship obj=getShip(i);
		if (obj!=null) {
		    obj = (Ship)obj.clone(); // deep copy
		    // obj.setUniverse(u);
		    u.putShip(obj);
		}
	    }
	} catch (Exception e) {
	    // clone must have been succesful.
	    System.err.println("Clone failed? "+e);
	}
	// TODO: use iterators
	for (int i=0;i<=getMaxMinefieldId();i++) {
	    Minefield obj=getMinefield(i);
	    if (obj!=null) u.putMinefield(obj);
	}
	for (int i=0;i<=getMaxIonstormId();i++) {
	    Ionstorm obj=getIonstorm(i);
	    if (obj!=null) u.putIonstorm(obj);
	}
	for (int i=0;i<=getMaxWormholeId();i++) {
	    Wormhole obj=getWormhole(i);
	    if (obj!=null) u.putWormhole(obj);
	}
	for (int i=0;i<getVcrCount();i++) {
	    u.putVcr(getVcr(i));
	}
	for (int i=0;i<getExplosionCount();i++) {
	    u.putExplosion(getExplosionAt(i));
	}
	return u;
    }

  /*  public void setTurnNumber(int newturn) {
      super.setTurnNumber(newturn); // now it has the new turn number itself
      for (int i=0;i<planets;i++) 
      if (planet[i]!=null) planet[i].setTurnNumber(newturn);
      else System.err.println("Universe turn "+getTurnNumber());
    }*/


  // reading data

  /**
   * Reads one result, already formed as an InputStream.
   */
  public void readRst(RstGen rstgen, int player) throws IOException {
    report("Reading RST (player "+player+", turn "+getTurnNumber()+")");
    Assert.assert(get() == RstOrTrn.RST);
    rstgen.generateAll(this,player);
    getPlayer(player).setStatus(Player.SELF);
  }

  /**
   * Read util??.dat. Only read after VCR's have been read for that
   * same turn, i.e. after all data has been read for that turn.
   */
  public void readUtil(UtilGen utilgen, int player) throws IOException {
    Assert.assert(get() == RstOrTrn.RST);
    report("Reading UTIL (player "+player+", turn "+getTurnNumber()+")");
    utilgen.generateAll(this,player);
  }

  /**
   * Reads all information of a RST universe for given player.
   * @param ifo InputFileOpener that knows where to look for files.
   * @param player Player for which data must be read.
   */
  public void readDisData(InputFileOpener ifo,int player) {
    InputStream infile;

    Assert.assert(get() == RstOrTrn.RST);
    report("Reading DIS data (player "+player+", turn "+getTurnNumber()+")");
    readData(ifo,player);

    // read messages
    try {
      infile=ifo.open("mdata"+player+".dat",false);
      MDataGen mdatagen=new MDataGen(infile);
      mdatagen.generateAll(this,player,0);
    } catch(IOException ioe) {
      crash(11, "Missing file ``mdata"+player+".dat'': "+ioe);
    }
  }

  /**
   * Reads all information of a TRN universe for given player.
   * @param ifo InputFileOpener that knows where to look for files.
   * @param player Player for which data must be read.
   */
  public void readDatData(InputFileOpener ifo,int player) {
    InputStream infile;

    Assert.assert(get() == RstOrTrn.TRN);
    report("Reading DAT data (player "+player+", turn "+getTurnNumber()+")");
    readData(ifo,player);

    // read messages
    try {
      infile=ifo.open("mess"+player+".dat",false);
      MessGen messgen=new MessGen(infile);
      messgen.generateAll(this,player);
    } catch(IOException ioe) {
      crash(11, "Missing file ``mess"+player+".dat'': "+ioe);
    }
  }

  /**
   * Read pconfig.src for PHost (or config.hst or so for TimHost).
   */
  public void readConfig(InputFileOpener ifo) {
    if (!configread) try {
      InputStream infile=ifo.open("pconfig.src",true);
      PConfigGen confgen=new PConfigGen(infile);
      confgen.generateAll(this);
      game.report("Game is called: ``"+config.getGameName()+"''");
    } catch (IOException ioe) {
      // hmm, must be a TimHost game
      report("No pconfig.src found; TimHost game?");
      config = new PConfig(); // should be TimConfig in a Config
    }
    configread=true;
  }

  private void readData(InputFileOpener ifo,int player) {
    InputStream infile;

    // mark Player as ``self''
    getPlayer(player).setStatus(Player.SELF);

    // read config: pconfig.src for PHost or [???] for TimHost
    readConfig(ifo);

    // Read pdata??.dat
    try {
      infile=ifo.open("pdata"+player+(isRst()?".dis":".dat"),false);
      pdatagen[player]=new PDataGen(infile);
      pdatagen[player].generateAll(this,player);
    } catch(IOException ioe) {
      crash(11, "Missing file ``pdata"+player+(isRst()?".dis":".dat")+"'': "+ioe);
    }

    // Read bdata??.dat
    try {
      infile=ifo.open("bdata"+player+(isRst()?".dis":".dat"),false);
      BDataGen bdatagen=new BDataGen(infile);
      bdatagen.generateAll(this);
    } catch(IOException ioe) {
      crash(11, "Missing file ``bdata"+player+(isRst()?".dis":".dat")+"'': "+ioe);
    }

    // read ship??.dat
    try {
      infile=ifo.open("ship"+player+(isRst()?".dis":".dat"),false);
      shipgen[player]=new ShipGen(infile);
      shipgen[player].generateAll(this);
    } catch(IOException ioe) {
      crash(11, "Missing file ``ship"+player+(isRst()?".dis":".dat")+"'': "+ioe);
    }

    if (isTrn()) return;
    report("Reading auxiliary Tim-files");

    // Read target??.dat
    if (isRst()) try {
      infile=ifo.open("target"+player+".dat",false);
      targetgen[player]=new TargetGen(infile);
      targetgen[player].generateAll(this,player);
    } catch(IOException ioe) {
      crash(11, "Missing file ``target"+player+".dat'': "+ioe);
    }

    // Read target??.ext from VPUTIL (and others?)
    if (isRst()) try {
      infile=ifo.open("target"+player+".ext",false);
      targetgen[player].setInfile(infile);
      targetgen[player].generateAll(this,player);
    } catch(IOException ioe) {
    }

    // Read shipxy??.dat
    if (isRst()) try {
      infile=ifo.open("shipxy"+player+".dat",false);
      ShipXYGen shipxygen=new ShipXYGen(infile);
      shipxygen.generateAll(this, player);
    } catch(IOException ioe) {
      crash(11, "Missing file ``shipxy"+player+".dat'': "+ioe);
    }

    // read vcr??.dat
    if (isRst()) try {
      infile=ifo.open("vcr"+player+".dat",false);
      vcrgen[player]=new VcrGen(infile);
      vcrgen[player].generateAll(this);
    } catch(IOException ioe) {
      crash(11, "Missing file ``vcr"+player+".dat'': "+ioe);
    }
  }

  public void figureVcrs(boolean docount,int[] count,LSBWriter[] lsbw) throws IOException {
    for (int i=0;i<vcr.size();i++) {
      Vcr v=(Vcr)vcr.elementAt(i);
      int id1=v.getCombatant(1).getOwner();
      int id2=v.getCombatant(2).getOwner();
      for (int player=1;player<=races;player++) if (getGen(player)!=null) {
	Alliance alli=getAlliance(player);
	if ((id1==player) || (id2==player) ||
	    (alli!=null) &&
	    ((alli.offersShipAlly(id1) && 
	      alli.offersCombatAlly(id1) && 
	      alli.offersVisionAlly(id1)) ||
	     (alli.offersShipAlly(id2) && 
	      alli.offersCombatAlly(id2) && 
	      alli.offersVisionAlly(id2)))) {
	  if (docount) count[player]++;
	  else v.writeTo(lsbw[player]);
	}
      }
    }
  }

  public void writeAll(InputFileOpener ifo,OutputFileOpener ofo) {
    InputStream infile;
    OutputStream outfile;
    int i,j;
    short player;
    String signature;

    // check for read-only option from user? hardly necessary, I think...

    game.report("Writing universe ("+getTurnNumber()+", "+(isRst()?"rst":"trn")+")");

    // read util??.dat in order to save transfered files
    for (j=1;j<=races;j++) {
      try { // again, rst/dis-data
	infile=ifo.open("util"+j+".dat",false);
	UtilGen utilgen=new UtilGen(game,infile);
	utilgen.setReportable(Reporter.BLACKHOLE);
	int turnnr=utilgen.getTurnNumber();
	if ((getGen(j)!=null) && (turnnr==getGen(j).getTurnNumber())) {
	  utilgen.generateAll(this,true,j);
	}
      } catch(IOException ioe) {}
    }

    // open control.dat for checksums
    outfile=ofo.open("control.dat");
    LSBWriter control=new LSBWriter(outfile);

    // write ships to ship??.dis/dat
    LSBWriter[] lsbw=new LSBWriter[races+1];
    Ship ship;
    // count ships per player, write as first short in ship-files
    short[] shipcount=new short[races+1];
    for (j=1;j<=races;j++) shipcount[j]=0; // silences compiler
    for (i=1;i<=ships;i++) {
      ship=getShip(i);
      if ((ship!=null) && (ship.isOurShip())) shipcount[ship.getOwner()]++;
    }

    try {
      for (j=1;j<=races;j++) {
	if (gen[j]!=null) {
	  outfile=ofo.open("ship"+j+(get()==RstOrTrn.RST?".dis":".dat"));
	  lsbw[j]=new LSBWriter(outfile);
	  lsbw[j].writeShort(shipcount[j]);
	  lsbw[j].getAndResetPartial();
	}
      }
      for (i=1;i<=ships;i++) {
	ship=getShip(i);
	if ((ship!=null) && ship.isOurShip()) {
	  int owner=ship.getOwner();
	  ship.writeShipTo(lsbw[owner]);
	  control.writeInt(lsbw[owner].getAndResetPartial());
	} else {
	  control.writeInt(0);
	}
      }
    } catch (IOException ioe) {
      crash(11, "Error while writing ship "+i+" to file: "+ioe);
    }

    // add signature
    for (i=0;i<races;i++) {
      player=(short)(i+1);
      if (lsbw[player]!=null) try {
	if (get()==RstOrTrn.RST) signature=gen[player].getDisSig();
	else signature=gen[player].getDatSig();
	lsbw[player].writeString(signature);
	lsbw[player].close();
	int sc=gen[player].getShipChecksum();
	gen[player].setShipChecksum(sc+lsbw[player].getChecksum());
      } catch (Exception ioe) {
	crash(11, "Failed to finish ship??.dat: "+ioe);
      }
    }

    // write planets to pdata.dis/dat
    lsbw=new LSBWriter[races+1];
    Planet planet;
    short[] planetCount=new short[races+1];
    for (j=1; j<=races; j++) planetCount[j]=0; // silences compiler
    for (i=1; i<=planets; i++) {
	planet = getPlanet(i);
	if ((planet != null) && planet instanceof PData) {
	    PData pdata = (PData)planet;
	    for (j=1; j<=races; j++) {
		if ((pdata.getOwner()==j) || pdata.isSeenBy(j))
		    planetCount[j]++;
	    }
	}
    }

    try {
      for (j=1; j<=races; j++) {
	if (gen[j] != null) {
	  outfile = ofo.open("pdata"+j+(get()==RstOrTrn.RST?".dis":".dat"));
	  lsbw[j] = new LSBWriter(outfile);
	  lsbw[j].writeShort(planetCount[j]);
	  lsbw[j].getAndResetPartial();
	}
      }
      for (i=1; i<=planets; i++) {
	  planet = getPlanet(i);
	  if ((planet != null) && planet instanceof PData) {
	      PData pdata = (PData)planet;
	      int owner = pdata.getOwner();
	      boolean firstPlayer = true;
	      for (j=1; j<=races; j++) {
		  if ((lsbw[j] != null) && ((owner==j) || pdata.isSeenBy(j))) {
		      ((PData)planet).writeTo(lsbw[j]);
		      int checkSum = lsbw[j].getAndResetPartial();
		      if (firstPlayer) {
			  control.writeInt(checkSum);
			  firstPlayer=false;
		      }
		  }
	      }
	  } else {
	      control.writeInt(0);
	  }
      }
    } catch (IOException ioe) {
      crash(11, "Failed to write planet "+i+" to file; "+ioe);
    }

    // add signature
    for (i=0;i<races;i++) {
      player=(short)(i+1);
      if (lsbw[player]!=null) try {
	if (get()==RstOrTrn.RST) signature=gen[player].getDisSig();
	else signature=gen[player].getDatSig();
	lsbw[player].writeString(signature);
	lsbw[player].close();
	int pc=gen[player].getPlanetChecksum();
	gen[player].setPlanetChecksum(pc+lsbw[player].getChecksum());
      } catch (Exception ioe) {
	crash(11, "Failed to finish pdata??.dat: "+ioe);
      }
    }

    // write starbases to bdata.dis/dat
    lsbw=new LSBWriter[races+1];
    BData bdata;
    short[] basecount=new short[races+1];
    for (j=1;j<=races;j++) basecount[j]=0; // silences compiler
    for (i=1;i<=planets;i++) {
      bdata=getBData(i);
      if (bdata!=null) basecount[bdata.getOwner()]++;
    }

    try {
      for (j=1;j<=races;j++) {
	if (gen[j]!=null) {
	  outfile=ofo.open("bdata"+j+(get()==RstOrTrn.RST?".dis":".dat"));
	  lsbw[j]=new LSBWriter(outfile);
	  lsbw[j].writeShort(basecount[j]);
	  lsbw[j].getAndResetPartial();
	}
      }
      for (i=1;i<=planets;i++) {
	bdata=getBData(i);
	if (bdata!=null) {
	  int owner=bdata.getOwner();
	  bdata.writeBaseTo(lsbw[owner]);
	  control.writeInt(lsbw[owner].getAndResetPartial());
	} else {
	  control.writeInt(0);
	}
      }
    } catch (IOException ioe) {
      crash(11, "Failed to write base "+i+" to file; "+ioe);
    }

    // add signature
    for (i=0;i<races;i++) {
      player=(short)(i+1);
      if (lsbw[player]!=null) try {
	if (get()==RstOrTrn.RST) signature=gen[player].getDisSig();
	else signature=gen[player].getDatSig();
	lsbw[player].writeString(signature);
	lsbw[player].close();
	int pc=gen[player].getBaseChecksum();
	gen[player].setBaseChecksum(pc+lsbw[player].getChecksum());
      } catch (Exception ioe) {
	crash(11, "Failed to finish bdata??.dat: "+ioe);
      }
    }

    // close control.dat
    try {
      control.writeShort(0); // two last bytes
      control.close();
    } catch (IOException ioe) {
      crash(11, "Could not finish control.dat: "+ioe);
    }

    // write messages
    try {
      if (get()==RstOrTrn.RST) {
	MDataGen mdatagen=new MDataGen(ofo); // hackerdiehack...
	mdatagen.saveMData(this);
      } else {
	MessGen messgen=new MessGen(ofo);
	messgen.saveMess(this);
      }
    } catch (IOException ioe) {
      crash(11, "Error while writing messages: "+ioe);
    }

    if (isRst()) {
      // write target??.dat
      game.report("Writing auxiliary Tim-files");
      lsbw=new LSBWriter[races+1];
      int[] targetcount=new int[races+1];
      for (j=1;j<=races;j++) targetcount[j]=0;
      for (i=1;i<=ships;i++) {
	ship=getShip(i);
	if (ship!=null) {
	  for (j=1;j<=races;j++) {
	    if (ship.isSeenBy(j) && (ship.getOwner() != j) && (ship.hulltype != UNKNOWN)) targetcount[j]++;
	  }
	}
      }
      try {
	for (j=1;j<=races;j++) {
	  if (getGen(j)!=null) { // isPlaying()...
	    outfile=ofo.open("target"+j+".dat");
	    lsbw[j]=new LSBWriter(outfile);
	    lsbw[j].writeShort(targetcount[j]);
	    lsbw[j].getAndResetPartial();
	  }
	}
	for (i=1;i<=ships;i++) {
	  ship=getShip(i);
	  if (ship!=null) {
	    for (j=1;j<=races;j++) {
	      if (ship.isSeenBy(j) && (ship.getOwner() != j) && (ship.hulltype != UNKNOWN)) { // isTarget
		Assert.assert(lsbw[j]!=null);
		ship.writeTargetTo(lsbw[j]);
	      }
	    }
	  }
	}
	for (j=1;j<=races;j++) {
	  if (lsbw[j]!=null) {
	    signature=gen[j].getDatSig(); // dis data, dat file. Really.
	    lsbw[j].writeString(signature);
	    lsbw[j].close();
	  }
	}
      } catch (IOException ioe) {
	crash(11, "Error while writing target??.dat: "+ioe);
      }

      // write shipxy??.dat
      lsbw=new LSBWriter[races+1];
      try {
	for (j=1;j<=races;j++) {
	  if (getGen(j)!=null) { // isPlaying()...
	    outfile=ofo.open("shipxy"+j+".dat");
	    lsbw[j]=new LSBWriter(outfile);
	  }
	}
	for (i=1;i<=ships;i++) {
	  ship=getShip(i);
	  for (j=1;j<=races;j++) {
	    if (lsbw[j]!=null) {
	      if ((ship!=null) && 
		  (ship.isSeenBy(j) || (ship.owner==j))) {
		ship.writeShipxyTo(lsbw[j]);
	      } else {
		if (lsbw[j]!=null) {
		  Ship.writeDummyShipxyTo(lsbw[j]);
		}
	      }
	    }
	  }
	}
	for (j=1;j<=races;j++) {
	  if (lsbw[j]!=null) {
	    signature=getGen(j).getDatSig();
	    lsbw[j].writeString(signature);
	    lsbw[j].close();
	  }
	}
      } catch (IOException ioe) {
	crash(11, "Error while writing shipxy??.dat: "+ioe);
      }

      // write init.tmp
      try {
	outfile=ofo.open("init.tmp");
	LSBWriter lw=new LSBWriter(outfile);
	for (player=1;player<=races;player++) {
	  if (getGen(player)!=null) lw.writeShort(1);
	  else lw.writeShort(0);
	}
	lw.close();
      } catch (IOException ioe) {
	crash(11, "Problem while writing init.tmp");
      }

      // write VCR's
      if (get()==RstOrTrn.RST) try {
	  int[] count=new int[races+1];
	  // count VCR's
	  figureVcrs(true,count,null); // figure means count
	  // open files for current players
	  for (player=1;player<=races;player++) {
	      if (getGen(player)!=null) {
		  outfile=ofo.open("vcr"+player+".dat");
		  lsbw[player]=new LSBWriter(outfile);
		  lsbw[player].writeShort(count[player]);
	      }
	  }
	  // write each VCR to the proper file(s)
	  figureVcrs(false,null,lsbw); // figure means write
	  // append datsig and close the files
	  for (player=1;player<=races;player++) {
	      if (getGen(player)!=null) {
		  signature=gen[player].getDatSig();
		  lsbw[player].writeString(signature);
		  lsbw[player].close();
	      }
	  }
      } catch (IOException ioe) {
	crash(11, "Something wrong: "+ioe);
      }

    } else { // we are TRN
      // finally, write gen??.dat, so that checksums are OK
      game.report("Writing gen??.dat, finally");
      for (player=1; player<=races; player++) {
	Gen gen=getGen(player);
	if (gen!=null) try {
	  outfile=ofo.open("gen"+player+".dat");
	  LSBWriter lw=new LSBWriter(outfile);
	  gen.writeGen(lw,player);
	  lw.close();
	} catch (IOException ioe) {
	  crash(11, "Couldn't save complete gen-file; you have a backup?");
	}
      }

      // now, write fizz.bin (uses checksums as well...)
      try {
	outfile=ofo.open("fizz.bin");
	LSBWriter lw=new LSBWriter(outfile);
	for (player=1; player<=races; player++) {
	  lw.writeInt((gen[player]!=null?gen[player].shipcs:0) + FIZZSHIP);
	  lw.writeInt((gen[player]!=null?gen[player].planetcs:0) + FIZZPLAN);
	  lw.writeInt((gen[player]!=null?gen[player].basecs:0) + FIZZBASE);
	}
	lw.writeInt(0);
	lw.writeByteArray(game.fizz);
	lw.close();
      } catch (IOException ioe) {
	crash(11, "Could not save FIZZ.BIN");
      }
    }
  }


    public boolean isActive(int player) {
	return (gen[player] != null);
    }

  // vga event throwers

  public void addPlanetListener(PlanetListener listener) {
    planetlistener.addElement(listener);
  }

  public void removePlanetListener(PlanetListener listener) {
    planetlistener.removeElement(listener);
  }

  protected void throwPlanetEvent(int planetid) {
    // preferably, this should be done in another Thread
    System.err.print("p");
    Planet planet = getPlanet(planetid);
    // Anything that's worth an event is worth a consistency check
    planet.getLocation().assertConsistency();
    for (int i=planetlistener.size()-1; i>=0; i--) {
      PlanetListener listener = (PlanetListener)planetlistener.elementAt(i);
      System.err.println("throwing event at "+listener);
      listener.planetChanged(new PlanetEvent(planetid));
    }
  }

  public void addShipListener(ShipListener listener) {
    shiplistener.addElement(listener);
  }

  public void removeShipListener(ShipListener listener) {
    shiplistener.removeElement(listener);
  }

  protected void throwShipEvent(int shipid) {
    System.err.print("s");
    Ship ship = getShip(shipid);
    ship.getLocation().assertConsistency();
    for (int i=shiplistener.size()-1; i>=0; i--) {
      ShipListener listener = (ShipListener)shiplistener.elementAt(i);
      listener.shipChanged(new ShipEvent(shipid));
    }
  }

  public void addBaseListener(BaseListener listener) {
    baselistener.addElement(listener);
  }

  public void removeBaseListener(BaseListener listener) {
    baselistener.removeElement(listener);
  }

  protected void throwBaseEvent(int baseid) {
    System.err.print("b");
    BData base = getBData(baseid);
    base.getLocation().assertConsistency();
    for (int i=baselistener.size()-1; i>=0; i--) {
      BaseListener listener = (BaseListener)baselistener.elementAt(i);
      listener.baseChanged(new BaseEvent(baseid));
    }
  }


  // put everything in the universe

  public void putGen(Gen g,int player) {
    gen[player]=g;
  }

    public void putPlanet(Planet p) {
	// warn("putPlanet("+p.id+")");
	p.setUniverse(this);
	// Put p into the hashtable
	String key=planethashprefix+p.id;
	thing.put(key,p);
	// put p into the planet-array
	planet[p.id]=p;
    }

    public void putBData(BData b) {
	putPlanet(b);
	return;
    }

    public void putShip(Ship s) {
	//warn("Universe.putShip("+s.id+") "+getTurnString());
	s.setUniverse(this);
	// Put s into the hashtable
	String key=shiphashprefix+s.id;
	thing.put(key,s);
	// put s into the ship-array
	ship[s.id]=s;
    }

  public void putVcr(Vcr v) {
    for (int i=0;i<vcr.size();i++) {
	Vcr v2=(Vcr)vcr.elementAt(i);
	if (v.equals(v2)) return;
    }
    // put v in the hashtable
    String key=vcrhashprefix+v.combatant1.id+"-"+v.combatant1.id;
    thing.put(key,v);
    // put v into the vcr-Vector
    vcr.addElement(v);
  }

    public void putMinefield(Minefield mf) {
	mf.setUniverse(this); // minefield MUST be in this universe already!
	String key=minefieldhashprefix+mf.id;
	thing.put(key,mf);
	if (minefield.size()<mf.id+1) minefield.setSize(mf.id+1);
	if (minefield.elementAt(mf.id)==null) // don't count more than once
	    minefieldcount++;
	minefield.setElementAt(mf,mf.id);
    }

    public void putExplosion(Explosion expl) {
	expl.setUniverse(this);
	String key=explosionhashprefix+expl.getShipID();
	thing.put(key,expl);
	explosion.addElement(expl);
    }

    public void putIonstorm(Ionstorm is) {
	is.setUniverse(this);
	String key=ionstormhashprefix+is.id;
	thing.put(key,is);
	if (ionstorm.size()<is.id+1) ionstorm.setSize(is.id+1);
	if (ionstorm.elementAt(is.id)==null) // don't count more than once
	    ionstormcount++;
	ionstorm.setElementAt(is,is.id);
    }

    public void putWormhole(Wormhole wh) {
	wh.setUniverse(this);
	String key=wormholehashprefix+wh.id;
	thing.put(key,wh);
	if (wormhole.size()<wh.id+1) wormhole.setSize(wh.id+1);
	if (wormhole.elementAt(wh.id)==null) // don't count more than once
	    wormholecount++;
	wormhole.setElementAt(wh,wh.id);
    }

    public void putMessage(Message m) {
	// if (isEditable()) {
	    message.addElement(m);
	//    game.unlock();
	// }
    }

  /**
   * @param pconf Should be Config conf
   */
  public void putConfig(PConfig conf) {
    config=conf;
  }

  public void putAlliance(Alliance all,int player) {
    alliance[player]=all;
  }


  // get what you need from the universe

  public Game getGame() {
    return game;
  }

  public int getTurnNumber() {
    return turnnr;
  }

  public int get() {
    return from;
  }

  public boolean isRst() {
    return get() == RstOrTrn.RST;
  }

  public boolean isTrn() {
    return get() == RstOrTrn.TRN;
  }

    public boolean isEditable() {
	int current = getGame().getCurrentTurn();
	if ((getTurnNumber()==current) && (get()==RstOrTrn.TRN)) {
	    return game.couldGrabLock();
	}
	return false;
    }

  public Planet getPlanet(int planetid) {
    return planet[planetid];
  }

  public PData gimmePData(int id) {
     Planet planet = getPlanet(id);
     PData pdata;
     if (planet instanceof PData) pdata=(PData)planet;
     else pdata = new PData(this, planet);
     removePlanet(id); // removes it only when available
     putPlanet(pdata);
     return pdata;
  }

  public BData getBData(int bdataid) {
    Planet p = planet[bdataid];
    if (p instanceof BData) return (BData)p;
    return null;
  }

  public Ship getShip(int shipid) {
    return ship[shipid];
  }

  public Gen getGen(int player) {
    if ((player>=1) && (player<=races)) return gen[player];
    return null;
  }

  public boolean atLeastPHostVersion(byte major, byte minor, char release) {
     for (int i=1; i<=races; i++) {
         if (gen[i] != null)
             return gen[i].atLeastPHostVersion(major, minor, release);
     }
     return false;
  }

  public String getTimestamp() {
    for (int i=1;i<=races;i++) {
      if (gen[i]!=null) return gen[i].getTimestamp();
    }
    return null;
  }

  public int getVcrCount() {
    return vcr.size();
  }

  /**
   * Get Vcr by Vcr-id.
   */
  public Vcr getVcr(int vcrid) {
    return (Vcr)vcr.elementAt(vcrid);
  }

  /**
   * Get Vcr by id's of ships and isplanet.
   */
  public Vcr getVcr(int id1,int id2,boolean isplanet) {
    for (int i=0;i<vcr.size();i++) {
      Vcr v=(Vcr)vcr.elementAt(i);
      if ((v.combatant1.id==id1) && 
	  (v.combatant2.id==id2) && 
	  v.isplanet==isplanet) return v;
    }
    return null;
  }

  public Alliance getAlliance(int player) {
    return alliance[player];
  }

  public int getMaxIonstormId() {
    return ionstorm.size()-1;
  }

  public int getIonstormCount() {
    return ionstormcount;
  }

  public Ionstorm getIonstorm(int ionstormid) {
    // return Object for key "Ion"+ionstormid from thing. Casting will succeed.
    return (Ionstorm)thing.get(ionstormhashprefix+ionstormid);
  }

  public int getMaxWormholeId() {
    return wormhole.size()-1;
  }

  public int getWormholeCount() {
    return wormholecount;
  }

  public Wormhole getWormhole(int wormholeid) {
    // return Object for key "W"+wormholeid from thing. Casting will succeed.
    return (Wormhole)thing.get(wormholehashprefix+wormholeid);
  }

  public int getMessageCount() {
    return message.size();
  }

  public Message getMessage(int nr) {
    if ((nr<0) || (nr>=message.size())) return null;
    return (Message)message.elementAt(nr);
  }

  public int getMaxMinefieldId() {
    int size=minefield.size();
    if (size==0) return -1;
    return size-1;
  }

  public int getMinefieldCount() {
    return minefieldcount;
  }

  /**
   * Get a Minefield (get it from the Hashtable)
   */
  public Minefield getMinefield(int minefieldid) {
    // return Object for key "M"+minefieldid from thing. Casting will succeed.
    return (Minefield)thing.get(minefieldhashprefix+minefieldid);
  }

  public int getExplosionCount() {
    return explosion.size();
  }

  public Explosion getExplosionAt(int index) {
    return (Explosion)explosion.elementAt(index);
  }

  public Explosion getExplosion(int explosionid) {
    // return Object for key "Expl"+explosionid from thing. Casting will succeed
    return (Explosion)thing.get(explosionhashprefix+explosionid);
  }

  public PConfig getConfig() {
    return config;
  }

  public Enumeration getLocatedObjects() {
    return thing.elements();
  }

  // Check for availabilty of some particular items

  public boolean containsPlanet(int planetid) {
    return planet[planetid]!=null;
  }

  public boolean containsShip(int shipid) {
    return ship[shipid]!=null;
  }

  public boolean containsIonstorm(int is_id) {
    return thing.contains(ionstormhashprefix+is_id);
  }

  public boolean containsWormhole(int whid) {
    return thing.contains(wormholehashprefix+whid);
  }

  // Remove some particular parts from the universe

  /**
   * Removes a planet. If it does not exist (yet), nothing happens.
   */
  public void removePlanet(int planetid) {
    String key=planethashprefix+planetid;
    if (thing.contains(key)) thing.remove(key);
    planet[planetid]=null;
  }

  /**
   * Removes a ship. If it does not exist (yet), nothing happens.
   */
  public void removeShip(int shipid) {
    String key=shiphashprefix+shipid;
    if (thing.contains(key)) thing.remove(key);
    ship[shipid]=null;
  }

  /**
   * Removes a minefield. If it does not exist (yet), nothing happens.
   */
  public void removeMinefield(Minefield mf) {
    String key=minefieldhashprefix+mf.id;
    if (thing.contains(key)) {
      thing.remove(key);
      minefield.removeElement(mf);
    }
  }

    public void removeMessageAt(int i) {
	if (i<0) return;
	if (i>=message.size()) return;
	message.removeElementAt(i);
    }

    public Location getLocationAt(int x, int y) {
	return mapsearch.getLocationAt(x, y);
    }

  public Location getNearestTo(int x,int y) {
    return mapsearch.getNearestTo(x,y);
  }

  public Enumeration getLocations() {
    return mapsearch.getLocations();
  }

  public String getTurnString() {
    return "turn "+getTurnNumber()+" ("+(isRst()?"rst":"trn")+")";
  }

  /**
   * Host current turn using own host methods.
   * Not accurate, but fast.
   * @author Rutger
   */
  public void hostSelf() {
    // System.err.println("hostSelf()");
    turnnr += 1;
    from = RstOrTrn.RST;
    vcr = new Vector();
    // Phase 1
    // TODO
    // Phase 2
    // Minefield decay
    for (int i=1; i<minefield.size(); i++) {
      Minefield m = (Minefield)minefield.elementAt(i);
      if (m != null)
	m.setUnits(m.getUnitsNT());
    }
    // Mine laying
    for (int i=1; i<ship.length; i++) {
      Ship s = ship[i];
      if (s != null) {
	int units = s.getUnitsLaid();
	if ((units != UNKNOWN) && (units > 0)) {
	  Location l;// = getLocation(s.getPoint());  incorrect interpretation
	  l = getNearestTo(s.getX(), s.getY()); // wrong as well... [Kero]
	  // REALLY need a proper getAllOverlappingMinefields(x, y)
	  Vector minefields = l.getMinefields();
	  boolean web = Mission.missionLaysWebMines(s.getMission(),
						    s.getConfig(),
						    s.getOwner());
	  int owner = s.getOwnerLaid();
	  // Find minefield to lay mines in
	  for (int j=1; j<minefields.size(); j++) {
	    Minefield m = (Minefield)minefields.elementAt(j);	  
	    if ((m != null) && (m.getOwner() == owner) && (m.isWeb() == web)) {
	      // Found!
	      // No, which id is the minefield? [Kero]
	      m.setUnits(m.getUnits() + units);
	      s.setTorpedoes(s.getTorpedoes() - s.getTorpsLaid());
	    }
	  }
	  // TODO make new field when none found
	}
      }
    }
    // Mine scooping + sweeping
    for (int i=0; i<ship.length; i++) {
      // TODO
    }
    // Ship movement
    for (int i=0; i<ship.length; i++) {
      Ship s = ship[i];
      if (s != null) {
	int neutr = s.getNeutrNT();
	//MyPoint move = s.getMovement(s.dx, s.dy);
	s.setPoint(s.getPositionNT());
	//s.setWaypoint(s.getWaypoint().sub(move));
	s.setNeutronium(neutr);
      }
    }
  }

  /**
   * Host the current turn, and return the hosted universe
   */
  public void hostPhost() {
    JavaPDK.init();
    // TODO: host :)
    JavaPDK.free();
  }

  public Player getPlayer(int p) {
    if ((p>=1) && (p<=races)) return player[p];
    return null;
  }

  public Player getPlayerByAdjective(String adj) {
    Player result = null;
    for (int i=1; i<=races; i++) {
      if (player[i] != null && player[i].getAdjective().equals(adj)) {
	if (result != null) {
	  System.err.println("Found more than one race for adj `"+adj+"'");
	  return null;
	}
	result = player[i];
      }
    }
    if (result == null) {
      for (int i=1; i<=races; i++) {
	if (player[i] != null && player[i].getAdjective().startsWith(adj)) {
	  if (result != null) {
	    System.err.println("Found more than one race for partial adj `"+adj+"'");
	    return null;
	  }
	  result = player[i];
	}
      }
    }
    return result;
  }
}
