// File: UtilGen.java

package jvpc.vga;

import java.util.Hashtable;
import java.io.*;
import kero.util.*;

/**
 * Generator for utilX.dat files. It generates lots of sorts of objects, all
 * more-than-30 that are passed on with utilX.dat. With the compliments to the
 * PHost team (it's <EM>much</em> better than scanning messages).
 *
 * an enum is 16 bits, i.e. a readShort() should be used.
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since     30 XI 1998
 * @version   21 XII 1999
 */
public class UtilGen
  extends Generator
{
  Game game;
  boolean numberread=false;

  public UtilGen(Game g,InputStream is) {
    super(is);
    game=g;
  }

  public int getTurnNumber() {
    Assert.assert(!numberread);
    try {
      int type=source.readShort();
      Assert.assert(type==13);
      // System.err.print(", "+type);
      int utillen=source.readShort();
      StringBuffer timestamp=source.readString(18);
      int turnnr=source.readShort();
      numberread=true;
      return turnnr;
    } catch (IOException ioe) {
      // forget it
      return UNKNOWN;
    }
  }

  public void generateAll(Universe u,int player) throws IOException {
    generateAll(u,false,player);
  }

  public void generateAll(Universe u,boolean maywrite,int player) throws IOException {
    short type,utillen;
    try {
      // read the remainings of record 13
      Assert.assert(numberread); // proper ordening in Game guaranteed
      u.getGen(player).readPHostControl(source,player);
      while (source.available()>0) {
	try {
	  type=source.readShort();
	  // System.err.print(" "+type+",");
	  utillen=source.readShort();
          // System.err.println("Scanning util.dat record type "+type+", length "+utillen);
	  switch (type) {
	  case 0:
	  case 46: { // Mine scan (46 for ID >=500, if ...)
	      int id = source.readShort();
	      Minefield minefield = u.getMinefield(id);
	      boolean hook_it = false;
	      if (minefield == null) {
		  minefield = new Minefield(u, id);
		  hook_it = true;
	      }
	      minefield.readFromUtil(source);
	      if (hook_it) u.putMinefield(minefield);
	      break;
	  }
	  case 1: { // Explosion
	    Explosion explosion=new Explosion(u);
	    explosion.readFromUtil(source);
	    u.putExplosion(explosion);
	    break;
	  }
	  case 2: { // Mine Hit
	    source.skip(utillen);
	    break;
	  }
	  case 3: { // Dark Sense
	    source.skip(utillen);
	    // get planet_id
	    // upgradePlanetToPDataIfNecessary(u, planet_id)
	    // owner = getShort();
	    // neutr getInt();
	    // trit getInt();
	    // dur getInt();
	    // molyb getInt();
	    // MC getInt();
	    // hasbase getShort(); // how to administrate this?
	    break;
	  }
	  case 4: { // Super Spy
	      int id = source.readShort();
	      PData p = u.gimmePData(id);
	      p.struct[Build.MINE] = source.readShort();
	      p.struct[Build.FACTORY] = source.readShort();
	      p.struct[Build.DEFENSE] = source.readShort();
	      p.setFC(source.readString(3).toString());
	      p.setNeutronium(source.readInt());
	      p.setTritanium(source.readInt());
	      p.setDuranium(source.readInt());
	      p.setMolybdenium(source.readInt());
	      p.setMC(source.readInt());
	      p.setSupplies(source.readInt());
	      break;
	  }
	  case 5: { // Exploration
	      int id = source.readShort();
	      PData p = u.gimmePData(id);
	      p.setTemp(source.readShort());
	      p.setOwner(source.readShort());
	      p.setColClans(source.readInt() / 100);
	      source.readShort(); // has base (but have no base?)
	      break;
	  }
	  case 6: { // Sensor Sweep
            int id = source.readShort();
            Planet p = u.getPlanet(id);
            p.setOwner(source.readShort());
            source.readShort(); // activity
	    break;
	  }
	  case 7: { // Battle Result
	    int id1=source.readShort();
	    int id2=source.readShort();
	    boolean isplanet=(source.readShort()!=0);
	    Vcr vcr=u.getVcr(id1,id2,isplanet);
	    vcr.readFromUtil(source);
	    break;
	  }
	  case 8: { // Meteor Strike
	    source.skip(utillen);
	    break;
	  }
	  case 9: { // Meteor Shower
	    source.skip(utillen);
	    break;
	  }
	  case 10: { // Ship Target (ship from ally or enemy)
	    Ship target;
	    int id=source.readShort();
	    if (u.containsShip(id)) {
	      target=u.getShip(id);
	      target.readTargetFrom(source,player);
	    } else {
	      target=new Ship(u,id);
	      target.readTargetFrom(source,player);
	      u.putShip(target);
	    }
	    break;
	  }
	  case 11: { // Base Target
	    source.skip(utillen);
	    break;
	  }
	  case 12: { // Planet Target (planet from ally)
	    source.skip(utillen);
	    break;
	  }
	  case 13: { // Control Record
	    crash(1002, "Can not encounter (second?) control-record in util??.dat");
	    source.skip(utillen);
	    break;
	  }
	  case 14: { // Wormhole Scan
	    Wormhole wormhole=new Wormhole(u);
	    wormhole.readFrom(source);
	    if (u.containsWormhole(wormhole.getId())) {
	      game.report("Wormhole "+wormhole.getId()+" read before");
	      // get it and compare it; should be the same
	    } else {
	      u.putWormhole(wormhole);
	    }
	    break;
	  }
	  case 15: { // Wormhole Travel
	    source.skip(utillen);
	    break;
	  }
	  case 16: { // Ship Recycled
	    source.skip(utillen);
	    break;
	  }
	  case 17: { // Ion Storm (yeah, right)
	    source.skip(utillen);
	    break;
	  }
	  case 18: { // Ship Colonized
	    source.skip(utillen);
	    break;
	  }
	  case 19: { // Ship Surrendering (either to or from you)
	    source.skip(utillen);
	    break;
	  }
	  case 20: { // Ship Construction
	    source.skip(utillen);
	    break;
	  }
	  case 21: { // Ship Trade (either old or new owner)
	    source.skip(utillen);
	    break;
	  }
	  case 22: { // Alliance Status Report
	    Alliance alliance=new Alliance(u,player);
	    alliance.readFrom(source);
	    u.putAlliance(alliance,player);
	    break;
	  }
	  case 23: { // Bioscan Report
	    source.skip(utillen);
	    break;
	  }
	  case 24: { // Glory Device Exploded
	    source.skip(utillen);
	    break;
	  }
	  case 25: { // Damage from Glory Device (both to ship- and GD-owner)
	    source.skip(utillen);
	    break;
	  }
	  case 26: { // Boarding Party
	    source.skip(utillen);
	    break;
	  }
	  // case 27: No longer used (pconfig.src transfer)
	  case 28: { // Ground Attack
	    source.skip(utillen);
	    break;
	  }
	  case 29: { // Minefields Exploding
	    source.skip(utillen);
	    break;
	  }
	  case 30: { // End of PHOST info
	    // no additional data
	    break;
	  }
	  case 31: { // Minefield Scooping
	    source.skip(utillen);
	    break;
	  }
	  case 32: { // Pillage
	    source.skip(utillen);
	    break;
	  }
	  case 33: { // General Object
	    int id=source.readShort();
	    GeneralObject go=new GeneralObject(u,id);
	    go.readFrom(source,utillen);
	    break;
	  }
	  case 34: { // File Transfer
	    String filename=source.readString(12).toString();
	    filename=MyString.stripTrailingZeroes(filename).toString();
	    byte flags=source.readByte();
	    StringBuffer contents=source.readString(utillen-13);
	    System.err.print("Encountered file ``"+filename+"''");
	    if ((filename.equals("pconfig.src")) ||
		(filename.equals("race.nm")) ||
		(filename.equals("xtrfcode.txt"))) {
	      // one of the three expected files
	      if (maywrite) {
		FileOutputStream fos=u.game.ofo.open(filename);
		if (fos!=null) {
		  PrintWriter ps=new PrintWriter(fos);
		  ps.print(contents);
		  ps.close();
		  System.err.println(" and wrote it.");
		} else {
		  System.err.println(", but could not write it.");
		}
	      } else {
		// may not write; do parse contents, though...
		System.err.println();
	      }
	    } else {
	      System.err.println(", which is unexpected.");
	    }
	    break;
	  }
	  case 35: { // Cloak Failure
	    source.skip(utillen);
	    break;
	  }
	  case 36: { // Anti Cloak Detection
	    source.skip(utillen);
	    break;
	  }
	  case 37: { // Remote Control Ships
	    source.skip(utillen);
	    break;
	  }
	  case 38: { // Activity Level Report
	    source.skip(utillen);
	    break;
	  }
	  case 39: { // Build Queue Report
	    source.skip(utillen);
	    break;
	  }
	  case 40: { // Web Drain Complete
	    int shipid=source.readShort();
	    report("Ship "+shipid+" out of fuel due to webdrain");
	    Ship ship=u.getShip(shipid);
	    if (ship!=null) ship.setWebDrainComplete();
	    break;
	  }
	  case 41: { // Rebel Ground Attack
	      source.skip(utillen);
	      break;
	  }
	  case 42: { // General Object Destruction
	      source.skip(utillen);
	      break;
	  }
	  case 43: { // Minefield Status
	      source.skip(utillen);
	      break;
	  }
	  case 44: { // Failure
	      source.skip(utillen);
	      break;
	  }
	  case 45: { // Planet Trade
	      source.skip(utillen);
	      break;
	  }
	  // 46: Mine Scan (high ID), see record type 0
	  case 47: { // Nonexistant Planets
	      source.skip(utillen);
	      break;
	  }
	  case 48: { // PAL Summary
	      source.skip(utillen);
	      break;
	  }
	  default:
	    System.err.println("Scanned unknown util??.dat record type "+type);
	    source.skip(utillen);
	    break;
	  }
	} catch(IOException ioe) {
	  game.report("UtilGen.generateAll() -- Oops");
	}
      }
    } catch (IOException ioe) {
      // end-of-file, probably, ignore
    }

    copyFileToDir(InputFileOpener.getLastOpenedFile().getAbsolutePath(), u.game.workdir.toString()+u.getTurnNumber());
  }
}
