// File: VCR.java

package jvpc.vga;

import java.io.*;

/**
 * Is a VCR (``video rcording'') of a VGA-Planets battle. It contains data of
 * the ships/ship plus planet that enact in the battle. Some seed sets the
 * random generator, but I have no clue as to which generator that would be,
 * nor how the resulting numbers are used.
 *
 * <P>For PHost, the battle-info from util??.dat is put into an instance as
 * well, for which the boolean isplanet is set. The (x,y) for Located also
 * comes from this.
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since     25 II 1999
 * @version   2 III 1999
 */
public class Vcr
  extends Located
{
  /**
   * Part of a battle that is stored for both of the enactors (combatants).
   */
  public class Combatant
    implements VGAPlanets
  {
    // Tim's fields; numbers are valid just <EM>before</em> the battle
    int mass;
    StringBuffer name;
    int damage;
    int crew,id,owner,hubba,picture; // hubba is unknown
    int beamtype,beams;
    int torptype,torps,launchs,bays,fighters;
    int shield;
    // PHost fields; valid just <EM>after</em> the battle
    int damageafter,torpsafter,fightersafter;
    int outcome;

    public boolean equals(Object o) {
      if (this==o) return true;
      if (!(o instanceof Combatant)) return false;
      Combatant comb;
      comb=(Combatant)o;
      return ((id==comb.id) &&
	      (owner==comb.owner) &&
	      (damage==comb.damage) &&
	      (shield==comb.shield) &&
	      (torps==comb.torps) &&
	      (fighters==comb.fighters));
    }

    public void readPartFrom(LSBReader source) throws IOException {
      name=source.readString(SHIPNMSIZE);
      damage=source.readShort();
      crew=source.readShort();
      id=source.readShort();
      owner=source.readByte();
      hubba=source.readByte();
      picture=source.readShort();
      beamtype=source.readShort();
      beams=source.readShort();
      bays=source.readShort();
      torptype=source.readShort();
      if (torptype!=0) {
	torps=source.readShort();
	fighters=0;
      } else {
	if (bays!=0) {
	  fighters=source.readShort();
	  torps=0;
	} else {
	  source.readShort(); // skip, in fact; should be a zero
	  fighters=0;
	  torps=0;
	}
      }
      launchs=source.readShort();
    }

    public void writePartTo(LSBWriter sink) throws IOException {
      sink.writeString(name,SHIPNMSIZE);
      sink.writeShort(damage);
      sink.writeShort(crew);
      sink.writeShort(id);
      sink.writeByte(owner);
      sink.writeByte(hubba);
      sink.writeShort(picture);
      sink.writeShort(beamtype);
      sink.writeShort(beams);
      sink.writeShort(bays);
      sink.writeShort(torptype);
      if (torptype!=0) {
	sink.writeShort(torps);
      } else {
	if (bays!=0) {
	  sink.writeShort(fighters);
	} else {
	  sink.writeShort(0);
	}
      }
      sink.writeShort(launchs);
    }

    public int getId() { return id; }
    public int getOwner() { return owner; }
    public int getOutcome() { return outcome; }
  }

  boolean isphostbattle=false;
  int seed,unknown;
  boolean isplanet;
  Combatant combatant1,combatant2;

  public Vcr(Universe u) {
    super(u);
    combatant1=new Combatant();
    combatant2=new Combatant();
  }

    public boolean equals(Object o) {
      if (this==o) return true;
      if (!(o instanceof Vcr)) return false;
      Vcr vcr;
      vcr=(Vcr)o;
      return ((seed==vcr.seed) &&
	      (unknown==vcr.unknown) &&
	      (isplanet==vcr.isplanet) && 
	      (combatant1.equals(vcr.combatant1)) &&
	      (combatant2.equals(vcr.combatant2)));
    }

  public void readFrom(LSBReader source) throws IOException {
    seed=source.readInt();
    unknown=source.readShort();
    isplanet=(source.readShort()!=0);
    combatant1.mass=source.readShort();
    combatant2.mass=source.readShort();

    combatant1.readPartFrom(source);
    combatant2.readPartFrom(source);

    combatant1.shield=source.readShort();
    combatant2.shield=source.readShort();
  }

  public void readFromUtil(LSBReader source) throws IOException {
    isphostbattle=true;
    combatant1.owner=source.readShort();
    combatant2.owner=source.readShort();
    combatant1.damageafter=source.readShort();
    combatant2.damageafter=source.readShort();
    combatant1.torpsafter=source.readShort();
    combatant2.torpsafter=source.readShort();
    combatant1.fightersafter=source.readShort();
    combatant2.fightersafter=source.readShort();
    combatant1.outcome=source.readShort();
    combatant2.outcome=source.readShort();
    x = source.readShort();
    y = source.readShort();
    if (getUniverse().atLeastPHostVersion((byte)3, (byte)4, 'b')) {
	warn("reading PHost >= 3.4b seed...");
	seed = source.readShort();
    }
    if (true) {
      System.err.print("#"+combatant1.id+" ("+combatant1.name+") <-> ");
      System.err.println("#"+combatant2.id+" ("+combatant2.name+")");
      System.err.print("outcome: ");
      switch (combatant1.outcome) {
      case 0: System.err.println("ship 1 won"); break;
      case 1: System.err.println("ship 1 captured"); break;
      case 2: System.err.println("ship 1 destroyed"); break;
      case 3: System.err.println("ship 1 out of ammo"); break;
      }

      System.err.println("1: damage: "+combatant1.damageafter+", fish: "+combatant1.fightersafter);
      System.err.println("2: damage: "+combatant2.damageafter+", fish: "+combatant2.fightersafter);
    }
    if (universe.getGen(combatant1.owner)==null) {
      // System.err.println("Copying info for first ship "+combatant1.id);
      Ship ship=universe.getShip(combatant1.id);
      /* if a ship does not exist or was created for another battle
       * copy info. If ship exploded and was created for another battle,
       * remove it.
       */
      if (ship==null) {
	ship=new Ship(universe,combatant1.id);
	universe.putShip(ship); // seenby never set => ship is never saved
      }
      // ship.copyFromBattle(combatant1);
    }
    if (!isplanet) {
      if (universe.getGen(combatant2.owner)==null) {
	// System.err.println("Copying info for second ship "+combatant2.id);
	Ship ship=universe.getShip(combatant2.id);
	if (ship==null) {
	  ship=new Ship(universe,combatant2.id);
	  universe.putShip(ship); // seenby never set => ship is never saved
	}
	// ship.copyFromBattle(combatant2);
      }
    }
  }

  public void writeTo(LSBWriter sink) throws IOException {
    sink.writeInt(seed);
    sink.writeShort(unknown);
    sink.writeShort(isplanet?1:0);
    sink.writeShort(combatant1.mass);
    sink.writeShort(combatant2.mass);

    combatant1.writePartTo(sink);
    combatant2.writePartTo(sink);

    sink.writeShort(combatant1.shield);
    sink.writeShort(combatant2.shield);
  }

  public Combatant getCombatant(int nr) {
    if (nr<=0) return null;
    if (nr>=3) return null;
    if (nr==1) return combatant1;
    return combatant2;
  }
}
