// File: Wormhole.java

package jvpc.vga;

import java.io.*;
import java.util.*;

/**
 * Represents a wormhole.
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since     30 XI 1998
 * @version   15 I 1999
 */
public class Wormhole
  extends Located
{

  public Vector getTypes() {
    Vector res = super.getTypes();
    res.addElement("id");
    res.addElement("radius");
    res.addElement("mass");
    return res;
  }

  public Vector getTypeMethods(String typeName) {
    Vector res = super.getTypeMethods(typeName);
    if ("id".equals(typeName)) {
      res.addElement("getId");
    } else if ("radius".equals(typeName)) {
      res.addElement("getRadius");
    } else if ("mass".equals(typeName)) {
      res.addElement("getMass");
    }
    return res;
  }

  short id,mass,stability;

  public Wormhole() {
    super(null);
  }

  public Wormhole(Universe u) {
    super(u);
    universe=u;
    id=mass=stability=(short)UNKNOWN;
  }

  public Object clone() throws CloneNotSupportedException {
    return (Wormhole)(super.clone());
  }

  public void readFrom(LSBReader source) throws IOException {
    setX(source.readShort());
    setY(y=source.readShort());
    mass=source.readShort();
    stability=source.readShort();
    id=source.readShort();
  }

  public short getId() {
    return id;
  }

  public short getMass() {
    return mass;
  }

  public int getRadius() {
    int wep = getConfig().wrmEntryPowerX100();
    return (int)(Math.pow(mass,wep / 100.0) / 2);
  }

  public String toString() {
    return("Wormhole #" + getId()+ "@("+x+","+y+")");
  }

  public String getKey() {
    return wormholehashprefix + getId();
  }
}
