// File: XYPlanGen.java

package jvpc.vga;

import java.util.Hashtable;
import java.io.*;

/**
 * Generates <CODE>Planet</code>objects.
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since     26 VII 1998
 * @version   21 XII 1999
 */
public class XYPlanGen extends Generator {
  /**
   * Constructs an Planet generator.
   * @param is The <CODE>InputStream</code> from which the <CODE>source</code>
   * can be created.
   */
  public XYPlanGen(InputStream is) {
    super(is);
  }

  /**
   * Generates all <CODE>Planet</code> objects found in <CODE>pis</code>.
   * The planets are stored in a hashtable. First, a check is made whether the
   * planet-to-be-constructed might already be in the hashtable. If so, its
   * x and y field are changed.
   * @param u Universe for which we are gegerating all objects.
   */
  public void generateAll(Universe u) {
    int i;
    Planet planet;
    for (i=1;i<=planets;i++) {
      planet=u.getPlanet(i);
      if (planet==null) {
	planet=new Planet(u);
	planet.id=i; // so that universe.putPlanet() understands
      }
      try {
	planet.readXYFrom(source);
	source.skip(DUMMYINXYPLAN); // Why, Tim?
      } catch (IOException ioe) {
	crash(11, "Error while generating XY coordinates: "+ioe);
      }
      if ((planet.getX()!=UNKNOWN) && !u.containsPlanet(i)) {
	u.putPlanet(planet);
      }
    }
  }
}
