/**
 * Plug-in for Java VGA-Planets clients, that enables a permanent network
 * connection to the hosting service. The Host will send warnings when the host
 * will be ran. In particular, "host 30" will be sent, which means the host
 * will be ran in 30 seconds. The client should result make a TRN and send it
 * back over the same line.
 */

package jvpc.vga.blitz;

import java.io.*;
import java.net.*;

public class Blitz {
    Socket socket;
    Thread thread;

    public Blitz(String hostname, int port)
	throws UnknownHostException, IOException
    {
	socket = new Socket(hostname, port);
	socket.setSoTimeout(0);
	thread = new BlitzThread(socket);
	thread.run();
    }

    public static void main(String arg[]) {
	Blitz blitz;
	try {
	    blitz = new Blitz("localhost", 9000);
	} catch (Exception e) {
	    System.err.println(e);
	}
    }
}

class BlitzThread extends Thread {
    InputStream is;
    OutputStream os;
    public BlitzThread(Socket socket) throws IOException {
	this.is = socket.getInputStream();
	this.os = socket.getOutputStream();
	os.write("name Java Client\n".getBytes());
	os.write("games\n".getBytes());
	os.write("join blitz 3\n".getBytes());
    }

    public void run() {
	while (true) {
	    StringBuffer buf = new StringBuffer();
	    int c = '\0';
	    try {
		while ((c = is.read()) != '\n') {
		    if (c < 0) return;
		    buf.append((char)c);
		}
		String str = new String(buf);
		int index = str.indexOf(' ');
		if (index < 0) return; // Odd line from host, bail out
		String command = str.substring(0, index);
		if ("host".equals(command)) {
		    System.err.println("Submitting TRN...");
		    /*   game.setEditable(false);
		     *   game.saveTurn();
		     *   maketurn(game);
		     */
		    File f = new File("player"+3+".trn");
		    int size = (int)f.length();
		    byte[] trn = new byte[size];
		    FileInputStream fis = new FileInputStream(f);
		    fis.read(trn);
		    fis.close();
		    os.write(("trn blitz 3 "+size+"\n").getBytes());
		    os.write(trn);
		} else if ("rst".equals(command)) {
		    System.err.println("Receiving RST...");
		    int size = Integer.parseInt(str.substring(index+1));
		    byte[] rst = new byte[size];
		    is.read(rst);
		    File f = new File("player"+3+".rst");
		    FileOutputStream fos = new FileOutputStream(f);
		    fos.write(rst);
		    fos.close();
		    /*   unpack(game);
		     *   game.setEditable(true);
		     */
		}
	    } catch (IOException ioe) {
		System.err.println(ioe);
	    }
	}
    }
}
