// File: ActionPlanetMaxStructures

package jvpc.vpcheck;

import java.util.*;
import kero.util.*;
import jvpc.vga.*;

/**
 * Maximizes structures (mines, factories and defense posts) on planets, but
 * only if that type of structure was maximized in the previous turn.
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since     2 II 2001
 * @version   2 II 2001
 */
public class ActionPlanetMaxStructures extends Action {

    public boolean action(Vector objects, Vector args, StringBuffer msg) {
	msg.setLength(0); // why does that happen here? Bloody input!

	Assert.assert(objects.size() == 1);
	if (!(objects.elementAt(0) instanceof PData)) return false;
	PData p = (PData)objects.elementAt(0);

	int turnNumber = p.getUniverse().getTurnNumber();
	if (turnNumber == 1) return false;

	Game g = p.getGame();
	Universe u = g.getUniverse(turnNumber - 1);
	if (u == null) return false;

	Planet prevPlanet = u.getPlanet(p.getId());
	if (prevPlanet == null) return false;
	if (!(prevPlanet instanceof PData)) return false;

	PData prevP = (PData)prevPlanet;
	boolean builtStruct = false;
	int prevMaxMines = prevP.getMaxMines();
	if ((prevMaxMines != 0) && (prevP.getMines() >= prevMaxMines)) {
	    if (p.getMines() < p.getMaxMines()) {
		addToMsg(msg, "Raising #mines from "+p.getMines()+
			 " to "+p.getMaxMines());
		p.buildMines(p.getMaxMines() - p.getMines());
		builtStruct = true;
	    }
	}
	int prevMaxFactories = prevP.getMaxFactories();
	if ((prevMaxFactories != 0) && (prevP.getFactories() >= prevMaxFactories)) {
	    if (p.getFactories() < p.getMaxFactories()) {
		addToMsg(msg, "Raising #factories from "+p.getFactories()+
			 " to "+p.getMaxFactories());
		p.buildFactories(p.getMaxFactories() - p.getFactories());
		builtStruct = true;
	    }
	}
	int prevMaxDefense = prevP.getMaxDefense();
	if ((prevMaxDefense != 0) && (prevP.getDefense() >= prevMaxDefense)) {
	    if (p.getDefense() < p.getMaxDefense()) {
		addToMsg(msg, "Raising #defense from "+p.getDefense()+
			 " to "+p.getMaxDefense());
		p.buildDefense(p.getMaxDefense() - p.getDefense());
		builtStruct = true;
	    }
	}

	return builtStruct;
    }
}
