// File: ActionShipWaypointClear

package jvpc.vpcheck;

import java.util.*;
import kero.util.*;
import jvpc.vga.*;

/**
 * Clear waypoint when it is inside a gravity well (i.e, previous turn you used
 * the well to reach the planet).
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since     24 V 1999
 */
public class ActionShipWaypointClear extends Action {
  public boolean action(Vector objects, Vector args, StringBuffer msg) {
    Assert.assert(objects.size() == 1);
    Assert.assert(args.size() == 0);
    msg.setLength(0);
    Ship s = (Ship)objects.elementAt(0);

    int warp=s.getWarp();
    if ((warp==UNKNOWN) || (warp<=1)) return false; // dunno

    int dx=s.getDX(),dy=s.getDY();
    if ((dx==UNKNOWN) || (dy==UNKNOWN)) return false; // dunno
    if ((dx==0) && (dy==0)) return false; // waypoint already clear(ed)

    Universe u = s.getUniverse();
    Location loc = s.getLocation();
    // should be something like getOverlappingGravityWells()
    Planet p=loc.getPlanet();
    if (p==null) return false; // no planet, no warpwell, no clearing needed

    PConfig config = s.getConfig();
    if (config==null) return false; // FIXME, Tim has other ways to tell

    if (GravityWell.liesIn(config, dx, dy)) { // pass radius & shape only...
      addToMsg(msg, "has warp>1 and waypoint within a warpwell,");
      s.setDX(0);
      s.setDY(0);
      addToMsg(msg, "Waypoint reset (dx=dy=0).");
      return true;
    }

    return false;
  }
}
