// File: CheckPlanetColNatTax.java

package jvpc.vpcheck;

import java.util.*;
import kero.util.*;
import jvpc.vga.*;

/**
 * Check whether there are enough colonists on the planet
 * to get all MCs from the natives without overtaxing
 *
 * @author    Rutger Nijlunsing
 * @copyright Rutger Nijlunsing (the GNU General Public License applies)
 * @version   18 XII 2000
 */
class CheckPlanetColNatTax extends Check {
    public boolean check(Vector col, StringBuffer msg) {
	boolean res = false;
	Assert.assert(col.size() == 1);
	msg.setLength(0);
	if (!(col.elementAt(0) instanceof PData)) return false;
	PData p = (PData)col.elementAt(0);

	int natsort = p.getNatSort();
	if ((natsort == UNKNOWN) || (natsort == 0)) return false;

	if (natsort == Natives.AMORPHOUS) {
	    // Amorphous give no tax
	    return false;
	}

	int nocol = p.getColClans();
	if (nocol == UNKNOWN) return false;

	int nonat = p.getNatClans();
	if (nonat == UNKNOWN) return false;

	boolean check = (natsort == Natives.BOVINOIDS);
	if (check) {
	    check = ((nonat / 100) > nocol);
	    if (check) {
		addToMsg(msg, "needs "+ nonat/100 +" clans (instead of "+
			 nocol+" for Bovinoid supplies production");
		res = true;
	    }
	}

	// Find highest tax without overtaxing
	int oldTax = p.getNatTax();
	int tax = 0;
	while (tax < 100) {
	    // TODO: use binary search
	    // TODO: Don't _SET_ the taxrate, just compute the income...
	    p.setNatTax(tax+1);
	    if (p.getNatHappyNT() < p.getNatHappy()) break;
	    tax++;
	}
	p.setNatTax(tax); // TODO: when not setting tax, need not set back...

	int mc;
	addToMsg(msg, "can have native tax of "+tax+"% without overtaxing");
	if (p.getOwner() == 0) {
	    mc = p.getNatTaxMCwoColMax();
	    addToMsg(msg, "for an estimated "+mc+" MC per turn");
	} else {
	    mc = p.getNatTaxMCwoCol();
	    addToMsg(msg, "which produces " + mc +" MC");
	    if (tax > oldTax) res = true;
	}

	p.setNatTax(oldTax); // TODO: when not setting tax, need not reset it

	int clans = p.getColClans();
	if (p.getNatSort() == Natives.INSECTOID) mc /= 2;
	if ((clans != UNKNOWN) && (clans < mc)) {
	    addToMsg(msg, "but has only " + clans + " colonist clans");
	    res = true;
	}

	return res;
    }
}
