// File: CheckPlanetDetectable.java

package jvpc.vpcheck;

import java.util.*;
import kero.util.*;
import jvpc.vga.*;

/**
 * Check whether the planet may be detectable to enemy ships performing
 * ``Sensor Sweep''.
 *
 * @author Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since 5 II 2001
 * @version 5 II 2001
 */
class CheckPlanetDetectable extends Check {
  public boolean check(Vector col, StringBuffer msg) {
    Assert.assert(col.size() == 1);
    Planet p = (Planet)col.elementAt(0);
    PConfig conf = p.getConfig();

    int defForUnd = conf.defenseForUndetectable();
    boolean detectable = (p.getDefense() < defForUnd) &&
	((p.getFactories() > conf.factoriesForDetectable()) ||
	 (p.getMines() > conf.minesForDetectable()));
    int chance = 0; // percentage of detection
    if (detectable) chance = 100 * (defForUnd - p.getDefense()) / defForUnd;

    msg.setLength(0);
    logCheck(detectable, msg,
	     "planet may be scanned by enemy ("+chance+"% per scan)",
	     "planet indetectable");
    return detectable;
  }
}
