// File: CheckShipWarp.java

package jvpc.vpcheck; import jvpc.vga.*; import kero.util.*;

import java.util.*;

/**
 * Check whether
 * another warp gives same ETA but <= 125% or max. 5 more 
 * of current fuel usage
 */
class CheckShipWarp extends Check {
  public boolean check(Vector col, StringBuffer msg) {
    Assert.assert(col.size() == 1);
    msg.setLength(0);
    Ship s = (Ship)col.elementAt(0);
    int curWarp = s.getWarp();
    if (curWarp == UNKNOWN) return false;
    
    // Determine max. fuel to be used 
    int curLeft = s.getNeutrAtWaypoint();
    if (curLeft == UNKNOWN) return false;
    int curUsed = s.getNeutronium() - curLeft;
    int maxUsed = Math.min((int)(curUsed * 1.25), 5);
    int minLeft = Math.max(s.getNeutronium() - maxUsed, 1);

    // Current ETA is max. ETA
    int curEta = s.getTurnsUntilWaypoint();

    // Try all warps
    int bestEta = curEta;
    int bestWarp = s.getWarp();
    int bestFuel = curLeft;
    for (int warp = s.getMaxWarp(); warp >= 1; warp--) {
      if (warp != curWarp) {
	s.setWarp(warp);
	int newEta = s.getTurnsUntilWaypoint();
	int newFuel = s.getNeutrAtWaypoint();
	if (newFuel > minLeft) {
	  // Found a possibility. Is it better?
	  if ((newEta < bestEta) ||
	      ((newEta == bestEta) && (newFuel > bestFuel))) {
	    bestEta = newEta;
	    bestWarp = warp;
	    bestFuel = newFuel;
	  }
	}
	s.setWarp(curWarp);
      }
    }

    if (bestWarp != s.getWarp()) {
      addToMsg(msg, "warp is " + curWarp + ", which gives ETA " + curEta +
	       " and leaves " + curLeft + " neutronium");
      addToMsg(msg, "but warp " + bestWarp + " gives ETA " + bestEta +
	       " and leaves " + bestFuel + " neutronium");
      return true;
    }

    return false;
  }
}
