//File: CheckShipWaypoint.java

package jvpc.vpcheck; import jvpc.vga.*; import kero.util.*;

import java.util.*;


/**
 * Checks whether a ship has a sensable waypoint set
 */  
class CheckShipWaypoint extends Check {
  public boolean check(Vector col, StringBuffer msg) {
    Assert.assert(col.size() == 1);
    Ship s = (Ship)col.elementAt(0);
    msg.setLength(0);

    int dx = s.getDX();
    if (dx==UNKNOWN) return false;
    int dy = s.getDY();
    // if dx is known, so is dy

    Hull h=s.getHull();
    if (h==null) return false;
    /* if (h.isAlchemy() || h.isRefinery() || h.isAdvancedRefinery() ||
       h.canHeatTo50() || h.canHeatTo100() || h.canCoolTo50() )
       return false; // ship does not need a waypoint */

    if (s.getMission() == Mission.INTERCEPT)
	return false; // will move, anyway
    boolean beingTowed = false;
    Vector stack = s.getLocation().getShips();
    for (int i=0; i<stack.size(); i++) {
	Ship tower = (Ship)stack.elementAt(i);
	if (tower == s) {
	    continue;
	}
	if ((tower.getMission() == Mission.TOW) && (tower.getTowTarget() == s.getId())) {
	    beingTowed = true;
	    // addToMsg(msg, "Ship being towed!");
	    if (tower.getWarp() <= s.getWarp()) {
		addToMsg(msg, "Ship being towed, but warp same or higher than tower");
	    }
	}
    }
    if ((dx == 0) && (dy == 0)) {
	if (!beingTowed) {
	    addToMsg(msg,"has no waypoint set (hulltype: "+s.getHull().getName()+")");
	    return true;
	} else {
	    return false;
	}
    }

    int warp = s.getWarp();
    if (warp == UNKNOWN) return false;

    if ((warp > 1) && GravityWell.liesIn(s.getConfig(), dx, dy)) {
      addToMsg(msg, "has warp>1 and a waypoint within warpwell distance");
      return true;
    }

    return false;
  }
}
