// File: RemoveFactories.java

package jvpc.vpcheck; import jvpc.vga.*; import kero.util.*;

import java.util.*;

/**
 * @author    Rutger Nijlunsing
 * @copyright Rutger Nijlunsing (the GNU General Public License applies)
 * @since     13 V 1999
 */
public class RemoveFactories
  implements VGAPlanets
{
  public static final String program="RemoveFactories";
  public static final String version="0.01";

  public static void main(String[] arg) {
    System.out.println(program + " v" + version);

    String gamedir = null;	// Directory for this game
    String planetdir = null;	// Global VGA Planets directory
    String confname = "default";  // Config name
    boolean allplayers = true;	// true iff no players are given
    boolean includeplayer[] = new boolean[players+1]; // players which play iff allplayers == false
    boolean readOnly = false;
    for (int i=0; i<players+1; i++)
      includeplayer[i] = false;
    Game game;
    for (int i=0; i<arg.length; i++) {
      if (arg[i].charAt(0) == '-') {
	// Process option
	if (arg[i].charAt(1) == 'p') {
	  // Include player
	  allplayers = false;
	  int p = Integer.parseInt(arg[i].substring(2));
	  if ((p<0) || (p>players)) {
	    System.out.println("Invalid player number after -p");
	    System.exit(1);
	  }
	  includeplayer[p] = true;
	} else if (arg[i].charAt(1) == 'c') {
	  // Config
	  confname = arg[i].substring(2);
	} else if (arg[i].charAt(1) == 'r') {
	  // ReadOnly
	  readOnly = true;
	} else {
	  System.out.println("Unknown option (" + arg[i] + ")");
	  System.exit(1);
	}
      } else {
	if (gamedir == null) {
	  gamedir = arg[i];
	} else if (planetdir == null) {
	  planetdir = arg[i];
	} else {
	  System.out.println("Only the first two directories are used");
	  System.exit(1);
	}
      }
    }
    if (allplayers) {
      for (int i=0; i<players+1; i++)
	includeplayer[i] = true;
    }
    if (gamedir == null) {
      game=new Game();
    } else if (planetdir == null) {
      game=new Game(arg[0]);
    } else {
      game=new Game(arg[0],arg[1]);
    }
    // game.setReadOnly(readOnly);

    boolean[] wantsToPlay = new boolean[races+1];
    for (int i=1; i<=races; i++) wantsToPlay[i] = true;
    game.readAll(wantsToPlay);
    //Universe hosted = game.getUniverse().host();

    // Ship checks & actions
    Universe utrn=game.getUniverse();
    Universe urst=game.getUniverse(utrn.getTurnNumber(),RstOrTrn.RST);

    boolean changedsomething=false;
    for (int i=1; i<=planets; i++) {
      Planet planet=utrn.getPlanet(i);
      if (planet instanceof PData) {
	PData pdatatrn=(PData)planet;
	planet=urst.getPlanet(i);
	System.err.println(planet.getClass().getName());
	PData pdatarst=(PData)urst.getPlanet(i);
	int defdiff=pdatatrn.getDefense()-pdatarst.getDefense();
	if (defdiff>0) {
	  changedsomething=true;
	  pdatatrn.breakDownDefense(defdiff);
	}
      }
    }

    // Write modified universe back, whenever
    // there is reason to write.
    if (changedsomething) {
      //game.writeAll();
    }
  }
}
