// File: VPCheck.java

package jvpc.vpcheck; import jvpc.vga.*; import kero.util.*;

import java.util.*;

/**
 * VPCheck checks a VGA planets turn for blatant errors and discrepancies.
 * Furthermore, it can assist in doing some automated tasks which
 * have to be done in every turn, like randomizing friendly codes.
 *
 * <P>JVPC is a project by Rutger Nijlunsing and Kero van Gelder. The basic
 * data read-and-write is written by Kero (with a history in Pascal, C and
 * C++), while Rutger added a lot of nice features. For the documentation,
 * @see jvpc.vga.Game
 *
 * @author    Rutger Nijlunsing
 * @copyright Rutger Nijlunsing (the GNU General Public License applies)
 * @since     10 VII 1998
 * @version   5 II 2001
 */
public class VPCheck
  implements VGAPlanets
{
  public static final String program="VPCheck";
  public static final String version="0.05";

  public static void main(String[] arg) {
    System.out.println(program + " v" + version);

    String gamedir = null;	// Directory for this game
    String planetdir = null;	// Global VGA Planets directory
    String confname = "default";  // Config name
    boolean allplayers = true;	// true iff no players are given
    boolean includeplayer[] = new boolean[players+1]; // players which play iff allplayers == false
    boolean readOnly = false;
    for (int i=0; i<players+1; i++)
      includeplayer[i] = false;
    Game game;
    for (int i=0; i<arg.length; i++) {
      if (arg[i].charAt(0) == '-') {
	// Process option
	if (arg[i].charAt(1) == 'p') {
	  // Include player
	  allplayers = false;
	  int p = Integer.parseInt(arg[i].substring(2));
	  if ((p<0) || (p>players)) {
	    System.out.println("Invalid player number after -p");
	    System.exit(1);
	  }
	  includeplayer[p] = true;
	} else if (arg[i].charAt(1) == 'c') {
	  // Config
	  confname = arg[i].substring(2);
	} else if (arg[i].charAt(1) == 'r') {
	  // ReadOnly
	  readOnly = true;
	} else {
	  System.out.println("Unknown option (" + arg[i] + ")");
	  System.exit(1);
	}
      } else {
	if (gamedir == null) {
	  gamedir = arg[i];
	} else if (planetdir == null) {
	  planetdir = arg[i];
	} else {
	  System.out.println("Only the first two directories are used");
	  System.exit(1);
	}
      }
    }
    if (allplayers) {
      for (int i=0; i<players+1; i++)
	includeplayer[i] = true;
    }
    if (gamedir == null) {
      game=new Game();
    } else if (planetdir == null) {
      game=new Game(arg[0]);
    } else {
      game=new Game(arg[0],arg[1]);
    }
    // game.setReadOnly(readOnly);

    boolean[] wantsToPlay = new boolean[races+1];
    for (int i=1; i<=races; i++) wantsToPlay[i] = true;
    game.readAll(wantsToPlay);
    //Universe hosted = game.getUniverse().host();

    VPCheckConfig conf = new VPCheckConfig(confname);
    Vector shipChecks = conf.getShipChecks();
    Vector planetChecks = conf.getPlanetChecks();
    Vector shipActions = conf.getShipActions();
    Vector planetActions = conf.getPlanetActions();

    System.out.println("Performing configuration called '"+confname+"'");

    // Ship checks & actions
    Universe universe=game.getUniverse();
    StringBuffer msg = new StringBuffer();
    Vector v = new Vector();
    v.setSize(1);
    for (int i=1; i<=ships; i++) { // TODO: use iterator
      boolean first = true;  // true if this warning is the first for this ship
      Ship ship=universe.getShip(i);
      if ((ship == null) ||
	  (ship.getOwner() == UNKNOWN) ||
	  !includeplayer[ship.getOwner()]) continue;
      msg.setLength(0);
      v.setElementAt(ship,0);
      // Checks
      for (int checks=0; checks<shipChecks.size(); checks++) {
	Check check = (Check) shipChecks.elementAt(checks);
	boolean res = check.check(v, msg);
	if (res) {
	  if (first) {
	    System.out.println(ship.toString() + " [Owner " + ship.getOwner() +
			       "] :");
	    first = false;
	  }
	  System.out.println(msg);
	}
      }
      // Actions
      for (int actions=0; actions<shipActions.size(); actions++) {
	Action action = (Action) shipActions.elementAt(actions);
	boolean res = action.action(v, new Vector(), msg);
	if (res) {
	  if (first) {
	    System.out.println(ship.toString() +
			       " [Owner " + ship.getOwner() + "] :");
	    first = false;
	  }
	  System.out.println(msg);
	}
      }
    }
    
    // Planet checks & actions
    for (int i=1; i<=planets; i++) { // TODO: use interator
      boolean first = true;
      msg.setLength(0);
      Planet planet=universe.getPlanet(i);
      if ((planet == null) ||
	  (planet.getOwner() == UNKNOWN) ||
	  !includeplayer[planet.getOwner()])
	continue;
      v.setElementAt(planet,0);
      // Checks
      for (int checks=0; checks<planetChecks.size(); checks++) {
	Check check = (Check) planetChecks.elementAt(checks);
	boolean res = check.check(v, msg);
	if (res) {
	  if (first) {
	    System.out.println(planet.toString() +
			       " [Owner " + planet.getOwner() + "]:\n");
	    first = false;
	  };
	  System.out.println(msg);
	}
      }
      // Actions
      for (int actions=0; actions<planetActions.size(); actions++) {
	Action action = (Action) planetActions.elementAt(actions);
	boolean res = action.action(v, new Vector(), msg);
	if (res) {
	  if (first) {
	    System.out.println(planet.toString() +
			       " [Owner " + planet.getOwner() + "] :");
	    first = false;
	  }
	  System.out.println(msg);
	}
      }
    }

    // Write modified universe back, if there is reason to write.
    if ((planetActions.size() != 0) || (shipActions.size() != 0)) {
	game.writeAll();
    }
  }
}
