// File: VPCheckConfig.java

package jvpc.vpcheck;

import java.util.*;
import kero.util.*;
import jvpc.vga.*;

/**
 * All VPCheck configurations.
 * <BR>Currently, "pre", "default", and "all"
 * @copyright (the GNU General Public License applies)
 * @version 5 II 2001
 */
public class VPCheckConfig {
    /* What to do when a check (for something bad) passes, i.e. what action to
     * take.
     */
    Hashtable check2action = new Hashtable(); // is this used? [Kero]

    Vector planetChecks = new Vector();
    Vector shipChecks = new Vector();
    Vector planetActions = new Vector();
    Vector shipActions = new Vector();
    Vector exclude = new Vector(); // int -> ExcludeCheck

    public Vector getPlanetChecks() {
	return planetChecks;
    }

    public Vector getShipChecks() {
	return shipChecks;
    }

    public Vector getPlanetActions() {
	return planetActions;
    }

    public Vector getShipActions() {
	return shipActions;
    }

    /**
     * Return true if check has to be performed
     */
    public boolean mustCheck(Check check, Object obj[]) {
	// Search exclude list
	ExcludeCheck ec = new ExcludeCheck(check, obj);
	for (int i=0; i<exclude.size(); i++) {
	    if (ec.equals(exclude.elementAt(i))) {
		return false;
	    }
	}
	return true;
    }

    /**
     * Return config with name @param name
     */
    public VPCheckConfig(String name) {
	if ("default".equals(name) || ("all".equals(name))) {
	    shipChecks.addElement(new CheckShipFCSpecial());
	    shipChecks.addElement(new CheckShipFCbdm());
	    shipChecks.addElement(new CheckShipFCWRT());
	    shipChecks.addElement(new CheckShipNeutrZero());
	    shipChecks.addElement(new CheckShipMissionSet());
	    shipChecks.addElement(new CheckShipMissionLayMines());
	    shipChecks.addElement(new CheckShipFCmkt());
	    shipChecks.addElement(new CheckShipWaypoint());
	    shipChecks.addElement(new CheckShipWarp());
      	    
	    planetChecks.addElement(new CheckPlanetNatTax());
	    planetChecks.addElement(new CheckPlanetDetectable());
	    planetChecks.addElement(new CheckPlanetFact());
	    planetChecks.addElement(new CheckPlanetColTooMany());
	    planetChecks.addElement(new CheckPlanetFCbum());
	    planetChecks.addElement(new CheckPlanetBaseBuilding());
	} else if ("pre".equals(name)) {
	    shipActions.addElement(new ActionShipNameClear());
	    shipActions.addElement(new ActionShipMissionClear());
	    shipActions.addElement(new ActionShipFCClear());
	    shipActions.addElement(new ActionShipWaypointClear());
	    shipActions.addElement(new ActionShipUnloadLadyRoyale());

	    planetActions.addElement(new ActionPlanetFCRandom());
	    planetActions.addElement(new ActionPlanetMaxStructures());
	} else {
	    System.out.println("Unknown config " + name);
	    System.exit(1);
	}

	if ("all".equals(name)) {
	    shipChecks.addElement(new CheckShipDamage());
	    shipChecks.addElement(new CheckShipCrew());

	    planetChecks.addElement(new CheckPlanetColNatTax());
	    planetChecks.addElement(new CheckPlanetTempTooLow());
	    planetChecks.addElement(new CheckPlanetTempTooHigh());
	    planetChecks.addElement(new CheckPlanetMineralZero());

	    planetChecks.addElement(new CheckBaseDamage());
	}
    }
}

/**
 * A check with objects to exclude
 */
class ExcludeCheck {
    Check check;
    Object obj[];

    public ExcludeCheck(Check check, Object obj[]) {
	this.check = check;
	this.obj = obj;
    }

    public boolean equals(ExcludeCheck ec) {
	return
	    ec.check == check &&
	    obj.equals(ec.obj);
    }
}
