// File: InputCollector.java

package kero.awt.event;

import java.util.Vector;
import java.util.Enumeration;
import java.awt.*;
import java.awt.event.*;
import kero.util.Reporter;

/**
 * Collects several types of events and passes them on to subscribers all as an
 * InputEvent. This should simplify event handling significantly.
 * <P>Furthermore, MyContainer can handle InputEvents in a simple manner,
 * automatically invoking one (parameterless) method.
 *
 * @author Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since 5 I 2001
 * @version 9 I 2001
 */
public class InputCollector
    extends Reporter
    implements ActionListener, FocusListener, TextListener,
	       ItemListener //, AdjustmentListener ?
{
    // We need to pass InputEvents to Listeners via some AWT Multicaster.
    Vector inputListener;

    public InputCollector() {
	inputListener = new Vector();
    }

    public void addInputListener(InputListener listener) {
	inputListener.addElement(listener);
    }

    public void removeInputListener(InputListener listener) {
	inputListener.removeElement(listener);
    }

    public void throwInputEvent(InputEvent ie) {
	Enumeration enum = inputListener.elements();
	while (enum.hasMoreElements()) {
	    InputListener listener = (InputListener)enum.nextElement();
	    listener.inputGiven(ie); // nah, don't edit an event ... clone()?
	}
    }

    public void actionPerformed(ActionEvent ae) {
	throwInputEvent(new InputEvent(ae.getSource(), InputEvent.INPUT_GIVEN));
    }

    public void focusGained(FocusEvent fe) {
	// This usually gives no input, so ignore
    }

    public void focusLost(FocusEvent fe) {
	throwInputEvent(new InputEvent(fe.getSource(), InputEvent.INPUT_GIVEN));
    }

    public void textValueChanged(TextEvent te) {
	throwInputEvent(new InputEvent(te.getSource(), InputEvent.INPUT_GIVEN,
				       te.paramString()));
    }

    public void itemStateChanged(ItemEvent ie) {
	throwInputEvent(new InputEvent(ie.getSource(), InputEvent.INPUT_GIVEN));
    }
}
